/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

static int abook_chg;
static USERBLK abook_ublk 	= {abook_obj, 0L};
static int abook_count = 0, abook_open;
static ABOOK_ENTRY *abook_list;
static int abook_page;

static int atree_ids[] = {AB_USER,
													AB_PASTE,
													AB_ADD,
													AB_DEL,
													AB_EDIT,
													AB_CLOSE,
													0};
static int aentry_ids[]= {AE_SHORT,
													AB_GROUP,
													AE_NAME,
													AE_STRT,
													AE_CITY,
													AE_PHONE,
													AE_FAX,
													AE_EM1,
													AE_EM2,
													0};
												
/* ------------------------------------
   | Find first matching recent entry |
   ------------------------------------ */
int find_matchingaddress(char *set)
{
ABOOK_ENTRY *xsrch;
int no, l;

DEBUG();
l = (int)strlen(set);
xsrch = abook_list;
no = 0;
while (no < abook_count)
	{
	if (!xsrch->group)
		if (!strnicmp(xsrch->email1, set, l))
			{
			strcpy(set, xsrch->email1);
			strcat(set, xsrch->email2);
			return TRUE;
			}
		
	xsrch++;
	no++;
	}
return FALSE;
}

/* ---------------------------
   | Get open Adress element |
   --------------------------- */
int AddressElement(int n)
{
int i, lvl;

DEBUG();
i = 0;
while (i < abook_count && n)
	{
	n--;
	
	if (abook_list[i].group && !abook_list[i].open)
		{
		lvl = abook_list[i].level;
		i++;
		while(i < abook_count && abook_list[i].level > lvl)
			i++;
		}
	else
		i++;
	}

return i;
}

/* ----------------------------
   | User object address book |
   ---------------------------- */
int cdecl abook_obj(PARMBLK *parm)
{
int xy[4], i, lvl, y, sel;
int dummy;

DEBUG();
obj_clip(parm, TRUE);

vst_height(vdi_handle, default_fsize, &dummy, &dummy, &dummy, &dummy);
vswr_mode(vdi_handle, MD_REPLACE);
vsf_interior(vdi_handle, FIS_HOLLOW);
xy[0] = parm->pb_x;
xy[1] = parm->pb_y;
xy[2] = parm->pb_x + parm->pb_w - 2;
xy[3] = parm->pb_y + parm->pb_h - 1;
vr_recfl(vdi_handle, xy);

y = xy[1];

i = AddressElement(abook_page);
while (i < abook_count && y < parm->pb_y + parm->pb_h)
	{
	lvl = abook_list[i].level;
	sel = abook_list[i].sel;
	
	if (abook_list[i].group)
		{
		v_gtext(vdi_handle, parm->pb_x + 2 * tx_width * lvl + 25, y, abook_list[i].shortname);
		
		if (!abook_list[i].open)	/* Skip entries of closed group */
			{
			icon_copy(symbols, 0, (smallfont ? 8 : 16) * 2, 
							 parm->pb_x + 2 * tx_width * lvl, y, 
							 20, smallfont ? 8 : 16);
			i++;
			while (i < abook_count && abook_list[i].level > lvl)
				i++;
			}
		else
			{
			icon_copy(symbols, 0, (smallfont ? 8 : 16) * 1, 
							 parm->pb_x + 2 * tx_width * lvl, y, 
							 20, smallfont ? 8 : 16);
			i++;
			}
		}
	else
		{
		icon_copy(symbols, 0, 0, 
						 parm->pb_x + 2 * tx_width * lvl, y, 
						 20, smallfont ? 8 : 16);
		v_gtext(vdi_handle, parm->pb_x + 2 * tx_width * lvl + 25, y, abook_list[i].shortname);
		i++;
		}

	if (sel)
		{
		vswr_mode(vdi_handle, MD_XOR);
		vsf_interior(vdi_handle, FIS_SOLID);
		vsf_color(vdi_handle, 1);

		xy[0] = parm->pb_x + 2 * tx_width * lvl - 1 + 25;
		xy[1] = y;
		xy[2] = parm->pb_x + parm->pb_w - 2;
		xy[3] = y + tx_height - 1;
		vr_recfl(vdi_handle, xy);
		vswr_mode(vdi_handle, MD_TRANS);
		}

	y += tx_height;
	}

obj_clip(parm, FALSE);
vswr_mode(vdi_handle, MD_REPLACE);

return 0;
}

/* ---------------------
   | Set to group mode |
   --------------------- */
void AbookSetGroup(OBJECT *aentry, int group_flg, int red_flag)
{
static int aeids[] = {AE_NAME, AE_STRT, AE_CITY, AE_PHONE,
											AE_FAX, AE_EM1, AE_EM2};
int i;
											
DEBUG();
for (i = 0; i < 7; i++)
	{
	if (group_flg)
		DisableObj(aentry, aeids[i]);
	else
		EnableObj(aentry, aeids[i]);
	if (red_flag)
		redraw_object(aentry, aeids[i]);
	}
}

/* ---------------------------
   | Edit address book entry |
   --------------------------- */
int	edit_abook(ABOOK_ENTRY *edit_entry)
{
OBJECT *aentry;
int obj;

DEBUG();
rsrc_gaddr(R_TREE, ABENTRY, &aentry);

aentry[AE_SHORT].ob_spec.tedinfo->te_ptext 	= edit_entry->shortname;
aentry[AE_NAME].ob_spec.tedinfo->te_ptext 	= edit_entry->name;
aentry[AE_STRT].ob_spec.tedinfo->te_ptext 	= edit_entry->street;
aentry[AE_CITY].ob_spec.tedinfo->te_ptext 	= edit_entry->city;
aentry[AE_PHONE].ob_spec.tedinfo->te_ptext 	= edit_entry->phone;
aentry[AE_FAX].ob_spec.tedinfo->te_ptext 		= edit_entry->fax;
aentry[AE_EM1].ob_spec.tedinfo->te_ptext 		= edit_entry->email1;
aentry[AE_EM2].ob_spec.tedinfo->te_ptext 		= edit_entry->email2;

if (edit_entry->shortname[0])
	DisableObj(aentry, AB_GROUP);
else
	EnableObj(aentry, AB_GROUP);
	
SetSelect(aentry, AB_GROUP, edit_entry->group);
AbookSetGroup(aentry, edit_entry->group, FALSE);

open_dialog(aentry, NULL, iconified_tree);
dlgset_help_info(aentry, helpfile, aentry_ids, "AE_");
do
	{
	obj = do_dialog(aentry, AE_SHORT, app_callback);
	if (obj == AB_GROUP)
		{
		AbookSetGroup(aentry, IsSelected(aentry, AB_GROUP), TRUE);
		}
		
	}while(obj != AE_OK && obj != AE_CANCL);
	
close_dialog(aentry, TRUE);

edit_entry->group = IsSelected(aentry, AB_GROUP);

return obj == AE_OK;
}

/* ----------------
   | Paste recent |
   ---------------- */
void paste_address(int num, int drag_paste)
{
char *buffer;
int wind;

DEBUG();
buffer = (char *)malloc(512);
if (!abook_list[num].group)
	{
	strcpy(buffer, abook_list[num].email1);
	strcat(buffer, abook_list[num].email2);
	}
else
	{
	strcpy(buffer, "^");
	strcat(buffer, abook_list[num].shortname);
	strcat(buffer, "^");
	}

if (!drag_paste)
	wind = abookset_wind;
else
	wind = wind_find(evnt.x, evnt.y);
	
if (wind == mail_wind)		
	{
	if (drag_paste) mail_button();
	paste_mailadr(buffer);
	}
else
	if (wind == get_dialog_wind(fwd_tree))
		{
#ifndef EVAL
		fwd_paste(buffer);
#endif		
		}
			
free(buffer);
}

/* ---------------------------
   | Set address book slider |
   --------------------------- */
void abook_slider(void)
{
int obs;
int i, lvl;

DEBUG();
obs = (abook_tree[AB_USER].ob_height + tx_height - 1) / tx_height;

abook_open = 0;
i = 0;
while (i < abook_count)
	{
	abook_open++;
	if (abook_list[i].group && !abook_list[i].open)
		{
		lvl = abook_list[i].level;
		i++;
		while(i < abook_count && abook_list[i].level > lvl)
			i++;
		}
	else
		i++;
	}

if (abook_page > abook_open - obs && abook_page)
	{
	abook_page = abook_open - obs;
	if (abook_page < 0) abook_page = 0;
	redraw_object(abook_tree, AB_USER);
	}

set_slider_objects(abook_tree, AB_SLD, AB_SLDB, abook_open, abook_page, obs, TRUE);	
}

/* ---------------------
   | Draw the drag box |
   --------------------- */
void drag_box(int x, int y, int w, int h)
{
int xy[10];

DEBUG();
vswr_mode(vdi_handle, MD_XOR);
vsl_type(vdi_handle, 7);
vsl_udsty(vdi_handle, 0x5555);
vsl_width(vdi_handle, 1);

xy[0] = x - (w >> 1);
xy[1] = y - (h >> 1);
xy[2] = x + (w >> 1);
xy[3] = xy[1];
xy[4] = xy[2];
xy[5] = y + (h >> 1);
xy[6] = xy[0];
xy[7] = xy[5];
xy[8] = xy[0];
xy[9] = xy[1];
graf_mouse(M_OFF, 0L);
v_pline(vdi_handle, 5, xy);
graf_mouse(M_ON, 0L);
}

/* --------------------------------
   | Mark destination information |
   -------------------------------- */
void dest_box(int x, int y, int w, int h)
{
int xy[10];

DEBUG();
vswr_mode(vdi_handle, MD_XOR);
vsf_interior(vdi_handle, FIS_PATTERN);
vsf_style(vdi_handle, 1);
vsf_color(vdi_handle, 1);

xy[0] = x;
xy[1] = y;
xy[2] = x + w - 1;
xy[3] = y + h - 1;
graf_mouse(M_OFF, 0L);
vr_recfl(vdi_handle, xy);
graf_mouse(M_ON, 0L);
}

/* ---------------------------
   | Drag address book entry |
   --------------------------- */
void drag_aentry(int pos)
{
int ox, oy, error;
int obs, obj, objx, objy, pobj = -1;
int objw, lvl, olvl, i;
EVNTDATA ev;

DEBUG();
objc_offset(abook_tree, AB_USER, &objx, &objy);
objw = abook_tree[AB_USER].ob_width - 1;

obs = (abook_tree[AB_USER].ob_height + tx_height - 1) / tx_height;

vswr_mode(vdi_handle, MD_XOR);
graf_mkstate(&ev);

drag_box(ev.x, ev.y, objw, tx_height);
ox = ev.x; oy = ev.y;
do
	{
	graf_mkstate(&ev);
	
	if (ev.x != ox || ev.y != oy)
		{
		if (pobj > -1)
			dest_box(objx, objy + pobj * tx_height, objw, tx_height);
		drag_box(ox, oy, objw, tx_height);

		if (ev.x >= objx && ev.x < objx + objw)
			{
			obj = (ev.y - objy - tx_height / 2) / tx_height;
			if (obj < 0)
				{
				if (abook_page > 0 && obj == -1)
					{
					abook_page--;
					redraw_object(abook_tree, AB_USER);
					abook_slider();
					delay(50);
					pobj = -1;
					}
				else
					pobj = -1;
				}
			else
				if (obj >= obs)
					{
					if (abook_page < abook_open - obs && obj == obs)
						{
						abook_page++;
						redraw_object(abook_tree, AB_USER);
						abook_slider();
						delay(50);
						pobj = -1;
						}
					else
						pobj = -1;
					}
				else
					{
					if (pobj != obj)
						{
						if (abook_page + obj <= abook_open)
							pobj = obj;
						else
							pobj = -1;
						}
					}
			}
		else
			pobj = -1;
			
		drag_box(ev.x, ev.y, objw, tx_height);
		if (pobj > -1)
			dest_box(objx, objy + pobj * tx_height, objw, tx_height);
		ox = ev.x; oy = ev.y;
		}
		
	}while(ev.bstate);

if (pobj > -1)
	dest_box(objx, objy + pobj * tx_height, objw, tx_height);
drag_box(ox, oy, abook_tree[AB_USER].ob_width - 1, tx_height);

vswr_mode(vdi_handle, MD_REPLACE);

if (pobj > -1)
	{
	obj = AddressElement(abook_page + pobj);
	if (obj != pos)
		{
		error = FALSE;
		
		if (obj < abook_count)
			if (abook_list[obj].group && abook_list[obj].open)
				{
				lvl = abook_list[obj].level + 1;
				obj++;
				}
			else
				lvl = abook_list[obj].level;
		else
			lvl = 0;
			
		olvl = abook_list[pos].level;
		
		if (abook_list[pos].group)
			{
			i = pos + 1;	/* Avoid moving inside yourself */
			while(i < abook_count && abook_list[i].level > olvl)
				{
				if (i == obj)
					{
					error = TRUE;
					break;
					}
				i++;
				}

			if (!error)
				{
				i = pos + 1;
				while(i < abook_count && abook_list[i].level > olvl)
					{
					abook_list[i].level = abook_list[i].level - olvl + lvl;
					i++;
					}
				i -= pos;
				}
			}
		else
			i = 1;
		
		if (!error)
			{	
			abook_list[pos].level = lvl;
				
			abook_list = (ABOOK_ENTRY *)realloc(abook_list,
																sizeof(ABOOK_ENTRY) * (abook_count + i));
	
			if (obj < abook_count)
				{
				memmove(abook_list + obj + i, abook_list + obj, 
								sizeof(ABOOK_ENTRY) * (abook_count - obj));
				if (pos > obj)
					pos += i;
				}
			memcpy(abook_list + obj, abook_list + pos, 
						 sizeof(ABOOK_ENTRY) * i);
						 
			if (pos + i <= abook_count)
				memmove(abook_list + pos, abook_list + pos + i,
								sizeof(ABOOK_ENTRY) * (abook_count - pos));
								
			abook_list = (ABOOK_ENTRY *)realloc(abook_list,
																sizeof(ABOOK_ENTRY) * abook_count);
			}
		redraw_object(abook_tree, AB_USER);
		}
	}
else
	{
	graf_mkstate(&ev); /* !!! Mouse_x mouse_y update? */
	evnt.x = ev.x;
	evnt.y = ev.y;
	evnt.bstate = 1;
	paste_address(pos, TRUE);
	}
}


/* -----------------------------
   | Handle Adress-Book events |
   ----------------------------- */
static void cdecl abook_handler (OBJECT *tree, int obj)
{
int i;
ABOOK_ENTRY edit_entry;

DEBUG();
switch (obj & 0x7FFF)
	{
	case AB_CLOSE:
		{
		save_addresses();
		close_dialog(tree, TRUE);
		break;
		}
		
	case AB_UP:
		change_object(tree, AB_UP, tree[AB_UP].ob_state & ~SELECTED);
		if (abook_page > 0)
			{
			abook_page--;
			redraw_object(tree, AB_USER);
			}
		break;
			
	case AB_DOWN:
		{
		int obs;
		
		change_object(tree, AB_DOWN, tree[AB_DOWN].ob_state & ~SELECTED);
		obs = (tree[AB_USER].ob_height + tx_height - 1) / tx_height;
		if (abook_page < abook_open - obs)
			{
			abook_page++;
			redraw_object(tree, AB_USER);
			}
		break;
		}
		
	case AB_SLD:
		{
		int ox, oy;
		EVNTDATA ev;
		int obs;
		
		obs = (tree[AB_USER].ob_height + tx_height - 1) / tx_height;
		graf_mkstate(&ev);
		objc_offset(tree, AB_SLDB, &ox, &oy);
		if (ev.y < oy)
			{
			abook_page -= obs;
			if (abook_page < 0) abook_page = 0;
			}
		else
			{
			abook_page += obs;
			if (abook_page > abook_open - obs)
				abook_page = abook_open - obs;
			}
		redraw_object(tree, AB_USER);
		break;
		}
					
	case AB_SLDB:
		{
		int pos;
		int obs;
		
		pos = graf_slidebox(tree, AB_SLD, AB_SLDB, 1);
		obs = (tree[AB_USER].ob_height + tx_height - 1) / tx_height;
		
		abook_page = (int)(pos * (long)(abook_open - obs) / 1000);
		redraw_object(tree, AB_USER);
		break;
		}
		
	case AB_ADD:
		memset(&edit_entry, 0, sizeof(edit_entry));
		if (edit_abook(&edit_entry))
			{
			if (!abook_count)
				abook_list = (ABOOK_ENTRY *)malloc(sizeof(ABOOK_ENTRY));
			else
				abook_list = (ABOOK_ENTRY *)realloc(abook_list,
												sizeof(ABOOK_ENTRY) * (abook_count + 1));

			i = 0;
			while(i < abook_count)
				{
				if (abook_list[i].sel)
					break;
				i++;
				}
					
			if (i < abook_count)
				{
				edit_entry.level = abook_list[i].level;
				if (abook_list[i].group)
					{
					edit_entry.level++;
					abook_list[i].open = 1;
					}
				
				i++;
				if (i < abook_count)
					memmove(abook_list + i + 1, abook_list + i, 
									sizeof(ABOOK_ENTRY) * (abook_count - i));
				}
			else
				i = abook_count;
				
			memcpy(abook_list + i, &edit_entry, sizeof(ABOOK_ENTRY));
			abook_count++;
#ifdef MILAN
			if (abook_count >= 5)
				DisableObj(tree, AB_ADD);
#endif
			abook_chg = TRUE;
			}
		break;
		
	case AB_DEL:
		{
		i = 0;
		while (i < abook_count)
			{
			if (abook_list[i].sel)
				{
				if (abook_list[i].group)
					{
					while (i < abook_count - 1 &&
								 abook_list[i + 1].level > abook_list[i].level)
						{
						if (i < abook_count - 2)
							memmove(abook_list + i + 1, abook_list + i + 2,
											sizeof(ABOOK_ENTRY) * (abook_count - i - 2));
						abook_count--;
						}
					}

				if (i < abook_count - 1)
					memmove(abook_list + i, abook_list + i + 1,
									sizeof(ABOOK_ENTRY) * (abook_count - i - 1));
				abook_count--;
				
				if (!abook_count)
					free(abook_list);
				else
					abook_list = (ABOOK_ENTRY *)realloc(abook_list, 
												sizeof(ABOOK_ENTRY) * abook_count);

				redraw_object(tree, AB_USER);
				break;
				}
			i++;
			}
#ifdef MILAN				
		if (abook_count < 5)
			EnableObj(tree, AB_ADD);
#endif				
		}
		break;
		
	case AB_EDIT:
		{
		i = abook_page;
		while (i < abook_count)
			{
			if (abook_list[i].sel)
				{
				abook_list[i].sel = 0;
				memcpy(&edit_entry, abook_list + i, sizeof(edit_entry));
				if (edit_abook(&edit_entry))
					{
					memcpy(abook_list + i, &edit_entry, sizeof(edit_entry));
					abook_chg = TRUE;
					}
					
				break;
				}
			i++;
			}
		break;
		}
		
	case AB_PASTE:
		{
		i = 0; 
		while(i < abook_count)
			{
			if (abook_list[i].sel)
				{
				paste_address(i, FALSE);
				break;
				}
		
			i++;
			}
		break;
		}
		
	case AB_USER:
		{
		int ox, oy, pos, obs;
		int i;
		EVNTDATA ev;
		
		obs = (tree[AB_USER].ob_height + tx_height - 1) / tx_height;
		
		objc_offset(tree, AB_USER, &ox, &oy);
		graf_mkstate(&ev);
		
		pos = (ev.y - oy) / tx_height;
		if (pos >= 0 && pos < obs)
			{
			pos = AddressElement(abook_page + pos);
			if (pos < abook_count)
				{
				for (i = 0; i < abook_count; i++)
					if (i != pos)
						abook_list[i].sel = 0;

				if (obj & 0x8000)
					{
					if (abook_list[pos].group)
						{
						abook_list[pos].open ^= 1;
						abook_list[pos].sel = 0;
						}
					else
						paste_address(pos, FALSE);
					}
				else	
					abook_list[pos].sel ^= 1;
					
				abook_chg = TRUE;
				redraw_object(tree, AB_USER);
				
				if (ev.bstate)
					drag_aentry(pos);
				}
			}
		break;
		}
	}			
set_abookdialog();
}

/* ------------------------------
   | Work with the address book |
   ------------------------------ */
void address_book(void)
{
DEBUG();
rsrc_gaddr(R_TREE, ABOOK, &abook_tree);

if (!is_open(abook_tree))
	{
	wind_get_int(ROOT, WF_TOP, &abookset_wind);
	
	abook_tree[AB_USER].ob_type = G_USERDEF;
	abook_tree[AB_USER].ob_spec.userblk = &abook_ublk;
	abook_page = 0;
	
	#ifdef MILAN
		if (abook_count >= 5)
			DisableObj(abook_tree, AB_ADD);
		else
			EnableObj(abook_tree, AB_ADD);
	#endif

	align_slider_objects(abook_tree, AB_SLDBKG, AB_UP, AB_SLD, AB_SLDB, AB_DOWN);
	
	open_dialog(abook_tree, NULL, iconified_tree);
	dlgset_help_info(abook_tree, helpfile, atree_ids, "AB_");
	set_abookdialog();
	set_active(abook_tree, abook_handler, TRUE, 0);
	}
else
	open_dialog(abook_tree, NULL, iconified_tree);
}

/* -----------------------------
   | Set Abook dialog controls |
   ----------------------------- */
void set_abookdialog(void)
{
int i;

DEBUG();
i = 0;
while(i < abook_count)
	{
	if (abook_list[i].sel)
		break;
	i++;
	}

if (i < abook_count)
	{
	if (abook_tree[AB_DEL].ob_state & DISABLED)
		{
		EnableObj(abook_tree, AB_DEL);
		EnableObj(abook_tree, AB_EDIT);
		EnableObj(abook_tree, AB_PASTE);
		redraw_object(abook_tree, AB_DEL);
		redraw_object(abook_tree, AB_EDIT);
		redraw_object(abook_tree, AB_PASTE);
		}
	}
else
	{
	if (!(abook_tree[AB_DEL].ob_state & DISABLED))
		{
		DisableObj(abook_tree, AB_DEL);
		DisableObj(abook_tree, AB_EDIT);
		DisableObj(abook_tree, AB_PASTE);
		redraw_object(abook_tree, AB_DEL);
		redraw_object(abook_tree, AB_EDIT);
		redraw_object(abook_tree, AB_PASTE);
		}
	}

abook_slider();
}

/* ----------------------------
   | Save recent address list |
   ---------------------------- */
void save_addresses(void)
{
long abook_fl;
char buf[256];

DEBUG();
if (!abook_chg) return;

base_path(buf);
strcat(buf, "ABOOK.DAT");
abook_fl = Fcreate(buf, 0);
if (abook_fl > 0)
	{
	Fwrite((int)abook_fl, sizeof(abook_count), &abook_count);
	if (abook_count)
		Fwrite((int)abook_fl, sizeof(ABOOK_ENTRY) * abook_count,
					 abook_list);
	Fclose((int)abook_fl);
	abook_chg = FALSE;
	}
}

/* ----------------------------
   | Save recent address list |
   ---------------------------- */
void load_addresses(void)
{
long abook_fl;
char buf[256];

DEBUG();
if (abook_count)
	free(abook_list);
	
base_path(buf);
strcat(buf, "ABOOK.DAT");

abook_fl = Fopen(buf, FO_READ);
if (abook_fl > 0)
	{
	Fread((int)abook_fl, sizeof(abook_count), &abook_count);
	if (abook_count)
		{
		abook_list = (ABOOK_ENTRY *)malloc(sizeof(ABOOK_ENTRY) * 
																		   abook_count);
																		   
		Fread((int)abook_fl, sizeof(ABOOK_ENTRY) * abook_count,
				  abook_list);
		}
		
	Fclose((int)abook_fl);
	}
else
	abook_count = 0;
	
abook_chg = FALSE;
}

/* -----------------------------
   | Get Adress group adresses |
   ----------------------------- */
char **get_group_address(char *name)
{
char **addrlist, *ptr;
int addrcnt = 0, i, j;

DEBUG();
for (i = 0; i < abook_count; i++)
	if (!strcmp(abook_list[i].shortname, name))
		break;

if (i < abook_count)
	{
	j = i + 1;
	while(j < abook_count && abook_list[j].level > abook_list[i].level)
		{
		if (!abook_list[j].group && abook_list[j].email1[0])
			{
			if (!addrcnt)
				addrlist = (char **)malloc(sizeof(char *));
			else
				addrlist = (char **)realloc(addrlist, sizeof(char *) * (addrcnt + 1));

			addrlist[addrcnt++] = ptr =
				(char *)malloc(strlen(abook_list[j].email1) + 
											 strlen(abook_list[j].email2) + 1);
			strcpy(ptr, abook_list[j].email1);
			strcat(ptr, abook_list[j].email2);
			}
		j++;
		}
	}

if (!addrcnt)
	addrlist = (char **)malloc(sizeof(char *));
else
	addrlist = (char **)realloc(addrlist, sizeof(char *) * (addrcnt + 1));
addrlist[addrcnt++] = NULL;

return addrlist;
}
