/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

#ifdef EVAL
# define ONE_ATTACHMENT
#endif

#ifdef MILAN
# define ONE_ATTACHMENT
#endif

static char **zw_attachments;
static int 	*zw_attencoding;
static int  zw_attach_count;
static int  att_page;
static char attach_path[256];
static OBJECT *att_tree, *att_popup;
static int first_att = TRUE;

void SetEncodingInfo(int no);

static int att_ids[] = {ENCODNAM,
												AT_1,
												AT_2,
												AT_3,
												AT_4,
												AT_5,
												AT_6,
												AT_7,
												AT_NEW,
												AT_DELET,
												AT_CANCL,
												AT_OKAY,
												0};
												
/* ---------------------------------
   | Set attaches in attach dialog |
   --------------------------------- */
void set_att_page(int flg)
{
int i;

DEBUG();
i = 0;
while (i < 7)
	{
	if (att_page + i < zw_attach_count)
		{
		att_tree[AT_1 + i].ob_state &= ~(DISABLED|SELECTED);
		if (strlen(zw_attachments[att_page + i]) > 45)
			att_tree[AT_1 + i].ob_spec.tedinfo->te_ptext = zw_attachments[att_page + i] + strlen(zw_attachments[att_page + i]) - 45;
		else
			att_tree[AT_1 + i].ob_spec.tedinfo->te_ptext = zw_attachments[att_page + i];
		}
	else
		{
		att_tree[AT_1 + i].ob_state |= DISABLED;
		att_tree[AT_1 + i].ob_state &= ~SELECTED;
		att_tree[AT_1 + i].ob_spec.tedinfo->te_ptext = "\0";
		}
	i++;
	}

set_slider_objects(att_tree, AT_SLD, AT_DRG, zw_attach_count, att_page, 7, flg);	
		
att_tree[ENCODNAM].ob_spec.tedinfo->te_ptext[0] = 0;
DisableObj(att_tree, ENCODNAM);
DisableObj(att_tree, ENC_SEL);

if (flg)
	{
	redraw_object(att_tree, ENCODNAM);
	redraw_object(att_tree, ENC_SEL);
	objc_draw(att_tree, AT_BKG, MAX_DEPTH, (GRECT *)&att_tree[0].ob_x);
	}
}

/* --------------------------------
   | Delete an attached from list |
   -------------------------------- */
void delete_attach(void)
{
int i, j;

DEBUG();
for (i = 0; i < 7; i++)
	if (att_tree[AT_1 + i].ob_state & SELECTED)
		{
		for (j = 0; j < epart_count; j++)
			if (zw_attachments[att_page + i] == edit_parts[j].name)
				break;
				
		if (j == epart_count)
			free(zw_attachments[att_page + i]);
		
		for (j = i; j < zw_attach_count - 1; j++)
			{
			zw_attencoding[att_page + j] = zw_attencoding[att_page + j + 1];
			zw_attachments[att_page + j] = zw_attachments[att_page + j + 1];
			}
		zw_attach_count--;

		if (!zw_attach_count)
			{
			free(zw_attachments);
			free(zw_attencoding);
			}
			
		if (att_page && att_page + 7 >= zw_attach_count)
			att_page--;
		
		set_att_page(TRUE);	
		break;
		}
}

/* --------------------------------
   | Select a new attachment file |
   -------------------------------- */
void select_attach(void)
{
int ex_button;
int l;
char attach_file[64];
char buf[256];

DEBUG();
if (first_att)
	{
	attach_path[0] = Dgetdrv() + 'A';
	attach_path[1] = ':';
	Dgetpath(attach_path + 2, 0);
	strcat(attach_path, "\\*.*");
	first_att = FALSE;
	}
	
attach_file[0] = 0;
if (fsel_input(attach_path, attach_file, &ex_button) && ex_button && attach_file[0])
	{
	strcpy(buf, attach_path);
	if (strchr(buf, '\\'))
		strrchr(buf, '\\')[1] = 0;
	strcat(buf, attach_file);

	l = (int)strlen(buf);
	
	if (!zw_attach_count)
		{
		zw_attachments = (char **)malloc(sizeof(char *));
		zw_attencoding = (int *)malloc(sizeof(int));
		}
	else
		{
		zw_attachments = (char **)realloc(zw_attachments, 
															sizeof(char *) * (zw_attach_count + 1));
		zw_attencoding = (int *)realloc(zw_attencoding,
																		sizeof(int) * (zw_attach_count + 1));
		}
	zw_attencoding[zw_attach_count] = -1;
	zw_attachments[zw_attach_count] = malloc(l + 1);
	strcpy(zw_attachments[zw_attach_count++], buf);
	if (zw_attach_count > 7)
		att_page++;

	set_att_page(TRUE);
	att_tree[AT_1 + zw_attach_count - att_page - 1].ob_state |= SELECTED;
	objc_draw(att_tree, AT_1 + zw_attach_count - att_page - 1, 
						MAX_DEPTH, 
						(GRECT *)&att_tree[0].ob_x);
	SetEncodingInfo(zw_attach_count - 1);
	}
}

/* -----------------------------
   | Show encoding information |
   ----------------------------- */
void SetEncodingInfo(int no)
{
static int enc_strgs[] = {ENC_AUTO, ENC_7BIT, ENC_QUOT,
												  ENC_B64, ENC_8BIT, 0,
												  /*ENC_UU*/0, 0, 0, 0, ENC_MSG};
int j;

DEBUG();
strcpy(att_tree[ENCODNAM].ob_spec.tedinfo->te_ptext,
			 att_popup[enc_strgs[zw_attencoding[no] + 1]].ob_spec.free_string + 1);

for (j = 0; j < epart_count; j++)
	if (edit_parts[j].name == zw_attachments[no])
		break;

if (j == epart_count)		
	{
	EnableObj(att_tree, ENCODNAM);
	EnableObj(att_tree, ENC_SEL);
	}
else
	{
	DisableObj(att_tree, ENCODNAM);
	DisableObj(att_tree, ENC_SEL);
	}
redraw_object(att_tree, ENCODNAM);
redraw_object(att_tree, ENC_SEL);
}

/* ----------------
   | Attach files |
   ---------------- */
void attach_file(void)
{
int i, e, j;

DEBUG();
rsrc_gaddr(R_TREE, D_ATT, &att_tree);
rsrc_gaddr(R_TREE, ENCPOPUP, &att_popup);

zw_attach_count = new_buffer.attach_count;
if (zw_attach_count)
	{
	zw_attachments = (char **)malloc(sizeof(char *) * zw_attach_count);
	memcpy(zw_attachments, new_buffer.attachments, sizeof(char *) * zw_attach_count);
	zw_attencoding = (int *)malloc(sizeof(int) * zw_attach_count);
	memcpy(zw_attencoding, new_buffer.att_encoding, sizeof(int) * zw_attach_count);
	}

align_slider_objects(att_tree, AT_SBK, AT_UP, AT_SLD, AT_DRG, AT_DOWN);

att_page = 0;
set_att_page(FALSE);

open_dialog(att_tree, NULL, iconified_tree);
dlgset_help_info(att_tree, helpfile, att_ids, "ATT_");

do
	{
	e = do_dialog(att_tree, 0, app_callback);
	
	switch(e)
		{
		case AT_UP:
			if (att_page > 0)
				{
				att_page--;
				set_att_page(TRUE);	
				}
			break;
		
		case AT_DOWN:
			if (att_page + 7 < zw_attach_count)
				{
				att_page++;
				set_att_page(TRUE);	
				}
			break;
		
		case AT_DRG:
			{
			int pos = graf_slidebox(att_tree, AT_SLD, AT_DRG, 1);
			att_page = (int)(pos * (long)(zw_attach_count - 7) / 1000);
			set_att_page(TRUE);	
			break;
			}
			
		case AT_SLD:
			{
			int ox, oy;
			EVNTDATA ev;
			
			objc_offset(att_tree, AT_DRG, &ox, &oy);
			graf_mkstate(&ev);
			
			if (ev.y < oy)
				{
				if (att_page > 0)
					{
					att_page -= 7;
					if (att_page < 0)
						att_page = 0;
					set_att_page(TRUE);	
					}
				}
			else
				{
				if (att_page + 7 < zw_attach_count)
					{
					att_page += 7;
					if (att_page + 7 >= zw_attach_count)
						att_page = zw_attach_count - 7;
					set_att_page(TRUE);	
					}
				}
			delay(20);
			break;
			}
				
		case ENCODNAM:
		case ENC_SEL:
			{
			int ox, oy;
			int sel, i;
			
			objc_offset(att_tree, ENCODNAM, &ox, &oy);
			switch (positioned_popup(att_popup, ox, oy))
				{
				case ENC_AUTO: sel = -1; 	break;
				case ENC_8BIT: sel = 3; 	break;
				case ENC_7BIT: sel = 0; 	break;
				case ENC_QUOT: sel = 1;		break;
				case ENC_B64:	 sel = 2;		break;
				case ENC_MSG:	 sel = 9;		break;
/*				case ENC_UU:	 sel = 5; 	break;*/
				default:			 sel = -1; 	break;
				}

			for (i = 0; i < 7; i++)
				if (att_tree[AT_1 + i].ob_state & SELECTED)
					{
					zw_attencoding[att_page + i] = sel;
					SetEncodingInfo(att_page + i);
					}
			break;
			}
			
		case AT_NEW:
#ifdef ONE_ATTACHMENT		
			if (!zw_attach_count)
#endif
				select_attach();
#ifdef ONE_ATTACHMENT		
			else
				{
				open_dialog(limits, NULL, NULL);
				do_dialog(limits, 0, (CALLBACK)NULL);
				close_dialog(limits, TRUE);
				}
#endif
			break;
		
		case AT_DELET:		
			delete_attach();
			break;
		
		default:
			if (e >= AT_1 && e <= AT_7)
				SetEncodingInfo(att_page + e - AT_1);
			break;
		}
		
	}while(e != AT_OKAY && e != AT_CANCL);

if (e == AT_OKAY)
	{
	if (new_buffer.attach_count)
		{
		for (i = 0; i < new_buffer.attach_count; i++)
			{
			for (j = 0; j < epart_count; j++)
				if (edit_parts[j].name == new_buffer.attachments[i])
					break;
					
			if (j == epart_count)
				{
				for (j = 0; j < zw_attach_count; j++)
					if (zw_attachments[j] == new_buffer.attachments[i])
						break;
				}
			else
				j = -1;
			
			if (j == zw_attach_count)
				free(new_buffer.attachments[i]);
			}
			
		free(new_buffer.attachments);
		free(new_buffer.att_encoding);
		}
		
	new_buffer.attach_count = zw_attach_count;
	new_buffer.attachments = zw_attachments;
	new_buffer.att_encoding = zw_attencoding;
	new_buffer.changed = TRUE;
	}
else
	{
	for (i = 0; i < zw_attach_count; i++)
		{
		for (j = 0; j < epart_count; j++)
			if (edit_parts[j].name == zw_attachments[i])
				break;
					
		if (j == epart_count)
			{
			for (j = 0; j < new_buffer.attach_count; j++)
				if (zw_attachments[i] == new_buffer.attachments[j])
					break;
			}
		else
			j = -1;
			
		if (j == new_buffer.attach_count)
			free(zw_attachments[i]);
		}
			
	free(zw_attachments);
	free(zw_attencoding);
	}
		
close_dialog(att_tree, TRUE);
}

/* --------------------
   | Store attachment |
   -------------------- */
void store_attachment(char *file, int msg, char *boundary, long endpos, char *type)
{
long out_att;
int i, cod;
char *line, *p;

DEBUG();
clear_inputbuffer();

out_att = Fcreate(file, 0);
if (out_att > 0)
	{
	line = (char *)malloc(1001);

	cod = -1;
	for (i = 0; i < 6; i++)
		if (!strnicmp(type, enc_tab[i], strlen(enc_tab[i])))
			cod = i;

	if (cod == -1)
		{
		do
			{
			if (!get_line(msg, line, 1000))
				break;
			
			if (!line[0]) break;
			
			if (!strnicmp(line, "Content-transfer-encoding:", 26))
				{
				p = parameter(line);
				for (i = 0; i < 6; i++)
					if (!strnicmp(p, enc_tab[i], strlen(enc_tab[i])))
						cod = i;
				}
			
			}while(1);
		}
	else
		{
		if (cod != 5)
			do
				{
				if (!get_line(msg, line, 1000))
					break;
				
				if (!line[0]) break;
				
				}while(1);
		}
		
	switch(cod)
		{
		case 0:		Decode7Bit(msg, (int)out_att, line, boundary);					break;
		case 1:		DecodeQuoted(msg, (int)out_att, line, boundary);				break;
		case 2:		DecodeBase64(msg, (int)out_att, line, boundary);				break;
		case 3:		Decode8Bit(msg, (int)out_att, line, boundary);					break;
		case 4:		DecodeBinary(msg, (int)out_att, line, boundary);				break;
		case 5:		DecodeUudecode(msg, (int)out_att, line, boundary, endpos);	break;
		default:	Decode7Bit(msg, (int)out_att, line, boundary);					break;
		}							
	
	free(line);
	Fclose((int)out_att);
	}
}
   
/* ----------------------------------
   | Select attachment file to save |
   ---------------------------------- */
void save_attach(char *name, int msg, char *boundary, long endpos, char *type, char *predefined)
{
int ex_button;
char attach_file[64];
char buf[256];

DEBUG();
if (name[0])
	strcpy(attach_file, name);
else
	attach_file[0] = 0;
	
if (predefined || (fsel_input(attach_path, attach_file, &ex_button) && ex_button))
	{
	if (predefined)
		strcpy(buf, predefined);
	else
		{
		strcpy(buf, attach_path);
		if (strchr(buf, '\\'))
			strrchr(buf, '\\')[1] = 0;
		strcat(buf, attach_file);
		}	
	
	start_info(saveatt_text, processing_text);	
	store_attachment(buf, msg, boundary, endpos, type);
	end_info();
	}
}

/* -----------------------------------------
   | Remove attachments from outgoing mail |
   ----------------------------------------- */
void remove_attaches(int box, int idx)
{
char 	*zw_mem, tmpfile[13];
long 	zw_blen, *zw_ptr;
int 	zw_width;
int 	zw_lines, f, i, wwrap, vwrap, ft;
FILE_MAIL_MEM header;
PART_INFO *parts;

DEBUG();
wwrap = word_wrap;
vwrap = mail_options.viewwrap;
word_wrap = 0;
mail_options.viewwrap = 0;

i = get_body(s_edit,
							boxes[box].file, box_manager[box].index + idx, 
							&zw_mem, &zw_blen, &zw_ptr,
					 		&zw_width, &zw_lines, FALSE);
word_wrap = wwrap;
mail_options.viewwrap = vwrap;

if (zw_blen > 0)
	free(zw_mem);

f = open_file(box_manager[box].index[idx].file, boxes[box].file);
if (f)
	{
	Fread(f, sizeof(FILE_MAIL_MEM), &header);
	if (header.magic != 0x4104)
		{
		memmove(&header.confirm_read, &header.magic, sizeof(header) - (21+21+6));
		Fseek(-(21+21+6L), f, 1);
		header.replyto_len = 0;
		header.identity[0] = 0;
		header.signature[0] = 0;
		header.magic = 0x4104;
		header.version = SVERSION_ID20;
		}
	else
		{
		if (header.version == SVERSION_ID17)
			{
			Fseek(-(21+21L), f, 1);
			header.identity[0] = 0;
			header.signature[0] = 0;
			header.version = SVERSION_ID20;
			}
		}
	if (header.part_count == 1)
		{
		Fclose(f);
		return;
		}
		
	ft = create_temp(tmpfile);
	if (ft)
		{
		long new_body_pos, new_part_pos;
		
		Fwrite(ft, sizeof(header), &header);
	
		CopyPart(ft, f, header.date_len);
		CopyPart(ft, f, header.from_len);
		CopyPart(ft, f, header.replyto_len);
		CopyPart(ft, f, header.organization_len);
		CopyPart(ft, f, header.to_len);
		CopyPart(ft, f, header.cc_len);
		CopyPart(ft, f, header.bcc_len);
		CopyPart(ft, f, header.subject_len);

		parts = (PART_INFO *)malloc(header.part_count * sizeof(PART_INFO));
		Fseek(header.partinfo_pos, f, 0);
		Fread(f, sizeof(PART_INFO) * header.part_count, parts);

		Fseek(header.body_pos, f, SEEK_SET);
		new_body_pos = Fseek(0, ft, SEEK_CUR);
		CopyPart(ft, f, (parts[0].offset & 0x7FFFFFFFL) - header.body_pos);
		
		header.body_pos = new_body_pos;
		header.body_len = Fseek(0, ft, SEEK_CUR) - new_body_pos;

		new_part_pos = Fseek(0, ft, SEEK_CUR);
		Fseek(parts[header.main_idx].offset & 0x7FFFFFFFL, f, SEEK_SET);
		CopyPart(ft, f, parts[header.main_idx].len);
		
		if (header.part_count > 1)
			{
			Fwrite(ft, 2, "\r\n");
			header.body_len += 2;
			Fwrite(ft, strlen(attachm_text), attachm_text);
			header.body_len += strlen(attachm_text);
			Fwrite(ft, 3, ":\r\n");
			header.body_len += 3;

			for (i = 0; i < header.part_count; i++)
				if (i != header.main_idx)
					{
					Fwrite(ft, 4, "  - ");
					Fwrite(ft, strlen(parts[i].name), parts[i].name);
					Fwrite(ft, 2, "\r\n");
					header.body_len += strlen(parts[i].name) + 6;
					}
				
			parts[0].offset = new_part_pos | (parts[header.main_idx].offset & 0x80000000L);
			parts[0].len    = Fseek(0, ft, SEEK_CUR) - new_part_pos;
						
			header.part_count = 1;
			header.main_idx = 0;
			}

		header.header_inf = Fseek(0L, ft, 2);
		header.partinfo_pos = Fseek(0L, ft, 1);
		Fwrite(ft, sizeof(PART_INFO), parts);
					
		Fseek(0L, ft, 0);
		Fwrite(ft, sizeof(header), &header);

		free(parts);
			
		Fclose(ft);
		}
		
	Fclose(f);
	}

f = wcreate_file(box_manager[box].index[idx].file, boxes[box].file);
if (f)
	{
	CopyFile(f, tmpfile);
	Fclose(f);
	box_manager[box].index[idx].body_len = header.body_len;
	}
#ifdef __AHCC__
Fdelete(tmpfile);
#else
unlink(tmpfile);
#endif
}
