/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"
#include <tos.h>

typedef struct
							{
							long magic;									/* 2BIT */
							char sample_name[8];
							int modus;									/* 0 = Mono, -1 = Stereo */
							int resolution;							/* 8 = 8Bit, 16 = 16Bit  */
							int sgned;									/* 0 = signed, 1 = unsigned */
							int loop;										/* 0 = loopen, 1 = nicht */
							int midi;										/* -1 */
							int unknown;
							unsigned frequenz;					/* Samplefrequenz */
							long laenge;								/* Anzahl samples */
							long loop_anfang;
							long loop_ende;
							int res1;
							int res2;
							int res3;
							char ext[20];								/* Erweiterter Dateiname */
							char reserved[64];
							}AVR_FILE;

typedef struct wav_struct
          {
					short f_categ;
					short channels;
					long srate;
					long bsize;
					short balign;
					short ssize;
					}WAVFMT_HEADER;
                         
#define PSG 0
#define DMA 1
#define CODEC 2

#define MWIRE 0xFFFF8922L
#define MWDATA 0
#define MWMASK 1

#define LTATTEN 0
#define RTATTEN 1
#define LTGAIN 2
#define RTGAIN 3
#define ADDERIN 4
#define ADCINPUT 5
#define SETPRESCALE 6
#define DMAPLAY 0
#define DSPXMIT 1
#define EXTINP 2
#define ADC 3
#define DMAREC 1
#define DSPREC 2
#define EXTOUT 4
#define DAC 8

#define CODECSTE 0
#define CODEC49170 1
#define CODEC33880 2
#define CODEC24585 3
#define CODEC20770 4
#define CODEC16940 5
#define CODEC12292 7
#define CODEC9834 9
#define CODEC8195 11

#define SDMACTL 	0 					 /* DMA-Sound Controlregister */
#define SDMASTAT	1						 /* DMA-Sound Statusregister	*/
#define SSTART	 	3						 /* Frame-Startadresse 			 */
#define SEND			0xf  				 /* Frame-Endadresse				  */
#define SDMAMOD 	0x20L  			 /* DMA-Sound Moderegister		*/

void send_mwire(void);
void play_onpsg(void);
void play_ondma(void);
void play_oncodec(void);
start_psg(long timer_tick);
long dma_connect(long play_spd);
int fill_play_buffer(char **data_poi, long *len_poi, char *buffer, long one_step, int two_step);
void set_dma(long from, long to);
long codec_connect(long play_spd);

static int mw_cmd[] = {0x4C0, 0x540, 0x500, 0x480, 0x440};
int mw_data[] = {40, 20, 20, 6, 6};

int dma_tab[] = {0,1,2,3};
long dma_spd[] = {6250, 12517, 25033, 50066L};

int codec_tab[] = {CODEC49170, CODEC33880,
									 CODEC24585, CODEC20770,
									 CODEC16940, CODEC12292,
									 CODEC9834, CODEC8195};
long codec_spd[] = {49170L, 33880L, 24585, 20770,
 							      16940, 12292, 9834, 8195};

char *sample_data = NULL;
long sample_length, ply_speed;
int play_dev = 1, falcon_flg = 0, mwire_flg = FALSE;
short mxMask;

void error(char *e1, char *e2, int x)
{
}

/* --------------------
	 | Get cookie value |
	 -------------------- */
int get_cookie(unsigned long cookie, unsigned long *value)
{
long old_stack;
long *jar;

DEBUG();
old_stack=Super(0L);
jar = *((long **)0x5a0L);
Super((void *)old_stack);

if (!jar)
	return 0;

do
	{
	if (*jar == cookie)
		{
		if (value)
			*value=jar[1];

		return 1;
		}
	else
		jar= &(jar[2]);

	}while(*jar);

return 0;
}

/* ------------------------------------
   | Initialize audio device settings |
   ------------------------------------ */
void init_audio(void)
{
unsigned long val;
int ret;

DEBUG();
play_dev = PSG;
mwire_flg = TRUE;

ret = get_cookie('_SND', &val);

if (!ret || !(val & 2))
	mwire_flg = FALSE;
else
	play_dev = DMA;

if (!ret || !(val & 4))
	{
	}
else
	{
	mwire_flg = FALSE;
	play_dev = CODEC;
	falcon_flg = 1;
	}
	
if (ret && !(val & 1) && play_dev == PSG)
	play_dev = -1;
/*
switch (play_dev)
	{
	case PSG:
		form_alert(1, "[1][Audio device: PSG][Ok]");
		break;
		
	case DMA:
		form_alert(1, "[1][Audio device: DMA][Ok]");
		break;
		
	case CODEC:
		form_alert(1, "[1][Audio device: CODEC][Ok]");
		break;
	}

if (mwire_flg)
	form_alert(1, "[1][Micro wire active][Ok]");
*/
mxMask = Mxmask();
/*	{
	char buf[20];
	sprintf(buf, "[1][Mask: %x][Ok]", mxMask);
	form_alert(1, buf);
	}
*/
}

/* -------------------
   | Play one sample |
   ------------------- */
void play_one(void)
{
DEBUG();
if (!sample_data)	/* Nothing to play */
	return;
	
send_mwire();
	
switch (play_dev)
	{
	case PSG:
		play_onpsg();
		break;
	
	case DMA:
		play_ondma();
		break;

	case CODEC:
		play_oncodec();
		break;
	}
}

/* ----------------------
	 | Play sample on psg |
	 ---------------------- */
void play_onpsg(void)
{
long stack;

DEBUG();
stack = Super(0L);

Ikbdws(0, "\23");
start_psg((2457600L/4L) / ply_speed);
Ikbdws(0, "\21");

Super((void *)stack);
}

/* ----------------------
	 | Play sample on DMA |
	 ---------------------- */
void play_ondma(void)
{
long out_speed;
char *snd_buf;
char *buf_mid;
char *akt_poi;
int two, buff_fill;
long frq, fill_len, one;
long stack;

DEBUG();
stack = Super(0L);

snd_buf = (char *)(mxMask ? Mxalloc(64000L, 0) : Malloc(64000L));
  
if (snd_buf)
	{
	buf_mid = snd_buf + 32000;
		
	out_speed = ply_speed;
	frq = dma_connect(out_speed);

	if (frq == out_speed)
		{
		one = 99999L;
		two = 32000;
		}
	else
		{
		buff_fill = (int)(frq - out_speed);
		one = (int)((out_speed + (buff_fill >> 1)) / buff_fill);
		fill_len = (out_speed / one) * (one + 1) + (out_speed  % one);
		if (frq < fill_len)
		  two = (int)((out_speed / one) / (fill_len - frq));
		 else
	  	two = (int)((out_speed / one) / ((fill_len - frq) >> 1));
	 	}
		 	
	akt_poi = sample_data;
	fill_len = sample_length;
	buff_fill = fill_play_buffer(&akt_poi, &fill_len, snd_buf, one, two);
	if (fill_len)
		{
		set_dma((long)snd_buf, (long)buf_mid - 1L);

		while(fill_len)
			{
			buff_fill = fill_play_buffer(&akt_poi, &fill_len, buf_mid, one, two);
			set_dma((long)buf_mid, (long)buf_mid + buff_fill);
			if (!fill_len)
				break;

			buff_fill = fill_play_buffer(&akt_poi, &fill_len, snd_buf, one, two);
			set_dma((long)snd_buf, (long)snd_buf + buff_fill);
			if (!fill_len)
				break;
			}
		}
	else
		set_dma((long)snd_buf, (long)snd_buf + buff_fill);
			
	while(*(char *)(0xffff8900L + SDMASTAT));
	Mfree(snd_buf);
	}
else
	error("Temporrer Bereich konnte", "nicht angelegt werden.", 5);

Super((void *)stack);
}

/* ---------------------------------------
   | Connect DMA with realiest frequency |
   --------------------------------------- */
long dma_connect(long play_spd)
{
int i, b = 3;
char *dma_chip = (char *)0xffff8900L;

DEBUG();
if (dma_spd[3] > play_spd)
	for (i = 2; i >= 0; i--)
		if (dma_spd[i] >= play_spd)
			b = i;

if (falcon_flg && !b)
	b = 1;

*(int *)(&dma_chip[SDMAMOD]) = 0x80 + dma_tab[b];
return dma_spd[b];
}

/* ---------------------
   | Set DMA registers |
   --------------------- */
void set_dma(long from, long to)
{
char *dma_chip = (char *)0xffff8900L;

DEBUG();
while(dma_chip[SDMASTAT]);

dma_chip[SSTART + 4] = from; from >>= 8;
dma_chip[SSTART + 2] = from; from >>= 8;
dma_chip[SSTART] = from;

dma_chip[SEND + 4] = to; to >>= 8;
dma_chip[SEND + 2] = to; to >>= 8;
dma_chip[SEND] = to;
*(int *)(&dma_chip[SDMACTL]) = 0x1;
}

/* ------------------------
	 | Play sample on CODEC |
	 ------------------------ */
void play_oncodec(void)
{
long out_speed;
char *snd_buf;
char *buf_mid;
char *akt_poi;
int buff_fill, two;
long frq, fill_len, buffers[4], one;
long stack;

DEBUG();
stack = Super(0L);
if (locksnd() < 0)
	error("Soundsystem bereits", "belegt!", 0x5);
else
	{
/*	soundcmd(LTATTEN, 
		soundcmd(RTATTEN,		*/

	snd_buf = (char *)(mxMask ? Mxalloc(64000L, 0) : Malloc(64000L));
	if (snd_buf)
		{
		buf_mid = snd_buf + 32000;
		
		setsndmode(2);
		settracks(0, 0);
		setmontracks(0);
		out_speed = ply_speed;
		frq = codec_connect(out_speed);

		if (frq == out_speed)
			{
			one = 99999L;
			two = 32000;
			}
		else
			{
			buff_fill = (int)(frq - out_speed);
			one = (int)((out_speed + (buff_fill >> 1)) / buff_fill);
			fill_len = (out_speed / one) * (one + 1) + (out_speed  % one);
			if (frq < fill_len)
			  two = (int)((out_speed / one) / (fill_len - frq));
			 else
		  	two = (int)((out_speed / one) / ((fill_len - frq) >> 1));
		 	}
		 	
		akt_poi = sample_data;
		fill_len = sample_length;
		buff_fill = fill_play_buffer(&akt_poi, &fill_len, snd_buf, one, two);

		if (fill_len)
			{
			setbuffer(0, snd_buf, buf_mid);
			buffoper(1);
			
			while(fill_len)
				{
				buff_fill = fill_play_buffer(&akt_poi, &fill_len, buf_mid, one, two);
				do
					{
					buffptr(buffers);
					}while((char *)buffers[0] != snd_buf);
				setbuffer(0, buf_mid, buf_mid + buff_fill);
				buffoper(1);
				
				if (!fill_len)
					break;

				buff_fill = fill_play_buffer(&akt_poi, &fill_len, snd_buf, one, two);
				do
					{
					buffptr(buffers);
					}while((char *)buffers[0] != buf_mid);
				setbuffer(0, snd_buf, snd_buf + buff_fill);
				buffoper(1);

				if (!fill_len)
					break;
				}
			}
		else
			{
			setbuffer(0, snd_buf, snd_buf + buff_fill);
			buffoper(1);
			buf_mid = snd_buf;
			}

    do
			{
			buffptr(buffers);
			}while((char *)buffers[0] != buf_mid);
		buffoper(0);
			
		Mfree(snd_buf);
		}
	else
		error("Temporrer Bereich konnte", "nicht angelegt werden.", 5);
		
	devconnect(DMAPLAY, DAC, 0, 0, 1);																						
	unlocksnd();
	}

Super((void *)stack);
}

/* -----------------------------------------
   | Connect CODEC with realiest frequency |
   ----------------------------------------- */
long codec_connect(long play_spd)
{
int i, b = 0;

DEBUG();
if (codec_spd[0] > play_spd)
	for (i = 1; i < 8; i++)
		if (codec_spd[i] >= play_spd)
			b = i;

devconnect(DMAPLAY, DAC, 0, codec_tab[b], 1);																						
return codec_spd[b];
}

/* --------------------------------------
   | Fill play buffer with current data |
   -------------------------------------- */
int fill_play_buffer(char **data_poi, long *len_poi, char *buffer, long one_step, int two_step)
{
int cnt = 32000, two_cnt;
char *poi;
long in_data, one_cnt;
static char note = 0, xnote;

DEBUG();
poi = data_poi[0];
in_data = *len_poi;
one_cnt = one_step;
two_cnt = (two_step > 0) ? two_step : -two_step;

while(in_data && cnt)
	{
	if (!one_cnt)
		{
		if (two_cnt <= 0)
			{
			if (two_step > 0)
				{
				two_cnt = two_step;
				one_cnt = one_step;
				continue;
				}
			else
				{
				if (!two_cnt)
					{
					xnote = note;
					note = (((*poi + note) >> 1) + note) >> 1;
					}
				else
					if (two_cnt == -1)
						{
						note = (*poi + xnote) >> 1;
						two_cnt = -two_step + 1;
						one_cnt = one_step;
						}
				}
			}
		else
			{
			note = (*poi + note) >> 1;
			one_cnt = one_step;
			}
		two_cnt--;
		}
	else
		{
		note = *(poi++);
		one_cnt--;
		in_data--;
		}
		
	*(buffer++) = note;
	cnt--;
	}

*data_poi = poi;
*len_poi = in_data;	
return 32000 - cnt;
}

/* ------------------------
   | Send it to Microwire |
   ------------------------ */
void send_mwire(void)
{
int *mw_reg = (int *)MWIRE;
int i;
long old_stack;

DEBUG();
old_stack = Super(0L);

mw_reg[MWMASK] = 0x7FF;
for (i = 0; i < 5; i++)
	{
	delay(50);
	mw_reg[MWDATA] = mw_cmd[i] | mw_data[i];
	}
delay(50);
mw_reg[MWDATA] = 0x401;
delay(50);

Super((void *)old_stack);
}

/* ------------------------------
   | Turn LONG to IBM PC format |
   ------------------------------ */
void turn_long(long *a)
{
char *adr, c;

adr = (char *)a;
c = adr[0];
adr[0] = adr[3];
adr[3] = c;
c = adr[1];
adr[1] = adr[2];
adr[2] = c;
}

/* ------------------------------
   | Turn WORD to IBM PC format |
   ------------------------------ */
void turn_word(short *a)
{
char *adr, c;

adr = (char *)a;
c = adr[0];
adr[0] = adr[1];
adr[1] = c;
}

static long convert_wav(char *data, WAVFMT_HEADER *header, long length)
{
long ret;

DEBUG();
if (header->channels == 2)
	{
	if (header->ssize == 16)
		{
		int *b;
		char *d;
		long c, v;
		
		d = data;
		b = (int *)data;
		ret = c = length / 4;
		while(c--)
			{
			short x, y;
			
			x = *(b++);
			y = *(b++);
			turn_word(&x);
			turn_word(&y);
			
			v = x + y;
			v >>= (8 + 1);
			*(d++) = (char)v;
			}
		}
	else
		{
		unsigned char *b;
		char *d;
		long c;
		unsigned v;
		
		d = data;
		b = (unsigned char *)data;
		ret = c = length / 2;
		while(c--)
			{
			v = *(b++) + *(b++);
			v >>= 1;
			*(d++) = (char)v;
			}
		}
	}
else
	{
	if (header->ssize == 16)
		{
		short *b;
		short v;
		char *d;
		long c;
		
		d = data;
		b = (short *)data;
		ret = c = length / 2;
		while(c--)
			{
			v = *(b++);
			turn_word(&v);
			v >>= 8;
			*(d++) = (char)v;
			}
		}
	}

return ret;
}

/* --------------
   | Wave maker |
   -------------- */
static void load_wave(int fd, long length)
{
WAVFMT_HEADER header;
long id;

DEBUG();
Fseek(8, fd, SEEK_SET);
Fread(fd, 4, &id);
if (id != 'WAVE')
	return;
	
while(Fseek(0, fd, SEEK_CUR) < length)
	{
	Fread(fd, 4, &id);
	if (id == 'fmt ')
		break;
	Fread(fd, 4, &id);
	turn_long(&id);
	Fseek(id, fd, SEEK_CUR);
	}

if (id == 'fmt ')
	{
	Fseek(4, fd, SEEK_CUR);
	Fread(fd, sizeof(header), &header);
	}
else
	return;

Fseek(12, fd, SEEK_SET);
while(Fseek(0, fd, SEEK_CUR) < length)
	{
	Fread(fd, 4, &id);
	if (id == 'data')
		break;
	Fread(fd, 4, &id);
	turn_long(&id);
	Fseek(id, fd, SEEK_CUR);
	}

if (id != 'data')
	return;

turn_long(&header.srate);
turn_word(&header.channels);
turn_word(&header.ssize);
ply_speed = header.srate;

Fread(fd, 4, &sample_length);
turn_long(&sample_length);

if (sample_length)
	{
	if ((sample_data = (char *)malloc(sample_length)) == NULL)
		error("Speicher voll!", "\0", 0xA);

	if (sample_data)
		{
		long i;
		
		graf_mouse(HOURGLASS, 0L);

		i = Fread((int)fd, sample_length, sample_data);
		
		sample_length = convert_wav(sample_data, &header, sample_length);
		
		graf_mouse(ARROW, 0L);
	
		if (i < sample_length)
			error("Datei defekt!!!", "\0", 0xA);
		}
	}
}		

/*
<RIFF><len>
<WAVE><len>
			<fmt><len>
						short			- format
						short			- channels
						long			- samples per sec.
						long			- avg samples
						short			- block align
						short			- bits per sample
			<data><len>
	
		turn_word(&WAV_STRUCT.f_categ);
		turn_word(&WAV_STRUCT.channels);
		turn_long(&WAV_STRUCT.srate);
		turn_long(&WAV_STRUCT.bsize);
		turn_word(&WAV_STRUCT.balign);
		turn_word(&WAV_STRUCT.ssize);
	*/
			
/* -------------------
   | Load audio file |
   ------------------- */
void load_audio(char *audio)
{
long fd;
long i;
AVR_FILE header;

DEBUG();
if (sample_data)
	{
	free(sample_data);
	sample_data = NULL;
	}

if ((fd = Fopen(audio, 0)) < 0)
	{
	error("Datei fehlt!", "\0", 0xA);
	sample_length = 0L;
	}
else
	{
	sample_length = Fseek(0, (int)fd, 2);
	Fseek(0, (int)fd, 0);

	Fread((int)fd, sizeof(AVR_FILE), &header);
	if (header.magic == '2BIT')
		{
		ply_speed = header.frequenz;
		sample_length -= sizeof(AVR_FILE);
		}
	else
		{
		if (header.magic == 'RIFF')
			load_wave((int)fd, sample_length);
			
		Fclose((int)fd);
		return;
		}

	if (sample_length)
		{
		if ((sample_data = (char *)malloc(sample_length)) == NULL)
			error("Speicher voll!", "\0", 0xA);

		if (sample_data)
			{
			graf_mouse(HOURGLASS, 0L);

			i = Fread((int)fd, sample_length, sample_data);
			graf_mouse(ARROW, 0L);
		
			if (i < sample_length)
				error("Datei defekt!!!", "\0", 0xA);
			}
		}

	Fclose((int)fd);
	}
}
	
