/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

static int fav_icony_flg = FALSE;
static int icn_selected = -1;

static int maxwidth, iconsperrow, favboxes, rowsneeded;

/* --------------------------------
   | Get number of favorite boxes |
   -------------------------------- */
int number_of_favorites(void)
{
return favboxes;
}

/* --------------------------
   | Update box information |
   -------------------------- */
void update_boxinfo(void)
{
int i;
int ext[8];
int wx, wy, ww, wh;

DEBUG();
for (favboxes = maxwidth = i = 0; i < box_count; i++)
	{
	if (boxes[i].fav_index > -1)
		{
		int w;
		
		vqt_extent(vdi_handle, boxes[i].desc, ext);
		w = abs(ext[0] - ext[2]);
		if (w > maxwidth) maxwidth = w;
		favboxes++;
		}
	}
if (icon_palheader.width > maxwidth) maxwidth = icon_palheader.width;
maxwidth += 4;

if (box_wind > -1)
	{
	wind_get(box_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
	iconsperrow = ww / maxwidth;
	if (!iconsperrow) iconsperrow  = 1;

	rowsneeded = ((favboxes + iconsperrow - 1) / iconsperrow);
	}
else
	iconsperrow = 1;
}

/* ------------------------
   | Redraw selected icon |
   ------------------------ */
void redraw_boxicon(int no)
{
int wx, wy, ww, wh;
int remain;

DEBUG();
if (box_wind == -1) return;

wind_get(box_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
no = boxes[no].fav_index;
remain = (ww - iconsperrow * maxwidth) >> 1;
no -= box_top * iconsperrow;
redraw_boxwind(wx + (no % iconsperrow) * maxwidth + remain, wy + (no / iconsperrow) * icon_height, maxwidth, icon_height);
}

/* --------------------------
   | Redraw box icon window |
   -------------------------- */
void redraw_boxwind(int rx, int ry, int rw, int rh)
{
GRECT t1, t2;
int xy[4], i, ext[8], ln[10];
int wx, wy, ww, wh;
int cx, cy, cw, ch;
int size, oldsize, pos, oldpos;
int x, y, remain;
int boxiconwidth, boxiconheight;

DEBUG();
wind_get(box_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
if (rx == -1)
	{
	t2.g_x = wx;
	t2.g_y = wy;
	t2.g_w = ww;
	t2.g_h = wh;
	}
else
	{
	t2.g_x = rx;
	t2.g_y = ry;
	t2.g_w = rw;
	t2.g_h = rh;
	}
	
wind_update(BEG_UPDATE);
graf_mouse(M_OFF, 0L);

vst_height(vdi_handle, 4, &cx, &cy, &cw, &ch);
wind_get(box_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);

boxiconheight = icon_palheader.height;
if (smallfont) boxiconheight >>= 1;
boxiconwidth = icon_palheader.width;

if (box_top + wh / icon_height > rowsneeded)
	{
	box_top = rowsneeded - (wh / icon_height - 1);
	if (box_top < 0) box_top = 0;
	}
remain = (ww - iconsperrow * maxwidth) >> 1;
		
wind_get(box_wind, WF_FIRSTXYWH, &t1.g_x, &t1.g_y, &t1.g_w, &t1.g_h);
while(t1.g_w && t1.g_h)
	{
	if (rc_intersect(&t2, &t1))
		{
		int xs;
		
		xy[0] = t1.g_x;
		xy[1] = t1.g_y;
		xy[2] = t1.g_x + t1.g_w - 1;
		xy[3] = t1.g_y + t1.g_h - 1;
		vs_clip(vdi_handle, 1, xy);

		if (fav_icony_flg)
			{
			iconified_tree[0].ob_x = wx;
			iconified_tree[0].ob_y = wy;
			iconified_tree[0].ob_width = ww;
			iconified_tree[0].ob_height = wh;
			
			iconified_tree[1].ob_x = (ww - iconified_tree[1].ob_width) >> 1;
			iconified_tree[1].ob_y = (wh - iconified_tree[1].ob_height) >> 1;

			objc_draw(iconified_tree, 0, MAX_DEPTH, &t1);
			}
		else
			{
			y = ((t1.g_y - wy) / boxiconheight) * boxiconheight + wy;
			xs = ((t1.g_x - wx) / boxiconwidth) * boxiconwidth + wx;
			while	(y < xy[3])
				{
				x = xs;
				while(x < xy[2])
					{
					blt_copy(boxicons, 0, 0, NULL, x,
						  		 y, boxiconwidth, boxiconheight, S_ONLY);
					x += boxiconwidth;
					}
				y += boxiconheight;
				}
				
			for (i = 0; i < box_count; i++)
				if (boxes[i].fav_index > -1)
					{
					int pos, y, x, xp, yp, w;
					
					pos = boxes[i].fav_index - box_top * iconsperrow;
	
					yp = pos / iconsperrow;
					xp = pos % iconsperrow;
					
					y = wy + yp * icon_height;
					x = wx + xp * maxwidth;
					if (y + icon_height > xy[1] && y < xy[3] &&
							x + maxwidth > xy[0] && x < xy[2])
						{
						vqt_extent(vdi_handle, boxes[i].desc, ext);
						w = abs(ext[0] - ext[2]);
	
						if (colours <= 4 && icn_selected == i)
							vswr_mode(vdi_handle, MD_ERASE);
						
						v_gtext(vdi_handle, x + (maxwidth - w) / 2 + remain, y + boxiconheight + 6, boxes[i].desc);
					
						blt_copy(boxicons, 0, boxes[i].icon * boxiconheight, NULL, x + (maxwidth - boxiconwidth) / 2 + remain,
						  		 	 y + 4, boxiconwidth, boxiconheight, 
						  		 	 (colours <= 4 && icn_selected == i) ? NOTS_OR_D :
						  		 	 S_ONLY);
		
						if (icn_selected == i)
							if (colours > 4)
								{
								ln[7] = ln[9] = y + 1;
								ln[3] = ln[5] = y + boxiconheight + 4;
								ln[1] = ln[5] + abs(ext[5] - ext[3]) + 3;
			
								ln[0] = ln[2] = x + (maxwidth - w) / 2 - 2 + remain;
								ln[4] = ln[6] = x + (maxwidth - boxiconwidth) / 2 - 2 + remain;
								ln[8] = ln[6] + boxiconheight + 4;
								
								if (ln[0] > ln[4])
									ln[0] = ln[2] = ln[4];
			
								vsl_color(vdi_handle, 1);
								v_pline(vdi_handle, 5, ln);
			
								ln[7] = ln[5];
								ln[3] = ln[1];
								ln[6] = ln[8];
								ln[2] = ln[4] = x + maxwidth - (maxwidth - w) / 2 + 2 + remain;
			
								if (ln[2] < ln[6])
									ln[2] = ln[4] = ln[6];
			
								vsl_color(vdi_handle, 0);
								v_pline(vdi_handle, 5, ln);
								}
							else
								vswr_mode(vdi_handle, MD_REPLACE);
						}
					}
			}				
		vs_clip(vdi_handle, 0, xy);
		}

	wind_get(box_wind, WF_NEXTXYWH, &t1.g_x, &t1.g_y, &t1.g_w, &t1.g_h);
	}

graf_mouse(M_ON, 0L);
wind_update(END_UPDATE);

if (rowsneeded * icon_height <= wh)
	{
	pos = 0;
	size = 1000;
	}
else
	{
	pos = (int)(1000L * box_top / (rowsneeded - wh / icon_height));
	size = (wh / icon_height) * 1000 / rowsneeded;
	}

wind_get_int(box_wind, WF_VSLIDE, &oldpos);
wind_get_int(box_wind, WF_VSLSIZE, &oldsize);

if (size != oldsize || pos != oldpos)
	{
	wind_set_int(box_wind, WF_VSLIDE, pos);
	wind_set_int(box_wind, WF_VSLSIZE, size);
	}
}

/* -------------------
   | Open box window |
   ------------------- */
void open_boxwind(void)
{
GRECT win, wout, wrec;

DEBUG();
wind_get_grect(ROOT, WF_WORKXYWH, &wrec);

win.g_x = win.g_y = 0;
win.g_w = 22 * tx_width;
win.g_h = 0;
wind_calc(WC_BORDER, NAME|CLOSER|MOVER|SIZER|
										UPARROW|DNARROW|VSLIDE|SMALLER,
					&win, &wout);
					
box_wind = wind_create(NAME|CLOSER|MOVER|SIZER|
											 UPARROW|DNARROW|VSLIDE|SMALLER, &wrec);
if (!box_wind)
	return;
	
wind_set_string(box_wind, WF_NAME, fav_title);

if (box_wd[0] == -1)
	{
	box_wd[0] = wrec.g_x + wrec.g_w - wout.g_w; 	/* Align right */
	box_wd[1] = wrec.g_y;
	box_wd[2] = wout.g_w; box_wd[3] = wrec.g_h;
	}
box_wd[0] &= ~0x4000;

if (box_wd[0] < 0)	box_wd[0] = 0;
if (box_wd[1] < 0)	box_wd[1] = 0;
	
if (box_wd[1] < wrec.g_y)
	box_wd[1] = wrec.g_y;
if (box_wd[3] > wrec.g_h)
	box_wd[3] = wrec.g_h;
if (box_wd[0] > wrec.g_x + wrec.g_w)
	box_wd[0] = wrec.g_x + wrec.g_w - wout.g_w;
if (box_wd[1] > wrec.g_y + wrec.g_h)	
	box_wd[1] = wrec.g_y + wrec.g_h - box_wd[3];
	
wind_open(box_wind, (GRECT *)&box_wd[0]);
register_window(box_wind);
update_boxinfo();
}

/* --------------------
   | Close box window |
   -------------------- */
void close_boxwind(void)
{
DEBUG();
if (box_wind > -1)
	{
	wind_get(box_wind, WF_CURRXYWH, &box_wd[0], &box_wd[1], &box_wd[2], &box_wd[3]);
	wind_close(box_wind);
	unregister_window(box_wind);
	wind_delete(box_wind);
	box_wind = -1;
	}
}

/* ------------------------------
   | Process box window message |
   ------------------------------ */
void box_window_message(int *msg)
{
int wx, wy, ww, wh;

DEBUG();
wind_get(box_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);

switch(msg[0])
	{
	case WM_REDRAW:
		redraw_boxwind(msg[4], msg[5], msg[6], msg[7]);
		break;
		
	case WM_CLOSED:
		close_boxwind();
		break;
		
	case WM_ICONIFY:
	case WM_ALLICONIFY:
		fav_icony_flg = TRUE;
		wind_set(msg[3], WF_ICONIFY, msg[4], msg[5], msg[6], msg[7]);
		break;

	case WM_UNICONIFY:
		fav_icony_flg = FALSE;
		wind_set(msg[3], WF_UNICONIFY, msg[4], msg[5], msg[6], msg[7]);
		break;
				
	case WM_ARROWED:
		{
		switch(msg[4])
			{
			case WA_UPPAGE:
				if (box_top > 0)
					{
					box_top -= wh / icon_height;
					if (box_top < 0) box_top = 0;
					redraw_boxwind(-1, 0, 0, 0);
					}
				break;
				
			case WA_DNPAGE:
				if (box_top + wh / icon_height < rowsneeded)
					{
					box_top += wh / icon_height;
					if (box_top + wh / icon_height > rowsneeded)
						{
						box_top = rowsneeded - (wh / icon_height - 1);
						if (box_top < 0) box_top = 0;
						}
					redraw_boxwind(-1, 0, 0, 0);
					}
				break;
				
			case WA_UPLINE:
				if (box_top > 0)
					{
					box_top--;
					redraw_boxwind(-1, 0, 0, 0);
					}
				break;
				
			case WA_DNLINE:
				{
				if (box_top + wh / icon_height < rowsneeded)
					{
					box_top++;
					redraw_boxwind(-1, 0, 0, 0);
					}
				break;
				}
			}
		break;
		}
		
	case WM_VSLID:
		{
		int new = (int)((long)msg[4] * (rowsneeded - wh / icon_height) / 1000);
		if (new != box_top)
			{
			box_top = new;
			redraw_boxwind(-1, 0, 0, 0);
			}
		break;
		}
	}			
}

/* -----------------------
   | Find a favorite box |
   ----------------------- */
int find_favbox(int no)
{
int i;

DEBUG();
for (i = 0; i < box_count; i++)
	if (boxes[i].fav_index > -1)
		if (boxes[i].fav_index == no)
			return i;
return -1;
}

/* ---------------------------
   | Drag selected mail list |
   --------------------------- */
void view_dragbox(int box, int posx, int posy, int *cxy)
{
int ln[10], ext[8], w;
int boxiconwidth, boxiconheight;

DEBUG();
boxiconheight = icon_palheader.height;
if (smallfont) boxiconheight >>= 1;
boxiconwidth = icon_palheader.width;

vqt_extent(vdi_handle, boxes[box].desc, ext);
w = abs(ext[0] - ext[2]);

vs_clip(vdi_handle, TRUE, cxy);
vswr_mode(vdi_handle, MD_XOR);
graf_mouse(M_OFF, 0L);

ln[7] = ln[9] = posy + 1;
ln[3] = ln[5] = posy + boxiconheight + 4;
ln[1] = ln[5] + abs(ext[5] - ext[3]) + 3;

ln[0] = ln[2] = posx + (maxwidth - w) / 2 - 2;
ln[4] = ln[6] = posx + (maxwidth - boxiconwidth) / 2 - 2;
ln[8] = ln[6] + boxiconheight + 4;

if (ln[0] > ln[4])
	ln[0] = ln[2] = ln[4];

vsl_color(vdi_handle, 1);
v_pline(vdi_handle, 5, ln);

ln[7] = ln[5];
ln[3] = ln[1];
ln[6] = ln[8];
ln[2] = ln[4] = posx + maxwidth - (maxwidth - w) / 2 + 2;

if (ln[2] < ln[6])
	ln[2] = ln[4] = ln[6];
v_pline(vdi_handle, 5, ln);

graf_mouse(M_ON, 0L);
vswr_mode(vdi_handle, MD_REPLACE);
vs_clip(vdi_handle, FALSE, cxy);
}

/* -------------------------
   | button clicked in box |
   ------------------------- */
void box_button(void)
{
int icn;
int wx, wy, ww, wh;
int icny, icnx, remain;
	
DEBUG();
wind_get(box_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
remain = (ww - iconsperrow * maxwidth) >> 1;
icny = (evnt.y - wy) / icon_height;
icnx = (evnt.x - wx - remain) / maxwidth;
	
if (icnx >= iconsperrow) icnx = iconsperrow - 1;
icn = icny * iconsperrow + icnx;

if (icn >= 0 && icn < favboxes)
	{
	int isel = box_top * iconsperrow + icn;
	int fav_box = find_favbox(isel);
	
	if (evnt.bstate & 2)
		{
		if (fav_box != box_number("QUEUE"))
			if (boxes[fav_box].type == -1)
				identity_popup(fav_box);
			else
				folder_popup(fav_box);
		else
			queue_popup(fav_box);
		update_boxinfo();
		}
	else
		if (clicks == 2)
			if (boxes[fav_box].type == -1)
				check_id(boxes[fav_box].identity, -1);
			else
				list_window(fav_box);
		else
			{
			int last_x = evnt.x, last_y = evnt.y;
			int sel_box, idx, i, cxy[4];
			int mw2 = maxwidth >> 1;
			int mh2 = icon_height >> 1;

			wind_get(box_wind, WF_WORKXYWH, &cxy[0], &cxy[1], &cxy[2], &cxy[3]);
			cxy[2] += cxy[0] - 1;
			cxy[3] += cxy[1] - 1;

			wind_update(BEG_MCTRL);
			view_dragbox(fav_box, evnt.x - mw2, evnt.y - mh2, cxy);
			sel_box = box_move(evnt.x + mw2, evnt.y + mh2);
			do
				{
			  graf_mkstate( &evnt);
				which = evnt_multi(MU_BUTTON|MU_TIMER,
								  1, 1, 0,
									0, &empty,
									0, &empty,
									NULL,
									50L,
									&evnt,
									&k_return, &clicks);
			
				if (last_x != evnt.x || last_y != evnt.y)
					{
					int last_selbox = sel_box;

					view_dragbox(fav_box, last_x - mw2, last_y - mh2, cxy);
					
					sel_box = box_move(evnt.x + mw2, evnt.y + mh2);
					if (sel_box != last_selbox)
						{
						if (sel_box == -1 || sel_box == fav_box)
							graf_mouse(BUSYBEE, 0L);
						else
							graf_mouse(POINT_HAND, 0L);
						}
								
					view_dragbox(fav_box, evnt.x - mw2, evnt.y - mh2, cxy);
					last_x = evnt.x;
					last_y = evnt.y;
					}
			
				}while(! (which & MU_BUTTON));			

			view_dragbox(fav_box, last_x - mw2, last_y - mh2, cxy);
			graf_mouse(ARROW, 0L);
			wind_update(END_MCTRL);
			
			if (sel_box != fav_box)
				{
				idx = boxes[fav_box].fav_index;

				boxes[fav_box].fav_index = -1;
				for (i = 0; i < box_count; i++)
					if (boxes[i].fav_index > idx)
						boxes[i].fav_index--;
				
				if (sel_box != -1)
					idx = boxes[sel_box].fav_index;
				else
					idx = favboxes - 1;

				for (i = 0; i < box_count; i++)
					if (boxes[i].fav_index >= idx)
						boxes[i].fav_index++;
				boxes[fav_box].fav_index = idx;
				update_boxinfo();
				redraw_boxwind(-1, -1, -1, -1);
				}
			}
	}
}

/* ------------------
   | Box mouse move |
   ------------------ */
int box_move(int mouse_x, int mouse_y)
{
int wnd;
int old, remain;
int wx, wy, ww, wh;

DEBUG();
old = icn_selected;

wind_get(box_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
remain = (ww - iconsperrow * maxwidth) >> 1;

wnd = wind_find(mouse_x, mouse_y);
if (wnd == box_wind)
	{
	int icny, icnx;
	
	icny = (mouse_y - wy) / icon_height;
	icnx = (mouse_x - wx - remain) / maxwidth;
	
	if (icnx >= iconsperrow) icnx = iconsperrow - 1;
	icn_selected = find_favbox(icny * iconsperrow + icnx + box_top * iconsperrow);
	}
else
	{
	if (is_listwindow(wnd))
		icn_selected = window_box(wnd);
	else
		icn_selected = -1;
	}
	
if (icn_selected != old)
	{
	redraw_boxicon(old);
	if (icn_selected > -1) redraw_boxicon(icn_selected);
	}
	
return icn_selected;
}

