/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

#define max(a,b) ((a > b) ? a : b)

/* Local prototype */
int cdecl ipreview(PARMBLK *paramblock);

USERBLK iconpreview = {ipreview, 0};

static int nfold_ids[] = {D_NFDESC,
													FOLDERSWTCH,
													POPSWTCH,
													INSWITCH,
													OUTSWTCH,
													DNEW_IDSEL,
													SICONUP,
													SICONDWN,
													SPREVIEW,
													NF_OKAY,
													NF_CANCL,
													0};

/* ------------------------
   | Scan mailbox folders |
   ------------------------ */
void scan_mailbox(char *box)
{
int i;

DEBUG();
if (!box[0])
	return;

load_box_options(box);
	
if (box_count == 0)
	return;
	
load_addresses();
load_identities();
setPollInitialize();
load_recvids();
load_signatures();
load_filter();

menu_ienable(menu_adr, M_NEWML, TRUE);
menu_ienable(menu_adr, M_CLSBOX, TRUE);
menu_ienable(menu_adr, M_ABOOK, TRUE);
menu_ienable(menu_adr, M_FILTER, TRUE);
menu_ienable(menu_adr, M_SIGNAT, TRUE);
menu_ienable(menu_adr, M_IDENTITY, TRUE);
menu_ienable(menu_adr, M_FAVOR, TRUE);
menu_ienable(menu_adr, M_PSETUP, TRUE);
enabler();

box_open = 1;

for (i = 0; i < box_count; i++)
	if (boxes[i].type >= 0)
		load_box_index(i);

if (box_wd[0] != -1 && (box_wd[0] & 0x4000))
	open_boxwind();
open_tree();

for (i = 0; i < box_count; i++)
	if (boxes[i].type >= 0 &&
			box_manager[i].box_coords[0] != -1 &&
			(box_manager[i].box_coords[0] & 0x4000))
		{
		list_window(i);
		}
}

/* ---------------
   | New mailbox |
   --------------- */
void new_box(void)
{
int ex_button;
char f[64], dir[256];

DEBUG();
if (!mail_options.maildir[0]) mail_options.maildir[0] = Dgetdrv() + 'A';
strcpy(dir, mail_options.maildir);
strcat(dir, "*.BOX");
strcpy(f, mail_options.mailfile);

fsel_input(dir, f, &ex_button);
if (ex_button == 1 && f[0])
	{
	close_box();

	if (strchr(dir, '\\'))
		*(strrchr(dir, '\\') + 1) = 0;
	else
		{
		strcpy(dir, "x:\\");
		dir[0] = Dgetdrv() + 'A';
		}

	if (!strchr(f, '.'))
		strcat(f, ".BOX");

	strcpy(mail_options.maildir, dir);
	strcpy(mail_options.mailfile, f);

	strcat(dir, f);
		
	create_mailbox(dir);
	scan_mailbox(dir);
	}
}

/* ----------------
   | Open mailbox |
   ---------------- */
void open_box(void)
{
int ex_button;
char f[64], dir[256];

DEBUG();
if (!mail_options.maildir[0]) 
	mail_options.maildir[0] = Dgetdrv() + 'A';
strcpy(dir, mail_options.maildir);
strcat(dir, "*.BOX");
strcpy(f, mail_options.mailfile);

fsel_input(dir, f, &ex_button);
if (ex_button == 1)
	{
	close_box();

	if (strchr(dir, '\\'))
		*(strrchr(dir, '\\') + 1) = 0;
	else
		{
		strcpy(dir, "x:\\");
		dir[0] = Dgetdrv() + 'A';
		}

	strcpy(mail_options.maildir, dir);
	strcpy(mail_options.mailfile, f);
	
	strcat(dir, f);
	if (!strchr(f, '.'))
		strcat(dir, ".BOX");
	
	scan_mailbox(dir);
	}
}

/* --------------------------
   | Close current mail box |
   -------------------------- */
void close_box(void)
{
int i;

DEBUG();
if (!box_open || !mail_options.maildir[0])
	return;

if (mail_wind > -1)
	if (!close_mailwindow())
		return;
	
for (i = 0; i < box_count; i++)
	{
	if (box_manager[i].list_wind > -1)
		{
		close_listwindow(i);
		if (mail_options.saveview)
			box_manager[i].box_coords[0] |= 0x4000;

		if (boxes[i].type >= 0)			
			save_box_index(i);
		}
	else
		if (box_manager[i].index_changed)
			if (boxes[i].type >= 0)			
				save_box_index(i);
	}

if (box_wind > -1)
	{
	close_boxwind();
	box_wd[0] |= 0x4000;
	}
		
save_box_options();
save_addresses();
save_identities();
stopPoll();
save_recvids();
save_filter();
save_signatures();
free(boxes);
free(box_manager);

/*free_index(i);*/

close_tree();
		
menu_ienable(menu_adr, M_CLSBOX, FALSE);
menu_ienable(menu_adr, M_NFOLD, FALSE);
menu_ienable(menu_adr, M_ABOOK, FALSE);
menu_ienable(menu_adr, M_FILTER, FALSE);
menu_ienable(menu_adr, M_SIGNAT, FALSE);
menu_ienable(menu_adr, M_IDENTITY, FALSE);
menu_ienable(menu_adr, M_FAVOR, FALSE);
menu_ienable(menu_adr, M_PSETUP, FALSE);
mail_options.maildir[0] = 0;

box_open = FALSE;
}

/* ---------------------------------
   | Userdefined redraw of preview |
   --------------------------------- */
int cdecl ipreview(PARMBLK *paramblock)
{
int offset;

DEBUG();
obj_clip(paramblock, TRUE);

if (paramblock->pb_parm >= 0)
	offset = 6 + (int)paramblock->pb_parm;
else
	offset = -(int)paramblock->pb_parm;
	
icon_copy(boxicons, 0, offset * paramblock->pb_h, 
					paramblock->pb_x, paramblock->pb_y, 
					paramblock->pb_w, paramblock->pb_h);

obj_clip(paramblock, FALSE);
return 0;
}

/* ---------------------
   | Create new folder |
   --------------------- */
void folder_properties(int box)
{
OBJECT *nfold;
GRECT wrec;
static GRECT empty = {0,0,0,0};
int obj;
char identity[21];

DEBUG();
rsrc_gaddr(R_TREE, D_NEWFLD, &nfold);

if (!(box & 0x8000))
	{
	strcpy(nfold[D_NFDESC].ob_spec.tedinfo->te_ptext, boxes[box].desc);
	iconpreview.ub_parm = boxes[box].icon;
	if (iconpreview.ub_parm < 6)
		iconpreview.ub_parm = -iconpreview.ub_parm;
	else
		iconpreview.ub_parm -= 6;

	SetSelect(nfold, POPSWTCH, 		boxes[box].type == -1);
	SetSelect(nfold, FOLDERSWTCH, boxes[box].type == -2);
	SetSelect(nfold, INSWITCH, 		boxes[box].type == 0);
	SetSelect(nfold, OUTSWTCH, 		boxes[box].type == 1 || boxes[box].type == 3);
	if (boxes[box].type >= 0)
		{
		if (boxes[box].type != 3 && 
				box != box_number("IN") && box != box_number("SENT") && box != box_number("SENTERR"))
			{
			EnableObj(nfold, INSWITCH);
			EnableObj(nfold, OUTSWTCH);
			}
		else
			{
			DisableObj(nfold, INSWITCH);
			DisableObj(nfold, OUTSWTCH);
			}
		DisableObj(nfold, POPSWTCH);
		DisableObj(nfold, FOLDERSWTCH);
		}
	else
		{
		if (boxes[box].type == -2)
			{
			DisableObj(nfold, SICONUP);
			DisableObj(nfold, SICONDWN);
			}
		else
			{
			EnableObj(nfold, SICONUP);
			EnableObj(nfold, SICONDWN);
			}
						
		DisableObj(nfold, POPSWTCH);
		DisableObj(nfold, FOLDERSWTCH);
		DisableObj(nfold, INSWITCH);
		DisableObj(nfold, OUTSWTCH);
		}
	if (!boxes[box].identity[0])
		if (boxes[box].type == -1)
			strcpy(nfold[DNEW_IDSEL].ob_spec.tedinfo->te_ptext, all_string);
		else
			strcpy(nfold[DNEW_IDSEL].ob_spec.tedinfo->te_ptext, none_string);
	else
		strcpy(nfold[DNEW_IDSEL].ob_spec.tedinfo->te_ptext, boxes[box].identity);
	}
else
	{
	nfold[D_NFDESC].ob_spec.tedinfo->te_ptext[0] = 0;
	iconpreview.ub_parm = -6;
	SetSelect(nfold, INSWITCH, FALSE);
	SetSelect(nfold, OUTSWTCH, FALSE);
	SetSelect(nfold, POPSWTCH, FALSE);
	SetSelect(nfold, FOLDERSWTCH, TRUE);
	EnableObj(nfold, INSWITCH);
	EnableObj(nfold, OUTSWTCH);
	EnableObj(nfold, POPSWTCH);
	EnableObj(nfold, FOLDERSWTCH);
	nfold[DNEW_IDSEL].ob_spec.tedinfo->te_ptext[0] = 0;
	}

nfold[SPREVIEW].ob_type = G_USERDEF;
nfold[SPREVIEW].ob_spec.userblk = &iconpreview;
nfold[SP_BKG].ob_width = nfold[SPREVIEW].ob_width = icon_palheader.width;
nfold[SP_BKG].ob_height = nfold[SPREVIEW].ob_height = 
	smallfont ? icon_palheader.height / 2 : icon_palheader.height;

open_dialog(nfold, nfold[CFOLD_HEADER].ob_spec.free_string, iconified_tree);
dlgset_help_info(nfold, helpfile, nfold_ids, "NF_");
do
	{
	obj = do_dialog(nfold, 0, app_callback) & 0x7FFF;
	switch(obj)
		{
		case SICONUP:
			if (iconpreview.ub_parm > 0)
				{
				iconpreview.ub_parm--;
				if (!iconpreview.ub_parm)
					{
					switch(boxes[box].type)
						{
						case 3:
							iconpreview.ub_parm = -4;
							break;
					
						case -2:
							iconpreview.ub_parm = -6;
							break;
					
						case -1:
							iconpreview.ub_parm = -3;
							break;
								
						default:
							if (IsSelected(nfold, INSWITCH))
								iconpreview.ub_parm = -1;
							else
								iconpreview.ub_parm = -2;
							break;
						}
					}
					
				redraw_object(nfold, SPREVIEW);
				}
			break;
			
		case SICONDWN:
			if (iconpreview.ub_parm < 20)
				{
				if (iconpreview.ub_parm < 0)
					iconpreview.ub_parm = 0;

				iconpreview.ub_parm++;
				redraw_object(nfold, SPREVIEW);
				}
			break;
			
		case POPSWTCH:
			EnableObj(nfold, SICONUP);
			EnableObj(nfold, SICONDWN);
			if (iconpreview.ub_parm < 0 && iconpreview.ub_parm != -3)
				{
				iconpreview.ub_parm = -3;
				redraw_object(nfold, SPREVIEW);
				}
			break;
			
		case FOLDERSWTCH:
			DisableObj(nfold, SICONUP);
			DisableObj(nfold, SICONDWN);
			if (iconpreview.ub_parm != -6)
				{
				iconpreview.ub_parm = -6;
				redraw_object(nfold, SPREVIEW);
				}
			break;
			
		case INSWITCH:
			EnableObj(nfold, SICONUP);
			EnableObj(nfold, SICONDWN);
			if (iconpreview.ub_parm < 0 || iconpreview.ub_parm == -2)
				{
				iconpreview.ub_parm = -1;
				redraw_object(nfold, SPREVIEW);
				}
			break;
				
		case OUTSWTCH:
			EnableObj(nfold, SICONUP);
			EnableObj(nfold, SICONDWN);
			if (iconpreview.ub_parm < 0 || iconpreview.ub_parm == -1)
				{
				iconpreview.ub_parm = -2;
				redraw_object(nfold, SPREVIEW);
				}
			break;
			
		case DNEW_IDSEL:
		case DNEW_IDSELDROP:
			{
			int x, y;
			
			objc_offset(nfold, DNEW_IDSEL, &x, &y);
			if (DialogIdentitySelect(identity, x, y, 
															 nfold[DNEW_IDSELDROP].ob_x + nfold[DNEW_IDSELDROP].ob_width - nfold[DNEW_IDSEL].ob_x,
															 IsSelected(nfold, POPSWTCH) ? all_string : none_string))
				{
				if (!identity[0])
					if (IsSelected(nfold, POPSWTCH))
						strcpy(nfold[DNEW_IDSEL].ob_spec.tedinfo->te_ptext, all_string);
					else
						strcpy(nfold[DNEW_IDSEL].ob_spec.tedinfo->te_ptext, none_string);
				else
					strcpy(nfold[DNEW_IDSEL].ob_spec.tedinfo->te_ptext, identity);
				redraw_object(nfold, DNEW_IDSEL);
				}
			break;
			}
		}
		
	}while(obj != NF_OKAY && obj != NF_CANCL);

if (obj == NF_OKAY)	
	{
	int icon, type;
	
	icon = (int)iconpreview.ub_parm;
	if (icon >= 0)
		icon += 6;
	else
		icon = -(int)iconpreview.ub_parm;
		
	if (box & 0x8000)
		{
		if (IsSelected(nfold, INSWITCH)) 		type = 0;
		if (IsSelected(nfold, OUTSWTCH)) 		type = 1;
		if (IsSelected(nfold, FOLDERSWTCH)) type = -2;
		if (IsSelected(nfold, POPSWTCH)) 		type = -1;
		
		create_new_folder(box & 0x7FFF, type,
											nfold[D_NFDESC].ob_spec.tedinfo->te_ptext,
											nfold[DNEW_IDSEL].ob_spec.tedinfo->te_ptext,
											icon);
		}
	else
		{
		if (IsSelected(nfold, INSWITCH)) 		type = 0;
		if (IsSelected(nfold, OUTSWTCH)) 		type = 1;
		if (IsSelected(nfold, FOLDERSWTCH)) type = -2;
		if (IsSelected(nfold, POPSWTCH)) 		type = -1;
		
		if (boxes[box].type != 3 && boxes[box].type >= 0)
			boxes[box].type = IsSelected(nfold, OUTSWTCH);
			
		strcpy(boxes[box].desc, nfold[D_NFDESC].ob_spec.tedinfo->te_ptext);
		strcpy(boxes[box].identity, identity);
		boxes[box].icon = icon;
		}
	
	wind_get_grect(box_wind, WF_WORKXYWH, &wrec);
	form_dial(FMD_FINISH, &empty, &wrec);
	}
	
close_dialog(nfold, TRUE);
}

/* ---------------------
   | Delete one folder |
   --------------------- */
void delete_one(int box)
{
char inf[256];
GRECT wrec;
static GRECT empty = {0,0,0,0};
		
DEBUG();
sprintf(inf, delfoldq, boxes[box].desc, box_manager[box].box_entries);
if (form_alert(2, inf) == 1)
	if (delete_folder(boxes[box].file))
		{
		int idx = boxes[box].fav_index;
		int i;
		
		free_index(box);

		if (box < box_count - 1)
			{
			memmove(boxes + box, boxes + box + 1, sizeof(BOX_INFO) * (box_count - box - 1));
			memmove(box_manager + box, box_manager + box + 1, sizeof(BOX_MANAGER) * (box_count - box - 1));
			}
		box_count--;

		for (i = 0; i < box_count; i++)
			if (boxes[i].fav_index > idx)
				boxes[i].fav_index--;
		move_filterfolders(box + 1, -1);

		if (box_wind > -1)
			{
			update_boxinfo();
			wind_get_grect(box_wind, WF_WORKXYWH, &wrec);
			form_dial(FMD_FINISH, &empty, &wrec);
			}
			
		wind_get_grect(tree_wind, WF_WORKXYWH, &wrec);
		form_dial(FMD_FINISH, &empty, &wrec);
		}
}

/* ----------------------------------
   | Return index for specified box |
   ---------------------------------- */
int box_number(char *box)
{
int i;

DEBUG();
for (i = 0; i < 20; i++)
	if (!stricmp(boxes[i].file, box))
		return i;
return -1;
}


