/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

extern int 	view_wind[20];
char *bubble_text;

#define BUBBLEGEM_REQUEST  0xBABA
#define BUBBLEGEM_SHOW     0xBABB
#define BUBBLEGEM_ACK      0xBABC
#define BUBBLEGEM_ASKFONT  0xBABD
#define BUBBLEGEM_FONT     0xBABE
#define BUBBLEGEM_HIDE     0xBABF

/* --------------------------------------
   | If bubble gem is available, use it |
   -------------------------------------- */
int show_bubblegem(char *txt, int x, int y)
{
int msg[8];
int bubble_id;

DEBUG();
if (!mail_options.bubblegem)
	return FALSE;
	
bubble_text = alloc_shared(256);

if (!bubble_text)   /* Pointer ungltig, kein Speicher da */
  return FALSE;

strncpy(bubble_text, txt, 255);
bubble_text[255] = 0;

bubble_id = appl_find("BUBBLE  ");

if (bubble_id >= 0)
  {
  msg[0] = BUBBLEGEM_SHOW;
  msg[1] = gl_apid;
  msg[2] = 0;
  msg[3] = x;
  msg[4] = y;
  msg[5] = (int)(((long) bubble_text >> 16)
           & 0x0000ffff);
  msg[6] = (int)((long) bubble_text & 0x0000ffff);
  msg[7] = 0;
  if (appl_write(bubble_id, 16, msg) == 0)
    {
      /* Fehler */
    }
  return TRUE;
  }
return FALSE;
}

int bubble_free(char *ptr)
{
if (ptr == bubble_text)
	{
	Mfree(ptr);
	return TRUE;
	}
return FALSE;
}

/* -------------------------
   | Mailwind info bubbles |
   ------------------------- */
void view_bubble(int idx, int mouse_x, int mouse_y)
{
int ret;
static long last_mouse = -1L;
static int last_shown = -1, last_but = -1;
static int offset[] = {LBUBBLE3, LBUBBLE4, 
											 B_CUT, B_COPY, B_PASTE, B_DEL, 
											 LBUBBLE5, LBUBBLE6, LBUBBLE7, LB_CLOSE};

DEBUG();
ret = get_viewwind_button(idx, mouse_x, mouse_y);
if (last_but != ret)
	{
	if (last_but != -1)
		{
		select_viewbutton(idx, -1);
		redraw_viewbutton(idx, last_but);
		}

	if (ret != last_shown)
		last_shown = -1;
	last_but = ret;

	if (ret != -1)
		if (tool_palheader.rollover)
			{
			select_viewbutton(idx, ret);
			redraw_viewbutton(idx, ret);
			}

	last_mouse = clock();
	}
else
	if (ret != -1 && clock() - last_mouse > CLK_TCK * 2)
		{
		if (last_shown != ret)
			show_bubble_info(view_wind[idx], header_tree[offset[ret]].ob_spec.free_string);
		last_mouse = clock();
		last_shown = ret;
		}
	else
		last_shown = -1;
}

/* -------------------------
   | Mailwind info bubbles |
   ------------------------- */
void list_bubble(int box, int mouse_x, int mouse_y)
{
int ret;
static long last_mouse = -1L;
static int last_shown = -1, last_but = -1;
static int offset[] = {LBUBBLE1, LBUBBLE2, LBUBBLE3, LBUBBLE4,
											 LBUBBLE5, LBUBBLE6, LBUBBLE7, LB_CLOSE};

DEBUG();
ret = get_listwind_button(box, mouse_x, mouse_y);
if (last_but != ret)
	{
	if (last_but != -1)
		{
		select_listbutton(box, -1);
		redraw_listbutton(box, last_but);
		}

	if (ret != last_shown)
		last_shown = -1;
	last_but = ret;

	if (ret != -1)
		if (tool_palheader.rollover)
			{
			select_listbutton(box, ret);
			redraw_listbutton(box, ret);
			}

	last_mouse = clock();
	}
else
	if (ret != -1 && clock() - last_mouse > CLK_TCK * 2)
		{
		if (last_shown != ret)
			show_bubble_info(box_manager[box].list_wind, header_tree[offset[ret]].ob_spec.free_string);
		last_mouse = clock();
		last_shown = ret;
		}
	else
		last_shown = -1;
}

/* -------------------------
   | Mailwind info bubbles |
   ------------------------- */
void pop_bubble(int mouse_x, int mouse_y)
{
int ret;
static long last_mouse = -1L;
static int last_shown = -1, last_but = -1;
static int offset[] = {PBUBBLE_0, PBUBBLE_1, PBUBBLE_2, PBUBBLE_3,
											 PBUBBLE_4};

DEBUG();
ret = get_popwind_button(mouse_x, mouse_y);
if (last_but != ret)
	{
	if (last_but != -1)
		{
		select_popbutton(-1);
		redraw_popbutton(last_but);
		}

	if (ret != last_shown)
		last_shown = -1;
	last_but = ret;

	if (ret != -1)
		if (tool_palheader.rollover)
			{
			select_popbutton(ret);
			redraw_popbutton(ret);
			}

	last_mouse = clock();
	}
else
	if (ret != -1 && clock() - last_mouse > CLK_TCK * 2)
		{
		if (last_shown != ret)
			show_bubble_info(pop_wind, header_tree[offset[ret]].ob_spec.free_string);
		last_mouse = clock();
		last_shown = ret;
		}
	else
		last_shown = -1;
}

/* -------------------------
   | Mailwind info bubbles |
   ------------------------- */
void bubble_info(int mouse_x, int mouse_y)
{
int ret;
static long last_mouse = -1L;
static int last_shown = -1, last_but = -1;
static int offset[] = {BUBBLE1, BUBBLE2, B_CUT, B_COPY, B_PASTE,
											 B_DEL, BUBBLE3, BUBBLE4, BUBBLE5, BUBBLE6, 
											 BUBBLE7, BUBBLE8};
int top;
			
DEBUG();
wind_get_int(ROOT, WF_TOP, &top);
			
if (is_listwindow(top)) 
	list_bubble(window_box(top), mouse_x, mouse_y);

if (is_viewwindow(top)) 
	view_bubble(window_view(top), mouse_x, mouse_y);

if (top == pop_wind)
	pop_bubble(mouse_x, mouse_y);

if (top != mail_wind)		
	return;

ret = get_mailwind_button(mouse_x, mouse_y);
if (last_but != ret)
	{
	if (last_but != -1)
		{
		select_mbbutton(-1);
		redraw_mbbutton(last_but);
		}

	if (ret != last_shown)
		last_shown = -1;
	last_but = ret;

	if (ret != -1)
		if (tool_palheader.rollover)
			{
			select_mbbutton(ret);
			redraw_mbbutton(ret);
			}
			
	last_mouse = clock();
	}
else
	if (ret != -1 && clock() - last_mouse > CLK_TCK * 2)
		{
		if (last_shown != ret)
			show_bubble_info(mail_wind, header_tree[offset[ret]].ob_spec.free_string);
		last_mouse = clock();
		last_shown = ret;
		}
	else
		last_shown = -1;
}

/* -------------------------
   | Show link information |
   ------------------------- */
void show_bubble_info(int wind, char *info)
{
int xy[4], w, w1, w2, font_w, font_h;
int wx, wy, ww, wh, d;
int mx, my;
GRECT t1, t2;
static GRECT empty = {0,0,0,0};
long start;

DEBUG();
if (show_bubblegem(info, evnt.x, evnt.y))
	return;

wind_update(BEG_UPDATE);
wind_get(ROOT, WF_WORKXYWH, &wx, &wy, &ww, &wh);

w1 = (int)strlen(info);
w2 = 0;

vst_font(vdi_handle, 1);
vst_color(vdi_handle, 1);
vst_effects(vdi_handle, 0);
vst_alignment(vdi_handle, 0, 3, &d, &d);
vst_height(vdi_handle, 4, &d, &d, &font_w, &font_h);

w = font_w * w1;

xy[0] = evnt.x - (w >> 1) - 2;
xy[1] = evnt.y - font_h - 2;
xy[2] = xy[0] + w + 4;
xy[3] = xy[1] + font_h + 4;

if (xy[0] < 0)
	{
	xy[2] -= xy[0];
	xy[0] = 0;
	}

if (xy[2] >= wx + ww)
	if (xy[0] > xy[2] - (wx + ww))
		{
		xy[0] -= xy[2] - (wx + ww);
		xy[2] = wx + ww;
		}
		
if (xy[0] <= wx && xy[2] >= wx + ww)
	{
	xy[0] = wx;
	xy[2] = wx + ww - 1;
	
	w2 = w1 = (ww - 4) / font_w - 4;
	}

if (xy[1] < wy)
	{
	xy[3] += wy - xy[1];
	xy[1] = wy;
	}

vswr_mode(vdi_handle, MD_REPLACE);
vsf_interior(vdi_handle, FIS_HOLLOW);
vsf_color(vdi_handle, 1);
vsl_type(vdi_handle, 1);
vsl_width(vdi_handle, 1);

t2.g_x = xy[0];
t2.g_y = xy[1];
t2.g_w = xy[2] - xy[0] + 1;
t2.g_h = xy[3] - xy[1] + 1;

wind_get(wind, WF_FIRSTXYWH, &t1.g_x, &t1.g_y, &t1.g_w, &t1.g_h);
while(t1.g_w && t1.g_h)
	{
	if (rc_intersect(&t2, &t1))
		{
		v_bar(vdi_handle, xy);
		v_gtext(vdi_handle, xy[0] + 2, xy[3] - 2, info);

		if (w2)
			{
			v_gtext(vdi_handle, xy[2] - 2 - (w2 + 4) * font_w, xy[3] - 2, "....");
			v_gtext(vdi_handle, xy[2] - 2 - (w2 + 4) * font_w, xy[3] - 2, 
							info + strlen(info) - w2);
			}
		}
		
	wind_get(wind, WF_NEXTXYWH, &t1.g_x, &t1.g_y, &t1.g_w, &t1.g_h);
	}
vst_alignment(vdi_handle, 0, 5, &d, &d);

start = clock();
check_user_message();
mx = evnt.x;
my = evnt.y;
do
	{
	if (check_user_message())
		break;
	}while(mx == evnt.x && my == evnt.y && clock() < start + CLK_TCK * 5);

wind_update(END_UPDATE);

form_dial(FMD_FINISH, &empty, &t2);
}
