/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

static char path[MAXDRIVE + MAXPATH];
static char file[MAXFILE + MAXEXT];
static int path_flg = FALSE;
char sharedpath[256];

long cdecl __sock_call__(long fnc, ...);

static int bootdrv;
static char slbpath[256*3];

/* ------------------
   | Get boot drive |
   ------------------ */
void get_bootdrive(void)
{
long old_stk;

DEBUG();
if (!Super((void *)1L))
	old_stk = Super(0L);
else	
	old_stk = 0;

bootdrv = *(int *)0x446;
	
if (old_stk)
	Super((void *)old_stk);	
}

/* -----------------
   | Scan one path |
   ----------------- */
void scan_one(char *scan_path)
{
DTA in_dta;
int s, f;
long dcfg;

DEBUG();
Fsetdta(&in_dta);
s = Fsfirst(scan_path, FA_READONLY|FA_ARCHIVE);
while(!s)
	{
	if (!prov_cnt++)
		providers = malloc(sizeof(PROVIDER));
	else
		providers = realloc(providers, prov_cnt * sizeof(PROVIDER));
		
	strcpy(providers[prov_cnt - 1].prov_path, scan_path);
	strcpy(strrchr(providers[prov_cnt - 1].prov_path, '\\') + 1, in_dta.d_fname);

	f = (int)Fopen(providers[prov_cnt - 1].prov_path, FO_READ);
	if (f > 0)	
		{
		Fread(f, sizeof(dcfg), &dcfg);
		if (dcfg == CFG_ID)
			Fread(f, 21, providers[prov_cnt - 1].provider);
		else
			prov_cnt--;
		Fclose(f);
		}
	else
		prov_cnt--;
		
	s = Fsnext();
	}
}

/* -------------------------------
   | Read installation directory |
   ------------------------------- */
void read_installdir(void)
{
char *c;
long fh;

DEBUG();
if (path_flg) return;

strcpy(mailpath, "marathon.rsc");
if (shel_find(mailpath) != 0)
	{
	char *s;

	if (!strcmp(mailpath, "marathon.rsc"))
		{
		mailpath[0] = Dgetdrv() + 'A';
		mailpath[1] = ':';
		Dgetpath(mailpath + 2, 0);
		}
	else
		{
		s = mailpath + strlen(mailpath) - 12;
		if (s == mailpath || (s[-1] != '\\' && s[-1] != '/'))
			mailpath[0] = 0;
		else
			s[-1] = 0;
		}
	}
else
	mailpath[0] = 0;
		
c = "x:\\draconis.inf";
c[0] = 'A' + bootdrv;
fh = Fopen(c, FO_READ);
if (fh > 0)
	{
	Fread((int)fh, 80, path);
	if (Fread((int)fh, 80, browserpath) != 80)
		browserpath[0] = 0;
	if (!mailpath[0])
		if (Fread((int)fh, 80, mailpath) != 80)
			mailpath[0] = 0;
  Fseek(8*80, (int)fh, SEEK_SET);
	if (Fread((int)fh, 80, sharedpath) != 80)
		sharedpath[0] = 0;
	Fclose((int)fh);
	}
else
	{
	strcpy(path, "x:\\draconis");
	path[0] = c[0];
	browserpath[0] = 0;
	}

if (!mailpath[0])
	{
	mailpath[0] = Dgetdrv() + 'A';
	mailpath[1] = ':';
	Dgetpath(mailpath + 2, 0);
	}

strcpy(slbpath, mailpath);
strcat(slbpath, ";");
strcat(slbpath, path);
strcat(slbpath, "\\shared;");
strcat(slbpath, sharedpath);

set_slbpath(slbpath);

strcpy(helpfile, mailpath);
strcat(helpfile, "\\help\\");
strcat(helpfile, languagecode);
strcat(helpfile, ".hlp");
}

/* ----------------------------
   | Load start configuration |
   ---------------------------- */
void start_config(void)
{
CFG_OPT *opt_set;
char buffer[255];

DEBUG();
opt_set = (CFG_OPT *)__sock_call__(FNC_GETOPTIONS);
if (opt_set != (CFG_OPT *)-1L)
	{
	memcpy(&cfg_options, opt_set, sizeof(CFG_OPT));
	strcpy(pop_password, cfg_options.mailpass);
	crypt(pop_password);
	strcpy(pop_username, cfg_options.popuser);
	return;
	}

if (!path_flg)
	{
	read_installdir();
	path_flg = TRUE;
	}
	
strcpy(buffer, path);
strcat(buffer, "\\CNF\\DEFAULT.CFG");

load_config(buffer);

strcat(path, "\\CNF\\*.CFG");
strcpy(file, "DEFAULT.CFG");
}

/* ----------------------
   | Load configuration |
   ---------------------- */
int load_config(char *f_load)
{
int button;
char full[MAXDRIVE + MAXPATH + MAXFILE + MAXEXT], *s;
int fhandle;
long dcfg;

DEBUG();
if (!f_load)
	{
	if (aes_version >= 0x104)
		fsel_exinput(path, file, &button, "Load DRACONIS configuration...");
	else
		fsel_input(path, file, &button);

	strcpy(full, path);
	s = strrchr(full, '\\');
	if (s)
		strcpy(s + 1, file);
	else
		strcat(full, file);
	}
else
	strcpy(full, f_load);
	
if (button == 1 || f_load)
	{
	fhandle = (int)Fopen(full, FO_READ);
	if (fhandle >  0)
		{
		Fread(fhandle, sizeof(dcfg), &dcfg);
		if (dcfg == CFG_ID)
			{
			Fseek(0, fhandle, 0);
			Fread(fhandle, sizeof(cfg_options), &cfg_options);
		
			strcpy(pop_password, cfg_options.mailpass);
			crypt(pop_password);
			strcpy(pop_username, cfg_options.popuser);
			}
		else
			form_alert(1, invalid_conf);
					
		Fclose(fhandle);
		return TRUE;
		}
	else
		{
		if (!f_load)
			{
			strcpy(full, "[3][Error #");
			itoa(fhandle, full + 11, 10);
			strcat(full, ".][OK]");		
			form_alert(1, full);
			}
		return FALSE;
		}
	}
return FALSE;
}
