/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

char default_confirm[] = 
"*****************************************************************\r"\
"\r"\
"Your message was read!\r"\
"\r"\
"				at the _DATE_ - _TIME_.\r"\
"-----------------------------------------------------------------\r"\
"\r"\
"This message is generated automatically by Marathon Mail of\r"\
"the Draconis Internet Package for ATARI and compatible computers.\r"\
"\r"\
"******************************************************************\r"\
"\r"\
"Ihre Nachricht wurde gelesen\r"\
"\r"\
"				am: _DATE_ um _TIME_ Uhr.\r"\
"\r"\
"------------------------------------------------------------------\r"\
"\r"\
"Diese Nachricht wurde automatisch von Marathon Mail aus dem\r"\
"Draconis-Internet-Paket fuer ATARI und kompatible Computer\r"\
"generiert.\r"\
"\r"\
"******************************************************************\r";

char *confirm_text;
long confirm_length;

/* --------------------------
   | Load confirmation text |
   -------------------------- */
void load_confirm(void)
{
long fh;

DEBUG();
fh = Fopen("CONFIRM.TXT", FO_READ);
if (fh > -1)
	{
	confirm_length = Fseek(0, (int)fh, 2);
	Fseek(0, (int)fh, 0);
	
	confirm_text = (char *)malloc(confirm_length + 1);
	Fread((int)fh, confirm_length, confirm_text);
	confirm_text[confirm_length] = 0;
	
	Fclose((int)fh);
	}
else
	{
	confirm_length = (int)strlen(default_confirm);

	confirm_text = (char *)malloc(confirm_length + 1);
	strcpy(confirm_text, default_confirm);
	confirm_text[confirm_length] = 0;
	}
}

/* ------------------------------
   | Generate read confirmation |
   ------------------------------ */
void gen_confirmation(char *mailbody, long body_len, char *addr, char *idname)
{
long p, p2, offset;
int reply_flag = FALSE, line_length;
char *y, *file;
MAIL_MEM zw_new;
time_t mtime;
struct tm  *mytime;
int idx;
IDENTITY *identity;

DEBUG();
if (!mail_options.allow_readconfirm)
	return;

idx = getIdentityIndex(idname);
if (idx == -1)
	return;
identity = getIdentityPtr(idx);
if (!identity)
	return;
	
memcpy(&zw_new, &new_buffer, sizeof(MAIL_MEM));

memset(&new_buffer, 0, sizeof(MAIL_MEM));
create_empty_header();

new_buffer.confirm_rcpt	=	0;
new_buffer.confirm_read	=	0;
new_buffer.priority		  = 0;

p = 0;
while(p < body_len)
	{
	while((mailbody[p] == 1 || mailbody[p] == ' ' || 
				 mailbody[p] == '\t')	&& p < body_len)
		p++;

	if (mailbody[p] == '\r') break;

	if ((!strnicmp(mailbody + p, "From:", 5) && !reply_flag) || 
			!strnicmp(mailbody + p, "Reply-To:", 9))
		{
		if (!strnicmp(mailbody + p, "From:", 5))
			p += 5;
		else
			{
			p += 9;
			reply_flag = TRUE;
			}
			
		while(mailbody[p] == ' ' || mailbody[p] == '\t')	
			p++;
	
		y = new_buffer.to;
		while (mailbody[p] != '\r' || 
					 !strncmp(mailbody + p + 1, "\1              ", 14))
			{
			if (mailbody[p] != '\r')
				{
				if (y - new_buffer.to + 2 > new_buffer.to_max)
					{
					offset = y - new_buffer.to;
					new_buffer.to_max += 60;
					new_buffer.to = (char *)realloc(new_buffer.to, new_buffer.to_max);
					y = new_buffer.to + offset;
					}
				*y++ = mailbody[p];
				}
			else
				p += 14;
			p++;
			}
		}
	else
		if (!strnicmp(mailbody + p, "Subject:", 8))
			{
			p += 8;
			while(mailbody[p] == ' ' || mailbody[p] == '\t')	
				p++;
	
			if (new_buffer.subj_max < 60)
				{
				new_buffer.subj_max = 60;
				new_buffer.subject = (char *)realloc(new_buffer.subject, 60);
				}
								
			y = new_buffer.subject;
			strcpy(y, "RC: ");
			y += 4;
			while (mailbody[p] != '\r' || 
						 !strncmp(mailbody + p + 1, "\1              ", 14))
				{
				if (mailbody[p] != '\r')
					{
					if (y - new_buffer.subject + 2 > new_buffer.subj_max)
						{
						offset = y - new_buffer.subject;
						new_buffer.subj_max += 60;
						new_buffer.subject = (char *)realloc(new_buffer.subject, new_buffer.subj_max);
						y = new_buffer.subject + offset;
						}
					*y++ = mailbody[p];
					}
				else
					p += 14;
				p++;
				}
			*y = 0;				
			}
		else
			while (mailbody[p] != '\r') 
				p++;

	p++;
	}
	
if (addr[0])
	strcpy(new_buffer.to, addr);
	
if (identity->username[0])
	{
	new_buffer.from[0] = '"';
	strcpy(new_buffer.from + 1, identity->username);
	strcat(new_buffer.from, "\" <");
	}

if (identity->use_providerdata)
	{
	strcat(new_buffer.from, cfg_options.email);
	strcat(new_buffer.from, cfg_options.email2);
	}
else
	{
	strcat(new_buffer.from, identity->email);
	strcat(new_buffer.from, identity->email2);
	}
	
if (identity->username[0])
	strcat(new_buffer.from, ">");

if (identity->organization)
	strcpy(new_buffer.organization, identity->organization);

strcpy(new_buffer.identity, idname);

mtime = time(NULL);
mytime = localtime(&mtime);
if (mytime->tm_year < 200) mytime->tm_year += 1900;

y = (char *)malloc(confirm_length + 1000);
p = p2 = 0;
line_length = 0;
while (confirm_text[p])
	{
	if (!strncmp(confirm_text + p, "_DATE_", 6))
		{
		p += 6;

		sprintf(y + p2, "%2d %03s %04d", 
						mytime->tm_mday, m_desc[mytime->tm_mon], 
						mytime->tm_year);
		
		p2 += (int)strlen(y + p2);
		}
	else
		if (!strncmp(confirm_text + p, "_TIME_", 6))
			{
			p += 6;
			sprintf(y + p2, "%02d:%02d:%02d", 
							mytime->tm_hour, mytime->tm_min, mytime->tm_sec);
			
			p2 += (int)strlen(y + p2);
			}
		else
			if (confirm_text[p] == '\n')
				p++;
			else
				{
				if (confirm_text[p] == '\r')
					{
					if (line_length > new_buffer.max_length)
						new_buffer.max_length = line_length;
						
					new_buffer.body_len++;
					line_length = 0;
					}
					
				y[p2++] = confirm_text[p++];
				}

	if (p2 + 10 > confirm_length + 1000)
		break;
	}

y = (char *)realloc(y, (p2 + 4095L) & ~4095L);
new_buffer.body = y;
new_buffer.body_len = p2;

file = gen_mail();
if (file != NULL)
	store_mail(file, box_number("QUEUE"), &new_buffer, FALSE);

memcpy(&new_buffer, &zw_new, sizeof(MAIL_MEM));
}
