/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#undef malloc
#undef free
#undef realloc

#include <stddef.h>
#include <tos.h>
#include <mt_aes.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#define MAGIC1 	0x21234532L
#define MAGIC11	0x21234533L
#define MAGIC2 	0x35216572L
#define MAGICF 	0x45352165L
#define MAGICR 	0x45342115L

struct	dbg_info
		{
		long magic;
		char *file;
		long line;
		long len;
		struct	dbg_info *next;
		struct	dbg_info *prev;
		long magic2;
		};

static long magic2 = MAGIC2;
static char warn[200];
static struct dbg_info *first = NULL;

void  *dbg_malloc(char *file, long line, size_t size )
{
void *mem = malloc(size + sizeof(struct dbg_info) + 4);

if (mem)
	{
	struct dbg_info *dbg = (struct dbg_info *)mem;
	char *ptr;
	
	dbg->magic = MAGIC1;
	dbg->file = file;
	dbg->line = line;
	dbg->len = size;
	dbg->magic2 = MAGIC11;

	ptr = ((char *)mem) + sizeof(struct dbg_info);
	memcpy(ptr + size, &magic2, 4);
	
	dbg->next = first;
	dbg->prev = NULL;
	if (first)
		first->prev = dbg;
	first = dbg;
	
	return ptr;
	}
else
	return NULL;
}

void  dbg_free(char *file, long line,  void *ptr )
{
if (ptr)
	{
	struct dbg_info *dbg = (struct dbg_info *)(((char *)ptr) - sizeof(struct dbg_info));
	
	if (dbg->magic != MAGIC1 || dbg->magic2 != MAGIC11)
		{
		if (dbg->magic == MAGICF)
			sprintf(warn, "[3][Free: Memory-Block allready freed:|File: %s/%ld|Freed: %s/%ld][Ok]", file, line, dbg->file, dbg->line);
		else if (dbg->magic == MAGICR)
			sprintf(warn, "[3][Free: Memory-Block allready realloced:|File: %s/%ld|Real.: %s/%ld][Ok]", file, line, dbg->file, dbg->line);
		else
			sprintf(warn, "[3][Free: Memory-Block destroyed:|File: %s|Line: %ld|Value: %lx][Ok]", file, line, dbg->magic);
		form_alert(1, warn);
		return;
		}
	else
		{
		if (memcmp( ((char *)ptr) + dbg->len, &magic2, 4))
			{
			sprintf(warn, "[3][Free: Memory-Block destroyed:|Alloc:%s/%ld|File: %s/%ld][Ok]", dbg->file, dbg->line, 
										file, line);
			form_alert(1, warn);
			}
		}

	dbg->magic = MAGICF;
	dbg->file = file;
	dbg->line = line;
	
	if (first == dbg)
		{
		first = dbg->next;
		if (first)
			first->prev = NULL;
		}
	else
		{
		dbg->prev->next = dbg->next;
		if (dbg->next)
			dbg->next->prev = dbg->prev;
		}
				
	free( dbg  );
	}
else
	{
	sprintf(warn, "[3][Free: Null pointer:|File: %s|Line: %ld][Ok]", file, line);
	form_alert(1, warn);
	}
}

void  *dbg_realloc(char *file, long line,  void *block, size_t newsize )
{
void *mem;

if (block)
	{
	struct dbg_info *dbg = (struct dbg_info *)(((char *)block) - sizeof(struct dbg_info));
	
	if (dbg->magic != MAGIC1 || dbg->magic2 != MAGIC11)
		{
		if (dbg->magic == MAGICF)
			sprintf(warn, "[3][Realloc: Memory-Block allready freed:|Freed: %s/%ld|File: %s/%ld][Ok]", dbg->file, dbg->line, 
										file, line);
		else if (dbg->magic == MAGICR)
			sprintf(warn, "[3][Realloc: Memory-Block allready realloced:|Realloc:%s/%ld|File: %s/%ld][Ok]", dbg->file, dbg->line, 
										file, line);
		else
			sprintf(warn, "[3][Realloc: Memory-Block destroyed:|Alloc:%s/%ld|File: %s/%ld][Ok]", dbg->file, dbg->line, 
										file, line);
		form_alert(1, warn);
		return NULL;
		}
	else
		{
		if (memcmp( ((char *)block) + dbg->len, &magic2, 4))
			{
			sprintf(warn, "[3][Free: Memory-Block	destroyed:|Alloc:%s/%ld|File: %s/%ld][Ok]", dbg->file, dbg->line, 
										file, line);
			form_alert(1, warn);
			}
		}

	dbg->magic = MAGICR;
	dbg->file = file;
	dbg->line = line;
			
	mem = realloc( dbg, newsize + sizeof(struct dbg_info) + 4 );
	
	if (mem)
		{
		struct dbg_info *dbgn = (struct dbg_info *)mem;
		char *ptr;
		
		dbgn->magic = MAGIC1;
		dbgn->len = newsize;
	
		ptr = ((char *)mem) + sizeof(struct dbg_info);
		memcpy(ptr + newsize, &magic2, 4);

		if (first == dbg)
			first = dbgn;
		else
			dbgn->prev->next = dbgn;

		if (dbgn->next)
			dbgn->next->prev = dbgn;
		
		return ptr;
		}
	else
		{
		return NULL;
		}
	}
else
	{
	sprintf(warn, "[3][Realloc: Null pointer:|File: %s|Line: %ld][Ok]", file, line);
	form_alert(1, warn);
	return NULL;
	}
}
  
void check_memory(char *file, long line)
{
struct dbg_info *chk;

chk = first;
while(chk)
	{
	if (chk->magic != MAGIC1 || chk->magic2 != MAGIC11)
		{
		sprintf(warn, "[3][Check: Memory-Block	destroyed:|File: %s|Line: %ld|Value: %08lx][Ok]", file, line, chk->magic);
		form_alert(1, warn);
		}
	else
		{
		if (memcmp( ((char *)(chk + 1)) + chk->len, &magic2, 4))
			{
			sprintf(warn, "[3][Check: Memory-Block	destroyed:|Alloc:%s/%ld|File: %s/%ld][Ok]", chk->file, chk->line, 
										file, line);
			form_alert(1, warn);
			}
		}
	
	chk = chk->next;
	}
}
