/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

/* ---------------------------------
   | Bitblit from screen/to screen |
   --------------------------------- */
void blt_copy(void *from, int xf, int yf, void *to, int xt, int yt, int w, int h, int cpy_mode)
{
MFDB src, des;
int xy[8];

DEBUG();
memset(&src, 0, sizeof(MFDB));
memset(&des, 0, sizeof(MFDB));

src.fd_addr			= from;
if (from)
	{
	src.fd_w 				= xf + w;
	src.fd_h				= yf + h;
	src.fd_wdwidth 	= (xf + w + 15) >> 4;
	src.fd_stand		= 0;
	src.fd_nplanes  = planes;
	src.fd_r1 = src.fd_r2 = src.fd_r3 = 0;
	}

des.fd_addr			= to;
if (to)
	{
	des.fd_w 				= xt + w;
	des.fd_h				= yt + h;
	des.fd_wdwidth 	= (xt + w + 15) >> 4;
	des.fd_stand		= 0;
	des.fd_nplanes  = planes;
	des.fd_r1 = des.fd_r2 = des.fd_r3 = 0;
	}
		
xy[0] = xf;
xy[1] = yf;
xy[2] = xf + w - 1;
xy[3] = yf + h - 1;
xy[4] = xt;
xy[5] = yt;
xy[6] = xt + w - 1;
xy[7] = yt + h - 1;

graf_mouse(M_OFF, 0L);
vro_cpyfm(vdi_handle, cpy_mode, xy, &src, &des);
graf_mouse(M_ON, 0L);
}

/* -------------------------------
   | Set default text attributes |
   ------------------------------- */
void set_default_text(void)
{
int h_in, v_in;
int cx, cy, cw, ch;

DEBUG();
vst_font(vdi_handle, 1);
vst_height(vdi_handle, default_fsize, &cx, &cy, &cw, &ch);
vst_alignment(vdi_handle, 0, 5, &h_in, &v_in);
vst_effects(vdi_handle, 0);
/*vst_color(vdi_handle, mail_options.text_colour);*/
vst_color(vdi_handle, 1);
}

/* -------------
   | Set label |
   ------------- */
void draw_label(int x, int y, char *label)
{
DEBUG();
set_default_text();

vswr_mode(vdi_handle, MD_TRANS);
v_gtext(vdi_handle, x, y, label);
vswr_mode(vdi_handle, MD_REPLACE);

if (colours > 4)
	{
	int xy[6];
	
	xy[0] = x - 2; xy[1] = y + tx_height - 1;
	xy[2] = x - 2; xy[3] = y - 1;
	xy[4] = x + tx_width * (int)strlen(label);	xy[5] = y - 1;
	
	vsl_type(vdi_handle, 1);
	vsl_color(vdi_handle, 1);
	vsl_width(vdi_handle, 1);
	v_pline(vdi_handle, 3, xy);
	
	xy[2] = xy[4];
	xy[3] = xy[1];
	vsl_color(vdi_handle, 0);
	v_pline(vdi_handle, 2, xy);
	}
}

/* ------------------
   | Set edit field |
   ------------------ */
void draw_editfield(int x, int y, int len, char *field, int mark_start, int mark_end)
{
char *buf, c = 0;

DEBUG();
set_default_text();

if (mark_start != -32767)
	{
	if (mark_start < 0) mark_start = 0;
	if (mark_end < 0 || mark_start > len) mark_start = -32767;
	if (mark_end > len) mark_end = len;
	}
	
buf = (char *)malloc(len + 1);
memset(buf, '_', len);
buf[len] = 0;
v_gtext(vdi_handle, x, y, buf);

vsf_color(vdi_handle, 1);
vsf_interior(vdi_handle, FIS_SOLID);

if (strlen(field) > len)
	{
	c = field[len];
	field[len] = 0;
	if (mark_start == -32767)
		v_gtext(vdi_handle, x, y, field);
	else
		{
		char c1;
		int fxy[4];
		
		c1 = field[mark_start];
		field[mark_start] = 0;
		v_gtext(vdi_handle, x, y, field);
		field[mark_start] = c1;
		
		c1 = field[mark_end];
		field[mark_end] = 0;
		
		vswr_mode(vdi_handle, MD_TRANS);
		fxy[0] = x + mark_start * tx_width;
		fxy[1] = y;
		fxy[2] = fxy[0] + (mark_end - mark_start) * tx_width;
		fxy[3] = y + tx_height - 1;
		vr_recfl(vdi_handle, fxy);		

		vst_color(vdi_handle, 0);
		v_gtext(vdi_handle, x + mark_start * tx_width, y, field + mark_start);
		vst_color(vdi_handle, 1);
		vswr_mode(vdi_handle, MD_REPLACE);
		
		field[mark_end] = c1;
		
		v_gtext(vdi_handle, x + mark_end * tx_width, y, field + mark_end);
		}
		
	field[len] = c;
	}
else
	{
	if (mark_start == -32767)
		v_gtext(vdi_handle, x, y, field);
	else
		{
		char c1;
		int fxy[4];
		
		c1 = field[mark_start];
		field[mark_start] = 0;
		v_gtext(vdi_handle, x, y, field);
		field[mark_start] = c1;
		
		c1 = field[mark_end];
		field[mark_end] = 0;

		vswr_mode(vdi_handle, MD_TRANS);
		fxy[0] = x + mark_start * tx_width;
		fxy[1] = y;
		fxy[2] = fxy[0] + (mark_end - mark_start) * tx_width;
		fxy[3] = y + tx_height - 1;
		vr_recfl(vdi_handle, fxy);		
		
		vst_color(vdi_handle, 0);
		v_gtext(vdi_handle, x + mark_start * tx_width, y, field + mark_start);
		vst_color(vdi_handle, 1);
		vswr_mode(vdi_handle, MD_REPLACE);
		
		field[mark_end] = c1;
		
		v_gtext(vdi_handle, x + mark_end * tx_width, y, field + mark_end);
		}
	}
	
free(buf);

if (colours > 4)
	{
	int xy[4];
	
	xy[0] = x;	xy[1] = y - 1;
	xy[2] = x + len * tx_width;	xy[3] = y - 1;

	vsl_type(vdi_handle, 1);
	vsl_color(vdi_handle, 1);
	vsl_width(vdi_handle, 1);
	v_pline(vdi_handle, 2, xy);
	}
}

