/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

int fwd_box;
static MAIL_INFO *fwd_entry;
static char fwd_identity[21], *fwd_ptext;

static int fwd_ids[] = {FW_ADDR,
												FWD_TO,
												EDIT_FWD,
												ATT_FWD,
												FTYPE_IDSEL,
												FW_ABOOK,
												FWD_CNC,
												FWD_OKAY,
												0};
													
/* ----------------------------------------
   | Restore an old header from mail file |
   ---------------------------------------- */
void restore_header(int f, FILE_MAIL_MEM *header, MAIL_MEM *new_buffer)
{
DEBUG();
Fread(f, sizeof(FILE_MAIL_MEM), header);
if (header->magic != 0x4104)
	{
	memmove(&header->confirm_read, &header->magic, sizeof(FILE_MAIL_MEM) - (21+21+6));
	Fseek(-(21+21+6L), f, 1);
	header->replyto_len = 0;
	header->identity[0] = 0;
	header->signature[0] = 0;
	}
else
	{
	if (header->version == SVERSION_ID17)
		{
		Fseek(-(21+21L), f, 1);
		header->identity[0] = 0;
		header->signature[0] = 0;
		}
	}

if (header->header_inf == header->partinfo_pos && !header->date_len)
	{
	Fseek(0L, f, 0);
	quick_check(f, new_buffer, header);
	
	strcpy(new_buffer->identity, header->identity);
	strcpy(new_buffer->signature, header->signature);
	return;
	}
		
new_buffer->from_max = (header->from_len + 1);
new_buffer->replyto_max = (header->replyto_len + 1);
new_buffer->to_max 	= (header->to_len + 1);
new_buffer->cc_max 	= (header->cc_len + 1);
new_buffer->bcc_max 	= (header->bcc_len + 1);
new_buffer->subj_max = (header->subject_len + 1);

new_buffer->from 				= (char *)malloc(new_buffer->from_max);
new_buffer->replyto			= (char *)malloc(new_buffer->replyto_max);
new_buffer->to 					= (char *)malloc(new_buffer->to_max);
new_buffer->cc 					= (char *)malloc(new_buffer->cc_max);
new_buffer->bcc 					= (char *)malloc(new_buffer->bcc_max);
new_buffer->subject 			= (char *)malloc(new_buffer->subj_max);
new_buffer->organization = (char *)malloc(header->organization_len + 1);
new_buffer->date 				= (char *)malloc(51);
	
new_buffer->to[0] = 0;
new_buffer->organization[0] = 0;
new_buffer->from[0] = 0;
new_buffer->replyto[0] = 0;
new_buffer->to[0] = 0;
new_buffer->cc[0] = 0;
new_buffer->bcc[0] = 0;
new_buffer->subject[0] = 0;
new_buffer->date[0] = 0;

new_buffer->confirm_rcpt	=	header->confirm_rcpt;
new_buffer->confirm_read	=	header->confirm_read;
new_buffer->word_wrap		  = header->word_wrap;
new_buffer->priority		  = header->priority;
	
Fseek(header->header_inf, f, 0);
if (header->date_len)
 	{
  Fread(f, header->date_len, new_buffer->date);
  new_buffer->date[header->date_len] = 0;
  }
if (header->from_len)
	{
  Fread(f, header->from_len, new_buffer->from);
  new_buffer->from[header->from_len] = 0;
  }
if (header->replyto_len)
	{
  Fread(f, header->replyto_len, new_buffer->replyto);
  new_buffer->replyto[header->replyto_len] = 0;
  }
if (header->organization_len)
	{
  Fread(f, header->organization_len, new_buffer->organization);
  new_buffer->organization[header->organization_len] = 0;
  }
if (header->to_len)
	{
  Fread(f, header->to_len, new_buffer->to);
  new_buffer->to[header->to_len] = 0;
  }
if (header->cc_len)
	{
  Fread(f, header->cc_len, new_buffer->cc);
  new_buffer->cc[header->cc_len] = 0;
  }
if (header->bcc_len)
	{
  Fread(f, header->bcc_len, new_buffer->bcc);
  new_buffer->bcc[header->bcc_len] = 0;
  }
if (header->subject_len)
	{
  Fread(f, header->subject_len, new_buffer->subject);
  new_buffer->subject[header->subject_len] = 0;
  }
 
if (!header->replyto_len)
	{
	header->replyto_len = header->from_len;
	new_buffer->replyto			= (char *)realloc(new_buffer->replyto, new_buffer->replyto_max = header->replyto_len + 1);
	strcpy(new_buffer->replyto, new_buffer->from);
	}
	
strcpy(new_buffer->identity, header->identity);
strcpy(new_buffer->signature, header->signature);

if (header->part_count > 1)
	new_buffer->has_attachments = TRUE;
}

/* ----------------------
   | Reply to a message |
   ---------------------- */
void edit_mail(int box, MAIL_INFO *entry)
{
char 	*edit_mem;
long 	edit_blen, *edit_ptr;
int 	edit_width;
int 	edit_lines, f, i, wwrap, j, vwrap;
FILE_MAIL_MEM header;
IDENTITY setid;
int has_identity;

DEBUG();
if (entry->identity[0])
	has_identity = getIdentity(entry->identity, &setid);
else
	{
	int sbox = box;
	int l;
	
	has_identity = FALSE;
	do
		{
		if (boxes[sbox].identity[0])
			has_identity = getIdentity(boxes[sbox].identity, &setid);
		
		if (!boxes[sbox].level) break;
		l = boxes[sbox].level;
		
		while(sbox > -1 && boxes[sbox].level >= l)
			sbox--;
		
		}while (!has_identity && sbox > -1);
	}

if (!has_identity)
	has_identity = getDefaultIdentity(&setid);

vwrap = mail_options.viewwrap;
wwrap = has_identity ? setid.word_wrap : TRUE;
word_wrap = 0;
mail_options.viewwrap = 0;

i = get_body(s_edit,
							boxes[box].file, entry, 
							&edit_mem, &edit_blen, &edit_ptr,
					 		&edit_width, &edit_lines, FALSE);
word_wrap = wwrap;
mail_options.viewwrap = vwrap;

if (!i)
	return;
	
memset(&new_buffer, 0, sizeof(new_buffer));

f = open_file(entry->file, boxes[box].file);
if (f)
	{
	restore_header(f, &header, &new_buffer);

	if (header.part_count > 1)
		{
		int cnt;
		
		edit_parts = (PART_INFO *)malloc(sizeof(PART_INFO) * header.part_count);
		Fseek(header.partinfo_pos, f, 0);
		Fread(f, sizeof(PART_INFO) * header.part_count, edit_parts);

		new_buffer.attach_count = header.part_count - 1;
		new_buffer.attachments = (char **)malloc(sizeof(char *) * new_buffer.attach_count);
		new_buffer.att_encoding = (int *)malloc(sizeof(int *) * new_buffer.attach_count);
		
		epart_count = header.part_count;
		cnt = 0;		
		for (i = 0; i < header.part_count; i++)
			if (i != header.main_idx)
				{
				if (!edit_parts[i].name[0])
					{
					strcpy(edit_parts[i].name, attachinf_text);
					strcat(edit_parts[i].name, " ");
					itoa(cnt, edit_parts[i].name + strlen(edit_parts[i].name), 10);
					}

				new_buffer.att_encoding[cnt] = -1;
				for (j = 0; j < 6; j++)
					if (!strnicmp(edit_parts[i].type, enc_tab[j], strlen(enc_tab[j])))
						new_buffer.att_encoding[cnt] = j;
					
				new_buffer.attachments[cnt++] = edit_parts[i].name;
				}
		}
		 	
	Fclose(f);
	}
else
	create_empty_header();
		
free(edit_ptr);

strcpy(mail_title, "Edit: ");
strcat(mail_title, new_buffer.subject);
	
edit_mem = (char *)realloc(edit_mem, (edit_blen + 4095L) & ~4095L);
if (has_identity)
	{
	new_buffer.mime_allowed = setid.mime_allowed;
	new_buffer.editwidth = setid.editwidth;
	}
else
	{
	new_buffer.mime_allowed = FALSE;
	new_buffer.editwidth = 78;
	}
	
new_buffer.body = edit_mem;
new_buffer.body_len = edit_blen;
new_buffer.body_lines = edit_lines;
new_buffer.max_length = edit_width;

forward_mode = FALSE;
edit_mode = TRUE;
strcpy(edit_file, entry->file);
strcpy(edit_box, boxes[box].file);
open_mailwindow();
}

/* --------------------
   | Edit and forward |
   -------------------- */
void edit_forward(int box, MAIL_INFO *entry, char *address, char *setidentity)
{
char 	*fwd_mem, *x, *y;
long 	fwd_blen, *fwd_ptr;
int 	fwd_width, fwd_textlen;
int 	fwd_lines, f, i;
FILE_MAIL_MEM header;
IDENTITY setid;
int has_identity;

DEBUG();
if (!get_body(s_fwdm,
							boxes[box].file, entry, 
							&fwd_mem, &fwd_blen, &fwd_ptr,
					 		&fwd_width, &fwd_lines, TRUE))
	return;

strcpy(mail_title, s_fwd);

memset(&new_buffer, 0, sizeof(new_buffer));

f = open_file(entry->file, boxes[box].file);
if (f)
	{
	restore_header(f, &header, &new_buffer);

	new_buffer.from = (char *)realloc(new_buffer.from,
											new_buffer.from_max = 255);
	new_buffer.replyto = (char *)realloc(new_buffer.replyto,
											new_buffer.replyto_max = 255);
	new_buffer.to 	= (char *)realloc(new_buffer.to,
											new_buffer.to_max = (int)strlen(address) + 1);

	if (setidentity[0])
		has_identity = getIdentity(setidentity, &setid);
	else
		has_identity = FALSE;
			
	if (!has_identity)
		if (entry->identity[0])
			has_identity = getIdentity(entry->identity, &setid);
		else
			{
			int sbox = box;
			int l;
			
			has_identity = FALSE;
			do
				{
				if (boxes[sbox].identity[0])
					has_identity = getIdentity(boxes[sbox].identity, &setid);
				
				if (!boxes[sbox].level) break;
				l = boxes[sbox].level;
				
				while(sbox > -1 && boxes[sbox].level >= l)
					sbox--;
				
				}while (!has_identity && sbox > -1);
			}
		
	if (!has_identity)
		has_identity = getDefaultIdentity(&setid);
																			
	new_buffer.from[0] = 0;
	if (has_identity)
		{
		if (setid.username[0])
			{
			new_buffer.from[0] = '"';
			strcpy(new_buffer.from + 1, setid.username);
			strcat(new_buffer.from, "\" <");
			}
			
		if (setid.use_providerdata)
			{
			strcat(new_buffer.from, cfg_options.email);
			strcat(new_buffer.from, cfg_options.email2);
			}
		else
			{
			strcat(new_buffer.from, setid.email);
			strcat(new_buffer.from, setid.email2);
			}
	
		if (setid.username[0])
			strcat(new_buffer.from, ">");

		strcpy(new_buffer.identity, setid.name);
		if (setid.organization[0])
			strcpy(new_buffer.organization, setid.organization);

		new_buffer.mime_allowed = setid.mime_allowed;
		new_buffer.word_wrap = setid.word_wrap;
		new_buffer.editwidth = setid.editwidth;

			/* Get priority from identity */
		if (setid.priority == -1)
			{
			if (setid.last_priority != -1)
				new_buffer.priority = setid.last_priority;
			}
		else
			new_buffer.priority = setid.priority;
			
			/* Get confirm receipt from identity */
		if (setid.confirm_rcpt == -1)
			{
			if (setid.last_rcpt != -1)
				new_buffer.confirm_rcpt = setid.last_rcpt;
			}
		else
			new_buffer.confirm_rcpt = setid.confirm_rcpt;
			
		if (setid.confirm_read == -1)
			{
			if (setid.last_read != -1)
				new_buffer.confirm_read = setid.last_read;
			}
		else
			new_buffer.confirm_read = setid.confirm_read;
	
		strcpy(new_buffer.signature, setid.signature);
		}
	else
		{
		new_buffer.mime_allowed = FALSE;
		new_buffer.editwidth = 78;
		}

	if (!has_identity || !setid.reply_to1[0])	
		strcpy(new_buffer.replyto, new_buffer.from);
	else
		{
		strcpy(new_buffer.replyto, setid.reply_to1);
		strcat(new_buffer.replyto, setid.reply_to2);
		}
			
	strcpy(new_buffer.to, address);

	if (header.part_count > 1)
		{
		int cnt;
		
		edit_parts = (PART_INFO *)malloc(sizeof(PART_INFO) * header.part_count);
		Fseek(header.partinfo_pos, f, 0);
		Fread(f, sizeof(PART_INFO) * header.part_count, edit_parts);

		new_buffer.attach_count = header.part_count - 1;
		new_buffer.attachments = (char **)malloc(sizeof(char *) * new_buffer.attach_count);
		
		epart_count = header.part_count;
		cnt = 0;		
		for (i = 0; i < header.part_count; i++)
			if (i != header.main_idx)
				{
				if (!edit_parts[i].name[0])
					{
					strcpy(edit_parts[i].name, attachinf_text);
					strcat(edit_parts[i].name, " ");
					itoa(cnt, edit_parts[i].name + strlen(edit_parts[i].name), 10);
					}
					
				new_buffer.attachments[cnt++] = edit_parts[i].name;
				}
		}
		 	
	Fclose(f);
	}
else
	create_empty_header();

i = 0;
while(i < fwd_lines)
	{
	x = fwd_ptr[i] + fwd_mem;
	
	while(*x == 1 || * x == 3 || *x == ' ' || *x == '\t')		x++;

	if (*x == '\r') break;
	
	if (!strnicmp(x, "Subject:", 8))
		{
		x += 8;
		while(*x == ' ' || *x == '\t')	x++;

		if (new_buffer.subj_max < 60)
			{
			new_buffer.subj_max = 60;
			new_buffer.subject = (char *)realloc(new_buffer.subject, 60);
			}
		
		y = new_buffer.subject;
		strcpy(y, "Fwd: ");
		y += 5;
		while (*x != '\r' || !strncmp(x + 1, "\1              ", 14))
			{
			if (*x != '\r')
				{
				if (y - new_buffer.subject + 2 > new_buffer.subj_max)
					{
					long offset;
					
					offset = y - new_buffer.subject;
					new_buffer.subj_max += 60;
					new_buffer.subject = (char *)realloc(new_buffer.subject, new_buffer.subj_max);
					y = new_buffer.subject + offset;
					}
				*y++ = *x;
				}				
			else
				x += 14;
			x++;
			}
		*y = 0;				
		}
				
	i++;
	}

i = fwd_lines - 1;
while(i >= 0)
	{
	x = fwd_ptr[i] + fwd_mem;
	
	if (*x == 1 || *x == 3)
		{
		memmove(x, x + 1,	fwd_blen - fwd_ptr[i] - 1);
		fwd_blen--;
		}

	i--;
	}
		
free(fwd_ptr);

fwd_textlen = (int)strlen(follows_text);
fwd_mem = (char *)realloc(fwd_mem, fwd_blen + fwd_textlen + 1);
memmove(fwd_mem + fwd_textlen + 1, fwd_mem, fwd_blen);
memcpy(fwd_mem, follows_text, fwd_textlen);
fwd_mem[fwd_textlen] = '\r';

new_buffer.changed = TRUE;
fwd_mem = (char *)realloc(fwd_mem, (fwd_blen + fwd_textlen + 1 + 4095L) & ~4095L);
new_buffer.body = fwd_mem;
new_buffer.body_len = fwd_blen + fwd_textlen + 1;
new_buffer.body_lines = fwd_lines;
new_buffer.max_length = fwd_width;

forward_mode = TRUE;
edit_mode = TRUE;
strcpy(edit_file, entry->file);
strcpy(edit_box, boxes[box].file);
open_mailwindow();
}

/* ----------------------------------------
   | Simple forwarding to another address |
   ---------------------------------------- */
void simple_forward(int box, MAIL_INFO *entry, char *address, char *setidentity)
{
static char tmp[9];
int f, in;
FILE_MAIL_MEM header, newheader;
IDENTITY setid;
int has_identity;

DEBUG();
ClearError();

f = create_outgoing(tmp);
if (f > 0)
	{
	in = open_file(entry->file, boxes[box].file);
	if (in)
		{
		Fread(in, sizeof(header), &header);

		if (header.magic != 0x4104)
			{
			memmove(&header.confirm_read, &header.magic, sizeof(header) - (21+21+6));
			Fseek(-(21+21+6L), in, 1);
			header.replyto_len = 0;
			header.identity[0] = 0;
			header.signature[0] = 0;
			}
		else
			if (header.version == SVERSION_ID17)
				{
				Fseek(-(21+21L), in, 1);
				header.identity[0] = 0;
				header.signature[0] = 0;
				}

		if (setidentity[0])
			has_identity = getIdentity(setidentity, &setid);
		else
			has_identity = FALSE;

		if (!has_identity)
			if (entry->identity[0])
				has_identity = getIdentity(entry->identity, &setid);
			else
				{
				int sbox = box;
				int l;
				
				has_identity = FALSE;
				do
					{
					if (boxes[sbox].identity[0])
						has_identity = getIdentity(boxes[sbox].identity, &setid);
					
					if (!boxes[sbox].level) break;
					l = boxes[sbox].level;
					
					while(sbox > -1 && boxes[sbox].level >= l)
						sbox--;
					
					}while (!has_identity && sbox > -1);
				}

		if (!has_identity)
			has_identity = getDefaultIdentity(&setid);
			
		memset(&newheader, 0, sizeof(newheader));
		newheader.magic 		= 0x4104;
		newheader.version 	= SVERSION_ID20;
		newheader.part_count = -1;

	  newheader.header_inf		= sizeof(newheader);
		newheader.confirm_rcpt	=	has_identity ? setid.confirm_rcpt : FALSE;
		newheader.confirm_read	=	has_identity ? setid.confirm_read : FALSE;
		newheader.priority		  = has_identity ? setid.priority : 0;
		
		if (has_identity)
			strcpy(newheader.identity, setid.name); /* Selection ? */

		Fwrite(f, sizeof(newheader), &newheader);

	  if (entry->date[0])
	  	{
		  newheader.date_len					= (int)strlen(entry->date);
		  Fwrite(f, newheader.date_len, entry->date);
		  }
	  if (entry->from[0])
	  	{
		  newheader.from_len					= (int)strlen(entry->from);
		  Fwrite(f, newheader.from_len, entry->from);
		  }
	  newheader.to_len						= (int)strlen(address);
	  Fwrite(f, newheader.to_len, address);
	  if (entry->subject[0])
	  	{
		  newheader.subject_len			= (int)strlen(entry->subject);
		  Fwrite(f, newheader.subject_len, entry->subject);
		  }

		newheader.body_pos = newheader.header_inf + 
		  newheader.date_len	+ newheader.from_len	+ 
		  newheader.to_len	+ newheader.subject_len;

		Fseek(header.body_pos, in, 0);
		CopyPart(f, in, header.body_len);
		
		newheader.body_len = header.body_len;
		Fseek(0L, f, 0);
		Fwrite(f, sizeof(newheader), &newheader);
		
		Fclose(in);
		
		new_buffer.from = entry->from;
		new_buffer.to   = address;
		new_buffer.subject = entry->subject;
		new_buffer.body_len = header.body_len;
		new_buffer.date = entry->date;
		}
	Fclose(f);
	store_mail(tmp, box_number("QUEUE"), &new_buffer, FALSE);
	}
return;
}

/* ----------------------------------------
   | Simple forwarding to another address |
   ---------------------------------------- */
void forward_attachment(int box, MAIL_INFO *entry, char *address, char *setidentity)
{
char *fwd_file;
IDENTITY setid;
int has_identity;

DEBUG();
fwd_file = save_message(box, entry);

strcpy(mail_title, "Marathon Mail (noname.eml)");
memset(&new_buffer, 0, sizeof(new_buffer));

create_empty_header();

if (setidentity[0])
	has_identity = getIdentity(setidentity, &setid);
else
	{
	int sbox = box;
	int l;
	
	has_identity = FALSE;
	do
		{
		if (boxes[sbox].identity[0])
			has_identity = getIdentity(boxes[sbox].identity, &setid);
		
		if (!boxes[sbox].level) break;
		l = boxes[sbox].level;
		
		while(sbox > -1 && boxes[sbox].level >= l)
			sbox--;
		
		}while (!has_identity && sbox > -1);
	}

if (!has_identity && entry->identity[0])
	has_identity = getIdentity(entry->identity, &setid);

if (!has_identity)
	has_identity = getDefaultIdentity(&setid);

if (has_identity)
	{
	if (setid.username[0])
		{
		new_buffer.from[0] = '"';
		strcpy(new_buffer.from + 1, setid.username);
		strcat(new_buffer.from, "\" <");
		}
		
	if (setid.use_providerdata)
		{
		strcat(new_buffer.from, cfg_options.email);
		strcat(new_buffer.from, cfg_options.email2);
		}
	else
		{
		strcat(new_buffer.from, setid.email);
		strcat(new_buffer.from, setid.email2);
		}

	if (setid.username[0])
		strcat(new_buffer.from, ">");

	strcpy(new_buffer.identity, setid.name);
	if (setid.organization[0])
		strcpy(new_buffer.organization, setid.organization);

	new_buffer.mime_allowed = setid.mime_allowed;
	new_buffer.word_wrap = setid.word_wrap;
	new_buffer.editwidth = setid.editwidth;

		/* Get priority from identity */
	if (setid.priority == -1)
		{
		if (setid.last_priority != -1)
			new_buffer.priority = setid.last_priority;
		}
	else
		new_buffer.priority = setid.priority;
		
		/* Get confirm receipt from identity */
	if (setid.confirm_rcpt == -1)
		{
		if (setid.last_rcpt != -1)
			new_buffer.confirm_rcpt = setid.last_rcpt;
		}
	else
		new_buffer.confirm_rcpt = setid.confirm_rcpt;
		
	if (setid.confirm_read == -1)
		{
		if (setid.last_read != -1)
			new_buffer.confirm_read = setid.last_read;
		}
	else
		new_buffer.confirm_read = setid.confirm_read;

	strcpy(new_buffer.signature, setid.signature);
	}
else
	{
	new_buffer.mime_allowed = FALSE;
	new_buffer.editwidth = 78;
	}
	
if (!has_identity || !setid.reply_to1[0])	
	strcpy(new_buffer.replyto, new_buffer.from);
else
	{
	strcpy(new_buffer.replyto, setid.reply_to1);
	strcat(new_buffer.replyto, setid.reply_to2);
	}

strcpy(new_buffer.to, address);

new_buffer.changed = TRUE;
new_buffer.attachments = (char **)malloc(sizeof(char *));
new_buffer.attach_count = 1;
new_buffer.att_encoding = (int *)malloc(sizeof(int));
*new_buffer.att_encoding = 9;

new_buffer.attachments[0] = (char *)malloc(strlen(fwd_file) + 1);
strcpy(new_buffer.attachments[0], fwd_file);

forward_mode = FALSE;
edit_mode = FALSE;
open_mailwindow();
}

/* --------------------------
   | Forward dialog handler |
   -------------------------- */
static void cdecl forward_handler (OBJECT *tree, int obj)
{
(void)tree;

DEBUG();
switch (obj & 0x7FFF)
	{
	case FTYPE_IDSEL:
	case FTYPE_IDSELDROP:
		{
		int x, y;
		char identity[21];
		
		objc_offset(fwd_tree, FTYPE_IDSEL, &x, &y);
		if (DialogIdentitySelect(identity, x, y, 
														 fwd_tree[FTYPE_IDSELDROP].ob_x + fwd_tree[FTYPE_IDSELDROP].ob_width - 
														 fwd_tree[FTYPE_IDSEL].ob_x,
														 "Default"))
			{
			if (!identity[0])
				strcpy(fwd_tree[FTYPE_IDSEL].ob_spec.tedinfo->te_ptext, "Default");
			else
				strcpy(fwd_tree[FTYPE_IDSEL].ob_spec.tedinfo->te_ptext, identity);
			strcpy(fwd_identity, identity);
			redraw_object(fwd_tree, FTYPE_IDSEL);
			}
		break;
		}
		
	case FW_ABOOK:
		address_book();
		break;
		
	case FWD_OKAY:
		if (IsSelected(fwd_tree, FWD_TO))
			{
			if (!fwd_ptext[0])
				{
				form_alert(1, noaddress_text);
				break;
				}

			simple_forward(fwd_box, fwd_entry, fwd_ptext, fwd_identity);
			}
		else
			if (IsSelected(fwd_tree, EDIT_FWD))
				edit_forward(fwd_box, fwd_entry, fwd_ptext, fwd_identity);
			else
				if (IsSelected(fwd_tree, ATT_FWD))
					forward_attachment(fwd_box, fwd_entry, fwd_ptext, fwd_identity);
			/* Fall through */
					
	case FWD_CNC:
		{
		close_dialog(fwd_tree, TRUE);
		break;
		}
	}
}

/* ------------------
   | Forward a mail |
   ------------------ */
void do_forward(int box, MAIL_INFO *entry)
{
IDENTITY setid;
int has_identity;

DEBUG();
if (entry->identity[0])
	has_identity = getIdentity(entry->identity, &setid);
else
	{
	int sbox = box;
	int l;
	
	has_identity = FALSE;
	do
		{
		if (boxes[sbox].identity[0])
			has_identity = getIdentity(boxes[sbox].identity, &setid);
		
		if (!boxes[sbox].level) break;
		l = boxes[sbox].level;
		
		while(sbox > -1 && boxes[sbox].level >= l)
			sbox--;
		
		}while (!has_identity && sbox > -1);
	}
	
if (!has_identity)
	has_identity = getDefaultIdentity(&setid);

rsrc_gaddr(R_TREE, FWD_TYPE, &fwd_tree);

DeselectObj(fwd_tree, FWD_TO);
SelectObj(fwd_tree, EDIT_FWD);
DeselectObj(fwd_tree, ATT_FWD);

if (has_identity)
	{
	strcpy(fwd_tree[FTYPE_IDSEL].ob_spec.tedinfo->te_ptext, setid.name);
	strcpy(fwd_identity, setid.name);
	}
else
	{
	fwd_tree[FTYPE_IDSEL].ob_spec.tedinfo->te_ptext[0] = 0;
	fwd_identity[0] = 0;
	}

if (!has_identity || !setid.mime_allowed)
	fwd_tree[ATT_FWD].ob_state |= DISABLED;
else
	fwd_tree[ATT_FWD].ob_state &= ~DISABLED;

fwd_box = box;
fwd_entry = entry;

dlginit_scrlted(fwd_tree + FW_ADDR, 127);
fwd_ptext = fwd_tree[FW_ADDR].ob_spec.tedinfo->te_ptext;
fwd_ptext[0] = 0;

open_dialog(fwd_tree, NULL, iconified_tree);
dlgset_help_info(fwd_tree, helpfile, fwd_ids, "FWD_");
set_active(fwd_tree, forward_handler, TRUE, 0);
}

/* -------------------------------------
   | Paste address into forward buffer |
   ------------------------------------- */
void fwd_paste(char *buffer)
{
int top;

DEBUG();
wind_get_int(ROOT, WF_TOP, &top);

if (top == get_dialog_wind(fwd_tree))
 	set_active(fwd_tree, (void cdecl (*) (OBJECT *tree, int obj))NULL, FALSE, 0);

if (fwd_ptext[0])
	strcat(fwd_ptext, ", ");

strcat(fwd_ptext, buffer);

redraw_object(fwd_tree, FW_ADDR);		
if (top == get_dialog_wind(fwd_tree))
	set_active(fwd_tree, forward_handler, TRUE, 0);
}

