/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

char errorstring[256];
int errorcode;
char errorprefix[10];

static int erri_ids[] = {	ERRORTEXT01,
													ERRORTEXT02,
													ERRORTEXT03,
													SHOW_SMTPERR,
													OKAY_ERROR,
													0};

/* ---------------------------
   | Store error information |
   --------------------------- */
void store_error(char *prefix, int code, char *string)
{
DEBUG();
strcpy(errorprefix, prefix);
strcpy(errorstring, string);
errorcode = code;
}

/* --------------------------
   | Show error information |
   -------------------------- */
void show_error(int smtperror)
{
OBJECT *tree;
int obj, i;
long fh;
char buf[256], *errinfo, id[20], *s, *scr;
static int errtext[] = {ERRORTEXT01, ERRORTEXT02, ERRORTEXT03, ERRORTEXT04, ERRORTEXT05,
												ERRORTEXT06, ERRORTEXT07, ERRORTEXT08, ERRORTEXT09, ERRORTEXT10};

DEBUG();
rsrc_gaddr(R_TREE, ERROR_INFO, &tree);

if (smtperror)
	EnableObj(tree, SHOW_SMTPERR);
else
	DisableObj(tree, SHOW_SMTPERR);

strcpy(buf, mailpath);
strcat(buf, "\\help\\");
strcat(buf, languagecode);
strcat(buf, ".err");

fh = Fopen(buf, FO_READ);
if (fh > 0)
	{
	long length;
	
	length = Fseek(0, (int)fh, SEEK_END);
	Fseek(0, (int)fh, SEEK_SET);
	errinfo = malloc(length + 1);
	Fread((int)fh, length, errinfo);
	errinfo[length] = 0;
	Fclose((int)fh);
	}
else
	{
	errinfo = malloc(1);
	*errinfo = 0;
	}

strcpy(id, errorprefix);
itoa(errorcode, id + strlen(id), 10);
strcat(id, "\r\n");

for (i = 0; i < 10; i++)
	tree[errtext[i]].ob_spec.free_string[0] = 0;
	
i = 0;
s = strstr(errinfo, id);
if (s)
	{
	for(;;)
		{
		scr = strstr(s, "\r\n");
		if (scr)
			*scr = 0;
			
		if (!strcmp(s, "."))
			break;
			
		strncpy(tree[errtext[i++]].ob_spec.free_string, s, 30);
		s = scr + 2;
		}
		
	if (!smtperror)
		s = 0;
	}

if (!s)
	{
	int l;
	
	l = (int)strlen(errorstring);
	s = errorstring;
	while(l > 0)
		{
		strncpy(tree[errtext[i++]].ob_spec.free_string, s, 30);
		s += 30;
		l -= 30;		
		}
	}
	
free(errinfo);
	
open_dialog(tree, NULL, iconified_tree);
dlgset_help_info(tree, helpfile, erri_ids, "ERR_");
obj = do_dialog(tree, 0, app_callback);
close_dialog(tree, TRUE);

if (obj == SHOW_SMTPERR)
	list_window(box_number("SENTERR"));
}
