/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"
#include "vaproto.h"

/* Local prototype */
int cdecl extern_obj(PARMBLK *parm);
void exec_program(char *program, char *file, int va);
short av_server( void );
void exec_avserver(char *file);

static int external_count = 0;
static EXTERNAL_DEF *externals;
static int use_av = TRUE;

static USERBLK extern_ublk 	= {extern_obj, 0L};
static int cfg_external, external_chg, external_page, selected_external;

static OBJECT *external_conf = NULL;
static char program_buf[256], *avserver_mem = NULL;

static int elist_ids[] = {EXTERNALLIST,
													EXT_USEAV,
													NEW_EXTERNAL,
													CFG_EXTERNAL,
													DEL_EXTERNAL,
													CLOSE_EXTERNALS,
													0};
static int econf_ids[] = {EXTCFG_PATTERN,
													EXTCFG_DESCR,
													EXTCFG_MIME,
													EXTCFG_PROGRAM,
													EXTCFG_SELECT,
													EXTCFG_VA,
													EXTCFG_CANCEL,
													EXTCFG_OKAY,
													0};

/* ----------------------
   | Select program buf |
   ---------------------- */
int select_program_buf(void)
{
char dir[256], file[64];
int button;

DEBUG();
if (!program_buf[0])
	{
	dir[0] = Dgetdrv() + 'A';
	dir[1] = ':';
	Dgetpath(dir + 2, 0);

	file[0] = 0;
	}
else
	{
	char *s;
	
	strcpy(dir, program_buf);
	s = strrchr(dir, '\\');
	if (s) 
		{
		strcpy(file, s + 1);
		*s = 0;
		}
	else
		file[0] = 0;
	}

strcat(dir, "\\*.prg");
	
if (aes_version >= 0x104)
	fsel_exinput(dir, file, &button, "Select program to execute...");
else
	fsel_input(dir, file, &button);
	
if (button == 1 && file[0])
	{
	if (strchr(dir, '\\'))
		*(strrchr(dir, '\\') + 1) = 0;
	else
		{
		strcpy(dir, "x:\\");
		dir[0] = Dgetdrv() + 'A';
		}

	strcpy(program_buf, dir);
	strcat(program_buf, file);
	return TRUE;
	}
return FALSE;
}

/* ------------------------------------
   | Set selected program information |
   ------------------------------------ */
void set_programinfo(void)
{	
char *s;

DEBUG();
s = strrchr(program_buf, '\\');
if (!s) s = program_buf; else s++;

strcpy(external_conf[EXTCFG_PROGRAM].ob_spec.tedinfo->te_ptext, s);
}

/* -------------------------------
	 | Fill external configuration |
	 ------------------------------- */
void fill_external_conf(int num)
{
DEBUG();
if (num == -1)
	{
	external_conf[EXTCFG_PATTERN].ob_spec.tedinfo->te_ptext[0] = 0;
	external_conf[EXTCFG_DESCR].ob_spec.tedinfo->te_ptext[0] = 0;
	external_conf[EXTCFG_MIME].ob_spec.tedinfo->te_ptext[0] = 0;
	external_conf[EXTCFG_PROGRAM].ob_spec.tedinfo->te_ptext[0] = 0;
	program_buf[0] = 0;
	SelectObj(external_conf, EXTCFG_VA);
	}
else
	{
	strcpy(external_conf[EXTCFG_PATTERN].ob_spec.tedinfo->te_ptext, externals[num].pattern);
	strcpy(external_conf[EXTCFG_DESCR].ob_spec.tedinfo->te_ptext, externals[num].description);
	strcpy(external_conf[EXTCFG_MIME].ob_spec.tedinfo->te_ptext, externals[num].mimetype);

	strcpy(program_buf, externals[num].program);
	set_programinfo();
	SetSelect(external_conf, EXTCFG_VA, externals[num].use_va);
  }
}

/* ------------------------------
	 | Get external configuration |
	 ------------------------------ */
void store_external_conf(int num)
{
EXTERNAL_DEF newcfg;

DEBUG();
strcpy(newcfg.pattern, external_conf[EXTCFG_PATTERN].ob_spec.tedinfo->te_ptext);
strcpy(newcfg.description, external_conf[EXTCFG_DESCR].ob_spec.tedinfo->te_ptext);
strcpy(newcfg.mimetype, external_conf[EXTCFG_MIME].ob_spec.tedinfo->te_ptext);
strcpy(newcfg.program, program_buf);

newcfg.use_va = IsSelected(external_conf, EXTCFG_VA);

if (num != -1)
	memcpy(externals + num, &newcfg, sizeof(EXTERNAL_DEF));
else
	{
	if (!external_count)
		externals = (EXTERNAL_DEF *)malloc(sizeof(EXTERNAL_DEF));
	else
		externals = (EXTERNAL_DEF *)realloc(externals, sizeof(EXTERNAL_DEF) * (external_count + 1));
		
	memcpy(externals + external_count, &newcfg, sizeof(EXTERNAL_DEF));
	external_count++;
	}
}

/* ----------------------------------------
   | Handle external configuration events |
   ---------------------------------------- */
static void cdecl extconfig_handler (OBJECT *tree, int obj)
{
DEBUG();
switch(obj)
	{
	case EXTCFG_SELECT:
		if (select_program_buf())
			{
			set_programinfo();
			redraw_object(tree, EXTCFG_PROGRAM);
			}
		break;
		
	case EXTCFG_OKAY:
		{
		OBJECT *externallist;
		
		external_chg = TRUE;
		store_external_conf(cfg_external);

		rsrc_gaddr(R_TREE, EXTERNALS, &externallist);

		if (is_open(externallist))
			redraw_object(externallist, EXTERNALLIST);
		}
		
	case EXTCFG_CANCEL:
		close_dialog(tree, TRUE);
		break;
	}		
}

/* --------------------------
	 | External configuration |
	 -------------------------- */
void config_external(int num)
{
DEBUG();
	/* Combine multipage dialogs */
if (external_conf == NULL)
	rsrc_gaddr(R_TREE, EXTERNALS_CFG, &external_conf);

cfg_external = num;
open_dialog(external_conf, NULL, iconified_tree);
dlgset_help_info(external_conf, helpfile, econf_ids, "ECNF_");
fill_external_conf(num);
set_active(external_conf, extconfig_handler, TRUE, 0);
}

/* ----------------------------------------
   | Handle External configuration events |
   ---------------------------------------- */
static void cdecl external_handler (OBJECT *tree, int obj)
{
DEBUG();
switch(obj)
	{
	case EXTERNALLIST:
		{
		int ox, oy, pos;
		EVNTDATA ev;
		
		objc_offset(tree, EXTERNALLIST, &ox, &oy);
		graf_mkstate(&ev);
		
		pos = (ev.y - oy) / tx_height + external_page;
		if (pos >= 0 && pos < external_count)
			{
			if (selected_external == -1)
				{
				EnableObj(tree, CFG_EXTERNAL);
				EnableObj(tree, DEL_EXTERNAL);
				redraw_object(tree, CFG_EXTERNAL);
				redraw_object(tree, DEL_EXTERNAL);
				}
			selected_external = pos;
			}
		else
			{
			if (selected_external != -1)
				{
				DisableObj(tree, CFG_EXTERNAL);
				DisableObj(tree, DEL_EXTERNAL);
				redraw_object(tree, CFG_EXTERNAL);
				redraw_object(tree, DEL_EXTERNAL);
				}
			selected_external = -1;
			}
		redraw_object(tree, EXTERNALLIST);
		
		break;
		}
		
	case NEW_EXTERNAL:
		config_external(-1);
		break;
		
	case CFG_EXTERNAL:
		config_external(selected_external);
		break;
		
	case DEL_EXTERNAL:
		{
		if (cfg_external == selected_external && is_open(external_conf))
			close_dialog(external_conf, TRUE);
			
		if (cfg_external > selected_external) cfg_external--;
		if (external_count > 1 && selected_external < external_count - 1)
			memmove(externals + selected_external, externals + selected_external + 1,
							sizeof(EXTERNAL_DEF) * (external_count - selected_external - 1)) ;
		else
			free(externals);	
		external_count--;
		external_chg = TRUE;
		selected_external = -1;
		redraw_object(tree, EXTERNALLIST);
		break;
		}
		
	case CLOSE_EXTERNALS:
		close_dialog(tree, TRUE);
		break;
	}
}

/* ----------------------------
   | User object address book |
   ---------------------------- */
int cdecl extern_obj(PARMBLK *parm)
{
int xy[4], i, y, lines;
int dummy;
char buf[20], *s;

DEBUG();
obj_clip(parm, TRUE);

vst_height(vdi_handle, default_fsize, &dummy, &dummy, &dummy, &dummy);
vswr_mode(vdi_handle, MD_REPLACE);
vsf_interior(vdi_handle, FIS_HOLLOW);
xy[0] = parm->pb_x;
xy[1] = parm->pb_y;
xy[2] = parm->pb_x + parm->pb_w - 2;
xy[3] = parm->pb_y + parm->pb_h - 1;
vr_recfl(vdi_handle, xy);

lines = (parm->pb_tree[EXTERNALLIST].ob_height + tx_height - 1) / tx_height;

y = xy[1];
for (i = external_page; i < external_count && i < external_page + lines; i++, y += tx_height)
	{
	strncpy(buf, externals[i].pattern, 9);
	buf[9] = 0;
	v_gtext(vdi_handle, parm->pb_x + tx_width, y, buf);		/* Pattern */

	strncpy(buf, externals[i].description, 14);
	buf[14] = 0;
	v_gtext(vdi_handle, parm->pb_x + tx_width + 10 * tx_width, y, buf);		/* Description */

	s = strrchr(externals[i].program, '\\');
	if (!s) s = externals[i].program; else s++;

	strncpy(buf, s, 15);
	buf[15] = 0;
	v_gtext(vdi_handle, parm->pb_x + tx_width + 25 * tx_width, y, buf);		/* Programs */
	
	if (selected_external == i)
		{
		vswr_mode(vdi_handle, MD_XOR);
		vsf_interior(vdi_handle, FIS_SOLID);
		vsf_color(vdi_handle, 1);

		xy[0] = parm->pb_x;
		xy[1] = y;
		xy[2] = parm->pb_x + parm->pb_w - 2;
		xy[3] = y + tx_height - 1;
		vr_recfl(vdi_handle, xy);
		vswr_mode(vdi_handle, MD_TRANS);
		}
	}


obj_clip(parm, FALSE);
vswr_mode(vdi_handle, MD_REPLACE);

return 0;
}

/* ------------------
   | Edit externals |
   ------------------ */
void edit_externals(void)
{
OBJECT *externallist;

DEBUG();
rsrc_gaddr(R_TREE, EXTERNALS, &externallist);

if (!is_open(externallist))
	{
	if (externallist[EXTERNALLIST].ob_type != G_USERDEF)
		{
		externallist[EXTERNALLIST].ob_type = G_USERDEF;
		externallist[EXTERNALLIST].ob_spec.userblk = &extern_ublk;
	
		external_page = 0;
		selected_external = -1;
		DisableObj(externallist, CFG_EXTERNAL);
		DisableObj(externallist, DEL_EXTERNAL);
		}

	if (av_server() != -1)
		EnableObj(externallist, EXT_USEAV);
	else
		DisableObj(externallist, EXT_USEAV);
	
	align_slider_objects(externallist, EXT_SLIDER, EXT_UP, EXT_SLDBKG, EXT_SLDBOX, EXT_DOWN);
	set_slider_objects(externallist, EXT_SLDBKG, EXT_SLDBOX, external_count, external_page, 
										 externallist[EXTERNALLIST].ob_height / tx_height, FALSE);	
	}
		
open_dialog(externallist, NULL, iconified_tree);
dlgset_help_info(externallist, helpfile, elist_ids, "EXT_");
set_active(externallist, external_handler, TRUE, 0);
}

/* -----------------------
   | Load externals file |
   ----------------------- */
void load_externals(void)
{
long ext_flg;

DEBUG();
if (external_count)
	free(externals);
	
ext_flg = Fopen("extern.dat", FO_READ);
if (ext_flg > 0)
	{
	Fread((int)ext_flg, sizeof(use_av), &use_av);
	Fread((int)ext_flg, sizeof(external_count), &external_count);
	if (external_count)
		{
		externals = (EXTERNAL_DEF *)malloc(sizeof(EXTERNAL_DEF) * 
																		   external_count);
																		   
		Fread((int)ext_flg, sizeof(EXTERNAL_DEF) * external_count,
				  externals);
		}
		
	Fclose((int)ext_flg);
	external_chg = FALSE;
	}
else
	{
	char adapath[256];
	
	strcpy(adapath, browserpath);
	strcat(adapath, "\\adapro.prg");
	if (!shel_find(adapath))
		{
		strcpy(adapath, browserpath);
		strcat(adapath, "\\adamas.prg");
		if (!shel_find(adapath))
			{
			strcpy(adapath, browserpath);
			strcat(adapath, "\\adapro30.prg");
			if (!shel_find(adapath))
				adapath[0] = 0;
			}
		}
		
	if (adapath[0])
		{
		externals = (EXTERNAL_DEF *)malloc(sizeof(EXTERNAL_DEF) * 2);
		external_count = 2;
		
		strcpy(externals->pattern, "*.htm*");
		strcpy(externals->description, "HTML");
		strcpy(externals->mimetype, "text/html");
		strcpy(externals->program, adapath);
		
		strcpy(externals[1].pattern, "http://*");
		strcpy(externals[1].description, "Web access");
		strcpy(externals[1].mimetype, "text/html");
		strcpy(externals[1].program, adapath);

		externals->use_va = TRUE;
		external_chg = TRUE;
		}
	else
		external_chg = FALSE;
	}
}

/* -----------------------
   | Save externals file |
   ----------------------- */
void save_externals(void)
{
long ext_flg;

DEBUG();
if (!external_chg) return;

ext_flg = Fcreate("extern.dat", 0);
if (ext_flg > 0)
	{
	Fwrite((int)ext_flg, sizeof(use_av), &use_av);
	Fwrite((int)ext_flg, sizeof(external_count), &external_count);
	if (external_count)
		Fwrite((int)ext_flg, sizeof(EXTERNAL_DEF) * external_count,
					 externals);
	Fclose((int)ext_flg);
	}
}

/* -----------------------------------------
   | Has the file an associated external ? |
   ----------------------------------------- */
int has_external(char *file, int only_non_av)
{
int i;

DEBUG();
if (use_av && !only_non_av) return TRUE;

for (i = 0; i < external_count; i++)
	if (matchpattern((unsigned char *)file, (unsigned char *)externals[i].pattern, FALSE))
		return TRUE;
		
return FALSE;
}

/* ----------------------------
   | Execute external program |
   ---------------------------- */
void exec_external(char *file, char *tmp)
{
int i;

DEBUG();
for (i = 0; i < external_count; i++)
	if (matchpattern((unsigned char *)file, (unsigned char *)externals[i].pattern, FALSE))
		{
		exec_program(externals[i].program, tmp, externals[i].use_va);
		return;
		}

if (use_av)
	exec_avserver(tmp);
}

/* -------------------------------
   | Create a name for appl_find |
   ------------------------------- */
int make_name(char *path, char *name)
{
char *s;

DEBUG();
s = strrchr(path, '\\');
if (!s)
	s = path;
else
	s++;

strncpy(name, s, 8);
name[8] = 0;
s = strchr(name, '.');
if (s) *s = 0;
strupr(name);

while(strlen(name) < 8)
	strcat(name, " ");

return TRUE;
}
   
/* -----------------------------------------------------
   | Execute a program with the specified command line |
   ----------------------------------------------------- */
void exec_program(char *program, char *file, int va)  
{
long start;
int extapp_id;
extern int bootdrv;
char cmdline[512], cmd_name[32];
 
DEBUG();
strcpy(cmdline + 1, file);
cmdline[0] = strlen(file);

make_name(program, cmd_name);
extapp_id = appl_find(cmd_name);
if (extapp_id < 0) 
	{	
	switch(CheckSystem())
		{
		case 1:
		case 3:
			if (va)
				{
	      shel_write(0, 1, 1, program, "\0");
  	    break;
    		}
    		  
    case 2:
    	if (va)
    		{
      	shel_write(1, 1, 100, program, "\0");
      	break;
      	}
      	
    default:
    	{
    	char buf[256], programpath[256], *s;
			int drv;
		
			if (!CheckSystem())
				{
				close_windows();
				menu_bar(menu_adr, FALSE);
				}
					
			drv = Dgetdrv();
		
			strcpy(programpath, program);
			s = strrchr(programpath, '\\');
			if (s) 
				{
				if (s < programpath + 3)	/* Don't delete first backslash */
					s++;
				*s = 0;
				}
				
			buf[0] = program[0];
			buf[1] = program[1];
    	Dgetpath(buf + 2, toupper(program[0]) - 'A' + 1);
    	Dsetpath(programpath);
    	Dsetdrv(toupper(program[0]) - 'A');
    	
      Pexec(0, program, cmdline, NULL);
      
      Dsetdrv(drv);
      Dsetpath(buf);

			if (!CheckSystem())
				{
				open_windows();
				menu_bar(menu_adr, TRUE);
				}
					
    	return;
    	}
		}
	
	start = clock();
	while (clock() - start < 2 * CLK_TCK)
		if (appl_find(cmd_name) >= 0)
			break;
	}
	
extapp_id = appl_find(cmd_name);
if (extapp_id >= 0)
	{
	int msg[8];
	char *va_mem;
		
	va_mem = (char *)alloc_shared((unsigned short)strlen(cmdline + 1) + 1);
	strcpy(va_mem, cmdline + 1);

	msg[0] = VA_START;
  msg[1] = gl_apid;
 	msg[2] = 0;
  *(char **)(msg + 3) = va_mem;
  msg[5] = 0;
 	msg[6] = 0;
  msg[7] = 0;
 	appl_write(extapp_id, 16, msg);
	}
}	   

/* -----------------------
   | Search for AVSERVER |
   ----------------------- */
short av_server( void )
{
int	   desk_id = -1;
char   desk_name[32];
char   desk_path[256];
char 	 *env;

DEBUG();
if ( (env = getenv("AVSERVER")) != NULL )
  {
  strcpy( desk_path, env );

  if ( make_name( desk_path, desk_name ) )
    {
    desk_id = appl_find( desk_name );
    }
  }

if ( check_cookie('MagX') && desk_id < 0)
  {
  desk_id = 0;
  }
else
  {
  desk_id = appl_find("THING   ");
  }

if ( desk_id < 0 )
  {
  desk_id = appl_find("JINNEE  ");
  }

if ( desk_id < 0 )
  {
  desk_id = appl_find("GEMINI  ");
  }

return desk_id >= 0 ? desk_id : -1;
}

/* -----------------------------
   | Call program via AVSERVER |
   ----------------------------- */
void exec_avserver(char *file)
{
int av_id = av_server();

DEBUG();
if (av_id >= 0)
	{
	int msg[8];

	avserver_mem = (char *)alloc_shared((unsigned short)strlen(file) + 1);
	strcpy(avserver_mem, file);
			
	msg[0] = AV_STARTPROG;
  msg[1] = gl_apid;
 	msg[2] = 0;
  *(char **)(msg + 3) = NULL;
  *(char **)(msg + 5) = avserver_mem;
  msg[7] = 0;
 	appl_write(av_id, 16, msg);
	}
}

/* ----------------------------------
   | Free allocated AVSERVER memory |
   ---------------------------------- */
void free_avserver(void)
{
DEBUG();
if (avserver_mem)
	{
	free(avserver_mem);
	avserver_mem = NULL;
	}
}
