/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"
#include "..\include\color.h"

/* Local prototype */
int cdecl filter_obj(PARMBLK *parm);
int execute_filter(int box, MAIL_INFO *mail, int filteridx);

int skip_filter;		/* Used by FILTERAC to notify about a relative move in the rules */
int filter_count = 0;
FILTER_DEF *filters;

static USERBLK filter_ublk 	= {filter_obj, 0L};
static int filter_chg, filter_page, selected_filter, section_start;
static int cfg_filter = -1, section_count;
static OBJECT *filter_conf, *filter_strings;
static int compare[3], condi[3], curroper[3], curresel[3];
static char curredit[6][31], condedit[3][31], filterfolder[9];
static char filtertitle[64];

static FILTER_DEF defaultentry = {"\0", "\0",
																	"\0", "\0", "\0", 	/* Condition edits 	*/
																	-1, 0, 0, 					/* Condition 				*/
																	0, 0, 0, 						/* Condition int 		*/
																	-1, 0, 0,						/* Operation 				*/
																	0, 0, 							/* And/Or 					*/
																	"\0", "\0", "\0", 
																	"\0", "\0", "\0",  	/* Edits 						*/
																	-1, -1, -1,					/* Editselects			*/
																	TRUE								/* Active						*/
																	};
																	
static int compare_ids[] = {FILTERCOMPAREBASE, FILTERCOMPARE0, FILTERCOMPARE1, FILTERCOMPARE2,
														FILTERCOMPARE3, FILTERCOMPARE4, FILTERCOMPARE5, FILTERCOMPARE6,
														FILTERCOMPARE7, FILTERCOMPARE8, FILTERCOMPARE9, FILTERCOMPARE10,
														FILTERCOMPARE11, FILTERCOMPARE12};
static int edittypes[] = {0, 0, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 0, 0};

static int operlbl_ids[] = {FILTEROPERLABELBASE, FILTEROPERLABEL0, FILTEROPLABEL1, FILTEROPLABEL2,
														FILTEROPLABEL3, FILTEROPLABEL4, FILTEROPLABEL5, FILTEROPLABEL6,
														FILTEROPLABEL7, FILTEROPLABEL8, FILTEROPLABEL9, FILTEROPLABEL10,
														FILTEROPLABEL11, FILTEROPLABEL12};
static int opertypes[] = {0, 0, 2, 2, 0, 1, 3, 0, 0, 0, 0, 0, 1, 2};

static int priostrings[] = {FILTEROPPRIORITYN, FILTEROPPRIORITYH, FILTEROPPRIORITYL};

static int efilt_ids[] = {APPLY_ON_OPEN,
													NEW_FILTER,
													EDIT_FILTER,
													DEL_FILTER,
													ACT_FILTER,
													DEACT_FILTER,
													EXIT_FILTER,
													0};
static int fconf_ids[] = {FILTERLABEL,
													FILTERCONDITION1,
													FILTERCONDEDIT1,
													FILTERCOMPSEL1,
													FILTERAND1,
													FILTEROR1,
													FILTERCONDITION2,
													FILTERCONDEDIT2,
													FILTERCOMPSEL2,
													FILTERAND2,
													FILTEROR2,
													FILTERCONDITION3,
													FILTERCONDEDIT3,
													FILTERCOMPSEL3,
													FILTEROPERATION1,
													FILTERCOLSEL1,
													FILTEREDIT1_1,
													FILTEREDIT1_2,
													FILTERSEL1,
													FILTEROPERATION2,
													FILTERCOLSEL2,
													FILTEREDIT2_1,
													FILTEREDIT2_2,
													FILTERSEL2,
													FILTEROPERATION3,
													FILTERCOLSEL3,
													FILTEREDIT3_1,
													FILTEREDIT3_2,
													FILTERSEL3,
													FILTER_CANCEL,
													FILTER_OKAY,
													0};

/* ----------------------
   | Set operation data |
   ---------------------- */
static void setOperation(int oper, int esel, char *seledit, int opersel, int sel, 
												 int seldrop, int editfield1, int editfield2, int colsel)
{
DEBUG();
filter_conf[opersel].ob_spec.tedinfo->te_ptext = 
	filter_strings[operlbl_ids[oper + 1]].ob_spec.free_string;

switch(opertypes[oper + 1])
	{
	case 0:
		filter_conf[sel].ob_flags |= HIDETREE;
		filter_conf[seldrop].ob_flags |= HIDETREE;
		filter_conf[editfield1].ob_flags |= HIDETREE;
		filter_conf[editfield2].ob_flags |= HIDETREE;
		filter_conf[editfield1].ob_flags &= ~EDITABLE;
		filter_conf[editfield2].ob_flags &= ~EDITABLE;
		filter_conf[colsel].ob_flags |= HIDETREE;
		break;
		
	case 1:
		filter_conf[sel].ob_flags |= HIDETREE;
		filter_conf[seldrop].ob_flags |= HIDETREE;
		filter_conf[editfield1].ob_flags &= ~HIDETREE;
		filter_conf[editfield2].ob_flags &= ~HIDETREE;
		filter_conf[editfield1].ob_flags |= EDITABLE;
		filter_conf[editfield2].ob_flags |= EDITABLE;
		filter_conf[colsel].ob_flags |= HIDETREE;
		break;
		
	case 2:
		switch (oper)
			{
			case 1:	/* Copy to */
			case 2:	/* Move to */
				if (esel != -1)
					strncpy(filter_conf[sel].ob_spec.tedinfo->te_ptext, boxes[esel].desc, 30);
				else
					filter_conf[sel].ob_spec.tedinfo->te_ptext[0]  = 0;
				break;

			case 12: /* Identity */
				strcpy(filter_conf[sel].ob_spec.tedinfo->te_ptext, seledit);
				break;
			
			case 5:		/* Mark with colour */
				filter_conf[sel].ob_spec.tedinfo->te_ptext[0]  = 0;
				break;
			}
			
		filter_conf[sel].ob_flags &= ~HIDETREE;
		filter_conf[seldrop].ob_flags &= ~HIDETREE;
		filter_conf[editfield1].ob_flags |= HIDETREE;
		filter_conf[editfield2].ob_flags |= HIDETREE;
		filter_conf[editfield1].ob_flags &= ~EDITABLE;
		filter_conf[editfield2].ob_flags &= ~EDITABLE;
		filter_conf[colsel].ob_flags |= HIDETREE;
		break;
		
	case 3:
		filter_conf[sel].ob_flags |= HIDETREE;
		filter_conf[seldrop].ob_flags &= ~HIDETREE;
		filter_conf[editfield1].ob_flags |= HIDETREE;
		filter_conf[editfield2].ob_flags |= HIDETREE;
		filter_conf[editfield1].ob_flags &= ~EDITABLE;
		filter_conf[editfield2].ob_flags &= ~EDITABLE;
		filter_conf[colsel].ob_flags &= ~HIDETREE;
		break;
	}
}												  

/* ---------------------
   | Set compare infos |
   --------------------- */
static void setCompare(int type, int condi, char *condedit, int condfield, int selfield, int dropfield, int editfield)
{
DEBUG();
filter_conf[condfield].ob_spec.tedinfo->te_ptext = 
	filter_strings[compare_ids[type + 1]].ob_spec.free_string;

switch(edittypes[type + 1])
	{
	case 0:
		filter_conf[selfield].ob_flags |= HIDETREE;
		filter_conf[dropfield].ob_flags |= HIDETREE;
		filter_conf[editfield].ob_flags |= HIDETREE;
		filter_conf[editfield].ob_flags &= ~EDITABLE;
		break;
		
	case 1:
		filter_conf[selfield].ob_flags |= HIDETREE;
		filter_conf[dropfield].ob_flags |= HIDETREE;
		filter_conf[editfield].ob_flags &= ~HIDETREE;
		filter_conf[editfield].ob_flags |= EDITABLE;
		break;
		
	case 2:
		switch (type)
			{
			case 7:		/* Priority */
				strcpy(filter_conf[selfield].ob_spec.tedinfo->te_ptext, 
							 filter_strings[priostrings[condi]].ob_spec.free_string);
				break;
			
			case 8:		/* Identity */
				strncpy(filter_conf[selfield].ob_spec.tedinfo->te_ptext, condedit, 30);
				break;
			}
			
		filter_conf[selfield].ob_flags &= ~HIDETREE;
		filter_conf[dropfield].ob_flags &= ~HIDETREE;
		filter_conf[editfield].ob_flags |= HIDETREE;
		filter_conf[editfield].ob_flags &= ~EDITABLE;
		break;
	}
}

/* -------------------
   | Select a folder |
   ------------------- */
static void IdentitySelect(int field, int x, int y)
{
char **strings;
char *buffer, *ptr;
int i, ret, num;

DEBUG();
num = getNumberOfIdentities();
strings = (char **)malloc(sizeof(char *) * (num + 1));
if (!strings)
	return;
	
ptr = buffer = (char *)malloc(num * 21);
if (!buffer)
	{
	free(strings);
	return;
	}
	
for (i = 0; i < num; i++)
	{
	strings[i] = ptr;
	strcpy(ptr, getIdentityName(i));
	ptr += 21;
	}
strings[i] = NULL;

ret = free_droplist(x, y, filter_conf[FILTERCOMPSELDROP1].ob_x + 
												  filter_conf[FILTERCOMPSELDROP1].ob_width - 
												  filter_conf[FILTERCOMPSEL1].ob_x, strings);
if (ret != -1)
	{
	if (field >= 3)
		strcpy(curredit[(field - 3) * 2], strings[ret]);
	else
		strcpy(condedit[field], strings[ret]);

	switch(field)
		{
		case 0:
			strcpy(filter_conf[FILTERCOMPSEL1].ob_spec.tedinfo->te_ptext, strings[ret]);
			redraw_object(filter_conf, FILTERCOMPSEL1);
			break;

		case 1:		
			strcpy(filter_conf[FILTERCOMPSEL2].ob_spec.tedinfo->te_ptext, strings[ret]);
			redraw_object(filter_conf, FILTERCOMPSEL2);
			break;
			
		case 2:
			strcpy(filter_conf[FILTERCOMPSEL3].ob_spec.tedinfo->te_ptext, strings[ret]);
			redraw_object(filter_conf, FILTERCOMPSEL3);
			break;
			
		case 3:
			strcpy(filter_conf[FILTERSEL1].ob_spec.tedinfo->te_ptext, curredit[0]);
			redraw_object(filter_conf, FILTERSEL1);
			break;
			
		case 4:
			strcpy(filter_conf[FILTERSEL2].ob_spec.tedinfo->te_ptext, curredit[1]);
			redraw_object(filter_conf, FILTERSEL2);
			break;
			
		case 5:
			strcpy(filter_conf[FILTERSEL3].ob_spec.tedinfo->te_ptext, curredit[2]);
			redraw_object(filter_conf, FILTERSEL3);
			break;
		}
	}

free(buffer);	
free(strings);
}

/* -------------------
   | Select a folder |
   ------------------- */
static void FolderSelect(int field, int x, int y)
{
char **strings;
char *buffer, *ptr;
int i, c, ret;

DEBUG();
strings = (char **)malloc(sizeof(char *) * (box_count + 1));
if (!strings)
	return;
	
ptr = buffer = (char *)malloc(box_count * (21 + 30));
if (!buffer)
	{
	free(strings);
	return;
	}
	
for (i = c = 0; i < box_count; i++)
	if (boxes[i].type != -1)
		{
		int l;
		
		strings[c++] = ptr;
		
		l = boxes[i].level;
		if (l)
			memset(ptr, ' ', l);
		strcpy(ptr + l, boxes[i].desc);
		ptr += l + 21;
		}
strings[c] = NULL;

ret = free_droplist(x, y, filter_conf[FILTERSELDROP1].ob_x + 
												  filter_conf[FILTERSELDROP1].ob_width - 
												  filter_conf[FILTERSEL1].ob_x, strings);
if (ret != -1)
	{
	for (i = c = 0; i < box_count; i++)
		if (boxes[i].type != -1)
			{
			if (c == ret)
				break;
			c++;
			}
	curresel[field] = i;

	switch(field)
		{
		case 0:
			strncpy(filter_conf[FILTERSEL1].ob_spec.tedinfo->te_ptext, boxes[i].desc, 30);
			redraw_object(filter_conf, FILTERSEL1);
			break;

		case 1:		
			strncpy(filter_conf[FILTERSEL2].ob_spec.tedinfo->te_ptext, boxes[i].desc, 30);
			redraw_object(filter_conf, FILTERSEL2);
			break;
			
		case 2:
			strncpy(filter_conf[FILTERSEL3].ob_spec.tedinfo->te_ptext, boxes[i].desc, 30);
			redraw_object(filter_conf, FILTERSEL3);
			break;
		}
	}
	
free(buffer);	
free(strings);
}

/* --------------------------
   | Select condition field |
   -------------------------- */
static void PrioritySelect(int field, int x, int y)
{
char *strings[4];
int i, ret;

DEBUG();
for (i = 0; i < 3; i++)
	strings[i] = filter_strings[priostrings[i]].ob_spec.free_string;
strings[3] = NULL;

ret = free_droplist(x, y, filter_conf[FILTERCOMPSELDROP1].ob_x + 
												  filter_conf[FILTERCOMPSELDROP1].ob_width - 
												  filter_conf[FILTERCOMPSEL1].ob_x, strings);
if (ret != -1)
	{
	condi[field] = ret;

	switch(field)
		{
		case 0:
			strcpy(filter_conf[FILTERCOMPSEL1].ob_spec.tedinfo->te_ptext, strings[ret]);
			redraw_object(filter_conf, FILTERCOMPSEL1);
			break;

		case 1:		
			strcpy(filter_conf[FILTERCOMPSEL2].ob_spec.tedinfo->te_ptext, strings[ret]);
			redraw_object(filter_conf, FILTERCOMPSEL2);
			break;
			
		case 2:
			strcpy(filter_conf[FILTERCOMPSEL3].ob_spec.tedinfo->te_ptext, strings[ret]);
			redraw_object(filter_conf, FILTERCOMPSEL3);
			break;
		}
	}
}

/* --------------------------
   | Select condition field |
   -------------------------- */
static void OperationSelect(int field, int x, int y)
{
char *strings[14];
int i, j = 0, ret, first;

DEBUG();
if (!field)
	{
	first = 1;
	i = 2;
	}
else
	{
	first = 0;
	i = 1;
	}

while(i < sizeof(operlbl_ids) / sizeof(operlbl_ids[0]))
	strings[j++] = filter_strings[operlbl_ids[i++]].ob_spec.free_string;
strings[j] = NULL;

ret = free_droplist(x, y, filter_conf[FILTEROPERDROP1].ob_x + 
												  filter_conf[FILTEROPERDROP1].ob_width - 
												  filter_conf[FILTEROPERATION1].ob_x, strings);
if (ret != -1)
	{
	ret += first;
	if (ret != curroper[field])
		{
		switch(ret)
			{
			case 1: 
			case 2: 
				if (curroper[field] != 1 && curroper[field] != 2)
					curresel[field] = -1;
				break;
				
/*			default:
				curredit[field * 2][0] = 0;
				curredit[field * 2 + 1][0] = 0;
				break;*/
			}
		}
	curroper[field] = ret;

	switch(field)
		{
		case 0:
			setOperation(curroper[0], curresel[0], curredit[0], FILTEROPERATION1, FILTERSEL1, FILTERSELDROP1, 
									 FILTEREDIT1_1, FILTEREDIT1_2, FILTERCOLSEL1);
			redraw_object(filter_conf, FILTEROPERATION1);
			redraw_object(filter_conf, ACTREFRESH1);
			break;

		case 1:		
			setOperation(curroper[1], curresel[1], curredit[2], FILTEROPERATION2, FILTERSEL2, FILTERSELDROP2, 
									 FILTEREDIT2_1, FILTEREDIT2_2, FILTERCOLSEL2);
			redraw_object(filter_conf, FILTEROPERATION2);
			redraw_object(filter_conf, ACTREFRESH2);
			break;
			
		case 2:
			setOperation(curroper[2], curresel[2], curredit[4], FILTEROPERATION3, FILTERSEL3, FILTERSELDROP3, 
									 FILTEREDIT3_1, FILTEREDIT3_2, FILTERCOLSEL3);
			redraw_object(filter_conf, FILTEROPERATION3);
			redraw_object(filter_conf, ACTREFRESH3);
			break;
		}
	}
}

/* --------------------------
   | Select condition field |
   -------------------------- */
static void ConditionSelect(int field, int x, int y)
{
char *strings[14];
int i, first, j = 0, ret;

DEBUG();
if (!field)
	{
	first = 1;
	i = 2;
	}
else
	{
	first = 0;
	i = 1;
	}

while(i < sizeof(compare_ids) / sizeof(compare_ids[0]))
	strings[j++] = filter_strings[compare_ids[i++]].ob_spec.free_string;
strings[j] = NULL;

ret = free_droplist(x, y, filter_conf[FILTERCONDDROP1].ob_x + 
												  filter_conf[FILTERCONDDROP1].ob_width - 
												  filter_conf[FILTERCONDITION1].ob_x, strings);
if (ret != -1)
	{
	ret += first;
	if (ret != compare[field])
		{
		switch(ret)
			{
			case 7: condi[field] = 0; break;
			case 8: condedit[field][0] = 0; break;
			default:
				if (compare[field] == 8)
					condedit[field][0] = 0;
				break;
			}
		}
	compare[field] = ret;

	switch(field)
		{
		case 0:
			setCompare(compare[0], condi[0], condedit[0], FILTERCONDITION1, FILTERCOMPSEL1, FILTERCOMPSELDROP1, FILTERCONDEDIT1);
			redraw_object(filter_conf, FILTERCONDITION1);
			redraw_object(filter_conf, CONDREFRESH1);
			break;
		case 1:	
			setCompare(compare[1], condi[1], condedit[1], FILTERCONDITION2, FILTERCOMPSEL2, FILTERCOMPSELDROP2, FILTERCONDEDIT2);
			redraw_object(filter_conf, FILTERCONDITION2);
			redraw_object(filter_conf, CONDREFRESH2);
			break;
		case 2:
			setCompare(compare[2], condi[2], condedit[2], FILTERCONDITION3, FILTERCOMPSEL3, FILTERCOMPSELDROP3, FILTERCONDEDIT3);
			redraw_object(filter_conf, FILTERCONDITION3);
			redraw_object(filter_conf, CONDREFRESH3);
			break;
		}
	}
}

/* -----------------------------
	 | Fill filter configuration |
	 ----------------------------- */
void fill_filter_conf(int num)
{
FILTER_DEF *cfg;
int i;

DEBUG();
if (num == -1)
	cfg = &defaultentry;
else
  cfg = filters + section_start + num;

strcpy(filter_conf[FILTERLABEL].ob_spec.tedinfo->te_ptext, cfg->name);

SetSelect(filter_conf, FILTERAND1, cfg->andor1 == 0);
SetSelect(filter_conf, FILTEROR1, cfg->andor1 == 1);
SetSelect(filter_conf, FILTERAND2, cfg->andor2 == 0);
SetSelect(filter_conf, FILTEROR2, cfg->andor2 == 1);

for (i = 0; i < 3; i++)
	{
	compare[i] = cfg->cond[i];
	condi[i] = cfg->condi[i];
	curroper[i] = cfg->oper[i];
	curresel[i] = cfg->esel[i];
	}

make_colcell(filter_conf, FILTERCOLSEL1, curresel[0]);
make_colcell(filter_conf, FILTERCOLSEL2, curresel[1]);
make_colcell(filter_conf, FILTERCOLSEL3, curresel[2]);

for (i = 0; i < 3; i++)
	strcpy(condedit[i], cfg->condedit[i]);

for (i = 0; i < 6; i++)
	strcpy(curredit[i], cfg->edit[i]);

filter_conf[FILTERCONDEDIT1].ob_spec.tedinfo->te_ptext = condedit[0];
filter_conf[FILTERCONDEDIT2].ob_spec.tedinfo->te_ptext = condedit[1];
filter_conf[FILTERCONDEDIT3].ob_spec.tedinfo->te_ptext = condedit[2];

setCompare(compare[0], condi[0], condedit[0], FILTERCONDITION1, FILTERCOMPSEL1, FILTERCOMPSELDROP1, FILTERCONDEDIT1);
setCompare(compare[1], condi[1], condedit[1], FILTERCONDITION2, FILTERCOMPSEL2, FILTERCOMPSELDROP2, FILTERCONDEDIT2);
setCompare(compare[2], condi[2], condedit[2], FILTERCONDITION3, FILTERCOMPSEL3, FILTERCOMPSELDROP3, FILTERCONDEDIT3);

filter_conf[FILTEREDIT1_1].ob_spec.tedinfo->te_ptext = curredit[0];
filter_conf[FILTEREDIT1_2].ob_spec.tedinfo->te_ptext = curredit[1];
filter_conf[FILTEREDIT2_1].ob_spec.tedinfo->te_ptext = curredit[2];
filter_conf[FILTEREDIT2_2].ob_spec.tedinfo->te_ptext = curredit[3];
filter_conf[FILTEREDIT3_1].ob_spec.tedinfo->te_ptext = curredit[4];
filter_conf[FILTEREDIT3_2].ob_spec.tedinfo->te_ptext = curredit[5];

setOperation(curroper[0], curresel[0], curredit[0], FILTEROPERATION1, FILTERSEL1, 
						 FILTERSELDROP1, FILTEREDIT1_1, FILTEREDIT1_2, FILTERCOLSEL1);
setOperation(curroper[1], curresel[1], curredit[2], FILTEROPERATION2, FILTERSEL2, 
						 FILTERSELDROP2, FILTEREDIT2_1, FILTEREDIT2_2, FILTERCOLSEL2);
setOperation(curroper[2], curresel[2], curredit[4], FILTEROPERATION3, FILTERSEL3, 
						 FILTERSELDROP3, FILTEREDIT3_1, FILTEREDIT3_2, FILTERCOLSEL3);
}

/* ----------------------------
	 | Get filter configuration |
	 ---------------------------- */
void store_filter_conf(int num)
{
FILTER_DEF newcfg;
int i;

DEBUG();
strcpy(newcfg.folder, filterfolder);
strcpy(newcfg.name, filter_conf[FILTERLABEL].ob_spec.tedinfo->te_ptext);

if (!newcfg.name[0])
	sprintf(newcfg.name, "Filter %d", (num != -1 ? num : (section_count + 1)));
	
newcfg.andor1 = IsSelected(filter_conf, FILTEROR1);
newcfg.andor2 = IsSelected(filter_conf, FILTEROR2);

for (i = 0; i < 3; i++)	newcfg.cond[i] = compare[i];
for (i = 0; i < 3; i++)
	{
	newcfg.cond[i] = compare[i];
	newcfg.condi[i] = condi[i];
	newcfg.oper[i] = curroper[i];
	newcfg.esel[i] = curresel[i];
	}

for (i = 0; i < 3; i++)
	strcpy(newcfg.condedit[i], condedit[i]);

for (i = 0; i < 6; i++)
	strcpy(newcfg.edit[i], curredit[i]);

if (num != -1)
	memcpy(filters + section_start + num, &newcfg, sizeof(FILTER_DEF));
else
	{
	if (!filter_count)
		filters = (FILTER_DEF *)malloc(sizeof(FILTER_DEF));
	else
		filters = (FILTER_DEF *)realloc(filters, sizeof(FILTER_DEF) * (filter_count + 1));
	
	if (section_start + section_count < filter_count)
		memmove(filters + section_start + section_count + 1,
						filters + section_start,
						sizeof(FILTER_DEF) * (filter_count - section_start - section_count));
							
	memcpy(filters + section_start + section_count, &newcfg, sizeof(FILTER_DEF));
	filter_count++;
	section_count++;
	}
}

/* ----------------------------------------
   | Handle Identity configuration events |
   ---------------------------------------- */
static void cdecl filterconfig_handler (OBJECT *tree, int obj)
{
int field = 0;

DEBUG();
	/* Pass 1 */
switch(obj)
	{
	case FILTERCONDDROP3:
		obj = FILTERCONDITION3;
		break;
	case FILTERCONDDROP2:
		obj = FILTERCONDITION2;
		break;
	case FILTERCONDDROP1:
		obj = FILTERCONDITION1;
		break;

	case FILTERCOMPSELDROP3:
		obj = FILTERCOMPSEL3;
		break;
	case FILTERCOMPSELDROP2:
		obj = FILTERCOMPSEL2;
		break;
	case FILTERCOMPSELDROP1:
		obj = FILTERCOMPSEL1;
		break;
		
	case FILTEROPERDROP3:
		obj = FILTEROPERATION3;
		break;
	case FILTEROPERDROP2:
		obj = FILTEROPERATION2;
		break;
	case FILTEROPERDROP1:
		obj = FILTEROPERATION1;
		break;
		
	case FILTERSELDROP3:
		if (curroper[2] == 5)
			obj = FILTERCOLSEL3;
		else
			obj = FILTERSEL3;
		break;
	case FILTERSELDROP2:
		if (curroper[1] == 5)
			obj = FILTERCOLSEL2;
		else
			obj = FILTERSEL2;
		break;
	case FILTERSELDROP1:
		if (curroper[0] == 5)
			obj = FILTERCOLSEL1;
		else
			obj = FILTERSEL1;
		break;
	}
	
switch(obj)
	{
	case FILTERCONDITION3:
		field++;
	case FILTERCONDITION2:
		field++;
	case FILTERCONDITION1:
		{
		int x, y;
		
		objc_offset(tree, obj & 0x7FFF, &x, &y);
		ConditionSelect(field, x, y);
		break;
		}		

	case FILTERCOMPSEL3:
		field++;
	case FILTERCOMPSEL2:
		field++;
	case FILTERCOMPSEL1:
		{
		int x, y;
		
		objc_offset(tree, obj & 0x7FFF, &x, &y);
		
		if (compare[field] == 7)
			PrioritySelect(field, x, y);
		else
			IdentitySelect(field, x, y);
		break;
		}
		
	case FILTEROPERATION3:
		field++;
	case FILTEROPERATION2:
		field++;
	case FILTEROPERATION1:
		{
		int x, y;
		
		objc_offset(tree, obj & 0x7FFF, &x, &y);
		OperationSelect(field, x, y);
		break;
		}		

	case FILTERSEL3:
		field++;
	case FILTERSEL2:
		field++;
	case FILTERSEL1:
		{
		int x, y;
		
		objc_offset(tree, obj & 0x7FFF, &x, &y);
		switch (curroper[field])
			{
			case 1:
			case 2:
				FolderSelect(field, x, y);
				break;
				
			case 12:
				IdentitySelect(field + 3, x, y);
				break;
			}
		break;
		}		

	case FILTERCOLSEL3:
		field++;
	case FILTERCOLSEL2:
		field++;
	case FILTERCOLSEL1:
		{
		OBJECT *pop;
		
		rsrc_gaddr(R_TREE, CSELPOP, &pop);	
		color_popup(tree, obj, pop, curresel[field] );
		
		curresel[field] = (tree[obj].ob_state >> 8) & 0xFF;
		break;
		}		
			
	case FILTER_OKAY:
		{
		OBJECT *filterlist;
		
		filter_chg = TRUE;
		store_filter_conf(cfg_filter);

		rsrc_gaddr(R_TREE, EDITFILTER, &filterlist);

		if (is_open(filterlist))
			redraw_object(filterlist, FILTERLIST);
		}
		
	case FILTER_CANCEL:
		close_dialog(tree, TRUE);
		break;
	}
}

/* --------------------------
	 | Identity configuration |
	 -------------------------- */
void config_filter(int num)
{
DEBUG();
if (!filter_conf)
	{
	rsrc_gaddr(R_TREE, FILTERSET, &filter_conf);
	rsrc_gaddr(R_TREE, FILTERSTRINGS, &filter_strings);
	}

cfg_filter = num;
open_dialog(filter_conf, NULL, iconified_tree);
dlgset_help_info(filter_conf, helpfile, fconf_ids, "RULE_");
fill_filter_conf(num);
set_active(filter_conf, filterconfig_handler, TRUE, 0);
}

/* --------------------------
   | Handle Identity events |
   -------------------------- */
static void cdecl filter_handler (OBJECT *tree, int obj)
{
DEBUG();
switch(obj)
	{
	case FILTERLIST:
		{
		int ox, oy, pos;
		EVNTDATA ev;
		
		objc_offset(tree, FILTERLIST, &ox, &oy);
		graf_mkstate(&ev);

		pos = (ev.y - oy) / tx_height + filter_page;
		if (pos >= 0 && pos < section_count)
			{
			if (selected_filter == -1)
				{
				EnableObj(tree, EDIT_FILTER);
				EnableObj(tree, DEL_FILTER);
				redraw_object(tree, EDIT_FILTER);
				redraw_object(tree, DEL_FILTER);
				}

			if (filters[section_start + pos].active)
				{
				DisableObj(tree, ACT_FILTER);
				EnableObj(tree, DEACT_FILTER);
				}
			else
				{
				EnableObj(tree, ACT_FILTER);
				DisableObj(tree, DEACT_FILTER);
				}
			redraw_object(tree, ACT_FILTER);
			redraw_object(tree, DEACT_FILTER);
			selected_filter = pos;
			}
		else
			{
			if (selected_filter != -1)
				{
				DisableObj(tree, EDIT_FILTER);
				DisableObj(tree, DEL_FILTER);
				DisableObj(tree, ACT_FILTER);
				DisableObj(tree, DEACT_FILTER);
				redraw_object(tree, EDIT_FILTER);
				redraw_object(tree, DEL_FILTER);
				redraw_object(tree, ACT_FILTER);
				redraw_object(tree, DEACT_FILTER);
				}
			selected_filter = -1;
			}
		redraw_object(tree, FILTERLIST);
		break;
		}

	case NEW_FILTER:
		config_filter(-1);
		break;
		
	case EDIT_FILTER:
		config_filter(selected_filter);
		break;
		
	case DEL_FILTER:
		{
		if (cfg_filter == selected_filter && is_open(filter_conf))
			close_dialog(filter_conf, TRUE);
			
		if (cfg_filter > selected_filter) cfg_filter--;
		if (section_count > 1 && selected_filter < section_count - 1)
			memmove(filters + section_start + selected_filter, 
							filters + section_start + selected_filter + 1,
							sizeof(FILTER_DEF) * (filter_count - section_start - selected_filter - 1)) ;
		else
			if (filter_count == 1)
				free(filters);	
		filter_chg = TRUE;
		filter_count--;
		section_count--;
		selected_filter = -1;

		DisableObj(tree, EDIT_FILTER);
		DisableObj(tree, DEL_FILTER);
		DisableObj(tree, ACT_FILTER);
		DisableObj(tree, DEACT_FILTER);
		redraw_object(tree, EDIT_FILTER);
		redraw_object(tree, DEL_FILTER);
		redraw_object(tree, ACT_FILTER);
		redraw_object(tree, DEACT_FILTER);
		
		redraw_object(tree, FILTERLIST);
		break;
		}

	case ACT_FILTER:
	case DEACT_FILTER:
		{
		filters[section_start + selected_filter].active = (obj == ACT_FILTER);
		redraw_object(tree, FILTERLIST);
		if (filters[section_start + selected_filter].active)
			{
			DisableObj(tree, ACT_FILTER);
			EnableObj(tree, DEACT_FILTER);
			}
		else
			{
			EnableObj(tree, ACT_FILTER);
			DisableObj(tree, DEACT_FILTER);
			}
		redraw_object(tree, ACT_FILTER);
		redraw_object(tree, DEACT_FILTER);
		break;
		}
		
	case EXIT_FILTER:
		{
		if (filterfolder[0])
			{
			int box = box_number(filterfolder);
			if (box != -1)
				boxes[box].auto_apply = IsSelected(tree, APPLY_ON_OPEN);
			}
		save_filter();
		close_dialog(tree, TRUE);
		break;
		}
	}
}


/* ----------------------------
   | User object address book |
   ---------------------------- */
int cdecl filter_obj(PARMBLK *parm)
{
int xy[4], i, y, lines;
int dummy;

DEBUG();
obj_clip(parm, TRUE);

vst_height(vdi_handle, default_fsize, &dummy, &dummy, &dummy, &dummy);
vswr_mode(vdi_handle, MD_REPLACE);
vsf_interior(vdi_handle, FIS_HOLLOW);
xy[0] = parm->pb_x;
xy[1] = parm->pb_y;
xy[2] = parm->pb_x + parm->pb_w - 2;
xy[3] = parm->pb_y + parm->pb_h - 1;
vr_recfl(vdi_handle, xy);

lines = (parm->pb_tree[FILTERLIST].ob_height + tx_height - 1) / tx_height;

y = xy[1];
for (i = filter_page; i < section_count && i < filter_page + lines; i++, y += tx_height)
	{
	if (filters[section_start + i].active) 
		icon_copy(symbols, 10, (smallfont ? 8 : 16) * 4, 
					 		parm->pb_x + 1, y, 
					 		10, smallfont ? 8 : 16);
	else
		icon_copy(symbols, 10, (smallfont ? 8 : 16) * 5, 
					 		parm->pb_x + 1, y, 
					 		10, smallfont ? 8 : 16);
		
	v_gtext(vdi_handle, parm->pb_x + 2 * tx_width, y, filters[section_start + i].name);

	if (selected_filter == i)
		{
		vswr_mode(vdi_handle, MD_XOR);
		vsf_interior(vdi_handle, FIS_SOLID);
		vsf_color(vdi_handle, 1);

		xy[0] = parm->pb_x;
		xy[1] = y;
		xy[2] = parm->pb_x + parm->pb_w - 2;
		xy[3] = y + tx_height - 1;
		vr_recfl(vdi_handle, xy);
		vswr_mode(vdi_handle, MD_TRANS);
		}
	}

obj_clip(parm, FALSE);
vswr_mode(vdi_handle, MD_REPLACE);

return 0;
}

/* -------------------
   | Edit filters |
   ------------------- */
void edit_filters(int box)
{
OBJECT *filterlist;
char *folder;
int i;

DEBUG();
rsrc_gaddr(R_TREE, EDITFILTER, &filterlist);

if (box != -1 && box != box_number("IN"))
	folder = boxes[box].file;
else
	folder = "\0";
	
if (is_open(filterlist) && strcmp(folder, filterfolder))
	{
	close_dialog(filterlist, TRUE);		/* Alert for okay?! */
	}
	
if (!is_open(filterlist))
	{
	if (filterlist[FILTERLIST].ob_type != G_USERDEF)
		{
		filterlist[FILTERLIST].ob_type = G_USERDEF;
		filterlist[FILTERLIST].ob_spec.userblk = &filter_ublk;
		}
			
	filter_page = 0;
	selected_filter = -1;
	DisableObj(filterlist, EDIT_FILTER);
	DisableObj(filterlist, DEL_FILTER);
	DisableObj(filterlist, ACT_FILTER);
	DisableObj(filterlist, DEACT_FILTER);
	
	if (folder[0])
		{
		SetSelect(filterlist, APPLY_ON_OPEN, boxes[box].auto_apply);
		EnableObj(filterlist, APPLY_ON_OPEN);
		}
	else
		{
		DeselectObj(filterlist, APPLY_ON_OPEN);
		DisableObj(filterlist, APPLY_ON_OPEN);
		}

	if (!folder[0])
		strcpy(filtertitle, filterlist[EF_LABEL_INCOMING].ob_spec.free_string);
	else
		sprintf(filtertitle, filterlist[EF_LABEL_FOR].ob_spec.free_string, boxes[box].desc);
	
	strcpy(filterfolder, folder);			

	section_start = filter_count;
	section_count = 0;
	for (i = 0; i < filter_count; i++)
		if (!strcmp(filters[i].folder, folder))
			{
			section_start = i;
			while(i < filter_count)
				{
				if (!strcmp(filters[i].folder, folder))
					section_count++;
				else
					break;
				i++;
				}
			break;
			}

	align_slider_objects(filterlist, FLT_SLIDER, FLT_UP, FLT_SLDBKG, FLT_SLDBOX, FLT_DOWN);
	set_slider_objects(filterlist, FLT_SLDBKG, FLT_SLDBOX, section_count, filter_page, 
										 filterlist[FILTERLIST].ob_height / tx_height, FALSE);	
	}
			
open_dialog(filterlist, filtertitle, iconified_tree);
dlgset_help_info(filterlist, helpfile, efilt_ids, "FRUL_");
set_active(filterlist, filter_handler, TRUE, 0);
}

/* ------------------------------
   | Move filter folder indexes |
   ------------------------------ */
void move_filterfolders(int from_index, int diff)
{
int i, j;

DEBUG();
for(i = 0; i < filter_count; i++)
	for (j = 0; j < 3; j++)
		if (filters[i].oper[j] == 1 || filters[i].oper[j] == 2)
			{
			if (filters[i].esel[j] >= from_index)
				{
				filters[i].esel[j] += diff;
				filter_chg = TRUE;
				}
			}
}

/* -----------------------
   | Remove folder rules |
   -----------------------*/
void remove_folder_rules(char *folder)
{
int i, start, count;

DEBUG();
if (!folder[0]) return;

for (i = 0; i < filter_count; i++)
	if (!strcmp(filters[i].folder, folder))
		{
		start = i;
		count = 0;
		
		while(i < filter_count)
			{
			if (!strcmp(filters[i].folder, folder))
				section_count++;
			else
				break;
			i++;
			}
			
		if (start + count < filter_count)
			memmove(filters + start, filters + start + count, 
							sizeof(FILTER_DEF) * (filter_count - start - count));
		
		filter_count -= count;
		if (!filter_count)
			free(filters);
		else
			filters = (FILTER_DEF *)realloc(filters, sizeof(FILTER_DEF) * filter_count);
		break;
		}
}

/* --------------------------
   | Load filter from  file |
   -------------------------- */
void load_filter(void)
{
long id_fl;
char buf[256];

DEBUG();
if (filter_count)
	free(filters);
	
base_path(buf);
strcat(buf, "FILTER.DAT");

id_fl = Fopen(buf, FO_READ);
if (id_fl > 0)
	{
	long ver;
	
	Fread((int)id_fl, sizeof(ver), &ver);
	
	Fread((int)id_fl, sizeof(filter_count), &filter_count);
	if (filter_count)
		{
		filters = (FILTER_DEF *)malloc(sizeof(FILTER_DEF) * 
																		   filter_count);
																		   
		Fread((int)id_fl, sizeof(FILTER_DEF) * filter_count,
				  filters);
		}
		
	Fclose((int)id_fl);
	}
else
	filter_count = 0;
	
filter_chg = FALSE;
}

/* --------------------
   | Save filter file |
   -------------------- */
void save_filter(void)
{
long id_fl;
char buf[256];

DEBUG();
if (!filter_chg) return;

base_path(buf);
strcat(buf, "FILTER.DAT");
id_fl = Fcreate(buf, 0);
if (id_fl > 0)
	{
	long ver = VERSION_ID20;
	
	Fwrite((int)id_fl, sizeof(ver), &ver);
	Fwrite((int)id_fl, sizeof(filter_count), &filter_count);
	if (filter_count)
		Fwrite((int)id_fl, sizeof(FILTER_DEF) * filter_count,
					 filters);
	Fclose((int)id_fl);
	filter_chg = FALSE;
	}
}

/* ----------------
   | Apply filter |
   ---------------- */
void apply_filter(int box)
{
int i;
char file[13];

DEBUG();
if (box == -1)
	box = box_number("IN");
	
i = 0;
while(i < box_manager[box].box_entries)
	{
	if (i + 1 < box_manager[box].box_entries)								/* Store the name of the next entry */
		strcpy(file, box_manager[box].index[i + 1].file);
	else
		file[0] = 0;
		
	apply_filter_to_msg(box, box_manager[box].index + i);
	
	if (file[0])
		{
			/* Try to find the next entry, because it may be moved */
		i = 0;
		while(i < box_manager[box].box_entries)
			if (!strcmp(file, box_manager[box].index[i].file))
				break;
			else
				i++;
		}
	else
		break;
	}
}

/* ---------------------------
   | Apply filter on message |
   --------------------------- */
void apply_filter_to_msg(int box, MAIL_INFO *mail)
{
int i;
char *folder;

DEBUG();
if (box != -1 && box != box_number("IN"))
	folder = boxes[box].file;
else
	folder = "\0";

for (i = 0; i < filter_count; i++)
	if (!strcmp(filters[i].folder, folder))
		{
		while(i < filter_count)
			{
			if (!strcmp(filters[i].folder, folder))
				{
				if (filters[i].active)
					{
					skip_filter = -32767;
					if (!execute_filter(box, mail, i))
						break;
						
					if (skip_filter != -32767)
						{
						i += skip_filter;
						continue;
						}
					}
				}
			else
				break;
			i++;
			}
			
		break;
		}

}

static int (*fcompares[])(int box, MAIL_INFO *mail, FILTER_DEF *fdef, int idx) = {
														fcompare_false, /* Please select -> Always false */
														fcompare_true,	/* Nothing -> Always true */
														fcompare_from, fcompare_subject, fcompare_text, fcompare_to, fcompare_cc,
														fcompare_to_and_cc, fcompare_priority, fcompare_member, fcompare_bigger,
														fcompare_smaller, fcompare_has_attachment, fcompare_always};

static int (*factions[])(int *box, MAIL_INFO **mail, FILTER_DEF *fdef, int idx) = {
														faction_nothing,	/* Please select  */
														faction_nothing,	/* Nothing */
														faction_moveto, faction_copyto, faction_remove, faction_forwardto, faction_markcolour,
														faction_select, faction_markread, faction_markunread, faction_stoprules,
														faction_skipnext, faction_jumpto, faction_setidentity};

/* ------------------------------
   | Check and execute one rule |
   ------------------------------ */
int execute_filter(int box, MAIL_INFO *mail, int filteridx)
{
int cond1, cond2, cond3, result;
FILTER_DEF *filterdef;

DEBUG();
filterdef = filters + filteridx;
cond1 = (*fcompares[filterdef->cond[0] + 1])(box, mail, filterdef, 0);
cond2 = (*fcompares[filterdef->cond[1] + 1])(box, mail, filterdef, 1);
cond3 = (*fcompares[filterdef->cond[2] + 1])(box, mail, filterdef, 2);

if (filterdef->andor1 == 0)
	result = cond1 & cond2;
else
	result = cond1 | cond2;

if (filterdef->andor2 == 0)
	result &= cond3;
else
	result |= cond3;

if (result)
	{
	(*factions[filterdef->oper[0] + 1])(&box, &mail, filterdef, 0);
	if (mail)	/* Because it may be deleted */
		(*factions[filterdef->oper[1] + 1])(&box, &mail, filterdef, 1);
	if (mail)	/* Because it may be deleted */
		(*factions[filterdef->oper[2] + 1])(&box, &mail, filterdef, 2);
	}
	
return TRUE;
}

