/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

extern int skip_filter;		/* Defined in FILTER.C */
extern int filter_count;
extern FILTER_DEF *filters;

/* 
	Comparisions:
		From
		Subject
		Text
		To
		CC
		To or CC
		Priority
		Member of
		Bigger
		Smaller
		has attachment
		Always
*/

/* ------------------------------------------------------------------
   | Always false (may be also called, if no condition is selection |
   ------------------------------------------------------------------ */
int fcompare_false(int box, MAIL_INFO *mail, FILTER_DEF *fdef, int idx)
{
(void)box;
(void)mail;
(void)fdef;
(void)idx;
return FALSE;
}

/* ----------------------------------------------------
   | Always true (called also for unfilled conditions |
   ---------------------------------------------------- */
int fcompare_true(int box, MAIL_INFO *mail, FILTER_DEF *fdef, int idx)
{
(void)box;
(void)mail;
(void)fdef;
(void)idx;
return TRUE;
}

/* --------------
   | Check FROM |
   -------------- */
int fcompare_from(int box, MAIL_INFO *mail, FILTER_DEF *fdef, int idx)
{
int f;

DEBUG();
f = open_file(mail->file, boxes[box].file);
if (f)
	{
	MAIL_MEM buffer;
	FILE_MAIL_MEM header;
	int ret;
	
	restore_header(f, &header, &buffer);
	Fclose(f);
	
	ret = matchpattern((unsigned char *)buffer.from, (unsigned char *)fdef->condedit[idx], FALSE);
	clear_buffer(&buffer);

	return ret;
	}
return FALSE;	
}

/* -----------------
   | Check SUBJECT |
   ----------------- */
int fcompare_subject(int box, MAIL_INFO *mail, FILTER_DEF *fdef, int idx)
{
int f;

DEBUG();
f = open_file(mail->file, boxes[box].file);
if (f)
	{
	MAIL_MEM buffer;
	FILE_MAIL_MEM header;
	int ret;
	
	restore_header(f, &header, &buffer);
	Fclose(f);
	
	ret = matchpattern((unsigned char *)buffer.subject, (unsigned char *)fdef->condedit[idx], FALSE);
	clear_buffer(&buffer);

	return ret;
	}
return FALSE;	
}

/* --------------
   | Check TEXT |
   -------------- */
int fcompare_text(int box, MAIL_INFO *mail, FILTER_DEF *fdef, int idx)
{
char 	*chk_mem;
long 	chk_blen, *chk_ptr;
int 	chk_width;
int 	chk_lines, i;

DEBUG();
i = get_body("\0",
							boxes[box].file, mail, 
							&chk_mem, &chk_blen, &chk_ptr,
					 		&chk_width, &chk_lines, FALSE);
if (i)
	{
	int ret;
	
	ret = matchpattern2((unsigned char *)chk_mem, (unsigned char *)(chk_mem + chk_blen),
											(unsigned char *)fdef->condedit[idx], FALSE);
	
	free(chk_mem);
	free(chk_ptr);
	
	return ret;
	}

return FALSE;
}

/* ------------
   | Check TO |
   ------------ */
int fcompare_to(int box, MAIL_INFO *mail, FILTER_DEF *fdef, int idx)
{
int f;

DEBUG();
f = rwopen_file(mail->file, boxes[box].file);
if (f)
	{
	MAIL_MEM buffer;
	FILE_MAIL_MEM header;
	int ret;
	
	restore_header(f, &header, &buffer);
	Fclose(f);
	
	ret = matchpattern((unsigned char *)buffer.to, (unsigned char *)fdef->condedit[idx], FALSE);
	clear_buffer(&buffer);

	return ret;
	}
return FALSE;	
}

/* ------------
   | Check CC |
   ------------ */
int fcompare_cc(int box, MAIL_INFO *mail, FILTER_DEF *fdef, int idx)
{
int f;

DEBUG();
f = open_file(mail->file, boxes[box].file);
if (f)
	{
	MAIL_MEM buffer;
	FILE_MAIL_MEM header;
	int ret;
	
	restore_header(f, &header, &buffer);
	Fclose(f);
	
	ret = matchpattern((unsigned char *)buffer.cc, (unsigned char *)fdef->condedit[idx], FALSE);
	ret |= matchpattern((unsigned char *)buffer.bcc, (unsigned char *)fdef->condedit[idx], FALSE);
	clear_buffer(&buffer);

	return ret;
	}
return FALSE;
}

/* ---------------
   | Check TO/CC |
   --------------- */
int fcompare_to_and_cc(int box, MAIL_INFO *mail, FILTER_DEF *fdef, int idx)
{
int f;

DEBUG();
f = open_file(mail->file, boxes[box].file);
if (f)
	{
	MAIL_MEM buffer;
	FILE_MAIL_MEM header;
	int ret;
	
	restore_header(f, &header, &buffer);
	Fclose(f);
	
	ret = matchpattern((unsigned char *)buffer.to, (unsigned char *)fdef->condedit[idx], FALSE);
	ret |= matchpattern((unsigned char *)buffer.cc, (unsigned char *)fdef->condedit[idx], FALSE);
	ret |= matchpattern((unsigned char *)buffer.bcc, (unsigned char *)fdef->condedit[idx], FALSE);
	clear_buffer(&buffer);

	return ret;
	}
return FALSE;
}

/* ------------------
   | Check PRIORITY |
	 | 	0 = Normal		|
	 |  1 = High		  |
	 |  2 = Low 			|
   ------------------ */
int fcompare_priority(int box, MAIL_INFO *mail, FILTER_DEF *fdef, int idx)
{
int f;

DEBUG();
f = open_file(mail->file, boxes[box].file);
if (f)
	{
	MAIL_MEM buffer;
	FILE_MAIL_MEM header;
	int ret;
	
	restore_header(f, &header, &buffer);
	Fclose(f);
	
	ret = (fdef->condi[idx] == header.priority);
	clear_buffer(&buffer);

	return ret;
	}
return FALSE;
}

/* ------------------
   | Check IDENTITY |
   ------------------ */
int fcompare_member(int box, MAIL_INFO *mail, FILTER_DEF *fdef, int idx)
{
int f;

DEBUG();
f = open_file(mail->file, boxes[box].file);
if (f)
	{
	MAIL_MEM buffer;
	FILE_MAIL_MEM header;
	int ret;
	
	restore_header(f, &header, &buffer);
	Fclose(f);

	ret = matchpattern((unsigned char *)buffer.identity, (unsigned char *)fdef->condedit[idx], FALSE);
	clear_buffer(&buffer);

	return ret;
	}
return FALSE;
}

/* --------------------------------
   | Check, if the size is bigger |
   -------------------------------- */
int fcompare_bigger(int box, MAIL_INFO *mail, FILTER_DEF *fdef, int idx)
{
int f;

DEBUG();
f = open_file(mail->file, boxes[box].file);
if (f)
	{
	MAIL_MEM buffer;
	FILE_MAIL_MEM header;
	int ret;
	long size;
	
	restore_header(f, &header, &buffer);
	Fclose(f);

	size = atol(fdef->condedit[idx]);
	
	ret = (header.body_len > size);
	clear_buffer(&buffer);

	return ret;
	}
return FALSE;
}

/* --------------------------------
   | Check, if the size is smaller |
   -------------------------------- */
int fcompare_smaller(int box, MAIL_INFO *mail, FILTER_DEF *fdef, int idx)
{
int f;

DEBUG();
f = open_file(mail->file, boxes[box].file);
if (f)
	{
	MAIL_MEM buffer;
	FILE_MAIL_MEM header;
	int ret;
	long size;
	
	restore_header(f, &header, &buffer);
	Fclose(f);

	size = atol(fdef->condedit[idx]);
	
	ret = (header.body_len < size);
	clear_buffer(&buffer);

	return ret;
	}
return FALSE;
}

/* ----------------------------------
   | Check, if it has an attachment |
   ---------------------------------- */
int fcompare_has_attachment(int box, MAIL_INFO *mail, FILTER_DEF *fdef, int idx)
{
int f;

(void)fdef;
(void)idx;

DEBUG();
f = open_file(mail->file, boxes[box].file);
if (f)
	{
	MAIL_MEM buffer;
	FILE_MAIL_MEM header;
	int ret;
	
	restore_header(f, &header, &buffer);
	Fclose(f);

	ret = (header.part_count > 1);
	clear_buffer(&buffer);

	return ret;
	}
return FALSE;
}

/* --------------------
   | Always condition |
   -------------------- */
int fcompare_always(int box, MAIL_INFO *mail, FILTER_DEF *fdef, int idx)
{
(void)box;
(void)mail;
(void)fdef;
(void)idx;
return TRUE;
}

/*		
	Actions:
	 	Move to
	 	Copy to
	 	Remove
	 	Forward to
	 	Mark with colour
	 	Select
	 	Mark as read
	 	Mark as unread
	 	Stop rules
	 	Skip next
	 	Jump to
	 */

/* --------------
   | Do nothing |
   -------------- */
int faction_nothing(int *box, MAIL_INFO **mail, FILTER_DEF *fdef, int idx)
{
(void)box;
(void)mail;
(void)fdef;
(void)idx;
return TRUE;
}

/* -------------------------------
   | Move mail to another folder |
   ------------------------------- */
int faction_moveto(int *box, MAIL_INFO **mail, FILTER_DEF *fdef, int idx)
{
int no;
char file[13];

DEBUG();
strcpy(file, (*mail)->file);

no = *mail - box_manager[*box].index;
if (move_onemail(*box, fdef->esel[idx], no))
	{
	if (box_manager[*box].list_wind > -1)
		{
		int wx, wy, ww, wh;
		
		wind_get(box_manager[*box].list_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
		redraw_listwindow(*box, wx, wy + iheight, ww, wh - iheight);
		}

	*box = fdef->esel[idx];
	
	if (box_manager[*box].list_wind > -1)
		{
		int wx, wy, ww, wh;
		
		wind_get(box_manager[*box].list_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
		redraw_listwindow(*box, wx, wy + iheight, ww, wh - iheight);
		}

	no = 0;
	while (no < box_manager[*box].box_entries)
		{
		if (!strcmp(box_manager[*box].index[no].file, file))
			{
			*mail = box_manager[*box].index + no;
			return TRUE;
			}
		no++;
		}
	}

return FALSE;
}

/* -------------
   | Copy mail |
   ------------- */
int faction_copyto(int *box, MAIL_INFO **mail, FILTER_DEF *fdef, int idx)
{
int no;
char file[13];

DEBUG();
strcpy(file, (*mail)->file);

no = *mail - box_manager[*box].index;
if (copy_onemail(*box, fdef->esel[idx], no))
	{
	if (box_manager[fdef->esel[idx]].list_wind > -1)
		{
		int wx, wy, ww, wh;
		
		wind_get(box_manager[fdef->esel[idx]].list_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
		redraw_listwindow(fdef->esel[idx], wx, wy + iheight, ww, wh - iheight);
		}

	no = 0;
	while (no < box_manager[*box].box_entries)
		{
		if (!strcmp(box_manager[*box].index[no].file, file))
			{
			*mail = box_manager[*box].index + no;
			break;
			}
		no++;
		}
	return TRUE;
	}

return FALSE;
}

/* ---------------
   | Remove mail |
   --------------- */
int faction_remove(int *box, MAIL_INFO **mail, FILTER_DEF *fdef, int idx)
{
char buf[51 * 2 + MAXFILE * 2 + 1];
int ret;

(void)fdef;
(void)idx;

DEBUG();
base_path(buf);

strcat(buf, boxes[*box].file);
strcat(buf, "\\");

strcat(buf, (*mail)->file);

ret = Fdelete(buf);		
if (!ret || ret == -33)
	{
	if ((*mail)->sel & 1)
		{
		box_manager[*box].list_marked -= (*mail)->body_len;
		box_manager[*box].list_marks--;
		}
		
	delete_index((*mail)->file, *box);
	*mail = NULL;
	return TRUE;
	}
return FALSE;
}

/* -------------------
   | Forward message |
   ------------------- */
int faction_forwardto(int *box, MAIL_INFO **mail, FILTER_DEF *fdef, int idx)
{
char address[41 * 2];

DEBUG();
strcpy(address, fdef->edit[idx * 2]);
strcat(address, fdef->edit[idx * 2 + 1]);

simple_forward(*box, *mail, address, "\0");

return TRUE;
}

/* --------------------
   | Mark with colour |
   -------------------- */
int faction_markcolour(int *box, MAIL_INFO **mail, FILTER_DEF *fdef, int idx)
{
(void)fdef;
(void)idx;

DEBUG();
(*mail)->color = fdef->esel[idx];
box_manager[*box].index_changed = TRUE;
				
if (box_manager[*box].list_wind > -1)
	{
	int wx, wy, ww, wh;
	
	wind_get(box_manager[*box].list_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
	redraw_listwindow(*box, wx, wy + iheight, ww, wh - iheight);
	}

return TRUE;
}

/* ---------------
   | Select mail |
   --------------- */
int faction_select(int *box, MAIL_INFO **mail, FILTER_DEF *fdef, int idx)
{
(void)fdef;
(void)idx;

DEBUG();
(*mail)->sel |= 1;
box_manager[*box].index_changed = TRUE;
				
if (box_manager[*box].list_wind > -1)
	{
	int wx, wy, ww, wh;
	
	wind_get(box_manager[*box].list_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
	redraw_listwindow(*box, wx, wy + iheight, ww, wh - iheight);
	}

return TRUE;
}

/* ----------------
   | Mark as read |
   ---------------- */
int faction_markread(int *box, MAIL_INFO **mail, FILTER_DEF *fdef, int idx)
{
(void)fdef;
(void)idx;

DEBUG();
(*mail)->sel &= ~2;
box_manager[*box].index_changed = TRUE;
				
if (box_manager[*box].list_wind > -1)
	{
	int wx, wy, ww, wh;
	
	wind_get(box_manager[*box].list_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
	redraw_listwindow(*box, wx, wy + iheight, ww, wh - iheight);
	}

return TRUE;
}

/* ------------------
   | Mark as unread |
   ------------------ */
int faction_markunread(int *box, MAIL_INFO **mail, FILTER_DEF *fdef, int idx)
{
(void)fdef;
(void)idx;

DEBUG();
(*mail)->sel |= 2;
box_manager[*box].index_changed = TRUE;
				
if (box_manager[*box].list_wind > -1)
	{
	int wx, wy, ww, wh;
	
	wind_get(box_manager[*box].list_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
	redraw_listwindow(*box, wx, wy + iheight, ww, wh - iheight);
	}

return TRUE;
}

/* ------------------------
	 | Stop executing rules |
	 ------------------------ */
int faction_stoprules(int *box, MAIL_INFO **mail, FILTER_DEF *fdef, int idx)
{
(void)box;
(void)fdef;
(void)idx;

DEBUG();
*mail = NULL;
skip_filter = 32767;
return TRUE;
}

/* ------------------
   | Skip next rule |
   ------------------ */
int faction_skipnext(int *box, MAIL_INFO **mail, FILTER_DEF *fdef, int idx)
{
(void)box;
(void)mail;
(void)fdef;
(void)idx;

DEBUG();
skip_filter = 1;
return TRUE;
}

/* -----------------------------
   | Jump to filter definition |
   ----------------------------- */
int faction_jumpto(int *box, MAIL_INFO **mail, FILTER_DEF *fdef, int idx)
{
int i, n;

(void)box;
(void)mail;

DEBUG();
n = (int)(fdef - filters);
i = n + 1;
while(i < filter_count)
	{
	if (!strcmp(filters[i].folder, fdef->folder))
		{
		if (!strcmp(filters[i].name, fdef->edit[idx * 2]))
			{
			skip_filter = i - n;
			return TRUE;
			}
		}
	else
		break;
	}
	
i = n - 1;
while(i > 0)
	{
	if (!strcmp(filters[i].folder, fdef->folder))
		{
		if (!strcmp(filters[i].name, fdef->edit[idx * 2]))
			{
			skip_filter = i - n;
			return TRUE;
			}
		}
	else
		break;
	}

return FALSE;
}

/* ----------------
   | Mark as read |
   ---------------- */
int faction_setidentity(int *box, MAIL_INFO **mail, FILTER_DEF *fdef, int idx)
{
(void)fdef;
(void)idx;

DEBUG();
strcpy((*mail)->identity, fdef->edit[idx * 2]);
box_manager[*box].index_changed = TRUE;
				
if (box_manager[*box].list_wind > -1)
	{
	int wx, wy, ww, wh;
	
	wind_get(box_manager[*box].list_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
	redraw_listwindow(*box, wx, wy + iheight, ww, wh - iheight);
	}

return TRUE;
}
