/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

static int find_in, repl_in;

static int find_ids[] = {FND_WORD,
												FND_CASE,
												FND_EXPR,
												FIND_FWD,
												FIND_BACKWD,
												FND_CANC,
												FND_FORW,
													0};

static int ropt_ids[] = {	REPL_WORD,
													REPL_WORD_SET,
													REPL_CASE,
													REPL_EXPR,
													REPL_FWD,
													REPL_BACKWD,
													REPL_CANC,
													REPL_ONE,
													REPL_ALL,
													0};

static int repl_ids[] = {	CANCEL_REPLACE,
													DONT_REPLACE,
													DO_REPLACE,
													0};

/* ----------------------
   | Handle find events |
   ---------------------- */
static void cdecl find_handler (OBJECT *tree, int obj)
{
DEBUG();
switch(obj)
	{
	case FND_FORW:
		{
		char *ptr = tree[FND_WORD].ob_spec.tedinfo->te_ptext;
		int sensitive = IsSelected(tree, FND_CASE);
		int use_expr	= IsSelected(tree, FND_EXPR);
		int backwards = IsSelected(tree, FIND_BACKWD);
		
		if (find_in == mail_wind)
			search_in_editor(ptr, sensitive, use_expr, backwards, TRUE);
		break;
		}
		
	case FND_CANC:
		close_dialog(tree, TRUE);
		break;
	}
}

/* --------------------------------------------------
   | Open find dialogue and register calling window |
   -------------------------------------------------- */
void open_find(int top)
{
DEBUG();
if (!is_open(find_tree))
	{
	open_dialog(find_tree, NULL, iconified_tree);
	dlgset_help_info(find_tree, helpfile, find_ids, "FND_");
	set_active(find_tree, find_handler, TRUE, FND_WORD);
	}
else
	open_dialog(find_tree, NULL, iconified_tree);

find_in = top;
}

/* -------------------
   | Ask for replace |
   ------------------- */
int replace_ask(void)
{
OBJECT *tree;
int obj;

DEBUG();
rsrc_gaddr(R_TREE, REPLACE_ASK, &tree);

open_dialog(tree, NULL, iconified_tree);
dlgset_help_info(tree, helpfile, repl_ids, "REPL_");
obj = do_dialog(tree, 0, app_callback) & 0x7FFF;
close_dialog(tree, TRUE);

return obj;
}

/* ----------------------
   | Handle find events |
   ---------------------- */
static void cdecl repl_handler (OBJECT *tree, int obj)
{
DEBUG();
switch(obj)
	{
	case REPL_ALL:
		{
		char *ptr = tree[REPL_WORD].ob_spec.tedinfo->te_ptext;
		char *ptr2 = tree[REPL_WORD_SET].ob_spec.tedinfo->te_ptext;
		int sensitive = IsSelected(tree, REPL_CASE);
		int use_expr	= IsSelected(tree, REPL_EXPR);
		int backwards = IsSelected(tree, REPL_BACKWD);
		
		if (repl_in == mail_wind)
			{
			int repl_done = FALSE;
			
			no_refresh = TRUE;
			do
				{
				search_in_editor(ptr, sensitive, use_expr, backwards, FALSE);
				if (!replace_editor(ptr2, FALSE))
					break;
				else
					repl_done = TRUE;
				}while(TRUE);
			no_refresh = FALSE;
			
			if (repl_done)
				{
				new_left();

				force_bodyredraw();
				set_mailslider();
				cursor_on();
	
				open_replace(mail_wind);
				}
			}
		break;
		}
		
	case REPL_ONE:
		{
		char *ptr = tree[REPL_WORD].ob_spec.tedinfo->te_ptext;
		char *ptr2 = tree[REPL_WORD_SET].ob_spec.tedinfo->te_ptext;
		int sensitive = IsSelected(tree, REPL_CASE);
		int use_expr	= IsSelected(tree, REPL_EXPR);
		int backwards = IsSelected(tree, REPL_BACKWD);
		
		if (repl_in == mail_wind)
			{
			do
				{
				search_in_editor(ptr, sensitive, use_expr, backwards, FALSE);
				if (!replace_editor(ptr2, TRUE))
					break;
				}while(TRUE);
			}
		break;
		}
		
	case REPL_CANC:
		close_dialog(tree, TRUE);
		break;
	}
}

/* --------------------------------------------
   | Open replace dialogue for current editor |
   -------------------------------------------- */
void open_replace(int top)
{
DEBUG();
if (!is_open(repl_tree))
	{
	open_dialog(repl_tree, NULL, iconified_tree);
	dlgset_help_info(repl_tree, helpfile, ropt_ids, "RPLO_");
	set_active(repl_tree, repl_handler, TRUE, REPL_WORD);
	}
else
	open_dialog(repl_tree, NULL, iconified_tree);

repl_in = top;
}
