/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

char **font_names = NULL;
int  *font_ids;

int select_font_size(int x, int y)
{
OBJECT *sizepop;
int obj;

DEBUG();
rsrc_gaddr(R_TREE, SIZEPOPUP, &sizepop);
obj = positioned_popup(sizepop, x, y);
if (obj > 0)
	{
	return atoi(sizepop[obj].ob_spec.free_string + 1);
	}
else
	return -1;
}

/* -------------------
   | Scan font names |
   ------------------- */
void scan_fontnames(void)
{
int i;
char *ptr;

DEBUG();
if (font_names)
	{
	free(font_names);
	font_names = NULL;
	}

ptr = (char *)malloc((sizeof(char *) + 33 + sizeof(int)) * num_fonts);

font_names = (char **)ptr;
ptr += sizeof(char *) * num_fonts;
font_ids = (int *)ptr;
ptr += sizeof(int) * num_fonts;

for (i = 0; i < num_fonts; i++)
	{
	font_ids[i] = vqt_name(vdi_handle, 1 + i, ptr);
	font_names[i] = ptr;
	ptr += 33;
	}
}

/* ------------------------
   | Find font identifier |
   ------------------------ */
int find_font_id(int id, char *fnt_name)
{
int i;

DEBUG();
if (id)
	{
	for (i = 0; i < num_fonts; i++)
		{
		if (id == font_ids[i])
			{
			if (fnt_name)
				{
				strncpy(fnt_name, font_names[i], 32);
				fnt_name[32] = 0;
				}
			return 1 + i;
			}
		}
	}
		
if (fnt_name)
	strcpy(fnt_name, font_names[0]);
return 1;
}

/* -----------------------
   | Font selector popup |
   ----------------------- */
void select_font_popup(int x, int y, int *zw, char *fname)
{
int i, j, page, obj;
OBJECT *font_pop;
static int ftext[] = {FONT_1, FONT_2, FONT_3, FONT_4, FONT_5, FONT_6};
char fnames[6][33], *font1, *font2;
GRECT fr;
int last_obj = 0, *font_sorts, zw1;

DEBUG();
rsrc_gaddr(R_TREE, FONTPOP, &font_pop);

form_center(font_pop, &fr);
font_pop[0].ob_x = x; font_pop[0].ob_y = y;

for (i = 0; i < 6; i++)
	{
	font_pop[ftext[i]].ob_spec.tedinfo->te_ptext = fnames[i];
	font_pop[ftext[i]].ob_width = font_pop[FONT_SBK].ob_x - 1;
	fnames[i][0] = 0;
	}

font_sorts = (int *)malloc(sizeof(int) * num_fonts);
for (i = 0; i < num_fonts; i++)
	font_sorts[i] = i;
for (i = 0; i < num_fonts - 1; i++)
	{
	font1 = font_names[font_sorts[i]];
	for (j = i + 1; j < num_fonts; j++)	
		{
		font2 = font_names[font_sorts[j]];
		if (strcmp(font2, font1) < 0)
			{
			font1 = font2;
			zw1 = font_sorts[i];
			font_sorts[i] = font_sorts[j];
			font_sorts[j] = zw1;
			}
		}
	}

if (num_fonts > 6)
	{
	zw1 = find_font_id(*zw, NULL);
	
	page = 0;
	while(font_sorts[page] + 1 != zw1)
		page++;
		
	if (page > num_fonts - 6)
		page = num_fonts - 6;
	}
else
	page = 0;

align_slider_objects(font_pop, FONT_SBK, FONT_UP, FONT_SLD, FONT_DRG, FONT_DWN);

wait_mouse_release();

for (i = 0; i < 6 && i < num_fonts; i++)	
	{
	strncpy(fnames[i], font_names[font_sorts[page + i]], 32);
	fnames[i][32] = 0;
	}
set_slider_objects(font_pop, FONT_SLD, FONT_DRG, num_fonts, page, 6, FALSE);	

open_frameless(font_pop, x - 1, y - 1);
wind_update(BEG_MCTRL);

do
	{
	do
		{	
		check_mouse();

		obj = objc_find(font_pop, ROOT, MAX_DEPTH, evnt.x, evnt.y);
		if (last_obj && last_obj != obj)
			{
			change_object(font_pop, last_obj,	font_pop[last_obj].ob_state & ~SELECTED);
			last_obj = 0;
			}
		
		if (obj > -1 && (font_pop[obj].ob_flags & SELECTABLE) && 
				!(font_pop[obj].ob_state & DISABLED) &&
				font_pop[obj].ob_spec.tedinfo->te_ptext[0] && last_obj != obj)
			{
			change_object(font_pop, obj,	font_pop[obj].ob_state | SELECTED);
			last_obj = obj;
			}
		}while(!evnt.bstate);
	
	switch(obj)
		{
		case FONT_UP:
			if (page > 0)
				{
				page--;
				delay(100);
				}
			break;
			
		case FONT_DWN:
			if (page < num_fonts - 6)
				{
				page++;
				delay(100);
				}
			break;
		
		case FONT_SLD:
			{
			int ox, oy;
			
			objc_offset(font_pop, FONT_DRG, &ox, &oy);
			if (evnt.y < oy)
				{
				page -= 6;
				if (page < 0) page = 0;
				}
			else
				{
				page += 6;
				if (page > num_fonts - 6)
					page = num_fonts - 6;
				}
			delay(100);
			break;
			}
			
		case FONT_DRG:
			{
			int pos = graf_slidebox(font_pop, FONT_SLD, FONT_DRG, 1);
			page = (int)(pos * (long)(num_fonts - 6) / 1000);
			break;
			}
		}
		
	for (i = 0; i < 6 && i < num_fonts; i++)	
		{
		strncpy(fnames[i], font_names[font_sorts[page + i]], 32);
		fnames[i][32] = 0;
		redraw_object(font_pop, ftext[i]);
		}
	set_slider_objects(font_pop, FONT_SLD, FONT_DRG, num_fonts, page, 6, TRUE);	

	}while(obj == FONT_UP || obj == FONT_DWN ||
				 obj == FONT_SLD || obj == FONT_DRG);

if (last_obj)
	change_object(font_pop, last_obj,	font_pop[last_obj].ob_state & ~SELECTED);

wait_mouse_release();
close_dialog(font_pop, TRUE);
	
wind_update(END_MCTRL);
	
if (obj > -1)
	{
	for (i = 0; i < 6; i++)
		if (obj == ftext[i])
			break;
			
	if (i < 6)
		{
		*zw = font_ids[font_sorts[page + i]];
		strncpy(fname, font_names[font_sorts[page + i]], 32);
		fname[32] = 0;
		}
	}
free(font_sorts);
}

