/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

static int scan_active = FALSE, header_scan = FALSE;
static int base_cod;
static int mime;
static int multipart, priority_flag, ansi_coding;
static char boundary[80];
static PART_INFO *parts;
static int part_cnt;
static char *mailbody;
static long body_len;
static int sbound;
static char *hdr_flds[] = {"MIME-VERSION:", "Content-type:", "Date:",
													 "From:", "Organization:", "To:", "Subject:", "Cc:",
													 "Content-transfer-encoding:", "Reply-to:",
													 "X-Confirm-Reading-To:", "X-pmrqc:", "Priority:"};
static int   hdr_lens[] = {13, 13, 5, 5, 13, 3, 8, 3, 26, 9, 21, 8, 9};
static char in_buf[8192];
static int in_buf_pos, in_buf_len;
static long last_seek;
static char inf[50];

char **wrap_text(char *s, int *l);

/* -----------------
   | Get parameter |
   ----------------- */
char *parameter(char *l)
{
while(*l != ':' && *l)	l++;
if (*l) l++;
while(*l == ' ' || *l == '\t')	l++;
return l;
}

/* --------------------
   | Seek to position |
   -------------------- */
void go_position(int in, long pos)
{
DEBUG();
last_seek = Fseek(pos, in, 0);
in_buf_len = in_buf_pos = 0;
}

/* --------------------------------
   | Return current file position |
   -------------------------------- */
long get_current_position(void)
{
return last_seek + in_buf_pos;
}

/* ----------------------
   | Clear input buffer |
   ---------------------- */
void clear_inputbuffer(void)
{
in_buf_len = in_buf_pos = 0;
}
   
/* ----------------------
   | Get line from mail |
   ---------------------- */
int get_line(int in, char *line, int max)
{
int i;
char c;

if (!in_buf_len)
	{
	in_buf_len = (int)Fread(in, 8192, in_buf);
	in_buf_pos = 0;
	}

if (in_buf_len <= 0)
	{
	line[0] = 0;
	return FALSE;
	}
	
i = 0;
while(i < max)
	{
	if (in_buf_pos >= in_buf_len)
		{
		last_seek += in_buf_len;
		in_buf_len = (int)Fread(in, 8192, in_buf);
		in_buf_pos = 0;
		if (in_buf_len <= 0)
			break;
		}
	
	c = in_buf[in_buf_pos++];
	
	if (c == '\r' || c == '\n')
		{
		if (in_buf_pos >= in_buf_len)
			{
			last_seek += in_buf_len;
			in_buf_len = (int)Fread(in, 8192, in_buf);
			in_buf_pos = 0;
			if (in_buf_len <= 0)
				break;
			}

		if ((in_buf[in_buf_pos] == '\r' || in_buf[in_buf_pos] == '\n') && in_buf[in_buf_pos] != c)
			in_buf_pos++;
			
		break;
		}
		
	line[i++] = c;	
	}
line[i] = 0;

if (i && line[i - 1] == '\\' && header_scan == TRUE)
	get_line(in, line + i - 1, max - i);

return TRUE;
}

/* ----------------------------------------------
   | Add the given line into the message header |
   ---------------------------------------------- */
void add_line_to_header(char *line, int hdr)
{
int l, hlen;

DEBUG();
if (hdr < 13 && !full_header)
	{
	if (line[0] == '\t' || line[0] == ' ')
		{
		memmove(line + hdr_lens[hdr] + 1, line + 1, strlen(line + 1) + 1);
		memset(line, ' ', hdr_lens[hdr] + 1);
		}
	else
		if (!line[hdr_lens[hdr]] || !line[hdr_lens[hdr] + 1])
			return;
			
	if (hdr_lens[hdr] < 13)
		hlen = hdr_lens[hdr];
	else
		hlen = 13;
	}
else
	hlen = 0;
			
l = (int)strlen(line);

if (!body_len)
	mailbody = (char *)malloc(l + 3 + 13 - hlen);
else
	mailbody = (char *)realloc(mailbody, body_len + l + 3 + 13 - hlen);
	
mailbody[body_len - 1] = 1;

if (hlen < 13 && hdr < 13 && !full_header)
	{
	memset(mailbody + body_len, ' ', 13 - hlen);
	body_len += 13 - hlen;
	}
	
strcpy(mailbody + body_len, line);
body_len += l;

mailbody[body_len++] = '\r';
mailbody[body_len++] = '\r';
}

/* --------------------
   | Scan mail header |
   -------------------- */
void scan_header(int in, long endpos)
{
char *line, *s, *s2;
int l, hdr;

DEBUG();
confirm_read = FALSE;
confirm_addr[0] = 0;

header_scan = TRUE;
base_cod = -1;
mime = FALSE;
priority_flag = 0;
multipart = FALSE;
ansi_coding = FALSE;
boundary[0] = 0;
sbound = 0;
line = (char *)malloc(1001);

line[0] = ' ';
while(line[0] && get_current_position() < endpos)
	{
	if (!get_line(in, line, 1000))
		break;

	if (line[0] != '\t' && line[0] != ' ')
		for (hdr = 0; hdr < 13; hdr++)
			if (!strnicmp(line, hdr_flds[hdr], hdr_lens[hdr]))
				break;

	if (!full_header && (hdr == 0 || hdr == 1 || hdr == 2 || hdr == 7
			|| hdr == 8 || hdr == 9 || hdr == 10 || hdr == 11))
		do
			{
			if (in_buf_pos >= in_buf_len)
				{
				last_seek += in_buf_len;
				in_buf_len = (int)Fread(in, 8192, in_buf);
				in_buf_pos = 0;
				}
				
			if (in_buf_len > 0 && (in_buf[in_buf_pos] == ' ' || in_buf[in_buf_pos] == '\t'))
				{
				l = (int)strlen(line);
				if (!get_line(in, line + l, 1000 - l))
					break;
				}
			else
				break;
			}while(1);
	
	decode_header(line);
			
	if (full_header && line[0]) add_line_to_header(line, hdr);
	switch(hdr)
		{
		case 0:
			if (!strnicmp(parameter(line), "1.0", 3))
				mime = TRUE;
			break;
			
		case 1:
			if (!strnicmp(parameter(line), "multipart", 9))
				{
				multipart = TRUE;
				s = stristr(line, "boundary=");
				if (s)
					{
					s += 9;
					if (*s == '"')
						{
						s2 = strchr(++s, '"');
						if (s2)
							{
							l = (int)(s2 - s);
							strncpy(boundary, s, l);
							boundary[l] = 0;
							sbound = l;
							}
						}
					else
						{
						s2 = strchr(s, ' ');
						if (!s2) s2 = strchr(s, 0);
						if (s2)
							{
							l = (int)(s2 - s);
							strncpy(boundary, s, l);
							boundary[l] = 0;
							sbound = l;
							}
						}
					}
				}
			else
				{
				s = stristr(line, "charset=");
				if (s)
					{
					s += 8;
					if (*s == '"' || *s == '\'') s++;
					if (!strnicmp(s, "iso-8", 5) || !strnicmp(s, "windows-", 8))
						ansi_coding = TRUE;
					}
				}
			break;
			
		case 2:										/* Date:					*/
		case 3:										/* From:					*/
		case 4:										/* Organization: 	*/
		case 5:										/* To:						*/
		case 6:										/* Subject:				*/
		case 7:										/* CC: 						*/
		case 9:										/* Reply-to: 			*/
			if (!full_header) add_line_to_header(line, hdr);
			break;

		case 8:									/* Content-transfer-encoding: */
			s = parameter(line);
			for (l = 0; l < 5; l++)
				if (!strnicmp(s, enc_tab[l], strlen(enc_tab[l])))
					base_cod = l;
			break;
			
		case 10:											/* Confirm read address */
			l = hdr_lens[hdr];
			while (isspace(line[l]))
				l++;
				
			if (!line[l])
				break;

			strcpy(confirm_addr, line + l);
			break;
			
		case 11:											/* Confirm read */
			l = hdr_lens[hdr];
			while (isspace(line[l]))
				l++;

			if (line[l] == '1')
				confirm_read = TRUE;
			break;
			
		case 12:											/* Priority */
			l = 9;
			while (isspace(line[l]))
				l++;
			if (!strnicmp(line + l, "urgent", 6))
				priority_flag = 1;
			if (!strnicmp(line + l, "low", 3))
				priority_flag = 2;
			break;
		}
	}

header_scan = FALSE;
	
free(line);
}

/* --------------------------
   | Search Multipart parts |
   -------------------------- */
void look_parts(int in, char *line, long len)
{
char*s;
int tflg = FALSE, empty = FALSE;
int last = 0, new;
long pos;

DEBUG();
pos = Fseek(0, in, SEEK_CUR);
Fseek(pos, in, SEEK_SET);

do
	{
	pos = get_current_position();
		
	new = (int)(pos * 100 / len);
	if (new >= last + 10)
		{
		sprintf(inf, analysing, new);
		change_info(inf);
		last = new;
		}
		
	if (!get_line(in, line, 1000))
		break;
	else
		{
		if (!strncmp(line, "--", 2) && !strncmp(line + 2, boundary, sbound) && sbound)
			{
			if (!part_cnt)
				parts = (PART_INFO *)malloc(sizeof(PART_INFO));
			else
				{
				parts[part_cnt - 1].len = pos - (parts[part_cnt - 1].offset & 0x7FFFFFFFL);
				parts = (PART_INFO *)realloc(parts, (part_cnt + 1) * sizeof(PART_INFO));
				}

			parts[part_cnt].offset = get_current_position();
			parts[part_cnt].name[0] = 0;
			parts[part_cnt].desc[0] = 0;
			strcpy(parts[part_cnt].type, "text");

			part_cnt++;
			tflg = TRUE;
			empty = TRUE;
			}
		else
			if (line[0])
				{
				if (tflg)
					{
					empty = FALSE;
					
					if (scan_active && part_cnt > 1)
						break;

					do
						{
						if (in_buf_pos >= in_buf_len)
							{
							last_seek += in_buf_len;
							in_buf_len = (int)Fread(in, 8192, in_buf);
							in_buf_pos = 0;
							}
							
						if (in_buf_len > 0 && (in_buf[in_buf_pos] == ' ' || in_buf[in_buf_pos] == '\t'))
							{
							int l = (int)strlen(line);
							if (!get_line(in, line + l, 1000 - l))
								break;
							}
						else
							break;
						}while(1);
								
					if (!strnicmp(line, "Content-type:", 13))
						{
						s = parameter(line);
						if (!strnicmp(parameter(line), "multipart", 9))
							{
							char prev_boundary[80], *s2;
							long prev_location;
							int l;
							
							strcpy(prev_boundary, boundary);
							part_cnt--;
							s = stristr(line, "boundary=");
							if (s)
								{
								s += 9;
								if (*s == '"')
									{
									s2 = strchr(++s, '"');
									if (s2)
										{
										l = (int)(s2 - s);
										strncpy(boundary, s, l);
										boundary[l] = 0;
										sbound = l;
										}
									}
								else
									{
									s2 = strchr(s, ' ');
									if (!s2) s2 = strchr(s, 0);
									if (s2)
										{
										l = (int)(s2 - s);
										strncpy(boundary, s, l);
										boundary[l] = 0;
										sbound = l;
										}
									}
								}
							prev_location = get_current_position();
							look_parts(in, line, len);
							go_position(in, prev_location);
							strcpy(boundary, prev_boundary);
							sbound = (int)strlen(boundary);
							}
						else
							{
							if (s)
								{
								strncpy(parts[part_cnt - 1].type, s, 20);
								parts[part_cnt - 1].type[20] = 0;
								s = strchr(parts[part_cnt - 1].type, '/');
								if (s) *s = 0;
								
								s = parameter(line);
								s = strchr(s, '/');
								if (!parts[part_cnt - 1].desc[0])
									{
									strncpy(parts[part_cnt - 1].desc, s + 1, 60);
									parts[part_cnt - 1].desc[60] = 0;
									
									s = strchr(parts[part_cnt - 1].desc, ';');
									if (s) *s = 0;
									}
								}
						
							s = stristr(line, "name=");
							if (s)
								{
								s += 5;
								
								if (*s == '"')
									strncpy(parts[part_cnt - 1].name, s + 1, 60);
								else
									strncpy(parts[part_cnt - 1].name, s, 60);
									
								parts[part_cnt - 1].name[60] = 0;
								if (*s == '"')
									{
									s = strchr(parts[part_cnt - 1].name, '"');
									if (s) *s = 0;
									}
								}
								
							s = stristr(line, "charset=");
							if (s)
								{
								s += 8;
								if (*s == '"' || *s == '\'') s++;
								if (!strnicmp(s, "iso-8", 5) || !strnicmp(s, "windows-", 8))
									parts[part_cnt - 1].offset |= 0x80000000L;
								}
							}
						}
					else
						if (!strnicmp(line, "Content-description:", 20))
							{
							strncpy(parts[part_cnt - 1].desc, parameter(line), 60);
							parts[part_cnt - 1].desc[60] = 0;
							}
					}
				}
			else
				tflg = FALSE;
		}
		
	}while(1);

if (part_cnt)
	if (tflg || empty)
		part_cnt--;
	else
		parts[part_cnt - 1].len = pos - (parts[part_cnt - 1].offset & 0x7FFFFFFFL);
}

/* --------------------------
   | Search Multipart parts |
   -------------------------- */
void find_parts(int in, long len)
{
char *line, *s;
int tflg = FALSE, empty = FALSE;
int last = 0, new;
long pos;

DEBUG();
pos = Fseek(0, in, SEEK_CUR);
Fseek(pos, in, SEEK_SET);

part_cnt = 0;
line = (char *)malloc(1001);
do
	{
	pos = get_current_position();
	
	new = (int)(pos * 100 / len);
	if (new >= last + 10)
		{
		sprintf(inf, analysing, new);
		change_info(inf);
		last = new;
		}
		
	if (!get_line(in, line, 1000))
		break;
	else
		{
		if (!strncmp(line, "--", 2) && !strncmp(line + 2, boundary, sbound) && sbound)
			{
			if (!part_cnt)
				parts = (PART_INFO *)malloc(sizeof(PART_INFO));
			else
				{
				parts[part_cnt - 1].len = pos - (parts[part_cnt - 1].offset & 0x7FFFFFFFL);
				parts = (PART_INFO *)realloc(parts, (part_cnt + 1) * sizeof(PART_INFO));
				}

			parts[part_cnt].offset = get_current_position();
			parts[part_cnt].name[0] = 0;
			parts[part_cnt].desc[0] = 0;
			strcpy(parts[part_cnt].type, "text");

			part_cnt++;
			tflg = TRUE;
			empty = TRUE;
			}
		else
			if (line[0])
				{
				if (tflg)
					{
					empty = FALSE;
					
					if (scan_active && part_cnt > 1)
						break;
						
					do
						{
						if (in_buf_pos >= in_buf_len)
							{
							last_seek += in_buf_len;
							in_buf_len = (int)Fread(in, 8192, in_buf);
							in_buf_pos = 0;
							}
							
						if (in_buf_len > 0 && (in_buf[in_buf_pos] == ' ' || in_buf[in_buf_pos] == '\t'))
							{
							int l = (int)strlen(line);
							if (!get_line(in, line + l, 1000 - l))
								break;
							}
						else
							break;
						}while(1);
						
					if (!strnicmp(line, "Content-type:", 13))
						{
						s = parameter(line);
						if (!strnicmp(parameter(line), "multipart", 9))
							{
							char prev_boundary[80], *s2;
							long prev_location;
							int l;
							
							strcpy(prev_boundary, boundary);
							part_cnt--;
							s = stristr(line, "boundary=");
							if (s)
								{
								s += 9;
								if (*s == '"')
									{
									s2 = strchr(++s, '"');
									if (s2)
										{
										l = (int)(s2 - s);
										strncpy(boundary, s, l);
										boundary[l] = 0;
										sbound = l;
										}
									}
								else
									{
									s2 = strchr(s, ' ');
									if (!s2) s2 = strchr(s, 0);
									if (s2)
										{
										l = (int)(s2 - s);
										strncpy(boundary, s, l);
										boundary[l] = 0;
										sbound = l;
										}
									}
								}
							prev_location = get_current_position();
							look_parts(in, line, len);
							go_position(in, prev_location);
							strcpy(boundary, prev_boundary);
							sbound = (int)strlen(boundary);
							}
						else
							{
							if (s)
								{
								strncpy(parts[part_cnt - 1].type, s, 20);
								parts[part_cnt - 1].type[20] = 0;
								s = strchr(parts[part_cnt - 1].type, '/');
								if (s) *s = 0;
								
								s = parameter(line);
								s = strchr(s, '/');
								if (!parts[part_cnt - 1].desc[0])
									{
									strncpy(parts[part_cnt - 1].desc, s + 1, 60);
									parts[part_cnt - 1].desc[60] = 0;
									
									s = strchr(parts[part_cnt - 1].desc, ';');
									if (s) *s = 0;
									}
								}
						
							s = stristr(line, "name=");
							if (s)
								{
								s += 5;
								
								if (*s == '"')
									strncpy(parts[part_cnt - 1].name, s + 1, 60);
								else
									strncpy(parts[part_cnt - 1].name, s, 60);
									
								parts[part_cnt - 1].name[60] = 0;
								if (*s == '"')
									{
									s = strchr(parts[part_cnt - 1].name, '"');
									if (s) *s = 0;
									}
								}
								
							s = stristr(line, "charset=");
							if (s)
								{
								s += 8;
								if (*s == '"' || *s == '\'') s++;
								if (!strnicmp(s, "iso-8", 5) || !strnicmp(s, "windows-", 8))
									parts[part_cnt - 1].offset |= 0x80000000L;
								}
							}
						}
					else
						if (!strnicmp(line, "Content-description:", 20))
							{
							strncpy(parts[part_cnt - 1].desc, parameter(line), 60);
							parts[part_cnt - 1].desc[60] = 0;
							}
					}
				}
			else
				tflg = FALSE;
		}
		
	}while(1);

if (part_cnt)
	if (tflg || empty)
		part_cnt--;
	else
		parts[part_cnt - 1].len = pos - (parts[part_cnt - 1].offset & 0x7FFFFFFFL);
		
free(line);
}

/* --------------------------
   | Search Multipart parts |
   -------------------------- */
void check_uuencode(int in, long len)
{
char *line;
int last = 0, new;
long pos, table_pos = 0;

DEBUG();
pos = Fseek(0, in, SEEK_CUR);
Fseek(pos, in, SEEK_SET);

line = (char *)malloc(1001);
do
	{
	pos = get_current_position();

	new = (int)(pos * 100 / len);
	if (new >= last + 10)
		{
		sprintf(inf, analysing, new);
		change_info(inf);
		last = new;
		}
		
	if (!get_line(in, line, 1000))
		break;
	else
		{
		if (!strncmp(line, "table", 5)) table_pos = pos;
		if (!strncmp(line, "begin", 5) && isspace(line[5]) &&
				(!strncmp(line + 6, "644", 3) || !strncmp(line + 6, "600", 3)))
			{
			char *p;
			int i;
			
			if (!table_pos) table_pos = pos;
			
			parts = (PART_INFO *)realloc(parts, (part_cnt + 1) * sizeof(PART_INFO));

			p = line + 9;
			while (*p == ' ' || *p == '\t')
				p++;
			
			i = 0;
			while(!isspace(*p))
				parts[part_cnt].name[i++] = *p++;
			parts[part_cnt].name[i] = 0;
			
			parts[part_cnt].desc[0] = 0;
			strcpy(parts[part_cnt].type, "uuencoded");

			parts[part_cnt].offset = table_pos;
			parts[part_cnt].len = parts[part_cnt - 1].len -
						(table_pos - (parts[part_cnt - 1].offset & 0x7FFFFFFFL));
			parts[part_cnt - 1].len = table_pos - (parts[part_cnt - 1].offset & 0x7FFFFFFFL);

			part_cnt++;
			table_pos = 0;
			
			if (scan_active && part_cnt > 1)
				break;
/*						
			break;*/
			}
		}
		
	}while(1);

if (part_cnt > 1 && !scan_active)
	parts[part_cnt - 1].len = pos - (parts[part_cnt - 1].offset & 0x7FFFFFFFL);

free(line);
}

/* ---------------------------------------------
   | Create an empty mail body for attachments |
   --------------------------------------------- */
void create_empty_mail(void)
{
DEBUG();
mailbody = (char *)malloc(1);
mailbody[0] = '\r';
body_len = 1;
}

/* ----------------------
   | Format a mail body |
   ---------------------- */
void get_mail_body(int in, int n, int *width, int *lines, int mark_reply)
{
char *line, *p;
int i, cod, w, wrapped, is_ansi, is_replyline;
long l, l_spc, strt, max_end;

DEBUG();
if (parts[n].offset & 0x80000000L)
	is_ansi = TRUE;
else
	is_ansi = FALSE;
	
go_position(in, parts[n].offset & 0x7FFFFFFFL);

strt = body_len;
max_end = (parts[n].offset & 0x7FFFFFFFL) + parts[n].len;

line = (char *)malloc(1001);
if (multipart)
	{
	cod = -1;
	do
		{
		if (!get_line(in, line, 1000))
			break;
		
		if (!line[0]) break;
		
		if (!strnicmp(line, "Content-transfer-encoding:", 26))
			{
			p = parameter(line);
			for (i = 0; i < 5; i++)
				if (!strnicmp(p, enc_tab[i], strlen(enc_tab[i])))
					cod = i;
			}
		
		}while(1);
	}
else
	cod = base_cod;

do
	{
	l = get_current_position();
	
	if (!get_line(in, line, 1000))
		break;

	}while(!line[0]);
	
go_position(in, l);
switch(cod)
	{
	case 0:		MemDecode7Bit(&mailbody, &body_len, in, line, boundary, max_end);		break;
	case 1:		MemDecodeQuoted(&mailbody, &body_len, in, line, boundary, max_end);	break;
	case 2:		MemDecodeBase64(&mailbody, &body_len, in, line, boundary, max_end);	break;
	case 3:		MemDecode8Bit(&mailbody, &body_len, in, line, boundary, max_end);		break;
	case 4:		MemDecodeBinary(&mailbody, &body_len, in, line, boundary, max_end);	break;
	default:	MemDecode7Bit(&mailbody, &body_len, in, line, boundary, max_end);		break;
	}							
	
free(line);

if (is_ansi)
	{
	l = 0L;
	while (l < body_len)
		{
		mailbody[l] = from_ansi[((unsigned char *)mailbody)[l]];
		l++;
		}
	}
	
*width = 0;
*lines = 0;
is_replyline = FALSE;
wrapped = FALSE;
w = 0;
l = 0L;
l_spc = -1;
while(l < body_len)
	{
	if (mailbody[l] == ' ' || mailbody[l] == ',' || mailbody[l] == ';' ||
	    mailbody[l] == '?' || mailbody[l] == '!')
		l_spc = l;
		
	if (mailbody[l] == '\t')
		{
		int zw;
		
		zw = (mail_options.viewtabs - 1) - (w % (mail_options.viewtabs - 1));
		if (zw)
			{
			mailbody = (char *)realloc(mailbody, body_len + zw);
			memmove(mailbody + l + zw, mailbody + l, body_len - l);
			memset(mailbody + l, ' ', zw + 1);
			body_len += zw;
			}
		else
			mailbody[l] = ' ';
		}

	if (mailbody[l] == '>' && !w && mark_reply)
		{
		mailbody = (char *)realloc(mailbody, body_len + 1);
		memmove(mailbody + l + 1, mailbody + l, body_len - l);
		mailbody[l] = 3;
		is_replyline = TRUE;
		body_len++;
		l++;
		}
						
	if (mailbody[l++] == '\r')
		{
		if (!wrapped || (l < body_len && (mailbody[l] == '\r' || mailbody[l] == '>')))
			{
			(*lines)++;
			if (w > *width) *width = w;
			w = 0;
			l_spc = -1L;

			if (mailbody[l] == '\n' &&  l < body_len)
				{
				if (body_len - l - 1 > 0)
					memmove(mailbody + l, mailbody + l + 1, body_len - l - 1);
				body_len--;
				}
			}
		else
			{
			l_spc = l - 1;
			mailbody[l_spc] = ' ';
			}
		wrapped = FALSE;
		is_replyline = FALSE;
		}
	else
		w++;
		
	if (w > mail_options.viewlength && mail_options.viewwrap && !is_replyline && l_spc > -1L && l > strt)
		if (l - l_spc < mail_options.viewlength)
			{
			if (w - (int)(l - l_spc) > *width) *width = w - (int)(l - l_spc);
	
			if (mailbody[l_spc] != ' ')
				{
				mailbody = (char *)realloc(mailbody, body_len + 1);
				memmove(mailbody + l_spc + 2, mailbody + l_spc + 1, body_len - l_spc - 1);
				body_len++;
				w++;
				l_spc++;
				}
				
			mailbody[l_spc] = '\r';
			(*lines)++;
			w = (int)(l - l_spc);
			wrapped = TRUE;
			}
	}
}

/* --------------------
   | Attach mail icon |
   -------------------- */
void attach_mail_icon(int n, int *lns)
{
int sz, l, i;
long zw;
char *s, **wrapped;

DEBUG();
sz = 8 + 8 + (int)strlen(parts[n].name) + 
		 (int)strlen(parts[n].desc) + (int)strlen(parts[n].type);
		 
if (!body_len)
	mailbody = (char *)malloc(sz);
else
	mailbody = (char *)realloc(mailbody, body_len + sz);

mailbody[body_len++] = 0x2;
memcpy(mailbody + body_len, &n, 2);
body_len += 2;

zw = parts[n].offset & 0x7FFFFFFFL;
memcpy(mailbody + body_len, &zw, 4);
body_len += 4;
memcpy(mailbody + body_len, &parts[n].len, 4);
body_len += 4;

strcpy(mailbody + body_len, parts[n].name);	
body_len += strlen(parts[n].name) + 1;
s = parts[n].name;

strcpy(mailbody + body_len, parts[n].desc);	
body_len += strlen(parts[n].desc) + 1;
if (!*s)
	s = parts[n].desc;
	
strcpy(mailbody + body_len, parts[n].type);	
body_len += strlen(parts[n].type) + 1;
if (!*s)
	s = parts[n].type;
mailbody[body_len++] = '\r';

wrapped = wrap_text(s, &l);
i = 0;
while(wrapped[i])
	i++;

*lns = i;
}

/* --------------------------
   | Generate pointer array |  
   -------------------------- */
void gen_ptr(long **ptr, int lines, int att)
{
char *s, *end;
long *p;

DEBUG();
*ptr = (long *)malloc((lines + att) * sizeof(long) + sizeof(long) * 2);
p = *ptr;
s = mailbody;

*p++ = 0L;
end = s + body_len;
do
	{
	if (*s == 0x2)
		{
		s += 11;							/* Id + Part-Nr + Offset + Length */
		s += strlen(s) + 1;		/* Name 					*/		
		s += strlen(s) + 1;		/* Description 		*/		
		s += strlen(s) + 1;		/* Type					 	*/		
		}

	if (s < end && *s == '\r')
		{
		*p++ = s + 1 - mailbody;
		lines--;
		}

	s++;		
	}while(s < end);
	
*p++ = body_len;
while (lines--)
	*p++ = body_len;
while (att--)
	*p++ = body_len;
}

/* ---------------------------------
   | Create formatted mail message |
   --------------------------------- */
int get_part(char *title, char *box, 
						 MAIL_INFO *entry, char **mem, long *blen,
						 long **ptr, int *width, int *lines, int *ansi,
						 int head_flg, int part_no)
{
int f;
int i;
FILE_MAIL_MEM header;

DEBUG();
in_buf_len = in_buf_pos = 0;
f = open_file(entry->file, box);
if (f)
	{
	Fread(f, sizeof(header), &header);
	if (header.magic != 0x4104)
		{
		memmove(&header.confirm_read, &header.magic, sizeof(header) - (6+21+21));
		Fseek(-(21+21+6L), f, SEEK_CUR);
		header.replyto_len = 0;
		header.identity[0] = 0;
		header.signature[0] = 0;
		}
	else
		if (header.version == SVERSION_ID17)
			{
			Fseek(-(21+21L), f, SEEK_CUR);
			header.identity[0] = 0;
			header.signature[0] = 0;
			}

	if (header.part_count == -1)
		{	
		sprintf(inf, analysing, 0);
		start_info(title, inf);	
		
		create_empty_mail();
		go_position(f, header.body_pos);
		scan_header(f, header.body_pos + header.body_len);
		
		if (!head_flg)
			{
			free(mailbody);
			body_len = 0;
			}
			
		if (multipart)
			find_parts(f, header.body_pos + header.body_len);
		else
			{
			parts = (PART_INFO *)malloc(sizeof(PART_INFO));
			part_cnt = 1;
			
			parts->offset = get_current_position();
			parts->len = header.body_pos + header.body_len - parts->offset;
			if (ansi_coding)
				parts->offset |= 0x80000000L;
			strcpy(parts->name, "No name");
			strcpy(parts->desc, "Text");
			strcpy(parts->type, "text");

			go_position(f, parts->offset);
			check_uuencode(f, header.body_pos + header.body_len);
			}

		i = -1;
		if (part_no == -1)
			{
			for (part_no = 0; part_no < part_cnt; part_no++)
				if (!strnicmp(parts[part_no].type, "text", 4))
					break;
			i = part_no;
			}
			
		if (part_no < part_cnt)
			{
			long sav;
			
			if (*ansi != -1)
				{
				sav = parts[part_no].offset;
				if (*ansi)
					parts[part_no].offset |= 0x80000000L;
				else
					parts[part_no].offset &= 0x7FFFFFFFL;
				}
				
			get_mail_body(f, part_no, width, lines, head_flg);
			
			if (*ansi == -1)
				*ansi = (parts[part_no].offset & 0x80000000L) != 0L;
			else
				parts[part_no].offset = sav;
			}
			
		gen_ptr(ptr, *lines, 0);
		*mem = mailbody;
		*blen = body_len;
				
		Fclose(f);

		if (i != -1)
			{
			f = rwopen_file(entry->file, box);
			if (f)
				{
				header.main_idx = i;
				header.part_count = part_cnt;
				header.partinfo_pos = Fseek(0L, f, SEEK_END);
				
				Fwrite(f, sizeof(PART_INFO) * part_cnt, parts);
						
				Fseek(0L, f, SEEK_SET);
				if (header.magic != 0x4104)
					Fwrite(f, sizeof(header) - (21+21+6), &header.confirm_read);
				else
					if (header.version == SVERSION_ID17)
						Fwrite(f, sizeof(header) - (21+21), &header);
					else
						Fwrite(f, sizeof(header), &header);
				Fclose(f);
				}
			}
					
		free(parts);
		end_info();
		}
	else
		{
		create_empty_mail();
		go_position(f, header.body_pos);
		scan_header(f, header.body_pos + header.body_len);

		if (!head_flg)
			{
			free(mailbody);
			body_len = 0;
			}
			
		parts = (PART_INFO *)malloc(header.part_count * sizeof(PART_INFO));
		Fseek(header.partinfo_pos, f, SEEK_SET);
		Fread(f, sizeof(PART_INFO) * header.part_count, parts);
		
		if (part_no == -1)
			part_no = header.main_idx;
				
		if (part_no < header.part_count)
			{
			long sav;
			
			if (*ansi != -1)
				{
				sav = parts[part_no].offset;
				if (*ansi)
					parts[part_no].offset |= 0x80000000L;
				else
					parts[part_no].offset &= 0x7FFFFFFFL;
				}

			get_mail_body(f, part_no, width, lines, head_flg);

			if (*ansi == -1)
				*ansi = (parts[part_no].offset & 0x80000000L) != 0L;
			else
				parts[part_no].offset = sav;
			}
			
		Fclose(f);
			
		gen_ptr(ptr, *lines, 0);
		*mem = mailbody;
		*blen = body_len;

		free(parts);
		}
			
	return TRUE;
	}
else
	return FALSE;
}
 
/* ---------------------------------
   | Create formatted mail message |
   --------------------------------- */
int get_body(char *title, char *box, 
						 MAIL_INFO *entry, char **mem, long *blen,
						 long **ptr, int *width, int *lines, int head_flg)
{
int a = -1;

DEBUG();
return get_part(title, box, entry, mem, blen, ptr, width, lines, &a,
								head_flg, -1);
}

/* ---------------------------------
   | Create formatted mail message |
   --------------------------------- */
int format_mail(char *box, 
								MAIL_INFO *entry, char **mem, long *blen, long **ptr,
								int *width, int *lines, char *bdry, int *ansi)
{
int f;
int i, j, max_att_lines = 0, att_height;
FILE_MAIL_MEM header;
int cx, cy, cw, ch;

vst_font(vdi_handle, 1);
set_default_text();
vst_height(vdi_handle, 4, &cx, &cy, &cw, &ch);

DEBUG();
in_buf_len = in_buf_pos = 0;
f = open_file(entry->file, box);
if (f)
	{
	Fread(f, sizeof(header), &header);
	if (header.magic != 0x4104)
		{
		memmove(&header.confirm_read, &header.magic, sizeof(header) - (21+21+6));
		Fseek(-(21+21+6L), f, SEEK_CUR);
		header.replyto_len = 0;
		header.identity[0] = 0;
		header.signature[0] = 0;
		}
	else
		if (header.version == SVERSION_ID17)
			{
			Fseek(-(21+21L), f, SEEK_CUR);
			header.identity[0] = 0;
			header.signature[0] = 0;
			}
	
	if (header.part_count == -1)
		{	
		sprintf(inf, analysing, 0);
		start_info(viewmail_text, inf);	
		
		create_empty_mail();
		go_position(f, header.body_pos);
		scan_header(f, header.body_pos + header.body_len);

		if (confirm_read)
			{
			long store_pos = get_current_position();
			
			gen_confirmation(mailbody, body_len, confirm_addr, entry->identity);
			go_position(f, store_pos);
			}
			
		strcpy(bdry, boundary);
		if (multipart)
			find_parts(f, header.body_pos + header.body_len);
		else
			{
			parts = (PART_INFO *)malloc(sizeof(PART_INFO));
			part_cnt = 1;
			
			parts->offset = get_current_position();
			parts->len = header.body_pos + header.body_len - parts->offset;
			if (ansi_coding)
				parts->offset |= 0x80000000L;
			strcpy(parts->name, "No name");
			strcpy(parts->desc, "Text");
			strcpy(parts->type, "text");
			
			go_position(f, parts->offset);
			check_uuencode(f, header.body_pos + header.body_len);
			}
		
		for (i = 0; i < part_cnt; i++)
			if (!strnicmp(parts[i].type, "text", 4))
				break;
				
DEBUG();
		if (i < part_cnt)
			{
			long sav;
			
			if (*ansi != -1)
				{
				sav = parts[i].offset;
				if (*ansi)
					parts[i].offset |= 0x80000000L;
				else
					parts[i].offset &= 0x7FFFFFFFL;
				}
			
			get_mail_body(f, i, width, lines, TRUE);
			
			if (*ansi == -1)
				*ansi = (parts[i].offset & 0x80000000L) != 0L;
			else
				parts[i].offset = sav;
			}
		Fclose(f);

DEBUG();
		if (part_cnt > 1)
			{
			entry->sel |= 8;
			mailbody = (char *)realloc(mailbody, body_len + strlen(attachm_text) + 10);

			strcpy(mailbody + body_len, "\r\1   ");
			body_len += 5;
			strcpy(mailbody + body_len, attachm_text);
			body_len += strlen(attachm_text);
			strcpy(mailbody + body_len, ":\r");
			body_len += 2;

			(*lines) += 2;
			}
			
		att_height = 0;
		for (j = 0; j < part_cnt; j++)
			if (j != i)
				{
				int l;
				attach_mail_icon(j, &l);
				
				if (l > max_att_lines)
					max_att_lines = l;

				if (((j + 1) % 4) == 0)
					{
					att_height += icon_height + (max_att_lines - 1) * ch;
					max_att_lines = 0;
					}
				}
		if ((part_cnt - 1) % 4)
			att_height += icon_height + (max_att_lines - 1) * ch;
		(*lines) += (att_height + tx_height - 1) / tx_height;

DEBUG();
		mailbody = (char *)realloc(mailbody, body_len + 2);
		strcpy(mailbody + body_len, "\r");
		(*lines)++;
		body_len++;

/*		if (part_cnt % 4)
			{
			j = part_cnt % 4;
			
			mailbody = (char *)realloc(mailbody, body_len + j);
			strncpy(mailbody + body_len, "\r\r\r", j);
			body_len += j;
			(*lines) += j;
			}*/
			
DEBUG();
		gen_ptr(ptr, *lines, part_cnt - 1);
		*mem = mailbody;
		*blen = body_len;

		f = rwopen_file(entry->file, box);
		if (f)
			{
			header.main_idx = i;
			header.part_count = part_cnt;
			header.partinfo_pos = Fseek(0L, f, SEEK_END);
			
			Fwrite(f, sizeof(PART_INFO) * part_cnt, parts);
					
			Fseek(0L, f, SEEK_SET);
			if (header.magic != 0x4104)
				Fwrite(f, sizeof(header) - (21+21+6), &header.confirm_read);
			else
				if (header.version == SVERSION_ID17)
					Fwrite(f, sizeof(header) - (21+21), &header);
				else
					Fwrite(f, sizeof(header), &header);
			Fclose(f);
			}

		free(parts);
		end_info();
		}
	else
		{
		create_empty_mail();
		go_position(f, header.body_pos);
		scan_header(f, header.body_pos + header.body_len);

		strcpy(bdry, boundary);
				
		parts = (PART_INFO *)malloc(header.part_count * sizeof(PART_INFO));
		Fseek(header.partinfo_pos, f, SEEK_SET);
		Fread(f, sizeof(PART_INFO) * header.part_count, parts);
			
		if (header.main_idx < header.part_count)
			{
			long sav;
			
			if (*ansi != -1)
				{
				sav = parts[header.main_idx].offset;
				if (*ansi)
					parts[header.main_idx].offset |= 0x80000000L;
				else
					parts[header.main_idx].offset &= 0x7FFFFFFFL;
				}

			get_mail_body(f, header.main_idx, width, lines, TRUE);

			if (*ansi == -1)
				*ansi = (parts[header.main_idx].offset & 0x80000000L) != 0L;
			else
				parts[header.main_idx].offset = sav;
			}

		Fclose(f);
			
DEBUG();
		if (header.part_count > 1)
			{
			entry->sel |= 8;

			mailbody = (char *)realloc(mailbody, body_len + strlen(attachm_text) + 8);
			strcpy(mailbody + body_len, "\r\1   ");
			body_len += 5;
			strcpy(mailbody + body_len, attachm_text);
			body_len += strlen(attachm_text);
			strcpy(mailbody + body_len, ":\r");
			body_len += 2;
			(*lines) += 2;
			}
			
		att_height = 0;
		for (j = 0; j < header.part_count; j++)
			if (j != header.main_idx)
				{
				int l;
				attach_mail_icon(j, &l);
				
				if (l > max_att_lines)
					max_att_lines = l;

				if (((j + 1) % 4) == 0)
					{
					att_height += icon_height + (max_att_lines - 1) * ch;
					max_att_lines = 0;
					}
				}
		if ((header.part_count - 1) % 4)
			att_height += icon_height + (max_att_lines - 1) * ch;

DEBUG();
		(*lines) += (att_height + tx_height - 1) / tx_height;
		mailbody = (char *)realloc(mailbody, body_len + 2);
		strcpy(mailbody + body_len, "\r");
		(*lines)++;
		body_len++;
	
/*		if (header.part_count % 4)
			{
			j = header.part_count % 4;
			
			mailbody = (char *)realloc(mailbody, body_len + j);
			strncpy(mailbody + body_len, "\r\r\r", j);
			body_len += j;
			(*lines) += j;
			}*/
			
DEBUG();
		gen_ptr(ptr, *lines, header.part_count - 1);
		*mem = mailbody;
		*blen = body_len;

		free(parts);
		}
	
	return TRUE;
	}
else
	return FALSE;
}

/* ---------------------------------
   | Create formatted mail message |
   --------------------------------- */
void quick_mailcheck(char *file, MAIL_MEM *buffer)
{
int f;
FILE_MAIL_MEM header;

DEBUG();
in_buf_len = in_buf_pos = 0;
f = rwopen_file(file, NULL);
if (f)
	{
	quick_check(f, buffer, &header);
	Fclose(f);
	}
}

/* -------------------
   | Check mail file |
   ------------------- */
void quick_check(int f, MAIL_MEM *buffer, FILE_MAIL_MEM *header)
{
int length[7], idx, err;
long i;
char *x, *memptr[7];

DEBUG();
Fread(f, sizeof(FILE_MAIL_MEM), header);
if (header->magic != 0x4104)
	{
	memmove(&header->confirm_read, &header->magic, sizeof(FILE_MAIL_MEM) - (21+21+6));
	Fseek(-(21+21+6L), f, SEEK_CUR);
	header->replyto_len = 0;
	header->identity[0] = 0;
	header->signature[0] = 0;
	}
else
	if (header->version == SVERSION_ID17)
		{
		Fseek(-(21+21L), f, SEEK_CUR);
		header->identity[0] = 0;
		header->signature[0] = 0;
		}
		
scan_active = TRUE;
		
create_empty_mail();
go_position(f, header->body_pos);
scan_header(f, header->body_pos + header->body_len);

if (multipart)
	find_parts(f, header->body_pos + header->body_len);
else
	{
	parts = (PART_INFO *)malloc(sizeof(PART_INFO));
	part_cnt = 1;
		
	parts->offset = get_current_position();
	parts->len = header->body_pos + header->body_len - parts->offset;
	if (ansi_coding)
		parts->offset |= 0x80000000L;
	strcpy(parts->name, "No name");
	strcpy(parts->desc, "Text");
	strcpy(parts->type, "text");
	
	go_position(f, parts->offset);
	check_uuencode(f, header->body_pos + header->body_len);
	}

for (idx = 0; idx < 7; idx++)
	{
	length[idx] = 0;
	memptr[idx] = NULL;
	}
	
i = 0;
while(i < body_len)
	{
	x = mailbody + i;
			
	while(*x == 1 || *x == 3 || *x == ' ' || *x == '\t')		x++;

	if (*x == '\r') break;
	
	if (!strnicmp(x, "Date:", 5))
		idx = 0;
	else if (!strnicmp(x, "From:", 5))
		idx = 1;
	else if (!strnicmp(x, "Reply-To:", 9))
		idx = 2;
	else if (!strnicmp(x, "Organization:", 13))
		idx = 3;
	else if (!strnicmp(x, "To:", 3))
		idx = 4;
	else if (!strnicmp(x, "CC:", 3))
		idx = 5;
	else if (!strnicmp(x, "Subject:", 8))
		idx = 6;
	else
		idx = -1;
			
	if (idx != -1)
		{
		while(*x != ':') x++;
		
		x++;	/* Skip colon */	

		while(*x == ' ' || *x == '\t')	x++;
		
		memptr[idx] = x;
		while (*x != '\r' || !strncmp(x + 1, "\1              ", 14))
			{
			if (*x == '\r')
				{
				memmove(x, x + 15, body_len - (x + 15 - mailbody));
				body_len -= 15;
				}
			x++;
			}
		
		length[idx] = x - memptr[idx];
		}
				
	while(i < body_len && mailbody[i] != '\r')
		i++;
	if (i < body_len)
		i++;
	}

header->date_len 					= length[0];
header->from_len 					= length[1];
header->replyto_len 			= length[2];
header->organization_len 	= length[3];
header->to_len 						= length[4];
header->cc_len 						= length[5];
header->subject_len 			= length[6];

header->header_inf = Fseek(0L, f, SEEK_END);

err = FALSE;
for (i = 0; i < 7; i++)
	if (length[i])
  	if (Fwrite(f, length[i], memptr[i]) < 0)
  		{
  		err = TRUE;
  		break;
			}

if (!err)
	{			
	Fseek(0L, f, SEEK_SET);
	if (header->magic != 0x4104)
		Fwrite(f, sizeof(FILE_MAIL_MEM) - (21+21+6), &header->confirm_read);
	else
		if (header->version == SVERSION_ID17)
			Fwrite(f, sizeof(FILE_MAIL_MEM) - (21+21), header);
		else
			Fwrite(f, sizeof(FILE_MAIL_MEM), header);
	}

buffer->body_len = header->body_len;							
buffer->from_max = (header->from_len + 1);
buffer->replyto_max = (header->replyto_len + 1);
buffer->to_max 	= (header->to_len + 1);
buffer->cc_max 	= (header->cc_len + 1);
buffer->bcc_max 	= 1;
buffer->subj_max = (header->subject_len + 1);

buffer->from 				= (char *)malloc(buffer->from_max);
buffer->replyto			= (char *)malloc(buffer->replyto_max);
buffer->to 					= (char *)malloc(buffer->to_max);
buffer->cc 					= (char *)malloc(buffer->cc_max);
buffer->bcc 					= (char *)malloc(1);
buffer->subject 			= (char *)malloc(buffer->subj_max);
buffer->organization = (char *)malloc(header->organization_len + 1);
buffer->date 				= (char *)malloc(51);
	
buffer->to[0] = 0;
buffer->organization[0] = 0;
buffer->from[0] = 0;
buffer->replyto[0] = 0;
buffer->to[0] = 0;
buffer->cc[0] = 0;
buffer->bcc[0] = 0;
buffer->subject[0] = 0;
buffer->date[0] = 0;

if (length[0])
	{
	memcpy(buffer->date, memptr[0], length[0]);
	buffer->date[length[0]] = 0;
	}
if (length[1])
	{
	memcpy(buffer->from, memptr[1], length[1]);
	buffer->from[length[1]] = 0;
	}
if (length[2])
	{
	memcpy(buffer->replyto, memptr[2], length[2]);
	buffer->replyto[length[2]] = 0;
	}
if (length[3])
	{
	memcpy(buffer->organization, memptr[3], length[3]);
	buffer->organization[length[3]] = 0;
	}
if (length[4])
	{
	memcpy(buffer->to, memptr[4], length[4]);
	buffer->to[length[4]] = 0;
	}
if (length[5])
	{
	memcpy(buffer->cc, memptr[5], length[5]);
	buffer->cc[length[5]] = 0;
	}
if (length[6])
	{
	memcpy(buffer->subject, memptr[6], length[6]);
	buffer->subject[length[6]] = 0;
	}
	
if (mailbody)
	free(mailbody);

if (priority_flag) 
	buffer->priority = 1;
else
	if (priority_flag)
		buffer->priority = 2;

if (part_cnt > 1)
	{
	buffer->has_attachments = 1;
		
	free(parts);
	end_info();
	}

scan_active = FALSE;
}
