/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

extern char *encode_type;

#ifdef EVAL
# define RESTRICTED_ATTACHMENT
#endif

#ifdef MILAN
# define RESTRICTED_ATTACHMENT
#endif

/* ---------------------
   | Convert into ansi |
   --------------------- */
void make_ansistring(char *str)
{
while(*str)
	{
	*str = to_ansi[*((unsigned char *)str)];
	str++;
	}
}

/* ------------------
   | Save mail body | 
   ------------------ */
int gen_mailbody(char *tmp2, long *enc_offset, char *signature, int eight_allowed)
{
char tmp[12], *p;
int f, in, enc;
long len;

DEBUG();
ClearError();

f = create_temp(tmp);
if (f > 0)
	{
#ifndef EVAL
	if (!edit_mode)
		{
		long lc;
		
		if (signature[0])
			{
			SIGNATURE *sigdata = getSignaturePtr(signature);
			int siglen;
			
			if (sigdata)
				{
				siglen = sigdata->len;
				
				if (new_buffer.body_len)
					new_buffer.body = (char *)realloc(new_buffer.body,
																		new_buffer.body_len + siglen + 1);
				else
					new_buffer.body = (char *)malloc(siglen + 1);
			
				new_buffer.body[new_buffer.body_len++] = '\r';
				memcpy(new_buffer.body + new_buffer.body_len, sigdata->sig, siglen);
			
				lc = 0;
				while(lc < siglen)
					{
					if (new_buffer.body[new_buffer.body_len + lc] == '\n')
						{
						if (siglen - lc - 1)
							memmove(new_buffer.body + new_buffer.body_len + lc, 
											new_buffer.body + new_buffer.body_len + lc + 1,
											siglen - lc - 1);
						siglen--;
						}
					else
						lc++;
					}
				new_buffer.body_len += siglen;
				}
			}
		}
#endif

	len = new_buffer.body_len;
	p = new_buffer.body;
	while(len--)
		{
		if ( *((unsigned char *)p) == 0x9E)
			*p = 0xE1;
		p++;
		}

	if (mail_options.ansi_flg)
		{
/*		make_ansistring(new_buffer.date);
		make_ansistring(new_buffer.from);
		make_ansistring(new_buffer.replyto);
		make_ansistring(new_buffer.organization);
		make_ansistring(new_buffer.to);
		make_ansistring(new_buffer.cc);
		make_ansistring(new_buffer.bcc);
		make_ansistring(new_buffer.subject);
	*/						
		len = new_buffer.body_len;
		p = new_buffer.body;
		while(len--)
			{
			*p = to_ansi[*((unsigned char *)p)];
			p++;
			}
		}
			
	len = 0;
	p = new_buffer.body;
	while (len < new_buffer.body_len)
		{
		if (new_buffer.body[len] == '\r')
			{
			WriteBytesLF(f, p, new_buffer.body + len - p);
			p = new_buffer.body + len + 1;
			}
		len++;
		}
	if (p < new_buffer.body + len)
		WriteBytesLF(f, p, new_buffer.body + len - p);
	Fclose(f);
	
	in = open_file(tmp, NULL);
	if (!in) 
		{
		delete_file(tmp);
		return -1;
		}
	}

f = create_temp(tmp2);
if (f > 0)
	{
	long length;
	
	length = Fseek(0, in, 2);
	Fseek(0, in, 0);
	enc = EncodeFile(in, f, "Message Body", TRUE, NULL, length, 
									 mail_options.ansi_flg ? -2 : -1, eight_allowed);
	*enc_offset = encoded_content;
	Fclose(f);
	}
else
	{
	delete_file(tmp);
	return -1;
	}
	
Fclose(in);
delete_file(tmp);
return enc;
}

/* -------------------------
   | Generate an ansi copy |
   ------------------------- */
char *gen_ansi(char *src)
{
char *dest;

DEBUG();
dest = (char *)malloc(strlen(src) + strlen(src) * 3 + 17 + 1);
strcpy(dest, src);
make_ansistring(dest);

if (strcmp(dest, src))
	{
	char *sp, *dp;
	unsigned char c;
	
	sp = src;
	dp = dest;
	
	while(*dp)
		{
		if (*dp != *sp || *dp == '=')
			{
			memmove(dp + 3, dp + 1, strlen(dp));
			c = (unsigned char)*dp;
			*(dp++) = '=';
			*(dp++) = hexchar[c >> 4];
			*(dp++) = hexchar[c & 0xF];
			}
		else
			if (*dp == ' ')
				*(dp++) = '_';
			else
				dp++;
		sp++;
		}
		
	memmove(dest + 15, dest, strlen(dest) + 1);
	memcpy(dest, "=?ISO-8859-1?Q?", 15);
	strcat(dest, "?=");
	}
	
return dest;
}

/* ------------------------------
   | Generate encoded mail file | 
   ------------------------------ */
char *gen_mail(void)
{
char body[13], out_date[30];
static char mailfile[13], bound_string[64], bound_base[128];
int out_mail, enc, in, part_cnt;
char *name;
time_t tm_val;
struct tm *tminfo;
FILE_MAIL_MEM new_mail;
PART_INFO *parts;
long e_offset;
char *ansi;
IDENTITY setid;
int has_identity;

DEBUG();
mime_flg = 0;

if (new_buffer.from && new_buffer.replyto)										/* Clear if reply-to is the same as the from field */
	if (!strcmp(new_buffer.from, new_buffer.replyto))
		new_buffer.replyto[0] = 0;

if (new_buffer.identity[0])
	has_identity = getIdentity(new_buffer.identity, &setid);
else
	has_identity = FALSE;

enc = gen_mailbody(body, &e_offset, has_identity ? setid.signature : NULL,
									 has_identity ? setid.eight_allowed : FALSE);
if (enc != -1)
	{
	out_mail = create_outgoing(mailfile);
	if (out_mail > 0)
		{
		ClearError();

		memset(&new_mail, 0, sizeof(new_mail));

		new_mail.magic 		= 0x4104;
		new_mail.version 	= SVERSION_ID20;
		new_mail.part_count = -1;
		strcpy(new_mail.identity, new_buffer.identity);
		strcpy(new_mail.signature, new_buffer.signature);

		Fwrite(out_mail, sizeof(new_mail), &new_mail);
		
		new_mail.confirm_read = new_buffer.confirm_read;
		new_mail.confirm_rcpt = new_buffer.confirm_rcpt;
		new_mail.word_wrap		= new_buffer.word_wrap;
		new_mail.priority 		= new_buffer.priority;

	  new_mail.header_inf					= sizeof(new_mail);
	  if (new_buffer.date && new_buffer.date[0])
	  	{
		  new_mail.date_len					= (int)strlen(new_buffer.date);
		  Fwrite(out_mail, new_mail.date_len, new_buffer.date);
		  }
	  if (new_buffer.from && new_buffer.from[0])
	  	{
		  new_mail.from_len					= (int)strlen(new_buffer.from);
		  Fwrite(out_mail, new_mail.from_len, new_buffer.from);
		  }
	  if (new_buffer.replyto && new_buffer.replyto[0])
	  	{
		  new_mail.replyto_len			= (int)strlen(new_buffer.replyto);
		  Fwrite(out_mail, new_mail.replyto_len, new_buffer.replyto);
		  }
	  if (new_buffer.organization && new_buffer.organization[0])
	  	{
		  new_mail.organization_len	= (int)strlen(new_buffer.organization);
		  Fwrite(out_mail, new_mail.organization_len, new_buffer.organization);
		  }
	  if (new_buffer.to && new_buffer.to[0])
	  	{
		  new_mail.to_len						= (int)strlen(new_buffer.to);
		  Fwrite(out_mail, new_mail.to_len, new_buffer.to);
		  add_recent(new_buffer.to);
		  }
	  if (new_buffer.cc && new_buffer.cc[0])
	  	{
		  new_mail.cc_len						= (int)strlen(new_buffer.cc);
		  Fwrite(out_mail, new_mail.cc_len, new_buffer.cc);
		  add_recent(new_buffer.cc);
		  }
	  if (new_buffer.bcc && new_buffer.bcc[0])
	  	{
		  new_mail.bcc_len					= (int)strlen(new_buffer.bcc);
		  Fwrite(out_mail, new_mail.bcc_len, new_buffer.bcc);
		  add_recent(new_buffer.bcc);
		  }
	  if (new_buffer.subject && new_buffer.subject[0])
	  	{
		  new_mail.subject_len			= (int)strlen(new_buffer.subject);
		  Fwrite(out_mail, new_mail.subject_len, new_buffer.subject);
		  }

		new_mail.body_pos = new_mail.header_inf + 
		  new_mail.date_len	+ new_mail.from_len	+ new_mail.replyto_len +
		  new_mail.organization_len	+ new_mail.to_len	+
		  new_mail.cc_len	+ new_mail.bcc_len + new_mail.subject_len;

		if (mime_flg || new_buffer.attach_count)
			{
			WriteStringLF(out_mail, "MIME-Version: 1.0 (Generated by marathon mail)");
			mime_flg = 1;
			}

		if (new_buffer.attach_count)
			{			
			WriteString(out_mail, "Content-Type: Multipart/Mixed; boundary=\"");
			strcpy(bound_string, "DRACONIS-boundary-");
			
			strcpy(bound_base, "Draconis mail boundary:");
			tm_val = time(NULL);
			strcat(bound_base, ctime(&tm_val));
			MDString(bound_base, bound_string + 18);
			
			WriteString(out_mail, bound_string);
			WriteStringLF(out_mail, "\"");
			}

		if (new_buffer.from[0])
			{
			if (mail_options.ansi_flg)	ansi = gen_ansi(new_buffer.from);
			BreakedStringLF(out_mail, "From: ", mail_options.ansi_flg ? ansi : new_buffer.from);
			if (mail_options.ansi_flg)	free(ansi);
			}
			
		if (new_buffer.replyto[0])
			{
			if (mail_options.ansi_flg)	ansi = gen_ansi(new_buffer.replyto);
			BreakedStringLF(out_mail, "Reply-To: ", mail_options.ansi_flg ? ansi : new_buffer.replyto);
			if (mail_options.ansi_flg)	free(ansi);
			}
			
		if (new_buffer.organization[0])
			{
			if (mail_options.ansi_flg)	ansi = gen_ansi(new_buffer.organization);
			BreakedStringLF(out_mail, "Organization: ", mail_options.ansi_flg ? ansi : new_buffer.organization);
			if (mail_options.ansi_flg)	free(ansi);
			}

		if (new_buffer.to[0])
			{
			if (mail_options.ansi_flg)	ansi = gen_ansi(new_buffer.to);
			BreakedStringLF(out_mail, "To: ", mail_options.ansi_flg ? ansi : new_buffer.to);
			if (mail_options.ansi_flg)	free(ansi);
			}

		if (new_buffer.cc[0])
			{
			if (mail_options.ansi_flg)	ansi = gen_ansi(new_buffer.cc);
			BreakedStringLF(out_mail, "Cc: ", mail_options.ansi_flg ? ansi : new_buffer.cc);
			if (mail_options.ansi_flg)	free(ansi);
			}
			
		if (mail_options.ansi_flg)	ansi = gen_ansi(new_buffer.subject);
		BreakedStringLF(out_mail, "Subject: ", mail_options.ansi_flg ? ansi : new_buffer.subject);
		if (mail_options.ansi_flg)	free(ansi);

		WriteString(out_mail, "Date: ");
		tm_val = time(NULL);
		tminfo = localtime(&tm_val);
/*		tminfo->tm_year %= 100;				 Avoid millenium bug */
		strftime(out_date, sizeof(out_date), 
						 "%a, %d %b %Y %H:%M:%S GMT", tminfo);
		WriteStringLF(out_mail, out_date);

		if (new_buffer.replyto && new_buffer.replyto[0])
			{
			if (mail_options.ansi_flg)	ansi = gen_ansi(new_buffer.replyto);
			
			if (new_buffer.confirm_read)
				{
				BreakedStringLF(out_mail, "X-Confirm-Reading-To: ", mail_options.ansi_flg ? ansi : new_buffer.replyto);
				WriteStringLF(out_mail, "X-pmrqc: 1");
				}

			if (new_buffer.confirm_rcpt)
				BreakedStringLF(out_mail, "Return-receipt-to: ", mail_options.ansi_flg ? ansi : new_buffer.replyto);

			if (mail_options.ansi_flg)	free(ansi);
			}
		else
			{
			if (mail_options.ansi_flg)	ansi = gen_ansi(new_buffer.from);
			
			if (new_buffer.confirm_read)
				{
				BreakedStringLF(out_mail, "X-Confirm-Reading-To: ", mail_options.ansi_flg ? ansi : new_buffer.from);
				WriteStringLF(out_mail, "X-pmrqc: 1");
				}

			if (new_buffer.confirm_rcpt)
				BreakedStringLF(out_mail, "Return-receipt-to: ", mail_options.ansi_flg ? ansi : new_buffer.from);

			if (mail_options.ansi_flg)	free(ansi);
			}
			
		switch (new_buffer.priority)
			{
			case 1:
				WriteStringLF(out_mail, "Priority: urgent");
				break;
			
			case 0:
				WriteStringLF(out_mail, "Priority: normal");
				break;
			
			case 2:
				WriteStringLF(out_mail, "Priority: low");
				break;
			}

		WriteStringLF(out_mail, "X-mailer: Marathon Mail 2.0 for ATARI and compatible");

		if (new_buffer.attach_count)
			{
			WriteStringLF(out_mail, "\0");
			WriteString(out_mail, "--");
			WriteStringLF(out_mail, bound_string);
			e_offset = 0;
			}

		new_mail.main_idx = 0;
		
		parts = (PART_INFO *)malloc(sizeof(PART_INFO) * (new_buffer.attach_count + 1));
		part_cnt = 1;

		parts[0].offset = Fseek(0L, out_mail, 1) + e_offset;
		strcpy(parts[0].name, "Body");
		strcpy(parts[0].desc, "Text");
		strcpy(parts[0].type, "text");
		CopyFile(out_mail, body);
		parts[0].len = Fseek(0L, out_mail, 1) - parts[0].offset;
		if (mail_options.ansi_flg) parts[0].offset |= 0x80000000L;
		
		new_buffer.has_attachments = (new_buffer.attach_count > 0);
		if (new_buffer.attach_count)
			{
			int att, edit_chk;
			long length;
			
			for (att = 0; att < new_buffer.attach_count; att++)
				{
				for (edit_chk = 0; edit_chk < epart_count; edit_chk++)
					if (new_buffer.attachments[att] == edit_parts[edit_chk].name)
						break;
						
				if (edit_chk == epart_count)
					{
					in = (int)Fopen(new_buffer.attachments[att], FO_READ);	
					if (in < 0)
						in = open_file(new_buffer.attachments[att], NULL);
					if (in > 0)
						{
						length = Fseek(0L, in, 2);
						Fseek(0L, in, 0);
						}
					}
				else
					{
					in = open_file(edit_file, edit_box);
					if (in > 0)
						{
						Fseek(edit_parts[edit_chk].offset, in, 0);
						length = edit_parts[edit_chk].len;
						}
					}

				if (in > 0)
					{
#ifdef RESTRICTED_ATTACHMENT
					if (length < 50000L)
						{
#endif
						WriteStringLF(out_mail, "\0");
						WriteStringLF(out_mail, "\0");
						WriteString(out_mail, "--");
						WriteStringLF(out_mail, bound_string);

						if (edit_chk == epart_count && 
								strncmp(new_buffer.attachments[att], 
												forwardmsg_text, strlen(forwardmsg_text)))
							{					
							if (strchr(new_buffer.attachments[att], '\\'))
								name = strrchr(new_buffer.attachments[att], '\\') + 1;
							else
								name = new_buffer.attachments[att];
						
							if (EncodeFile(in, out_mail, NULL, FALSE, name, length, new_buffer.att_encoding[att],
														 has_identity ? setid.eight_allowed : FALSE) == 9)
								name[0] = 0;
							
							strcpy(parts[part_cnt].name, name);
							parts[part_cnt].offset = encoding_offset;
							parts[part_cnt].desc[0] = 0;
							strcpy(parts[part_cnt].type, encode_type);
							parts[part_cnt].len = Fseek(0L, out_mail, 1) - parts[part_cnt].offset;
							}
						else
							{
							parts[part_cnt].offset = Fseek(0, out_mail, 1);

							CopyPart(out_mail, in, length);
							
							strcpy(parts[part_cnt].name, edit_parts[edit_chk].name);
							strcpy(parts[part_cnt].desc, edit_parts[edit_chk].desc);
							strcpy(parts[part_cnt].type, edit_parts[edit_chk].type);
							parts[part_cnt].len = edit_parts[edit_chk].len;
							}
							
						part_cnt++;
#ifdef RESTRICTED_ATTACHMENT
						}
					else
						{
						form_alert(1, bigatt_msg);
						}
#endif						
					Fclose(in);
					}
					
				free(new_buffer.attachments[att]);
				}
			free(new_buffer.attachments);
			free(new_buffer.att_encoding);
			new_buffer.attach_count = 0;
			
			WriteStringLF(out_mail, "\0");
			WriteStringLF(out_mail, "\0");
			WriteString(out_mail, "--");
			WriteString(out_mail, bound_string);
			WriteStringLF(out_mail, "--");
			}

		new_mail.partinfo_pos = Fseek(0, out_mail, 1);
		new_buffer.body_len =
		new_mail.body_len = new_mail.partinfo_pos -
												new_mail.body_pos;
		new_mail.part_count = part_cnt;
		Fwrite(out_mail, sizeof(PART_INFO) * part_cnt, parts);
		Fseek(0, out_mail, 0);
		Fwrite(out_mail, sizeof(new_mail), &new_mail);
		
		Fclose(out_mail);
		free(parts);
		}
	
	delete_file(body);
	}
return mailfile;
}

