/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

int intel_mode, red_shift, green_shift, blue_shift;

/* -------------------------------
   | Analyse pixel index mapping |
   ------------------------------- */
void check_indexmapping(void)
{
unsigned char res_buf[4 * 16];
int i, xy[2], *c, j, val;

DEBUG();
xy[0] = 0;
xy[1] = 0;
for (i = 0; i < dither_colours; i++)
	{
	vsm_color(vdi_handle, i);
	v_pmarker(vdi_handle, 1, xy);
	
	blt_copy(NULL, 0, 0, res_buf, 0, 0, 1, 1, S_ONLY);
	c = (int *)res_buf;
	
	if (!pixel_type)
		{
		c += planes - 1;
		
		val = 0;
		for (j = planes - 1; j >= 0; j--)
			if (*(c--) & 0x8000)
				val |= 1 << j;
		}
	else
		val = (*res_buf) >> (8 - planes);

	tr_cols[i] = val;
	tr_base[val] = i;
	}
}

/* -------------------------------
   | Analyse pixel index mapping |
   ------------------------------- */
void check_truecolor(void)
{
unsigned char res_buf[4 * 32];
int xy[2], rgb[3], old_rgb[3];
unsigned long c, c1;

DEBUG();
xy[0] = 0;
xy[1] = 0;
rgb[0] = 1000;
rgb[1] = 500;
rgb[2] = 250;
vq_color(vdi_handle, 40, 0, old_rgb);
vs_color(vdi_handle, 40, rgb);
vq_color(vdi_handle, 40, 1, rgb);

rgb[0] = (int)((rgb[0] + 3) * 255L / 1000);
rgb[1] = (int)((rgb[1] + 3) * 255L / 1000);
rgb[2] = (int)((rgb[2] + 3) * 255L / 1000);

vsm_color(vdi_handle, 40);
v_pmarker(vdi_handle, 1, xy);
	
blt_copy(NULL, 0, 0, res_buf, 0, 0, 3, 1, S_ONLY);

c = *(unsigned long *)res_buf;
c >>= (32 - planes);

switch(planes)
	{
	case 16:
		{
		/* 16 Bit (32768 Colour 5/5/5) */		
		c1 = (((rgb[0] & 0xF8) << 7) | ((rgb[1] & 0xF8) << 2) | (rgb[2] >> 3)) & 0xFFFF;
		if (c1 == c)	
			{
			truecolor_type = 0;	/* 5/5/5 Motorola */
			break;
			}

		c1 = ((c1 << 8) | ((c1 >> 8) & 0xFF)) & 0xFFFF;
		if (c1 == c)
			{
			truecolor_type = 5;	/* 5/5/5 Intel */
			break;
			}
			
		/* 16 Bit (65536 Colour 5/6/5) */
		c1 = (((rgb[0] & 0xF8) << 8) | ((rgb[1] & 0xFC) << 3) | (rgb[2] >> 3)) & 0xFFFF;
		if (c1 == c)
			{
			truecolor_type = 2;		/* 5/6/5 Motorola */
			break;
			}
			
		c1 = ((c1 << 8) | ((c1 >> 8) & 0xFF)) & 0xFFFF;
		if (c1 == c)
			{
			truecolor_type = 7;	/* 5/6/5 Intel */
			break;
			}
			
		c1 = ((rgb[0] & 0xF8) << 8) | ((rgb[1] & 0xF8) << 3) | (rgb[2] >> 3);
		c1 = ((c1 << 8) | ((c1 >> 8) & 0xFF)) & 0xFFFF;
		if (c1 == c)
			{
			truecolor_type = 6;	/* 5/5/5 Falcon Intel */
			break;
			}
									
/*			16 Bit Falcon - c1 = ((rgb[0] & 0xF8) << 8) | ((rgb[1] & 0xF8) << 3) | (rgb[2] >> 3);*/

		break;
		}
		
	case 24:
		c1 = ((unsigned long)rgb[0] << 16L) | ((unsigned long)rgb[1] << 8L) | rgb[2];
		if (c1 == c)
			{
			truecolor_type = 3;
			break;
			}
		c1 = ((unsigned long)rgb[2] << 16L) | ((unsigned long)rgb[1] << 8L) | rgb[0];
		if (c1 == c)
			{
			truecolor_type = 9;
			break;
			}
		break;

	case 32:
		c1 = ((unsigned long)rgb[0] << 16L) | ((unsigned long)rgb[1] << 8L) | rgb[2];
		if (c == c1)
			{
			truecolor_type = 4;
			break;
			}
		c1 <<= 8L;
		if (c == c1)
			{
			truecolor_type = 8;
			break;
			}
		c1 = ((unsigned long)rgb[2] << 24L) | ((unsigned long)rgb[1] << 16) | ((unsigned long)rgb[0] << 8);
		if (c == c1)
			{
			truecolor_type = 10;
			break;
			}
		break;		
	}

vs_color(vdi_handle, 40, old_rgb);
}

/* --------------------------
   | Detect pixel type mode |
   -------------------------- */
void detect_pixeltype(void)
{
unsigned char zw_buf[4 * 32], res_buf[4 * 32];
int col1, pel1, col2, pel2;

DEBUG();
graf_mouse(M_OFF, 0);
blt_copy(NULL, 0, 0, zw_buf, 0, 0, planes, 1, S_ONLY);

memset(res_buf, 0, sizeof(res_buf));							/* Clear area */
blt_copy(res_buf, 0, 0, NULL, 0, 0, planes, 1, S_ONLY);

if (dither_colours > 2)
	{
	v_get_pixel(vdi_handle, 1, 0, &pel1, &col1);

	switch (dither_colours)
		{
		case 4:										/* ? */
			res_buf[0] ^= 0x30;
			break;
		
		case 16:
			res_buf[0] ^= 0x0F;
			break;
	
		case 256:
			res_buf[1] ^= 0xFF;
			break;
		}

	blt_copy(res_buf, 0, 0, NULL, 0, 0, 2, 1, S_ONLY);
	v_get_pixel(vdi_handle, 1, 0, &pel2, &col2);

	if (pel1 != pel2 || col1 != col2)
		pixel_type = 1;
	else
		pixel_type = 0;
		
	res_buf[0] = res_buf[1] = 0;
	blt_copy(res_buf, 0, 0, NULL, 0, 0, 2, 1, S_ONLY);
	check_indexmapping();
	}
else
	{
	pixel_type = 0;
	if (!dither_colours)
		check_truecolor();
	else
		check_indexmapping();
	}
blt_copy(zw_buf, 0, 0, NULL, 0, 0, planes, 1, S_ONLY);
graf_mouse(M_ON, 0);
}

