/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

/* Local prototype */
int cdecl ident_obj(PARMBLK *parm);

static USERBLK ident_ublk 	= {ident_obj, 0L};
static int identity_chg, identity_page, selected_identity;
static int identity_count = 0, next_identifier;
static IDENTITY *identities;
static int cfg_identity, default_identity = -1;
static OBJECT *identity_conf = NULL;
static int idcnf_pages[5];

static IDENTITY defaultentry = {1, "<default>", TRUE, "\0", "\0", "\0", "\0", "\0", "\0",
																TRUE, TRUE, FALSE, TRUE, 0, -1, -1, TRUE, "\0", "\0",
																-1, FALSE, TRUE, FALSE, TRUE, TRUE, 74, 8, FALSE, TRUE, '>', -1, TRUE,
																-1, TRUE, -1, 0, "\0", "\0", "\0", 1+2};
static int idcfg_tabids[] = {ID_TAB_ADDR, ID_TAB_EDIT, ID_TAB_SMTP, ID_TAB_POP, ID_TAB_ACCOUNT};

extern int notify, dsn, smtp_s;

static int idcnf_ids[] = {ID_NAME,						/* Base */
													ID_TAB_ACCOUNT,
													ID_TAB_POP,
													ID_TAB_SMTP,
													ID_TAB_EDIT,
													ID_TAB_ADDR,
													IDCFG_OKAY,
													IDCFG_CANCEL,					/* 7 */

													IDCNF_USE_PROVIDER,	/* Page 0 */
													IDCNF_ML_ADDR,
													IDCNF_ML_ADDR2,
													IDCNF_POPUSER,
													IDCNF_POPPASS,
													IDCNF_POPSERVER,
													IDCNF_SMTPSERVER,			/* 14 */

													ID_INCLCHK,					/* Page 1 */ 
													IDCNF_ENCRYPT,
													IDCNF_IMMEDOWN,
													IDCNF_DELERECV,
													IDCNF_MAXDOWN,
													IDCNF_POPPORT,
													IDCNF_DEFPOP,
													IDCNF_CHKEVERY,
													IDCNF_COUNT,					/* 23 */

													IDCNF_USERNAME,			/* Page 2 */ 
													IDCNF_ORGANIZATION,
													IDCNF_ALLOW_8BIT,
													IDCNF_USE_MIME,
													IDCND_SENDIMME,
													IDCNF_POPAUTH,
													IDCNF_SMTPPORT,
													IDCNF_SMTPDEFAULT,		/* 31 */
/*
#define IDCNF_AUTHNONE 8  /* BUTTON in tree ID_SMTP */
#define IDCNF_SMTPAUTH 9  /* BUTTON in tree ID_SMTP */
#define IDCNF_SMTPUSER 10  /* FTEXT in tree ID_SMTP */
#define IDCNF_SMTPPASS 11  /* FTEXT in tree ID_SMTP */
*/

													IDCNF_WORDWRAP,			/* Page 3 */ 
													IDCNF_EDITLEN,
													IDCNF_TABWIDTH,
													IDCNF_QUOTE,
													IDCNF_QUOTECHAR,
													IDCNF_RCPT_YES,
													IDCNF_RCPT_NO,
													IDCNF_RCPT_LAST,
													IDCNF_READ_YES,				/* 40 */
													IDCNF_READ_NO,
													IDCNF_READ_LAST,
													IDCNF_HIGH,
													IDCNF_NORMAL,
													IDCNF_LOW,
													IDCNF_PRIO_LAST,			/* 46 */
													
													IDCNF_REPLY1,				/* Page 4 */ 
													IDCNF_REPLY2,
													IDCNF_SIG_BUTTON,
													IDCNF_SIGDROP,
													IDCNF_RECENT,
													IDCNF_ABOOK,					/* 52 */
													
													0};

static int ilist_ids[] = {IDENTITYLIST,
													NEW_IDENTITY,
													CFG_IDENTITY,
													DEL_IDENTITY,
													DEF_IDENTITY,
													CLOSE_IDENTITY,
													0};

/* --------------------
   | Set changed flag |
   -------------------- */
void setIdentityChange(void)
{
identity_chg = TRUE;
}

/* -------------------------------
   | Is there a default identity |
   ------------------------------- */
int hasDefaultIdentity(void)
{
return (default_identity != -1);
}

/* --------------------------------
   | Get data of default identity |
   -------------------------------- */
int getDefaultIdentity(IDENTITY *store)
{
DEBUG();
if (default_identity != -1)
	{
	memcpy(store, identities + default_identity, sizeof(IDENTITY));
	return TRUE;
	}
return FALSE;
}

/* --------------------
   | Get Identity idx |
   -------------------- */
int getIdentityIndex(char *name)
{
int i;

for (i = 0; i < identity_count; i++)
	if (!strcmp(identities[i].name, name))
		return i;
return -1;
}
   
/* --------------------------------------------------
   | Get identity pointer (for short time use only) |
   -------------------------------------------------- */
IDENTITY *getIdentityPtr(int i)
{
return identities + i;
}

/* ---------------------
   | Get identity data |
   --------------------- */
int getIdentity(char *name, IDENTITY *store)
{
int i;

DEBUG();
for (i = 0; i < identity_count; i++)
	if (!strcmp(identities[i].name, name))
		{
		memcpy(store, identities + i, sizeof(IDENTITY));
		return TRUE;
		}
return FALSE;
}

/* -------------------------------------------
   | Return the number of defined identities |
   ------------------------------------------- */
int getNumberOfIdentities(void)
{
return identity_count;
}

/* ---------------------
   | Get Identity name |
   --------------------- */
char *getIdentityName(int i)
{
return identities[i].name;
}

/* -------------------------------
	 | Fill identity configuration |
	 ------------------------------- */
void fill_identity_conf(int num)
{
IDENTITY *cfg;

DEBUG();
if (num == -1)
	{
	cfg = &defaultentry;
	identity_conf[ID_NAME].ob_spec.tedinfo->te_ptext[0] = 0;
	}
else
	{
  cfg = identities + num;
  strcpy(identity_conf[ID_NAME].ob_spec.tedinfo->te_ptext, cfg->name);
  }

  /* Account settings */
SetSelect(identity_conf, idcnf_pages[0] + IDCNF_USE_PROVIDER, cfg->use_providerdata);
if (cfg->use_providerdata)
	{
	DisableObj(identity_conf, idcnf_pages[0] + IDCNF_ML_ADDR);
	DisableObj(identity_conf, idcnf_pages[0] + IDCNF_ML_ADDR2);
	DisableObj(identity_conf, idcnf_pages[0] + IDCNF_POPUSER);
	DisableObj(identity_conf, idcnf_pages[0] + IDCNF_POPPASS);
	DisableObj(identity_conf, idcnf_pages[0] + IDCNF_POPSERVER);
	DisableObj(identity_conf, idcnf_pages[0] + IDCNF_SMTPSERVER);
	}		
else
	{
	EnableObj(identity_conf, idcnf_pages[0] + IDCNF_ML_ADDR);
	EnableObj(identity_conf, idcnf_pages[0] + IDCNF_ML_ADDR2);
	EnableObj(identity_conf, idcnf_pages[0] + IDCNF_POPUSER);
	EnableObj(identity_conf, idcnf_pages[0] + IDCNF_POPPASS);
	EnableObj(identity_conf, idcnf_pages[0] + IDCNF_POPSERVER);
	EnableObj(identity_conf, idcnf_pages[0] + IDCNF_SMTPSERVER);
	}		

strcpy(identity_conf[idcnf_pages[0] + IDCNF_ML_ADDR].ob_spec.tedinfo->te_ptext, cfg->email);
strcpy(identity_conf[idcnf_pages[0] + IDCNF_ML_ADDR2].ob_spec.tedinfo->te_ptext, cfg->email2);
strcpy(identity_conf[idcnf_pages[0] + IDCNF_POPUSER].ob_spec.tedinfo->te_ptext, cfg->popuser);
strcpy(identity_conf[idcnf_pages[0] + IDCNF_POPPASS].ob_spec.tedinfo->te_ptext, cfg->mailpass);
strcpy(identity_conf[idcnf_pages[0] + IDCNF_POPSERVER].ob_spec.tedinfo->te_ptext, cfg->popserver);
strcpy(identity_conf[idcnf_pages[0] + IDCNF_SMTPSERVER].ob_spec.tedinfo->te_ptext, cfg->smtpserver);

 /* POP settings */
SetSelect(identity_conf, idcnf_pages[1] + ID_INCLCHK, cfg->check);
SetSelect(identity_conf, idcnf_pages[1] + IDCNF_ENCRYPT, cfg->pass_coded);
SetSelect(identity_conf, idcnf_pages[1] + IDCNF_IMMEDOWN, cfg->download_immediately);
SetSelect(identity_conf, idcnf_pages[1] + IDCNF_DELERECV, cfg->delete_rcvd);
ltoa(cfg->maxsize / 1000L, 
		 identity_conf[idcnf_pages[1] + IDCNF_MAXDOWN].ob_spec.tedinfo->te_ptext, 10);

if (cfg->pop_port != -1)
	{
	DeselectObj(identity_conf, idcnf_pages[1] + IDCNF_DEFPOP);
	EnableObj(identity_conf, idcnf_pages[1] + IDCNF_POPPORT);
	itoa(cfg->pop_port, 
		 	 identity_conf[idcnf_pages[1] + IDCNF_POPPORT].ob_spec.tedinfo->te_ptext, 10);
	}
else
	{
	SelectObj(identity_conf, idcnf_pages[1] + IDCNF_DEFPOP);
	DisableObj(identity_conf, idcnf_pages[1] + IDCNF_POPPORT);
	strcpy(identity_conf[idcnf_pages[1] + IDCNF_POPPORT].ob_spec.tedinfo->te_ptext, "110");
	}

if (cfg->poll_every != -1)
	itoa(cfg->poll_every, 
			 identity_conf[idcnf_pages[1] + IDCNF_CHKEVERY].ob_spec.tedinfo->te_ptext, 10);
else
	identity_conf[idcnf_pages[1] + IDCNF_CHKEVERY].ob_spec.tedinfo->te_ptext[0] = 0;
	
SetSelect(identity_conf, idcnf_pages[1] + IDCNF_COUNT, cfg->count_only);

  /* SMTP settings */
strcpy(identity_conf[idcnf_pages[2] + IDCNF_USERNAME].ob_spec.tedinfo->te_ptext, cfg->username);
strcpy(identity_conf[idcnf_pages[2] + IDCNF_ORGANIZATION].ob_spec.tedinfo->te_ptext, cfg->organization);

if (cfg->smtp_port != -1)
	{
	DeselectObj(identity_conf, idcnf_pages[2] + IDCNF_SMTPDEFAULT);
	EnableObj(identity_conf, idcnf_pages[2] + IDCNF_SMTPPORT);
	itoa(cfg->smtp_port, 
		 	 identity_conf[idcnf_pages[2] + IDCNF_SMTPPORT].ob_spec.tedinfo->te_ptext, 10);
	}
else
	{
	SelectObj(identity_conf, idcnf_pages[2] + IDCNF_SMTPDEFAULT);
	DisableObj(identity_conf, idcnf_pages[2] + IDCNF_SMTPPORT);
	strcpy(identity_conf[idcnf_pages[2] + IDCNF_SMTPPORT].ob_spec.tedinfo->te_ptext, "25");
	}

SetSelect(identity_conf, idcnf_pages[2] + IDCNF_ALLOW_8BIT, cfg->eight_allowed);
SetSelect(identity_conf, idcnf_pages[2] + IDCNF_USE_MIME, cfg->mime_allowed);
SetSelect(identity_conf, idcnf_pages[2] + IDCND_SENDIMME, cfg->send_immediately);

SetSelect(identity_conf, idcnf_pages[2] + IDCNF_AUTHNONE, cfg->auth_required == 0);
SetSelect(identity_conf, idcnf_pages[2] + IDCNF_POPAUTH, cfg->auth_required == 1);
SetSelect(identity_conf, idcnf_pages[2] + IDCNF_SMTPAUTH, cfg->auth_required == 2);

strcpy(identity_conf[idcnf_pages[2] + IDCNF_SMTPUSER].ob_spec.tedinfo->te_ptext, cfg->smtpuser);
strcpy(identity_conf[idcnf_pages[2] + IDCNF_SMTPPASS].ob_spec.tedinfo->te_ptext, cfg->smtppass);

if (cfg->auth_required == 2)
	{
	EnableObj(identity_conf, idcnf_pages[2] + IDCNF_SMTPUSER);
	EnableObj(identity_conf, idcnf_pages[2] + IDCNF_SMTPPASS);
	}
else
	{
	DisableObj(identity_conf, idcnf_pages[2] + IDCNF_SMTPUSER);
	DisableObj(identity_conf, idcnf_pages[2] + IDCNF_SMTPPASS);
	}

	/* Editor settings */
SetSelect(identity_conf, idcnf_pages[3] + IDCNF_WORDWRAP, cfg->word_wrap);
itoa(cfg->editwidth, 
	 	 identity_conf[idcnf_pages[3] + IDCNF_EDITLEN].ob_spec.tedinfo->te_ptext, 10);
itoa(cfg->tabwidth, 
	 	 identity_conf[idcnf_pages[3] + IDCNF_TABWIDTH].ob_spec.tedinfo->te_ptext, 10);
SetSelect(identity_conf, idcnf_pages[3] + IDCNF_QUOTE, cfg->quote_mail);
SetSelect(identity_conf, idcnf_pages[3] + IDCNF_FLOATING, cfg->floating);

identity_conf[idcnf_pages[3] + IDCNF_QUOTECHAR].ob_spec.tedinfo->te_ptext[0] = cfg->quote_char;
SetSelect(identity_conf, idcnf_pages[3] + IDCNF_RCPT_YES, 	cfg->confirm_rcpt == 1);
SetSelect(identity_conf, idcnf_pages[3] + IDCNF_RCPT_NO, 		cfg->confirm_rcpt == 0);
SetSelect(identity_conf, idcnf_pages[3] + IDCNF_RCPT_LAST, 	cfg->confirm_rcpt == -1);

SetSelect(identity_conf, idcnf_pages[3] + IDCNF_READ_YES, 	cfg->confirm_read == 1);
SetSelect(identity_conf, idcnf_pages[3] + IDCNF_READ_NO, 		cfg->confirm_read == 0);
SetSelect(identity_conf, idcnf_pages[3] + IDCNF_READ_LAST, 	cfg->confirm_read == -1);

SetSelect(identity_conf, idcnf_pages[3] + IDCNF_HIGH, 			cfg->priority == 1);
SetSelect(identity_conf, idcnf_pages[3] + IDCNF_NORMAL, 		cfg->priority == 0);
SetSelect(identity_conf, idcnf_pages[3] + IDCNF_LOW, 				cfg->priority == 2);
SetSelect(identity_conf, idcnf_pages[3] + IDCNF_PRIO_LAST, 	cfg->priority == -1);

	/* Address settings */
strcpy(identity_conf[idcnf_pages[4] + IDCNF_REPLY1].ob_spec.tedinfo->te_ptext, cfg->reply_to1);
strcpy(identity_conf[idcnf_pages[4] + IDCNF_REPLY2].ob_spec.tedinfo->te_ptext, cfg->reply_to2);

strcpy(identity_conf[idcnf_pages[4] + IDCNF_SIG_BUTTON].ob_spec.free_string, cfg->signature);

SetSelect(identity_conf, idcnf_pages[4] + IDCNF_RECENT, cfg->address_completion & 1);
SetSelect(identity_conf, idcnf_pages[4] + IDCNF_ABOOK, cfg->address_completion & 2);
}

/* ------------------------------
	 | Get identity configuration |
	 ------------------------------ */
void store_identity_conf(int num)
{
IDENTITY newcfg;

DEBUG();
strcpy(newcfg.name, identity_conf[ID_NAME].ob_spec.tedinfo->te_ptext);

  /* Account settings */
newcfg.use_providerdata = IsSelected(identity_conf, idcnf_pages[0] + IDCNF_USE_PROVIDER);

strcpy(newcfg.email, 			identity_conf[idcnf_pages[0] + IDCNF_ML_ADDR].ob_spec.tedinfo->te_ptext);
strcpy(newcfg.email2, 		identity_conf[idcnf_pages[0] + IDCNF_ML_ADDR2].ob_spec.tedinfo->te_ptext);
strcpy(newcfg.popuser, 		identity_conf[idcnf_pages[0] + IDCNF_POPUSER].ob_spec.tedinfo->te_ptext);
strcpy(newcfg.mailpass, 	identity_conf[idcnf_pages[0] + IDCNF_POPPASS].ob_spec.tedinfo->te_ptext);
strcpy(newcfg.popserver,	identity_conf[idcnf_pages[0] + IDCNF_POPSERVER].ob_spec.tedinfo->te_ptext);
strcpy(newcfg.smtpserver, identity_conf[idcnf_pages[0] + IDCNF_SMTPSERVER].ob_spec.tedinfo->te_ptext);

 /* POP settings */
newcfg.check 								= IsSelected(identity_conf, idcnf_pages[1] + ID_INCLCHK);
newcfg.pass_coded 					= IsSelected(identity_conf, idcnf_pages[1] + IDCNF_ENCRYPT);
newcfg.download_immediately = IsSelected(identity_conf, idcnf_pages[1] + IDCNF_IMMEDOWN);
newcfg.delete_rcvd 					= IsSelected(identity_conf, idcnf_pages[1] + IDCNF_DELERECV);

newcfg.maxsize = atol(identity_conf[idcnf_pages[1] + IDCNF_MAXDOWN].ob_spec.tedinfo->te_ptext) * 1000L;

if (IsSelected(identity_conf, idcnf_pages[1] + IDCNF_DEFPOP))
	newcfg.pop_port = -1;
else
	newcfg.pop_port = atoi(identity_conf[idcnf_pages[1] + IDCNF_POPPORT].ob_spec.tedinfo->te_ptext);

if (identity_conf[idcnf_pages[1] + IDCNF_CHKEVERY].ob_spec.tedinfo->te_ptext[0])
	newcfg.poll_every = atoi(identity_conf[idcnf_pages[1] + IDCNF_CHKEVERY].ob_spec.tedinfo->te_ptext);
else
	newcfg.poll_every = -1;
newcfg.count_only = IsSelected(identity_conf, idcnf_pages[1] + IDCNF_COUNT);

  /* SMTP settings */
strcpy(newcfg.username, identity_conf[idcnf_pages[2] + IDCNF_USERNAME].ob_spec.tedinfo->te_ptext);
strcpy(newcfg.organization, identity_conf[idcnf_pages[2] + IDCNF_ORGANIZATION].ob_spec.tedinfo->te_ptext);

if (IsSelected(identity_conf, idcnf_pages[2] + IDCNF_SMTPDEFAULT))
	newcfg.smtp_port = -1;
else
	newcfg.smtp_port = atoi(identity_conf[idcnf_pages[2] + IDCNF_SMTPPORT].ob_spec.tedinfo->te_ptext);

newcfg.eight_allowed 		= IsSelected(identity_conf, idcnf_pages[2] + IDCNF_ALLOW_8BIT);
newcfg.mime_allowed 		= IsSelected(identity_conf, idcnf_pages[2] + IDCNF_USE_MIME);
newcfg.send_immediately = IsSelected(identity_conf, idcnf_pages[2] + IDCND_SENDIMME);

if (IsSelected(identity_conf, idcnf_pages[2] + IDCNF_POPAUTH))
	newcfg.auth_required = 1;
else if (IsSelected(identity_conf, idcnf_pages[2] + IDCNF_SMTPAUTH))
	newcfg.auth_required = 2;
else
	newcfg.auth_required = 0;

strcpy(newcfg.smtpuser, identity_conf[idcnf_pages[2] + IDCNF_SMTPUSER].ob_spec.tedinfo->te_ptext);
strcpy(newcfg.smtppass, identity_conf[idcnf_pages[2] + IDCNF_SMTPPASS].ob_spec.tedinfo->te_ptext);

	/* Editor settings */
newcfg.word_wrap = IsSelected(identity_conf, idcnf_pages[3] + IDCNF_WORDWRAP);
newcfg.editwidth = atoi(identity_conf[idcnf_pages[3] + IDCNF_EDITLEN].ob_spec.tedinfo->te_ptext);
newcfg.tabwidth  = atoi(identity_conf[idcnf_pages[3] + IDCNF_TABWIDTH].ob_spec.tedinfo->te_ptext);

newcfg.quote_mail = IsSelected(identity_conf, idcnf_pages[3] + IDCNF_QUOTE);
newcfg.floating = IsSelected(identity_conf, idcnf_pages[3] + IDCNF_FLOATING);
newcfg.quote_char = identity_conf[idcnf_pages[3] + IDCNF_QUOTECHAR].ob_spec.tedinfo->te_ptext[0];

if (IsSelected(identity_conf, idcnf_pages[3] + IDCNF_RCPT_YES))
	newcfg.confirm_rcpt = 1;
else 
	newcfg.confirm_rcpt = IsSelected(identity_conf, idcnf_pages[3] + IDCNF_RCPT_NO) ? 0 : -1;
	
if (IsSelected(identity_conf, idcnf_pages[3] + IDCNF_READ_YES))
	newcfg.confirm_read = 1;
else 
	newcfg.confirm_read = IsSelected(identity_conf, idcnf_pages[3] + IDCNF_READ_NO) ? 0 : -1;

if (IsSelected(identity_conf, idcnf_pages[3] + IDCNF_HIGH))
	newcfg.priority = 1;
else 
	if (IsSelected(identity_conf, idcnf_pages[3] + IDCNF_NORMAL))
		newcfg.priority = 0;
	else
		newcfg.priority = IsSelected(identity_conf, idcnf_pages[3] + IDCNF_LOW) ? 2 : -1;

	/* Address settings */
strcpy(newcfg.reply_to1, identity_conf[idcnf_pages[4] + IDCNF_REPLY1].ob_spec.tedinfo->te_ptext);
strcpy(newcfg.reply_to2, identity_conf[idcnf_pages[4] + IDCNF_REPLY2].ob_spec.tedinfo->te_ptext);

strcpy(newcfg.signature, identity_conf[idcnf_pages[4] + IDCNF_SIG_BUTTON].ob_spec.free_string);

newcfg.address_completion  = IsSelected(identity_conf, idcnf_pages[4] + IDCNF_RECENT) ? 1 : 0;
newcfg.address_completion |= IsSelected(identity_conf, idcnf_pages[4] + IDCNF_ABOOK) ? 2 : 0;

if (num != -1)
	memcpy(identities + num, &newcfg, sizeof(IDENTITY));
else
	{
	newcfg.identifier = next_identifier++;
	if (!identity_count)
		identities = (IDENTITY *)malloc(sizeof(IDENTITY));
	else
		identities = (IDENTITY *)realloc(identities, sizeof(IDENTITY) * (identity_count + 1));
		
	memcpy(identities + identity_count, &newcfg, sizeof(IDENTITY));
	identity_count++;
	}
}

/* ----------------------------------------
   | Handle Identity configuration events |
   ---------------------------------------- */
static void cdecl idconfig_handler (OBJECT *tree, int obj)
{
DEBUG();
switch(obj)
	{
	case ID_TAB_ACCOUNT: 	SetTabPage(tree, ID_CFG_TAB, idcnf_pages[0], ID_TAB_ACCOUNT, 	5, idcfg_tabids); break;
	case ID_TAB_POP: 			SetTabPage(tree, ID_CFG_TAB, idcnf_pages[1], ID_TAB_POP, 		 	5, idcfg_tabids); break;
	case ID_TAB_SMTP: 		SetTabPage(tree, ID_CFG_TAB, idcnf_pages[2], ID_TAB_SMTP, 		5, idcfg_tabids); break;
	case ID_TAB_EDIT: 		SetTabPage(tree, ID_CFG_TAB, idcnf_pages[3], ID_TAB_EDIT, 		5, idcfg_tabids); break;
	case ID_TAB_ADDR: 		SetTabPage(tree, ID_CFG_TAB, idcnf_pages[4], ID_TAB_ADDR, 		5, idcfg_tabids); break;

	case IDCFG_OKAY:
		{
		OBJECT *identitylist;
		
		identity_chg = TRUE;
		store_identity_conf(cfg_identity);
		if (cfg_identity == -1)
			{
			OBJECT *tree;
			int obj;

			rsrc_gaddr(R_TREE, CREATE_ID, &tree);

			open_dialog(tree, NULL, NULL);
			obj = do_dialog(tree, 0, app_callback) & 0x7FFF;
			close_dialog(tree, TRUE);
			if (obj == DO_CREATEID)
				{
				int i, last_folder = -1;
				
				for (i = 0; i < box_count; i++)
					if (boxes[i].type == -1)
						break;
						
				if (i < box_count)
					{
					int l = boxes[i].level;
					
					while(i >= 0 && boxes[i].level >= l)
						i--;
					
					if (i >= 0)
						last_folder = i;
					}
				
				if (last_folder > -1)
					boxes[last_folder].expanded = TRUE;
					
				create_new_folder(last_folder, -1,
													identity_conf[ID_NAME].ob_spec.tedinfo->te_ptext,
													identity_conf[ID_NAME].ob_spec.tedinfo->te_ptext,
													3);
				}
			}

		rsrc_gaddr(R_TREE, IDENTITIES, &identitylist);

		if (is_open(identitylist))
			{
			redraw_object(identitylist, IDENTITYLIST);
			}
		}
		
	case IDCFG_CANCEL:
		close_dialog(tree, TRUE);
		break;
		
	default:
		if (IsSelected(tree, ID_TAB_ACCOUNT))	/* Account page active ? */
			{
			switch(obj - idcnf_pages[0])
				{
				case IDCNF_USE_PROVIDER:
					if (IsSelected(tree, obj))
						{
						DisableObj(identity_conf, idcnf_pages[0] + IDCNF_ML_ADDR);
						DisableObj(identity_conf, idcnf_pages[0] + IDCNF_ML_ADDR2);
						DisableObj(identity_conf, idcnf_pages[0] + IDCNF_POPUSER);
						DisableObj(identity_conf, idcnf_pages[0] + IDCNF_POPPASS);
						DisableObj(identity_conf, idcnf_pages[0] + IDCNF_POPSERVER);
						DisableObj(identity_conf, idcnf_pages[0] + IDCNF_SMTPSERVER);
						}		
					else
						{
						EnableObj(identity_conf, idcnf_pages[0] + IDCNF_ML_ADDR);
						EnableObj(identity_conf, idcnf_pages[0] + IDCNF_ML_ADDR2);
						EnableObj(identity_conf, idcnf_pages[0] + IDCNF_POPUSER);
						EnableObj(identity_conf, idcnf_pages[0] + IDCNF_POPPASS);
						EnableObj(identity_conf, idcnf_pages[0] + IDCNF_POPSERVER);
						EnableObj(identity_conf, idcnf_pages[0] + IDCNF_SMTPSERVER);
						}		
					redraw_object(identity_conf, idcnf_pages[0] + IDCNF_ML_ADDR);
					redraw_object(identity_conf, idcnf_pages[0] + IDCNF_ML_ADDR2);
					redraw_object(identity_conf, idcnf_pages[0] + IDCNF_POPUSER);
					redraw_object(identity_conf, idcnf_pages[0] + IDCNF_POPPASS);
					redraw_object(identity_conf, idcnf_pages[0] + IDCNF_POPSERVER);
					redraw_object(identity_conf, idcnf_pages[0] + IDCNF_SMTPSERVER);
					break;
				}				
			}
		else if (IsSelected(tree, ID_TAB_POP))	/* Account page active ? */
			{
			switch(obj - idcnf_pages[1])
				{
				case IDCNF_DEFPOP:
					if (IsSelected(tree, obj))
						DisableObj(identity_conf, idcnf_pages[1] + IDCNF_POPPORT);
					else
						EnableObj(identity_conf, idcnf_pages[1] + IDCNF_POPPORT);
					redraw_object(identity_conf, idcnf_pages[1] + IDCNF_POPPORT);
					break;
				}
			}
		else if (IsSelected(tree, ID_TAB_SMTP))	/* Account page active ? */
			{
			switch(obj - idcnf_pages[2])
				{
				case IDCNF_SMTPDEFAULT:
					if (IsSelected(tree, obj))
						DisableObj(identity_conf, idcnf_pages[2] + IDCNF_SMTPPORT);
					else
						EnableObj(identity_conf, idcnf_pages[2] + IDCNF_SMTPPORT);
					redraw_object(identity_conf, idcnf_pages[2] + IDCNF_SMTPPORT);
					break;
					
				case IDCNF_AUTHNONE:
				case IDCNF_POPAUTH:
					DisableObj(identity_conf, idcnf_pages[2] + IDCNF_SMTPUSER);
					DisableObj(identity_conf, idcnf_pages[2] + IDCNF_SMTPPASS);
					redraw_object(identity_conf, idcnf_pages[2] + IDCNF_SMTPUSER);
					redraw_object(identity_conf, idcnf_pages[2] + IDCNF_SMTPPASS);
					break;
					
				case IDCNF_SMTPAUTH:
					EnableObj(identity_conf, idcnf_pages[2] + IDCNF_SMTPUSER);
					EnableObj(identity_conf, idcnf_pages[2] + IDCNF_SMTPPASS);
					redraw_object(identity_conf, idcnf_pages[2] + IDCNF_SMTPUSER);
					redraw_object(identity_conf, idcnf_pages[2] + IDCNF_SMTPPASS);
					break;
				}
			}
		else if (IsSelected(tree, ID_TAB_ADDR))	/* Account page active ? */
			{
			switch(obj - idcnf_pages[4])
				{
				case IDCNF_SIG_BUTTON:
				case IDCNF_SIGDROP:
					{
					int x, y;
					char sidentity[21];
					
					objc_offset(tree, IDCNF_SIG_BUTTON + idcnf_pages[4], &x, &y);
					if (select_signature(x, y, tree[IDCNF_SIGDROP + idcnf_pages[4]].ob_x + 
															 tree[IDCNF_SIGDROP + idcnf_pages[4]].ob_width - 
															 tree[IDCNF_SIG_BUTTON + idcnf_pages[4]].ob_x,
															 sidentity))
						{
						if (!sidentity[0])
							strcpy(tree[IDCNF_SIG_BUTTON + idcnf_pages[4]].ob_spec.free_string, defid_string);
						else
							strcpy(tree[IDCNF_SIG_BUTTON + idcnf_pages[4]].ob_spec.free_string, sidentity);
						redraw_object(tree, IDCNF_SIG_BUTTON + idcnf_pages[3]);
						}
					break;
					}
				}
			}
	}
}

/* --------------------------
	 | Identity configuration |
	 -------------------------- */
void config_identity(int num)
{
DEBUG();
	/* Combine multipage dialogs */
if (identity_conf == NULL)
	{
	int i;
	OBJECT *tree, *tab_x;
	static int idopttabs[] = {ID_ACCOUNT, ID_POP, ID_SMTP, ID_EDIT, ID_ADDR};
	
	rsrc_gaddr(R_TREE, IDENTITY_CFG, &tree);
	for (i = 0; i < 5; i++)
		{
		rsrc_gaddr(R_TREE, idopttabs[i],	&tab_x);
		if (!i)
			idcnf_pages[i] = CombineDialogs(&identity_conf, tree, tab_x);
		else
			idcnf_pages[i] = CombineDialogs(&identity_conf, identity_conf, tab_x);
		}

	for (i =  8; i <= 14; i++)	idcnf_ids[i] += idcnf_pages[0];		
	for (i = 15; i <= 23; i++)	idcnf_ids[i] += idcnf_pages[1];		
	for (i = 24; i <= 31; i++)	idcnf_ids[i] += idcnf_pages[2];		
	for (i = 32; i <= 46; i++)	idcnf_ids[i] += idcnf_pages[3];		
	for (i = 47; i <= 52; i++)	idcnf_ids[i] += idcnf_pages[4];		
	}

cfg_identity = num;
SetTabPage(identity_conf, ID_CFG_TAB, idcnf_pages[0], ID_TAB_ACCOUNT, 5, idcfg_tabids);

open_dialog(identity_conf, NULL, iconified_tree);
dlgset_help_info(identity_conf, helpfile, idcnf_ids, "IDCNF_");
fill_identity_conf(num);
set_active(identity_conf, idconfig_handler, TRUE, 0);
}

/* --------------------------
   | Handle Identity events |
   -------------------------- */
static void cdecl identity_handler (OBJECT *tree, int obj)
{
DEBUG();
switch(obj)
	{
	case IDENTITYLIST:
		{
		int ox, oy, pos;
		EVNTDATA ev;
		
		objc_offset(tree, IDENTITYLIST, &ox, &oy);
		graf_mkstate(&ev);
		
		pos = (ev.y - oy) / tx_height + identity_page;
		if (pos >= 0 && pos < identity_count)
			{
			if (selected_identity == -1)
				{
				EnableObj(tree, CFG_IDENTITY);
				EnableObj(tree, DEL_IDENTITY);
				EnableObj(tree, DEF_IDENTITY);
				redraw_object(tree, CFG_IDENTITY);
				redraw_object(tree, DEL_IDENTITY);
				redraw_object(tree, DEF_IDENTITY);
				}
			selected_identity = pos;
			}
		else
			{
			if (selected_identity != -1)
				{
				DisableObj(tree, CFG_IDENTITY);
				DisableObj(tree, DEL_IDENTITY);
				DisableObj(tree, DEF_IDENTITY);
				redraw_object(tree, CFG_IDENTITY);
				redraw_object(tree, DEL_IDENTITY);
				redraw_object(tree, DEF_IDENTITY);
				}
			selected_identity = -1;
			}
		redraw_object(tree, IDENTITYLIST);
		
		break;
		}

	case NEW_IDENTITY:
		config_identity(-1);
		break;
		
	case CFG_IDENTITY:
		config_identity(selected_identity);
		break;
		
	case DEL_IDENTITY:
		{
		if (default_identity > selected_identity)
			default_identity--;
		else
			if (default_identity == selected_identity)
				default_identity = -1;
				
		if (cfg_identity == selected_identity && is_open(identity_conf))
			close_dialog(identity_conf, TRUE);
			
		if (cfg_identity > selected_identity) cfg_identity--;
		if (identity_count > 1 && selected_identity < identity_count - 1)
			memmove(identities + selected_identity, identities + selected_identity + 1,
							sizeof(IDENTITY) * (identity_count - selected_identity - 1)) ;
		else
			free(identities);	
		identity_count--;
		identity_chg = TRUE;

		DisableObj(tree, CFG_IDENTITY);
		DisableObj(tree, DEL_IDENTITY);
		DisableObj(tree, DEF_IDENTITY);
		redraw_object(tree, CFG_IDENTITY);
		redraw_object(tree, DEL_IDENTITY);
		redraw_object(tree, DEF_IDENTITY);
		
		selected_identity = -1;
		redraw_object(tree, IDENTITYLIST);
		break;
		}
		
	case DEF_IDENTITY:
		default_identity = selected_identity;
		identity_chg = TRUE;
		redraw_object(tree, IDENTITYLIST);
		break;

	case CLOSE_IDENTITY:
		close_dialog(tree, TRUE);
		if (identity_chg)
			save_identities();
		break;
	}
}


/* ----------------------------
   | User object address book |
   ---------------------------- */
int cdecl ident_obj(PARMBLK *parm)
{
int xy[4], i, y, lines;
int dummy;

DEBUG();
obj_clip(parm, TRUE);

vst_height(vdi_handle, default_fsize, &dummy, &dummy, &dummy, &dummy);
vswr_mode(vdi_handle, MD_REPLACE);
vsf_interior(vdi_handle, FIS_HOLLOW);
xy[0] = parm->pb_x;
xy[1] = parm->pb_y;
xy[2] = parm->pb_x + parm->pb_w - 2;
xy[3] = parm->pb_y + parm->pb_h - 1;
vr_recfl(vdi_handle, xy);

lines = (parm->pb_tree[IDENTITYLIST].ob_height + tx_height - 1) / tx_height;

y = xy[1];
for (i = identity_page; i < identity_count && i < identity_page + lines; i++, y += tx_height)
	{
	if (default_identity == i)
		icon_copy(symbols, 10, (smallfont ? 8 : 16) * 4, 
					 		parm->pb_x + 1, y, 
					 		10, smallfont ? 8 : 16);
					 		
	v_gtext(vdi_handle, parm->pb_x + 2 * tx_width, y, identities[i].name);

	if (selected_identity == i)
		{
		vswr_mode(vdi_handle, MD_XOR);
		vsf_interior(vdi_handle, FIS_SOLID);
		vsf_color(vdi_handle, 1);

		xy[0] = parm->pb_x;
		xy[1] = y;
		xy[2] = parm->pb_x + parm->pb_w - 2;
		xy[3] = y + tx_height - 1;
		vr_recfl(vdi_handle, xy);
		vswr_mode(vdi_handle, MD_TRANS);
		}
	}

obj_clip(parm, FALSE);
vswr_mode(vdi_handle, MD_REPLACE);

return 0;
}

/* -------------------
   | Edit identities |
   ------------------- */
void edit_identities(void)
{
OBJECT *identitylist;

DEBUG();
rsrc_gaddr(R_TREE, IDENTITIES, &identitylist);

if (!is_open(identitylist))
	{
	if (identitylist[IDENTITYLIST].ob_type != G_USERDEF)
		{
		identitylist[IDENTITYLIST].ob_type = G_USERDEF;
		identitylist[IDENTITYLIST].ob_spec.userblk = &ident_ublk;
	
		identity_page = 0;
		selected_identity = -1;
		DisableObj(identitylist, CFG_IDENTITY);
		DisableObj(identitylist, DEL_IDENTITY);
		DisableObj(identitylist, DEF_IDENTITY);
		}
		
	align_slider_objects(identitylist, ID_SLIDER, ID_UP, ID_SLDBKG, ID_SLDBOX, ID_DOWN);
	set_slider_objects(identitylist, ID_SLDBKG, ID_SLDBOX, identity_count, identity_page, 
										 identitylist[IDENTITYLIST].ob_height / tx_height, FALSE);	
	}

open_dialog(identitylist, NULL, iconified_tree);
dlgset_help_info(identitylist, helpfile, ilist_ids, "ILIST_");
set_active(identitylist, identity_handler, TRUE, 0);
}

/* ------------------------
   | Load identities file |
   ------------------------ */
void load_identities(void)
{
long id_fl;
char buf[256];

DEBUG();
if (identity_count)
	free(identities);
	
base_path(buf);
strcat(buf, "IDENTITY.DAT");

id_fl = Fopen(buf, FO_READ);
if (id_fl > 0)
	{
	long ver;
	int i;
	
	Fread((int)id_fl, sizeof(ver), &ver);
	Fread((int)id_fl, sizeof(identity_count), &identity_count);
	if (identity_count)
		{
		identities = (IDENTITY *)malloc(sizeof(IDENTITY) * 
																		   identity_count);
		if (ver < VERSION_ID201)
			{
			for (i = 0; i < identity_count; i++)
				{
				Fread((int)id_fl, sizeof(IDENTITY) - 86, identities + i);
				identities[i].smtpuser[0] = 0;
				identities[i].smtppass[0] = 0;
				}
			}
		else
			Fread((int)id_fl, sizeof(IDENTITY) * identity_count, identities);

		for (i = 0; i < identity_count; i++)
			identities[i].identifier = i;
		next_identifier = identity_count;
		}
	Fread((int)id_fl, sizeof(default_identity), &default_identity);
		
	Fclose((int)id_fl);
	}
else
	{
	identities = (IDENTITY *)malloc(sizeof(IDENTITY));
	memcpy(identities, &defaultentry, sizeof(IDENTITY));

	identities[0].identifier = 0;
	identities[0].pass_coded = mail_options._pass_coded;
	identities[0].download_immediately = mail_options._download_immediately;
	identities[0].delete_rcvd = mail_options._delete_rcvd;
	identities[0].maxsize = mail_options._maxsize;

	strcpy(identities[0].username, mail_options._username);
	strcpy(identities[0].organization, mail_options._organization);
	identities[0].eight_allowed = mail_options._eight_allowed;
	identities[0].mime_allowed = mail_options._mime_allowed;
	identities[0].send_immediately = mail_options._send_immediately;
	identities[0].auth_required = mail_options._popauth_required;

	identities[0].word_wrap = mail_options._word_wrap;
	identities[0].editwidth = mail_options._editwidth;
	identities[0].last_read = mail_options._confirm_read;
	identities[0].last_rcpt = mail_options._confirm_rcpt;
	identities[0].last_priority = mail_options._priority;

	identities[0].smtpuser[0] = 0;
	identities[0].smtppass[0] = 0;

	default_identity = 0;							
	identity_count = 1;
	next_identifier = 1;
	}
	
identity_chg = FALSE;
}

/* ------------------------
   | Save identities file |
   ------------------------ */
void save_identities(void)
{
long id_fl;
char buf[256];

DEBUG();
if (!identity_chg) return;

base_path(buf);
strcat(buf, "IDENTITY.DAT");
id_fl = Fcreate(buf, 0);
if (id_fl > 0)
	{
	long ver = VERSION_ID201;
	
	Fwrite((int)id_fl, sizeof(ver), &ver);
	Fwrite((int)id_fl, sizeof(identity_count), &identity_count);
	if (identity_count)
		{
		Fwrite((int)id_fl, sizeof(IDENTITY) * identity_count,
					 identities);
		}
	Fwrite((int)id_fl, sizeof(default_identity), &default_identity);
	Fclose((int)id_fl);
	identity_chg = FALSE;
	}
}

/* ---------------------------
   | identity selector popup |
   --------------------------- */
int select_identity(int x, int y, IDENTITY *id)
{
int i, page, obj;
OBJECT *ident_pop;
static int stext[] = {SELPOP_1, SELPOP_2, SELPOP_3, SELPOP_4, SELPOP_5, SELPOP_6};
char inames[6][21];
GRECT fr;
int last_obj = 0;

DEBUG();
rsrc_gaddr(R_TREE, SELPOP, &ident_pop);

form_center(ident_pop, &fr);
ident_pop[0].ob_x = x; ident_pop[0].ob_y = y;

for (i = 0; i < 6; i++)
	{
	ident_pop[stext[i]].ob_spec.tedinfo->te_ptext = inames[i];
	ident_pop[stext[i]].ob_width = ident_pop[SELPOP_SBK].ob_x - 1;
	inames[i][0] = 0;
	}

page = 0;
align_slider_objects(ident_pop, SELPOP_SBK, SELPOP_UP, SELPOP_SLD, SELPOP_DRG, SELPOP_DWN);

wait_mouse_release();

for (i = 0; i < 6 && i < identity_count; i++)	
	strcpy(inames[i], identities[page + i].name);

set_slider_objects(ident_pop, SELPOP_SLD, SELPOP_DRG, identity_count, page, 6, FALSE);	

open_frameless(ident_pop, x - 1, y - 1);
wind_update(BEG_UPDATE);

do
	{
	do
		{	
		check_mouse();

		obj = objc_find(ident_pop, ROOT, MAX_DEPTH, evnt.x, evnt.y);
		if (last_obj && last_obj != obj)
			{
			change_object(ident_pop, last_obj,	ident_pop[last_obj].ob_state & ~SELECTED);
			last_obj = 0;
			}
		
		if (obj > -1 && (ident_pop[obj].ob_flags & SELECTABLE) && 
				!(ident_pop[obj].ob_state & DISABLED) &&
				ident_pop[obj].ob_spec.tedinfo->te_ptext[0] && last_obj != obj)
			{
			change_object(ident_pop, obj,	ident_pop[obj].ob_state | SELECTED);
			last_obj = obj;
			}
		}while(!evnt.bstate);
	
	switch(obj)
		{
		case SELPOP_UP:
			if (page > 0)
				{
				page--;
				delay(100);
				}
			break;
			
		case SELPOP_DWN:
			if (page < identity_count - 6)
				{
				page++;
				delay(100);
				}
			break;
		
		case SELPOP_SLD:
			{
			int ox, oy;
			
			objc_offset(ident_pop, FONT_DRG, &ox, &oy);
			if (evnt.y < oy)
				{
				page -= 6;
				if (page < 0) page = 0;
				}
			else
				{
				page += 6;
				if (page > identity_count - 6)
					page = identity_count - 6;
				}
			delay(100);
			break;
			}
			
		case SELPOP_DRG:
			{
			int pos = graf_slidebox(ident_pop, SELPOP_SLD, SELPOP_DRG, 1);
			page = (int)(pos * (long)(identity_count - 6) / 1000);
			break;
			}
		}
		
	for (i = 0; i < 6 && i < identity_count; i++)	
		{
		strcpy(inames[i], identities[page + i].name);
		redraw_object(ident_pop, stext[i]);
		}
	set_slider_objects(ident_pop, SELPOP_SLD, SELPOP_DRG, num_fonts, page, 6, TRUE);	

	}while(obj == SELPOP_UP || obj == SELPOP_DWN ||
				 obj == SELPOP_SLD || obj == SELPOP_DRG);

if (last_obj)
	change_object(ident_pop, last_obj,	ident_pop[last_obj].ob_state & ~SELECTED);

wait_mouse_release();
close_dialog(ident_pop, TRUE);
	
wind_update(END_UPDATE);
	
if (obj > -1)
	{
	for (i = 0; i < 6; i++)
		if (obj == stext[i])
			break;
			
	if (i < 6)
		{
		memcpy(id, identities + page + i, sizeof(IDENTITY));
		return TRUE;
		}
	}
return FALSE;
}

/* ------------------------
   | Opem SMTP connection |
   ------------------------ */
int open_idsmtp(int idx)
{
struct sockaddr_in remote;
struct servent *serv;
struct hostent *host;
int e, ret, auth_type;
char buf[256];
char *smtpserver, *popserver;
char *popuser, *poppass;
char zwuser[43], zwpass[43];
IDENTITY *identity;

DEBUG();
identity = identities + idx;

blockPoll(TRUE);
if (identity->use_providerdata)
	{
	smtpserver 	= cfg_options.smtpserver;
	popserver 	= cfg_options.popserver;
	popuser			= pop_username;
	poppass			= pop_password;
	}
else
	{
	strcpy(zwuser, identity->popuser);
	strcpy(zwpass, identity->mailpass);
	
	smtpserver 	= identity->smtpserver;
	popserver 	= identity->popserver;
	popuser			= zwuser;
	poppass			= zwpass;
	}

if (identity->auth_required == 1)
	{
	int s;

	if (!poppass[0])
		if (!pop_login(popuser, poppass))
			{
			blockPoll(FALSE);
			return FALSE;
			}
			
	start_info(mailchk_text, popconn_text);	
	s = connect_pop(popserver, identity->pop_port, popuser, poppass, identity->pass_coded);
	if (!s)
		{
		show_error(FALSE);
		blockPoll(FALSE);
		return FALSE;
		}
		
	disconnect_pop(s);
	}
else
	start_info(send_text, connect_msg);	

stopPoll();
notify = FALSE;
dsn = FALSE;
auth_type = 0;

remote.sin_family = AF_INET;

if (identity->smtp_port == -1)
	{
	serv = getservbyname("smtp", "tcp");
	if (!serv)
		{
		form_alert(1, smtp_unk_msg);
		end_info();
		blockPoll(FALSE);
		return FALSE;
		}
	remote.sin_port = serv->s_port;
	}
else
	remote.sin_port = identity->smtp_port;
	
if (isdigit(smtpserver[0]))
	remote.sin_addr.s_addr = inet_addr(smtpserver);
else
	{
	int	state = async_gethostbyname(smtpserver);
	while (state)
		{
		long stime = clock();
		if (!info_stop())
			{
			doPoll();
			if (clock() - stime >= CLK_TCK)
				state = async_gethostbyname(smtpserver);
			}
		else
			{
			end_info();
			blockPoll(FALSE);
			return FALSE;
			}
		}
				
	do
		{
		host = async_queryname();
		if (info_stop())
			{
			async_discard();
			blockPoll(FALSE);
			end_info();
			return FALSE;
			}
					
		}while((long)host == -EINPROGRESS);
	async_discard();
	
	/*	host = gethostbyname(smtpserver);*/
	if (!host)
		{
		form_alert(1, unknownserver);
		end_info();
		blockPoll(FALSE);
		return FALSE;
		}
	memcpy(&remote.sin_addr.s_addr, host->h_addr, host->h_length);
	}
	
smtp_s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
if (!smtp_s)
	{
	form_alert(1, sockfail_msg);
	blockPoll(FALSE);
	end_info();
	return FALSE;
	}

if (get_tcpversion() >= 0x00010007L)
	sockfcntl(smtp_s, SO_SETFL, SO_O_NONBLOCK);
	
ret = connect(smtp_s, (struct sockaddr *)&remote, (int)sizeof(remote));
while(ret != 0 && (errno == -EINPROGRESS || errno == -EALREADY))
	{
	if (info_stop())
		{
		close_socket(smtp_s);
		end_info();
		blockPoll(FALSE);
		smtp_s = 0;
		return 0;
		}
	ret = connect(smtp_s, NULL, 0);
	}
if (ret && errno == -EISCONN)
	ret = 0;

if (get_tcpversion() >= 0x00010007L)
	sockfcntl(smtp_s, SO_SETFL, 0);

if (!ret)
	{
	e = get_response(smtp_s, buf);
	if (e < 200 || e > 399)
		{ 
		close_smtp(); 	
		blockPoll(FALSE);
		return FALSE; 
		}

	change_info(auth_text);	
	
	send_cmd(smtp_s, "EHLO", smtpserver, NULL, NULL);
	e = get_response(smtp_s, buf);
	if (e < 200 || e > 399)
		{
		send_cmd(smtp_s, "HELO", smtpserver, NULL, NULL);
		e = get_response(smtp_s, buf);
		if (e < 200 || e > 399)
			{ 
			store_error("SMTP", e, buf);
			close_smtp(); 	
			blockPoll(FALSE);
			return FALSE; 
			}
		notify = FALSE;
		}
	else
		if (e == 250)
			{
			while (buf[3] == '-')
				{
				if (!strncmp(buf, "250-DSN", 7))
					{
					dsn = TRUE;
					notify = TRUE;
					}
				if (!strncmp(buf, "250-AUTH", 8))
					{
					if (strstr(buf + 8, "PLAIN") != NULL)
						if (auth_type < 1) auth_type = 1;
					if (strstr(buf + 8, "LOGIN") != NULL)
						if (auth_type < 2) auth_type = 2;
					if (strstr(buf + 8, "CRAM-MD5") != NULL)
						if (auth_type < 3) auth_type = 3;
					}
					
				e = get_response(smtp_s, buf);
				if (e < 200 || e > 399)
					{ 
					store_error("SMTP", e, buf);
					close_smtp(); 	
					blockPoll(FALSE);
					return FALSE; 
					}
				}
				
			if (buf[3] == ' ')
				{
				if (!strncmp(buf, "250 DSN", 7))
					{
					dsn = TRUE;
					notify = TRUE;
					}
				if (!strncmp(buf, "250 AUTH", 8))
					{
					if (strstr(buf + 8, "PLAIN") != NULL)
						if (auth_type < 1) auth_type = 1;
					if (strstr(buf + 8, "LOGIN") != NULL)
						if (auth_type < 2) auth_type = 2;
					if (strstr(buf + 8, "CRAM-MD5") != NULL)
						if (auth_type < 3) auth_type = 3;
					}
				}
			}

	if (identity->auth_required == 2 && auth_type)
		if (!smtp_auth(smtp_s, identity, auth_type))
			{
			change_info(disconnect_msg);	
			close_socket(smtp_s);
			end_info();
			blockPoll(FALSE);
			smtp_s = 0;

			return FALSE;
			}
	}
else
	{
	form_alert(1, smtpprob_msg);
	
	change_info(disconnect_msg);	
	close_socket(smtp_s);
	end_info();
	blockPoll(FALSE);
	smtp_s = 0;

	return FALSE;
	}
blockPoll(FALSE);
return TRUE;
}

/* ------------------------
   | Send mail for a SMTP |
   ------------------------ */
static int send_id_smtp(int idx, int all)
{
int i, s, to_box, box;
char bufs[51 * 2 + MAXFILE * 2 + 1];
int wx, wy, ww, wh, err;

DEBUG();
box = box_number("QUEUE");

for (i = 0; i < box_manager[box].box_entries; i++)
	if (!strcmp(box_manager[box].index[i].identity, identities[idx].name) &&
			((box_manager[box].index[i].sel & 1) != 0 || all == TRUE))
		break;

if (i == box_manager[box].box_entries)
	return TRUE;
	
base_path(bufs);
strcat(bufs, boxes[box].file);
strcat(bufs, "\\");

if (!open_idsmtp(idx))
	{
	show_error(TRUE);
	return FALSE;
	}
	
err = FALSE;
for (i = 0; i < box_manager[box].box_entries; i++)
	if (!strcmp(box_manager[box].index[i].identity, identities[idx].name) &&
			((box_manager[box].index[i].sel & 1) != 0 || all == TRUE))
		{
		strcpy(strrchr(bufs, '\\') + 1, box_manager[box].index[i].file);
		if (!strncmp(box_manager[box].index[i].subject, "RC: ", 4))
			{
			s = 1;
			mail_send(bufs);
			
			if (!Fdelete(bufs))
				{
				if (box_manager[box].index[i].sel & 1)
					{
					box_manager[box].list_marked -= box_manager[box].index[i].body_len;
					box_manager[box].list_marks--;
					}
							
				delete_index(box_manager[box].index[i].file, box);
				}
			}
		else
			{
			if (mail_send(bufs))
				{
				if (mail_options.remove_attaches)
					remove_attaches(box, i);
				to_box = box_number("SENT");
				}
			else
				{
				err = TRUE;
				to_box = box_number("SENTERR");
				}
				
			if (box_manager[box].index[i].sel & 1)
				{
				box_manager[box].index[i].sel &= ~1;
				box_manager[box].list_marked -= box_manager[box].index[i].body_len;
				box_manager[box].list_marks--;
				}
				
			s = move_onemail(box, to_box, i);
			}
			
		if (s) 
			{
			if (box_manager[box].list_wind > -1)
				{
				wind_get(box_manager[box].list_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
				redraw_listwindow(box, wx, wy, ww, wh);
				}
	
			if (box_manager[to_box].list_wind > -1) 
				{
				wind_get(box_manager[to_box].list_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
				redraw_listwindow(to_box, wx, wy, ww, wh);
				}
			set_listinfo(to_box);
			set_listinfo(box);
			i--;
			}
		}
close_smtp();

if (err)
	show_error(TRUE);

return TRUE;
}

/* --------------------------------
   | Send mails for this identity |
   -------------------------------- */
int send_idqueue(char *identity, int all)
{
int i;

DEBUG();
if (!check_connection(2, identity, all))
	return FALSE;

if (!identity[0])
	{
	clear_infostop();
	for (i = 0; i < identity_count; i++)
		if (identities[i].check)
			if (!send_id_smtp(i, all))
				return FALSE;

	return !info_stop();
	}
else
	{
	for (i = 0; i < identity_count; i++)
		if (!strcmp(identities[i].name, identity))
			return send_id_smtp(i, all);

	return FALSE;
	}
}

/* ----------------------------
   | Check pop for a identity |
   ---------------------------- */
static int check_id_pop(int idx, int flg)
{
char *popserver;
char *popuser, *poppass;
char zwuser[43], zwpass[43];
IDENTITY *identity;

DEBUG();
identity = identities + idx;

if (identity->use_providerdata)
	{
	popserver 	= cfg_options.popserver;
	popuser			= pop_username;
	poppass			= pop_password;
	}
else
	{
	strcpy(zwuser, identity->popuser);
	strcpy(zwpass, identity->mailpass);
	
	popserver 	= identity->popserver;
	popuser			= zwuser;
	poppass			= zwpass;
	}

switch (scan_pop(popserver, identity->pop_port, popuser, poppass, identity->pass_coded, 
								 flg, identity->delete_rcvd, identity->maxsize, identity->name))
	{
	case TRUE:	
		return TRUE;
		
	case FALSE:
		form_alert(1, scanerr_msg);
		return FALSE;
		
	case -2:	
		return TRUE;
	}
return FALSE;
}

/* -------------------------------
   | Check pop for this identity |
   ------------------------------- */
int check_id(char *identity, int flg)
{
int i, immediate;

DEBUG();
if (!check_connection(1, identity, flg))
	return FALSE;

clear_pop();

if (!identity[0])
	{
	clear_infostop();
	
	immediate = TRUE;
	for (i = 0; i < identity_count && !info_stop(); i++)
		if (identities[i].check)
			{
			int imme = (flg != -1 ? flg : identities[i].download_immediately);
			if (!imme)
				immediate = imme;
				
			if (!check_id_pop(i, imme))
				return FALSE;
			}
	
	if (!immediate)
		pop_window();
	return !info_stop();
	}
else
	{
	for (i = 0; i < identity_count; i++)
		if (!strcmp(identities[i].name, identity))
			{
			immediate = (flg != -1 ? flg : identities[i].download_immediately);
			if (check_id_pop(i, immediate))
				{
				if (!immediate)
					pop_window();
				return TRUE;
				}			
			}
			
	return FALSE;
	}
}

/* -------------------
   | Select a folder |
   ------------------- */
int DialogIdentitySelect(char *buf, int x, int y, int w, char *first)
{
char **strings;
char *buffer, *ptr;
int i, ret, num;

DEBUG();
num = getNumberOfIdentities();
strings = (char **)malloc(sizeof(char *) * (num + 2));
if (!strings)
	return FALSE;
	
ptr = buffer = (char *)malloc(num * 41);
if (!buffer)
	{
	free(strings);
	return FALSE;
	}

strings[0] = first;
for (i = 0; i < num; i++)
	{
	strings[i + 1] = ptr;
	strcpy(ptr, getIdentityName(i));
	ptr += 41;
	}
strings[i + 1] = NULL;

ret = free_droplist(x, y, w, strings);
if (ret != -1)
	{
	if (!ret)
		buf[0] = 0;
	else
		strcpy(buf, strings[ret]);
	}

free(buffer);	
free(strings);

return ret != -1;
}
