/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 *
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 *
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 *
 */

#include "extern.h"

PALETTE_FILE palette_header;
unsigned char cpalette[16 * 3 + 256 * 3];

typedef struct reso_inf
											{
											int cols, smallfont;
											}RESO_INF;

typedef struct cpal_inf
											{
											char *desc, *file;
											int  cpal;
											}COLL_INF;

static RESO_INF reso_tab[] = {{  2, 0},
										 	   	    {  2, 1},
												      {  4, 0},
												      {  4, 1},
												      { 16, 0},
												      { 16, 1},
												  	  {256, 0},
												  	  {256, 1},
												 	    {  0, 0},
												 	    {  0, 1}};

static unsigned char *butcols, nearcols[256];
unsigned char col2_4[] = {0xFF, 0xFF, 0xFF, 0, 0, 0, 0, 0, 0, 0, 0, 0};
COLL_INF *collections, *toolcollections, *boxcollections;
int collection_cnt = 0, toolcollection_cnt = 0, boxcollection_cnt = 0;
char **cpals, **toolcpals, **boxcpals;
int cpal_cnt = 0, toolcpal_cnt = 0, boxcpal_cnt = 0;

int colors_stored[256][4];
long mult[1001];

/* Local prototypes */
void fill_pals(OBJECT *tree, int base, int collection_cnt, COLL_INF *collections);
int select_buttons(int collection_cnt, COLL_INF *collections);
void load_buttons(char **iconmem, PALETTE_FILE *icon_palheader,
									int *num, char *bfile, int collection_cnt, COLL_INF *collections, char **cpals);

/* --------------------------
   | Init color cache array |
   -------------------------- */
void init_colorcache(void)
{
int i, maincol;

DEBUG();
for (i = 0; i < colours; i++)
	{
	vq_color(vdi_handle, i, 0, colors_stored[i]);

	if (abs(colors_stored[i][0] - colors_stored[i][1]) < 100 &&
			abs(colors_stored[i][1] - colors_stored[i][2]) < 100 &&
			abs(colors_stored[i][0] - colors_stored[i][2]) < 100)
		maincol = 7;
	else
		maincol = 0;

	if (!maincol)
		{
		if (colors_stored[i][0] > colors_stored[i][1] && colors_stored[i][0] > colors_stored[i][2])
			maincol = 1;
		else
			if (colors_stored[i][1] > colors_stored[i][0] && colors_stored[i][1] > colors_stored[i][2])
				maincol = 2;
			else
				if (colors_stored[i][2] > colors_stored[i][0] && colors_stored[i][2] > colors_stored[i][1])
					maincol = 4;

		if (abs(colors_stored[i][0] - colors_stored[i][1]) < 100 && (maincol & 3))
			maincol |= 3;

		if (abs(colors_stored[i][0] - colors_stored[i][2]) < 100 && (maincol & 5))
			maincol |= 5;

		if (abs(colors_stored[i][1] - colors_stored[i][2]) < 100 && (maincol & 6))
			maincol |= 6;
		}

	colors_stored[i][3] = maincol;
	}

for (i = 0; i <= 1000; i++)
	mult[i] = (long)i * i;
}

/* ---------------------------
   | Find nearest RGB colour |
   --------------------------- */
int find_colour(int *srgb)
{
int i, smaincol, maincol;
int best;
long diff, d;
int r, g, b, ds1, ds2, ds3, d2;

DEBUG();
for (i = 0; i < 3; i++)
	srgb[i] = (int)(srgb[i] * 1000L / 255);

ds1 = srgb[0] - srgb[1];
ds2 = srgb[0] - srgb[2];
ds3 = srgb[1] - srgb[2];

if (abs(ds1) < 100 && abs(ds2) < 100 &&	abs(ds3) < 100)
	smaincol = 7;
else
	smaincol = 0;

if (!smaincol)
	{
	if (srgb[0] > srgb[1] && srgb[0] > srgb[2])
		smaincol = 1;
	else
		if (srgb[1] > srgb[0] && srgb[1] > srgb[2])
			smaincol = 2;
		else
			if (srgb[2] > srgb[0] && srgb[2] > srgb[1])
				smaincol = 4;

	if (abs(ds1) < 100 && (smaincol & 3))
		smaincol |= 3;

	if (abs(ds2) < 100 && (smaincol & 5))
		smaincol |= 5;

	if (abs(ds3) < 100 && (smaincol & 6))
		smaincol |= 6;
	}

diff = 0x7FFFFFFFL;
for (i = 0; i < colours; i++)
	{
	r = colors_stored[i][0];
	g = colors_stored[i][1];
	b = colors_stored[i][2];
	maincol = colors_stored[i][3];

	d  = abs(ds1 - (r - g)) + abs(ds2 - (r - b)) + abs(ds3 - (g - b));
	d2 = srgb[0] - r;
	d += mult[abs(d2)];
	d2 = srgb[1] - g;
	d += mult[abs(d2)];
	d2 = srgb[2] - b;
	d += mult[abs(d2)];

	if (d < diff && maincol == smaincol)
		{
		diff = d;
		best = i;
		if (!d)
			break;
		}
	}

if (diff == 0x7FFFFFFFL)
	{
	for (i = 0; i < colours; i++)
		{
		int x;

		r = colors_stored[i][0];
		g = colors_stored[i][1];
		b = colors_stored[i][2];

		x = (int) ((srgb[0] - r) * 2 * 256L / 1000);
		d  = mult[abs(x)];
		x = (int) ((srgb[1] - g) * 3 * 256L / 1000);
		d  += mult[abs(x)];
		x = (int) ((srgb[2] - b) * 1 * 256L / 1000);
		d  += mult[abs(x)];

		if (d < diff)
			{
			diff = d;
			best = i;
			}
		}
	}

return best;
}

/* ----------------------------
   | Generate mapping colours |
   ---------------------------- */
void gen_colours(void)
{
int i;
int srgb[3], black;
unsigned char *ptr;

DEBUG();
if (dither_colours <= 4)
	{
	nearcols[0] = 0;
	nearcols[1] = nearcols[2] = nearcols[3] = 1;
	}
else
	{
	srgb[0] = srgb[1] = srgb[2] = 0;
	black = find_colour(srgb);

	ptr = butcols;
	for (i = 0; i < dither_colours; i++)
		{
		srgb[0] = *ptr++;
		srgb[1] = *ptr++;
		srgb[2] = *ptr++;
		if (srgb[0] != 0 || srgb[1] != 0 || srgb[2] != 0)
			nearcols[i] = find_colour(srgb);
		else
			nearcols[i] = black;
		}
	}
}

/* --------------------------
   | Gen interleaved bitmap |
   -------------------------- */
void gen_interleave(long size, unsigned char *img)
{
unsigned planmap[8], *imgp, c, mask, *pp;
int x, y, shift, padd;
unsigned char c2;
long p;

DEBUG();
gen_colours();

mask = (1 << planes) - 1;
imgp = (unsigned *)img;
padd = 16 / (8 / planes);
p = 0;
while (p < size)
	{
	shift = -1;

	for (y = 16; y > 0; y--)
		{
		if (shift < 0)
			{
			c = *img++;
			shift = 8 - planes;
			}

		c2 = tr_cols[ nearcols[(c >> shift) & mask] ];
		shift -= planes;

		x = planes;
		pp = planmap;
		do
			{
			*pp <<= 1;
			*pp |= c2 & 1;
			c2 >>= 1;
			pp++;
			}while (--x);
		}

	x = 0;
	while (x < planes)
		*imgp++ = planmap[x++];

	p += padd;
	}
}

/* --------------------------
   | Gen interleaved bitmap |
   -------------------------- */
void gen_pixels(long size, unsigned char *img)
{
long p;
int m, shift;
unsigned char c1, c2;

DEBUG();
gen_colours();

m = 0xFF >> (8 - planes);
p = 0;
while (p < size)
	{
	c1 = *img;
	c2 = 0;
	shift = 8 - planes;
	while(shift >= 0)
		{
		c2 <<= planes;
		c2 |= tr_cols[nearcols[(c1 >> shift) & m]];
		shift -= planes;
		}
	*img = c2;
	img++;
	p++;
	}
}

/* ------------------------
   | Gen truecolor bitmap |
   ------------------------ */
unsigned char *gen_truecolor(long *sz, unsigned char *img)
{
long p, size;
unsigned *c, *cd, zw;

DEBUG();
if (truecolor_type == 1 || truecolor_type == 2) return img;

size = *sz;
switch (truecolor_type)
	{
	case 3:
	case 9:
    *sz = size * 3 / 2;
    img = (unsigned char *)realloc(img, *sz);
    cd = (unsigned *)(img + *sz);
		break;

	case 4:
	case 8:
	case 10:
    *sz = size * 2;
    img = (unsigned char *)realloc(img, *sz);
    cd = (unsigned *)(img + *sz);
		break;

	default:
		cd = (unsigned *)(img + size);
		break;
	}

c = (unsigned *)(img + size);
p = 0;
while (p < size)
	{
	--c;
	switch (truecolor_type)
		{
		case 0:		/* 5/5/5 Motorola */
			*(--cd) = ((*c & 0xFFC0) >> 1) | (*c & 0x001F);
			break;

/*		case 2:		* 5/6/5 Motorola *
			*(--cd) = ((*c & 0x03E0) << 1) | (*c & 0xFC1F);
			break;*/

		case 3:							/* 24 Bit */
			*(--((unsigned char *)cd)) = (*c & 0x001F) << 3;
			*(--((unsigned char *)cd)) = (*c & 0x07C0) >> 3;
			*(--((unsigned char *)cd)) = (*c & 0xF800) >> 8;
			break;

		case 4:							/* 32 Bit */
			*(--((unsigned char *)cd)) = (*c & 0x001F) << 3;
			*(--((unsigned char *)cd)) = (*c & 0x07C0) >> 3;
			*(--((unsigned char *)cd)) = (*c & 0xF800) >> 8;
			*(--((unsigned char *)cd)) = 0;
			break;

		case 5:							/* 5/5/5 Intel */
			zw = ((*c & 0xFFC0) >> 1) | (*c & 0x001F);
			*(--cd) = (zw >> 8) | (zw << 8);
			break;

		case 6:							/* 5/5/5 Falcon Intel */
			*(--cd) = (*c >> 8) | (*c << 8);
			break;

		case 7:							/* 5/6/5 Intel */
/*			zw = ((*c & 0x03E0) << 1) | (*c & 0xFC1F);*/
			zw = *c;
			*(--cd) = (zw >> 8) | (zw << 8);
			break;

		case 8:							/* 32Bit (0 Byte ending) */
			*(--((unsigned char *)cd)) = 0;
			*(--((unsigned char *)cd)) = (*c & 0x001F) << 3;
			*(--((unsigned char *)cd)) = (*c & 0x07C0) >> 3;
			*(--((unsigned char *)cd)) = (*c & 0xF800) >> 8;
			break;

		case 9:							/* 24 Bit Intel */
			*(--((unsigned char *)cd)) = (*c & 0xF800) >> 8;
			*(--((unsigned char *)cd)) = (*c & 0x07C0) >> 3;
			*(--((unsigned char *)cd)) = (*c & 0x001F) << 3;
			break;

		case 10:						/* 32Bit (0 Byte ending) */
			*(--((unsigned char *)cd)) = 0;
			*(--((unsigned char *)cd)) = (*c & 0xF800) >> 8;
			*(--((unsigned char *)cd)) = (*c & 0x07C0) >> 3;
			*(--((unsigned char *)cd)) = (*c & 0x001F) << 3;
			break;
		}

	p += 2;
	}

return img;
}

/* -----------------
   | Load an image |
   ----------------- */
char *load_images(char *file, int flg)
{
long fh, size, i, fh2;
char *mem;
char cachecopy[256], *cdot;

DEBUG();

strcpy(cachecopy, file);
cdot = strrchr(cachecopy, '.');
if (cdot > strrchr(cachecopy, '\\'))
  {
  int val;

  val = (dither_index << 5) + (smallfont << 4) +
        (!dither_colours ? truecolor_type : pixel_type);

  itoa(val, cdot + 1, 10);
  }
else
  cachecopy[0] = 0;

fh = Fopen(file, FO_READ);
if (fh < 0L)
	{
	if (flg)
		{
		form_alert(1, fileoerr);
		return NULL;
		}
	}

fh2 = Fopen(cachecopy, FO_READ);
if (fh2 > -1L)
  {
  if (mail_options.icon_cache)
    {
    long l;
		DOSTIME dtimecache, dtimeori;

		Fdatime(&dtimecache, (int)fh2, 0 );
		Fdatime(&dtimeori, (int)fh, 0 );

		if (dtimecache.date > dtimeori.date ||
				(dtimecache.date == dtimeori.date && dtimecache.time >= dtimeori.time))
			{
	  	Fread((int)fh2, sizeof(PALETTE_FILE), &palette_header);
  	  Fread((int)fh2, sizeof(l), &l);
	    mem = (char *)malloc(l);
  	  Fread((int)fh2, l, mem);

    	Fclose((int)fh2);
	    Fclose((int)fh);
  	  return mem;
  	  }
  	else
  		{
  		Fclose((int)fh2);
	    Fdelete(cachecopy);
  		}
    }
  else
    {
    Fclose((int)fh2);
    Fdelete(cachecopy);
    }
  }

i = 0;
while (i < 10)
	{
	if (reso_tab[i].cols == dither_colours && reso_tab[i].smallfont == smallfont)
		break;
	i++;
	}

/*fh = Fopen(file, FO_READ);*/
if (fh > -1L)
	{
	if (Fread((int)fh, sizeof(PALETTE_FILE), &palette_header) != sizeof(PALETTE_FILE))
		{
		if (flg)
			{
			form_alert(1, notpalet);
			}

		Fclose((int)fh);
		return NULL;
		}

	if (palette_header.id != 'DPAL')
		{
		if (flg)
			{
			form_alert(1, notpalet);
			}
		Fclose((int)fh);
		return NULL;
		}

	if (!palette_header.images[i].size)
		{
		if (flg)
			{
			char *msg;
			rsrc_gaddr(R_STRING, NOICONS, &msg);
			form_alert(1, msg);
			}
		Fclose((int)fh);
		return NULL;
		}

	size = palette_header.images[i].size;
	Fseek(palette_header.images[i].pos, (int)fh, 0);

	if (dither_colours)
		{
/*		nearcols = (unsigned char *)malloc(dither_colours);*/

		if (dither_colours == 16)
			butcols = cpalette;
		else
			if (dither_colours < 16)
				butcols = col2_4;
			else
				butcols = cpalette + 16 * 3;
		}

	mem = (char *)malloc(size);
	Fread((int)fh, size, mem);
	Fclose((int)fh);

	switch(dither_colours)
		{
		case 4:
		case 16:
		case 256:
			if (pixel_type == 0)
				gen_interleave(size, (unsigned char *)mem);
			else
				gen_pixels(size, (unsigned char *)mem);
			break;

		case 0:
      mem = (char *)gen_truecolor(&size, (unsigned char *)mem);
			break;
		}

  if (mail_options.icon_cache)
    {
    fh2 = Fcreate(cachecopy, 0);
    if (fh2 > -1L)
      {
	  	Fwrite((int)fh2, sizeof(PALETTE_FILE), &palette_header);
      Fwrite((int)fh2, sizeof(size), &size);
      Fwrite((int)fh2, size, mem);
      Fclose((int)fh2);
      }
    }

/*	if (dither_colours)
		free(nearcols);*/
	return mem;
	}

return NULL;
}

/* ------------------------
   | Initialize color map |
   ------------------------ */
int load_cpalette(char *file)
{
long fh;

DEBUG();
fh = Fopen(file, FO_READ);
if (fh > -1)
	{
	Fread((int)fh, sizeof(cpalette), cpalette);
	Fclose((int)fh);
	return TRUE;
	}
else
	{
	char *msg;
	rsrc_gaddr(R_STRING, COLNFND, &msg);
	form_alert(1, msg);
	return FALSE;
	}
}

/* -------------------------------
   | Get one parameter as string |
   ------------------------------- */
char *getParam(char **p, char *e)
{
char *ptr, *s;
char *mem;

DEBUG();
ptr = *p;

while(ptr < e && isspace(*ptr)) ptr++;

if (*ptr == '"')
	{
	s = ++ptr;
	while (ptr < e && *ptr != '"' && *ptr != '\r' && *ptr != '\n')
		ptr++;
	}
else
	{
	s = ptr;
	while(ptr < e && !isspace(*ptr) && *ptr != ',' && *ptr != ';')
		ptr++;
	}

mem = (char *)malloc(ptr - s + 1);
memcpy(mem, s, ptr - s);
mem[ptr - s] = 0;

if (*ptr == '"') ptr++;
while(ptr < e && isspace(*ptr)) ptr++;
if (*ptr == ',' || *ptr == ';') ptr++;

*p = ptr;

return mem;
}

/* ----------------------------
   | Load toolbar collections |
   ---------------------------- */
void load_toolcollections(void)
{
load_collections("MARATHON.COL");

toolcollections = collections;
toolcollection_cnt = collection_cnt;

toolcpals = cpals;
toolcpal_cnt = cpal_cnt;

collection_cnt = 0;
cpal_cnt = 0;
}

/* ----------------------------
   | Load toolbar collections |
   ---------------------------- */
void load_boxcollections(void)
{
DEBUG();
load_collections("BOXICONS.COL");

boxcollections = collections;
boxcollection_cnt = collection_cnt;

boxcpals = cpals;
boxcpal_cnt = cpal_cnt;

collection_cnt = 0;
cpal_cnt = 0;
}

/* ----------------------------
   | Load palette information |
   ---------------------------- */
void load_collections(char *file)
{
char buf[256], *zw, *ptr, *end, *parm, *p, *cdesc, *cfile;
int cpal;
long fh, l;
int mode = 0;

DEBUG();
strcpy(buf, mailpath);
strcat(buf, "\\images\\");
strcat(buf, file);

fh = Fopen(buf, FO_READ);
if (fh > -1)
	{
	l = Fseek(0L, (int)fh, 2);
	Fseek(0L, (int)fh, 0);

	zw = (char *)malloc(l);
	Fread((int)fh, l, zw);

	Fclose((int)fh);

	end = zw + l;
	ptr = zw;
	while(ptr < end)
		{
		char buf[20];
		strncpy(buf, ptr, 20);
		buf[19] = 0;

		if (*ptr == '[')
			{
			mode = 0;
			if (!strncmp(ptr, "[Palette]", 9))
				mode = 1;
			else
				if (!strncmp(ptr, "[Colours]", 9))
					mode = 2;
			}
		else
			{
			if (mode == 1)
				if (!strncmp(ptr, "Palette", 7) && isdigit(ptr[7]))
					{
					p = ptr + 7;
					while(p < end && *p != '=')
						p++;

					if (p < end)
						{
						p++;

						parm = getParam(&p, end);
						if (!parm) continue;
						cdesc = parm;

						parm = getParam(&p, end);
						if (!parm)
							{
							free(cdesc);
							continue;
							}
						cfile = parm;

						parm = getParam(&p, end);
						if (!parm)
							{
							free(cdesc);
							free(cfile);
							continue;
							}
						free(parm);

						parm = getParam(&p, end);
						if (!parm)
							{
							free(cdesc);
							free(cfile);
							continue;
							}
						free(parm);

						parm = getParam(&p, end);
						if (!parm)
							{
							free(cdesc);
							free(cfile);
							continue;
							}
						cpal = atoi(parm);
						free(parm);

						if (!collection_cnt)
							collections = (COLL_INF *)malloc(sizeof(COLL_INF));
						else
							collections = (COLL_INF *)realloc(collections,
										sizeof(COLL_INF) * (collection_cnt + 1));

						collections[collection_cnt].desc = cdesc;
						collections[collection_cnt].file = cfile;
						collections[collection_cnt++].cpal = cpal;
						}
					}

			if (mode == 2)
				if (!strncmp(ptr, "Colour", 6) && isdigit(ptr[6]))
					{
					p = ptr + 6;
					while(p < end && *p != '=')
						p++;

					if (p < end)
						{
						p++;

						parm = getParam(&p, end);
						if (!parm) continue;

						if (!cpal_cnt)
							cpals = (char **)malloc(sizeof(char *));
						else
							cpals = (char **)realloc(cpals, sizeof(char *) * (cpal_cnt + 1));

						cpals[cpal_cnt++] = parm;
						}
					}
			}

		while(ptr < end && *ptr != '\r' && *ptr != '\n')
			ptr++;

		if (ptr < end)
			ptr++;
		}

	free(zw);
	}
}

/* -------------------
   | Load intro data |
   ------------------- */
char *load_intro(void)
{
char buf[256];

DEBUG();
strcpy(buf, mailpath);
strcat(buf, "\\images\\marathon.clc");
load_cpalette(buf);

strcpy(buf, mailpath);
strcat(buf, "\\images\\intro.pal");
return load_images(buf, FALSE);
}

/* ---------------
   | Load images |
   --------------- */
void load_my_images(void)
{
int srgb[3];
char buf[256];

DEBUG();
strcpy(buf, mailpath);
strcat(buf, "\\images\\marathon.clc");
if (!load_cpalette(buf))
	{
	exit_flg = TRUE;
	return;
	}

strcpy(buf, mailpath);
strcat(buf, "\\images\\symbols.pal");
symbols = load_images(buf, FALSE);

load_toolbuttons();
load_boxicons();

if (colours > 4)
	{
	if (!dither_colours)
		srgb[0] = srgb[1] = srgb[2] = 240;
	else
		srgb[0] = srgb[1] = srgb[2] = 217;
	box_background = find_colour(srgb);
	}
}

/* ------------------------
   | Load toolbar buttons |
   ------------------------ */
void load_toolbuttons(void)
{
DEBUG();
load_buttons(&mw_toolbar, &tool_palheader, &tool_button_no,
	     mail_options.toolbfile, toolcollection_cnt,
	     toolcollections, toolcpals);
calc_butpos();
}

/* ------------------------
   | Load toolbar buttons |
   ------------------------ */
void load_boxicons(void)
{
int cx, cy, cw, ch, t_xy[8];

DEBUG();
load_buttons(&boxicons, &icon_palheader, &box_icons_no, iconbfile, boxcollection_cnt, boxcollections, boxcpals);

vst_effects(vdi_handle, 0);
vst_height(vdi_handle, 4, &cx, &cy, &cw, &ch);
vqt_extent(vdi_handle, "x", t_xy);
icon_height = (smallfont ? icon_palheader.height / 2 : icon_palheader.height) + abs(t_xy[5] - t_xy[1]) + 15;
}

/* ----------------------------------
   | Load user selected button file |
   ---------------------------------- */
void load_buttons(char **iconmem, PALETTE_FILE *icon_palheader,
									int *num, char *bfile, int collection_cnt, COLL_INF *collections, char **cpals)
{
char *nbuts;
char buf[256];

DEBUG();
if (*num == -1)
	{
	*num = 0;

	while(*num < collection_cnt)
		{
		if (!strcmp(collections[*num].file, bfile))
			break;
		(*num)++;
		}

	if (*num == collection_cnt)
		*num = 0;
	}

strcpy(buf, mailpath);
strcat(buf, "\\images\\");
strcat(buf, cpals[collections[*num].cpal - 1]);
strcat(buf, ".clc");

load_cpalette(buf);

strcpy(buf, mailpath);
strcat(buf, "\\images\\");
strcat(buf, collections[*num].file);
strcat(buf, ".pal");

nbuts = load_images(buf, TRUE);
if (!nbuts)
	{
	if (*num)
		{
		if (*iconmem == NULL)
			{
			*num = 0;
			load_buttons(iconmem, icon_palheader, num, bfile, collection_cnt, collections, cpals);
			}
		}
	else
		exit_flg = TRUE;
	return;
	}

if (*iconmem)	free(*iconmem);
*iconmem = nbuts;

strcpy(bfile, collections[*num].file);
memcpy(icon_palheader, &palette_header, sizeof(palette_header));
}

static int f_pals[] = {PAL_1, PAL_2, PAL_3, PAL_4, PAL_5, PAL_6};

/* -------------------------
   | Fill in provider list |
   ------------------------- */
void fill_pals(OBJECT *tree, int base, int collection_cnt, COLL_INF *collections)
{
int i, obj;

DEBUG();
for (i = 0; i < 6; i++)
	{
	obj = f_pals[i];

	if (base + i < collection_cnt)
		{
		strcpy(tree[obj].ob_spec.tedinfo->te_ptext, collections[base + i].desc);
		tree[obj].ob_state &= ~DISABLED;
		}
	else
		{
		tree[obj].ob_spec.tedinfo->te_ptext[0] = 0;
		tree[obj].ob_state |= DISABLED;
		}
	}
}

/* --------------------------
   | Select toolbar buttons |
   -------------------------- */
void select_toolbuttons(void)
{
int ret;

DEBUG();
ret = select_buttons(toolcollection_cnt, toolcollections);
if (ret != -1)
	{
	tool_button_no = ret;
	load_toolbuttons();
	}
}

/* --------------------------
   | Select toolbar buttons |
   -------------------------- */
void select_boxicons(void)
{
int ret;

DEBUG();
ret = select_buttons(boxcollection_cnt, boxcollections);
if (ret != -1)
	{
	box_icons_no = ret;
	load_boxicons();
	}
}

/* -------------------------
   | Select button palette |
   ------------------------- */
int select_buttons(int collection_cnt, COLL_INF *collections)
{
OBJECT *sel_buttree;
int obj, i;
int but_base = 0;

DEBUG();
rsrc_gaddr(R_TREE, PAL_SEL, &sel_buttree);

align_slider_objects(sel_buttree, PAL_SBKG, PAL_UP, PAL_SLD, PAL_DRG, PAL_DWN);

for (i = 0; i < 6; i++)
	sel_buttree[f_pals[i]].ob_state &= ~SELECTED;

set_slider_objects(sel_buttree, PAL_SLD, PAL_DRG, collection_cnt, 0, 6, FALSE);

sel_buttree[PAL_DRG].ob_y = 0;
fill_pals(sel_buttree, but_base, collection_cnt, collections);
open_dialog(sel_buttree, NULL, iconified_tree);
do
	{
	set_slider_objects(sel_buttree, PAL_SLD, PAL_DRG, collection_cnt, but_base, 6, TRUE);

	obj = do_dialog(sel_buttree, 0, app_callback);

	switch(obj & 0x7FFF)
		{
		case PAL_1:
		case PAL_2:
		case PAL_3:
		case PAL_4:
		case PAL_5:
		case PAL_6:
			if (sel_buttree[obj & 0x7FFF].ob_spec.tedinfo->te_ptext[0])
				{
				set_scrollselect(sel_buttree, obj & 0x7FFF);
				if (obj & 0x8000)
					obj = PAL_OK;
				}
			break;

		case PAL_UP:
			if (but_base > 0)
				{
				but_base--;
				fill_pals(sel_buttree, but_base, collection_cnt, collections);
				redraw_object(sel_buttree, PAL_BKG);
				}
			break;

		case PAL_DWN:
			if (but_base + 6 < collection_cnt)
				{
				but_base++;
				fill_pals(sel_buttree, but_base, collection_cnt, collections);
				redraw_object(sel_buttree, PAL_BKG);
				}
			break;

		case PAL_DRG:
			but_base = graf_slidebox(sel_buttree, PAL_SLD, PAL_DRG, 1) * (collection_cnt - 6) / 1000;
			fill_pals(sel_buttree, but_base, collection_cnt, collections);
			redraw_object(sel_buttree, PAL_BKG);
			break;

		case PAL_SLD:
			{
			int ox, oy;

			objc_offset(sel_buttree, PAL_DRG, &ox, &oy);
			if (evnt.y < oy)
				{
				but_base -= 6;
				if (but_base < 0) but_base = 0;
				}
			else
				{
				but_base += 6;
				if (but_base > collection_cnt - 6)
					but_base = collection_cnt - 6;
				}
			fill_pals(sel_buttree, but_base, collection_cnt, collections);
			redraw_object(sel_buttree, PAL_BKG);
			delay(100);
			break;
			}
		}

	}while(obj != PAL_OK && obj != PAL_CNC);
close_dialog(sel_buttree, TRUE);

if (obj == PAL_OK)
	{
	for (i = 0; i < 6 && !(sel_buttree[f_pals[i]].ob_state & SELECTED); i++);

	if (i < 6)
		{
		return but_base + i;
		}
	}
return -1;
}
