/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

static OBJECT *info_dlg = NULL;
static int info_cancelled;
static long progress_bytes, last_bytes, last_size, last_total;
static int progress_number, last_number, is_progress;
static clock_t progress_start, last_time;

static int info2_ids[] = {TOTALMAILS,
													TOTALBYTES,
													TIMEREMAINING,
													OPSLDBKG,
													STOPTRNS,
													0};
													
/* ------------------------
   | Handle info dialogue |
   ------------------------ */
static void cdecl info_handler (OBJECT *tree, int obj)
{
(void)tree;

DEBUG();
switch (obj & 0x7FFF)
	{
	case P_CANCEL:
		{
		info_cancelled = TRUE;
		break;
		}
	}
}

/* -------------------
   | Clear cancelled |
   ------------------- */
void clear_infostop(void)
{
info_cancelled = FALSE;
}
   
/* ---------------------
   | Start information |
   --------------------- */
void start_info(char *t, char *l)
{
DEBUG();
rsrc_gaddr(R_TREE, P_INFO, &info_dlg);

strcpy(info_dlg[P_INFHD].ob_spec.tedinfo->te_ptext + 14, t);
info_dlg[P_INFOLN].ob_spec.tedinfo->te_ptext = l;

is_progress = FALSE;
info_cancelled = FALSE;
open_dialog(info_dlg, NULL, NULL);
redraw_dialog(info_dlg);
set_active(info_dlg, info_handler, TRUE, 0);
}

/* --------------------
   | Change info line |
   -------------------- */
void change_info(char *ninfo)
{
DEBUG();
if (info_dlg)
	{
	if (is_progress)
		{
		info_dlg[OPSLDBKG].ob_spec.tedinfo->te_ptext = ninfo;
		redraw_object(info_dlg, OPSLDBKG);
		}
	else
		{
		info_dlg[P_INFOLN].ob_spec.tedinfo->te_ptext = ninfo;
		redraw_object(info_dlg, P_INFOLN);
		}
	}
}
   
/* ----------------------
   | End of information |
   ---------------------- */
void end_info(void)
{
DEBUG();
if (info_dlg)
	{
	close_dialog(info_dlg, TRUE);
	info_dlg = NULL;
	}
}

/* -----------------------
   | Check cancel button |
   ----------------------- */
int info_stop(void)
{
DEBUG();
check_user_message();
return info_cancelled;
}

/* ------------------------
   | Handle info dialogue |
   ------------------------ */
static void cdecl progressinfo_handler (OBJECT *tree, int obj)
{
(void)tree;

DEBUG();
switch (obj & 0x7FFF)
	{
	case STOPTRNS:
		{
		info_cancelled = TRUE;
		break;
		}
	}
}

/* ---------------------
   | Start information |
   --------------------- */
void start_progressinfo(int number, long bytes, clock_t starttime, int download)
{
DEBUG();
rsrc_gaddr(R_TREE, PROGRESSBARS, &info_dlg);

is_progress = TRUE;
info_cancelled = FALSE;
info_dlg[SEND_LABEL].ob_y = info_dlg[DOWNLOAD_LABEL].ob_y;

if (download)
	{
	info_dlg[DOWNLOAD_LABEL].ob_flags &= ~HIDETREE;
	info_dlg[SEND_LABEL].ob_flags |= HIDETREE;
	}
else
	{
	info_dlg[DOWNLOAD_LABEL].ob_flags |= HIDETREE;
	info_dlg[SEND_LABEL].ob_flags &= ~HIDETREE;
	}
	
open_dialog(info_dlg, NULL, NULL);
dlgset_help_info(info_dlg, helpfile, info2_ids, "INF2_");

progress_start = starttime;
last_total = 0;
progress_bytes = bytes;
ltoa(bytes, info_dlg[TOTALBYTES].ob_spec.tedinfo->te_ptext, 10);

last_number = 0;
progress_number = number;
strcpy(info_dlg[TOTALMAILS].ob_spec.tedinfo->te_ptext, "0/");
itoa(number, info_dlg[TOTALMAILS].ob_spec.tedinfo->te_ptext + 2, 10);

last_bytes = 0;
last_size = 0;
info_dlg[OPSLIDER].ob_width = 0;

info_dlg[TIMEREMAINING].ob_spec.tedinfo->te_ptext[0] = 0;

redraw_dialog(info_dlg);
set_active(info_dlg, progressinfo_handler, TRUE, 0);
last_time = clock();
}

/* --------------------
   | Change info line |
   -------------------- */
void change_progressinfo(int numproc, long currbytes, long currsize, long currtotal)
{
DEBUG();
if (info_dlg && clock() - last_time > CLK_TCK)
	{
	if (numproc != last_number)
		{
		char *ptr;
		
		ptr = info_dlg[TOTALMAILS].ob_spec.tedinfo->te_ptext;
		itoa(numproc, ptr, 10);
		strcat(ptr, "/");
		itoa(progress_number, ptr + strlen(ptr), 10);
		last_number = numproc;
		redraw_object(info_dlg, TOTALMAILS);
		}

	if (currbytes > currsize) currbytes = 0;
	if ((currbytes != last_bytes || currsize != last_size) && currbytes > 0)
		{
		if (currsize > 0)
			info_dlg[OPSLIDER].ob_width = (int)(info_dlg[OPSLDBKG].ob_width * currbytes / currsize);
		else
			info_dlg[OPSLIDER].ob_width = 0;
			
		info_dlg[OPSLDBKG].ob_spec.tedinfo->te_ptext = "\0";
		last_size = currsize;
		last_bytes = currbytes;
		redraw_object(info_dlg, OPSLDBKG);
		}
		
	if (currtotal != last_total)
		{
		long secs;
		int hour, min, sec;
		long remaining = progress_bytes - currtotal;
		secs = remaining / (currtotal * CLK_TCK / (clock() - progress_start));
	
		hour = (int)(secs / 3600);
		secs %= 3600;
		min = (int)(secs / 60);
		secs %= 60;
		sec = (int)secs;

		sprintf(info_dlg[TIMEREMAINING].ob_spec.tedinfo->te_ptext,
							"%02d:%02d:%02d", hour, min, sec);
		redraw_object(info_dlg, TIMEREMAINING);
		}
	last_time = clock();
	}
}
   
