/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

/* -----------------------------------------
   | Create an empty header for a new mail |
   ----------------------------------------- */
void create_empty_header(void)
{
DEBUG();
new_buffer.organization = (char *)malloc(60);
new_buffer.from 				= (char *)malloc(new_buffer.from_max = 128);
new_buffer.replyto			= (char *)malloc(new_buffer.replyto_max = 128);
new_buffer.to 					= (char *)malloc(new_buffer.to_max = 60);
new_buffer.cc 					= (char *)malloc(new_buffer.cc_max = 60);
new_buffer.bcc 					= (char *)malloc(new_buffer.bcc_max = 60);
new_buffer.subject 			= (char *)malloc(new_buffer.subj_max = 60);
new_buffer.date 				= (char *)malloc(51);

new_buffer.to[0] = 0;
new_buffer.replyto[0] = 0;
new_buffer.organization[0] = 0;
new_buffer.from[0] = 0;
new_buffer.cc[0] = 0;
new_buffer.bcc[0] = 0;
new_buffer.subject[0] = 0;
new_buffer.date[0] = 0;

new_buffer.confirm_rcpt	=	TRUE;
new_buffer.confirm_read	=	TRUE;
new_buffer.priority		  = 0;
}
	
/* -------------------------
   | Execute external mail |
   ------------------------- */
void external_mail(char *in_cmd)
{
char *cpy = NULL, *to, *subj;
int i;

DEBUG();
if (!strnicmp(in_cmd, "file=", 5))
	{
	long fh, len;

	fh = Fopen(in_cmd + 5, FO_READ);
	if (fh  > 0)
		{
		cpy = (char *)malloc((len = Fseek(0L, (int)fh, 2)) + 1);
		if (cpy)
			{
			Fseek(0L, (int)fh, 0);
			Fread((int)fh, len, cpy);
			cpy[len] = 0;
			}
		Fclose((int)fh);
		}
	}
else
	{
	cpy = (char *)malloc(strlen(in_cmd) + 1);
	if (cpy)
		strcpy(cpy, in_cmd);
	}
	
if (cpy)
	{	
	if (!strnicmp("mailto:", cpy, 7))
		to = cpy + 7;
	else
		to = cpy;

	i = 0;
	subj = NULL;
	while (cpy[i])
		{
		if (!strnicmp(cpy + i, "?subject=", 9))
			{
			cpy[i] = 0;
			subj = cpy + i + 9;
			break;
			}
		i++;
		}
	
	new_mail(to, subj);
	
	free(cpy);
	}
}

/* -----------------
   | Send new mail |
   ----------------- */
void new_mail(char *to, char *subject)
{
IDENTITY def;
int has_default;

DEBUG();
if (mail_wind != -1)
	return;

strcpy(mail_title, "Marathon Mail (noname.eml)");
memset(&new_buffer, 0, sizeof(new_buffer));

create_empty_header();

has_default = getDefaultIdentity(&def);

if (to)
	strncpy(new_buffer.to, to, 59);
if (subject)
	strncpy(new_buffer.subject, subject, 59);

if (has_default)
	{
	new_buffer.mime_allowed = def.mime_allowed;
	new_buffer.word_wrap = def.word_wrap;
	new_buffer.editwidth = def.editwidth;

		/* Get priority from identity */
	if (def.priority == -1)
		{
		if (def.last_priority != -1)
			new_buffer.priority = def.last_priority;
		}
	else
		new_buffer.priority = def.priority;
		
		/* Get confirm receipt from identity */
	if (def.confirm_rcpt == -1)
		{
		if (def.last_rcpt != -1)
			new_buffer.confirm_rcpt = def.last_rcpt;
		}
	else
		new_buffer.confirm_rcpt = def.confirm_rcpt;
		
	if (def.confirm_read == -1)
		{
		if (def.last_read != -1)
			new_buffer.confirm_read = def.last_read;
		}
	else
		new_buffer.confirm_read = def.confirm_read;

	strcpy(new_buffer.signature, def.signature);
	strcpy(new_buffer.identity, def.name);
	if (def.organization)
		strcpy(new_buffer.organization, def.organization);

	if (def.username[0])
		{
		new_buffer.from[0] = '"';
		strcpy(new_buffer.from + 1, def.username);
		strcat(new_buffer.from, "\" <");
		}
	}

if (!has_default || def.use_providerdata)
	{
	strcat(new_buffer.from, cfg_options.email);
	strcat(new_buffer.from, cfg_options.email2);
	}
else
	if (has_default)
		{
		strcat(new_buffer.from, def.email);
		strcat(new_buffer.from, def.email2);
		}

if (has_default)	
	if (def.username[0])
		strcat(new_buffer.from, ">");

if (has_default || !def.reply_to1[0])	
	strcpy(new_buffer.replyto, new_buffer.from);
else
	{
	strcpy(new_buffer.replyto, def.reply_to1);
	strcat(new_buffer.replyto, def.reply_to2);
	}
	
forward_mode = FALSE;
edit_mode = FALSE;
open_mailwindow();
}

/* --------------------------
   | Clear edit buffer data |
   -------------------------- */
void clear_buffer(MAIL_MEM *buffer)
{
DEBUG();
if (buffer->body_len > 0)
	free(buffer->body);

if (buffer->to) 						free(buffer->to);
if (buffer->organization) 	free(buffer->organization);
if (buffer->from) 					free(buffer->from);
if (buffer->replyto) 				free(buffer->replyto);
if (buffer->cc) 						free(buffer->cc);
if (buffer->bcc) 						free(buffer->bcc);
if (buffer->subject) 				free(buffer->subject);
if (buffer->date) 					free(buffer->date);
}