#include "extern.h"

FILE_MAIL_MEM header;

int notify, dsn, smtp_s;

/* -------------------------
   | Close SMTP connection |
   ------------------------- */
int close_smtp(void)
{
int e, ret = TRUE;
char buf[256];

DEBUG();
if (smtp_s)
	{
	change_info(disconnect_msg);	
	send_cmd(smtp_s, "QUIT", NULL, NULL, NULL);
	e = get_response(smtp_s, buf);
	if (e < 200 || e > 399)
		ret = FALSE;
		
	close_socket(smtp_s);
	smtp_s = 0;
	}
end_info();
	
return ret;
}

/* -------------------------
   | Do RSET on mail error |
   ------------------------- */
void mail_error(void)
{
DEBUG();
send_cmd(smtp_s, "RSET", NULL, NULL, NULL); 
}

/* ------------------
   | Send mail body |
   ------------------ */
int send_message(int s, char *mailfile)
{
char *buf;
int in;
long l, pos, len, infl;
int last = 0, new;
char inf[40];

DEBUG();
buf = (char *)malloc(1024);
infl = Fopen(mailfile, FO_READ);
if (infl > 0)
	in = (int)infl;
else
	in = 0;

if (in)
	{	
	pos = header.body_pos;
	len = l = header.body_len;
	Fseek(pos, in, 0);

	while(l > 0)
		{
		if (l >= 1024)
			{
			Fread(in, 1024, buf);
			send(s, buf, 1024, 0);
			l -= 1024;
			pos += 1024;
			}
		else
			{
			Fread(in, l, buf);
			send(s, buf, l, 0);
			pos += l;
			l = 0;
			}

		new = (int)((pos - header.body_pos) * 100 / len);
		if (new >= last + 2)
			{
			if (new == 100 && !last)
				for (new = 10; new < 100; new += 10)
					{
					sprintf(inf, send_perc, new);
					change_info(inf);
					}
					
			sprintf(inf, send_perc, new);
			change_info(inf);
			last = new;
			}
		}
	Fclose(in);
	}
	
free(buf);

return in != 0;
}

/* --------------------------
   | Send it to SMTP server |
   -------------------------- */
int send_smtp(char *mailfile)
{
int e;
char buf[256];

DEBUG();
if (!smtp_s)
	return FALSE;
			
/*	e = send_quoted(smtp_s, buf, "MAIL FROM:", cfg_options.email, cfg_options.email2, "\1RET=HDRS");
	if (e == 555)
		e = send_quoted(smtp_s, buf, "MAIL FROM:", cfg_options.email, cfg_options.email2, NULL);*/

if (dsn)
	e = send_quoted(smtp_s, buf, "MAIL FROM:", new_buffer.from, "\1RET=HDRS", NULL);
if (!dsn || e == 555)
	e = send_quoted(smtp_s, buf, "MAIL FROM:", new_buffer.from, NULL, NULL);
		
if (e < 200 || e > 399)
	{
	store_error("SMTP", e, buf);
	mail_error();
	return FALSE; 
	}

if (new_buffer.to)
	if (new_buffer.confirm_rcpt && notify)
		e = send_quoted(smtp_s, buf, "RCPT TO:", new_buffer.to, "\1NOTIFY=SUCCESS,FAILURE", NULL);
	else
		e = send_quoted(smtp_s, buf, "RCPT TO:", new_buffer.to, NULL, NULL);

if ((e < 200 || e > 399) && notify)
	{
	notify = FALSE;
	e = send_quoted(smtp_s, buf, "RCPT TO:", new_buffer.to, NULL, NULL);
	}

if (e < 200 || e > 399)
	{ 
	store_error("SMTP", e, buf);
	mail_error();
	return FALSE; 
	}

if (new_buffer.cc && new_buffer.cc[0])
	{
	if (new_buffer.confirm_rcpt && notify)
		e = send_quoted(smtp_s, buf, "RCPT TO:", new_buffer.cc, "\1NOTIFY=SUCCESS,FAILURE", NULL);
	else
		e = send_quoted(smtp_s, buf, "RCPT TO:", new_buffer.cc, NULL, NULL);
		
	if (e < 200 || e > 399)
		{ 
		store_error("SMTP", e, buf);
		mail_error();
		return FALSE; 
		}
	}
		
if (new_buffer.bcc && new_buffer.bcc[0])
	{
	if (new_buffer.confirm_rcpt && notify)
		e = send_quoted(smtp_s, buf, "RCPT TO:", new_buffer.bcc, "\1NOTIFY=SUCCESS,FAILURE", NULL);
	else
		e = send_quoted(smtp_s, buf, "RCPT TO:", new_buffer.bcc, NULL, NULL);

	if (e < 200 || e > 399)
		{ 
		store_error("SMTP", e, buf);
		mail_error();
		return FALSE; 
		}
	}

send_cmd(smtp_s, "DATA", NULL, NULL, NULL);
e = get_response(smtp_s, buf);
if (e != 354)
	{ 
	store_error("SMTP", e, buf);
	mail_error();
	return FALSE; 
	}

change_info(sending_text);	
if (send_message(smtp_s, mailfile))
	{
	send_cmd(smtp_s, ".", NULL, NULL, NULL);
	e = get_response(smtp_s, buf);
	if (e < 200 || e > 399)
		{ 
		store_error("SMTP", e, buf);
		mail_error();
		return FALSE; 
		}
	}
else
	{
	store_error("SMTP", e, buf);
	mail_error();
	return FALSE; 
	}
	
return TRUE;
}

/* -------------
   | Send mail |
   ------------- */
int mail_send(char *mailfile)
{
long infl;
int in, ret;

DEBUG();
infl = Fopen(mailfile, FO_READ);
if (infl > 0)
	in = (int)infl;
else
  in = 0;
  
if (in)
	{
	Fread(in, sizeof(header), &header);

	if (header.magic != 0x4104)
		{
		memmove(&header.confirm_read, &header.magic, sizeof(header) - (21+21+6));
		Fseek(-(21+21+6L), in, 1);
		header.replyto_len = 0;
		header.identity[0] = 0;
		header.signature[0] = 0;
		}
	else
		if (header.version == SVERSION_ID17)
			{
			Fseek(-(21+21L), in, 1);
			header.identity[0] = 0;
			header.signature[0] = 0;
			}

	memset(&new_buffer, 0, sizeof(MAIL_MEM));

	new_buffer.confirm_rcpt	=	header.confirm_rcpt;
	new_buffer.confirm_read	=	header.confirm_read;
	new_buffer.priority		  = header.priority;
	strcpy(new_buffer.identity, header.identity);
	strcpy(new_buffer.signature, header.signature);
	
	Fseek(header.header_inf, in, 0);
  if (header.date_len)
  	{
  	new_buffer.date = (char *)malloc(header.date_len + 1);
	  Fread(in, header.date_len, new_buffer.date);
	  new_buffer.date[header.date_len] = 0;
	  }
  if (header.from_len)
  	{
  	new_buffer.from = (char *)malloc(header.from_len + 1);
	  Fread(in, header.from_len, new_buffer.from);
	  new_buffer.from[header.from_len] = 0;
	  }
  if (header.replyto_len)
  	{
  	new_buffer.replyto = (char *)malloc(header.replyto_len + 1);
	  Fread(in, header.replyto_len, new_buffer.replyto);
	  new_buffer.replyto[header.replyto_len] = 0;
	  }
  if (header.organization_len)
  	{
  	new_buffer.organization = (char *)malloc(header.organization_len + 1);
	  Fread(in, header.organization_len, new_buffer.organization);
	  new_buffer.organization[header.organization_len] = 0;
	  }
  if (header.to_len)
  	{
  	new_buffer.to = (char *)malloc(header.to_len + 1);
	  Fread(in, header.to_len, new_buffer.to);
	  new_buffer.to[header.to_len] = 0;
	  }
  if (header.cc_len)
  	{
  	new_buffer.cc = (char *)malloc(header.cc_len + 1);
	  Fread(in, header.cc_len, new_buffer.cc);
	  new_buffer.cc[header.cc_len] = 0;
	  }
  if (header.bcc_len)
  	{
  	new_buffer.bcc = (char *)malloc(header.bcc_len + 1);
	  Fread(in, header.bcc_len, new_buffer.bcc);
	  new_buffer.bcc[header.bcc_len] = 0;
	  }
  if (header.subject_len)
  	{
  	new_buffer.subject = (char *)malloc(header.subject_len + 1);
	  Fread(in, header.subject_len, new_buffer.subject);
	  new_buffer.subject[header.subject_len] = 0;
	  }	  	  

	Fclose(in);

	ret = send_smtp(mailfile);

	if (header.date_len) 					free(new_buffer.date);
	if (header.from_len) 					free(new_buffer.from);
	if (header.replyto_len) 			free(new_buffer.replyto);
	if (header.organization_len) 	free(new_buffer.organization);
	if (header.to_len) 						free(new_buffer.to);
	if (header.cc_len) 						free(new_buffer.cc);
	if (header.bcc_len) 					free(new_buffer.bcc);
	if (header.subject_len) 			free(new_buffer.subject);

	return ret;
	}
else
	return FALSE;
}		
