/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"
#include "global.h"
#include "vaproto.h"

extern int debug;

#ifdef TRAPIT
#  include <setjmp.h>

	long oldbus, oldaddr, oldillegal;
	long textseg_base, textseg_length;
	void trapbus( void );
	void trapaddr( void );
	void trapillegal( void );
#endif

extern int dial_function;
extern char dial_identity[21];
extern int dial_flg;

int stored_msg = FALSE;
#define BUBBLEGEM_ACK      0xBABC

/* ---------------------------- */
/* | Arbeitsstation schlieen | */
/* ---------------------------- */
void cls_vwork(void)
{
DEBUG();
v_clsvwk(vdi_handle);
}

/* ---------------------------- */
/* | Virtuelle Arbeitsstation | */
/* ---------------------------- */
void open_vwork(void)
{
register int i;
int dummy, t_attr[10];
int wout[57];
static int colnum[] = {2, 4, 16, 256, 0};

DEBUG();
vdi_handle = graf_handle(&dummy, &dummy, &dummy, &dummy);

for(i = 0; i < 10; work_in[i++] = 1);
work_in[10] = 2;

v_opnvwk(work_in, &vdi_handle, work_out);

vqt_attributes(vdi_handle, t_attr);
default_fsize = t_attr[7];
tx_width = t_attr[8];
tx_height = t_attr[9];
smallfont = (tx_height < 16);

vq_extnd(vdi_handle, TRUE, wout);
planes = wout[4];
colours = work_out[13];
if (planes >= 16)
	dither_colours = 0;
else
	dither_colours = colours;

for (i = 0; i < 5; i++)
	if (dither_colours == colnum[i])
		dither_index = i;
		
gdos = vq_gdos();
if (gdos)
	num_fonts = vst_load_fonts(vdi_handle, 0);
else
	num_fonts = 1;
	
detect_pixeltype();
set_default_text();
}

#ifdef TRAPIT
extern jmp_buf trapcont;
#endif

/* -------------------------
   | Main routine of STTCP |
   ------------------------- */
void main(int argc, char *argv[])
{
int bclicks;

DEBUG();

gl_apid = appl_init();
open_vwork();

get_aesversion();

#ifdef TRAPIT
{
long old_stk;
old_stk = Super(0L);
oldbus = *(long *)0x8;
oldaddr = *(long *)0xC;
oldillegal = *(long *)0x10;

textseg_base = (long)_BasPag->p_tbase;
textseg_length = textseg_base + _BasPag->p_tlen;

*(long *)0x8 = (long)trapbus;
*(long *)0xC = (long)trapaddr;
/**(long *)0x10 = (long)trapillegal;*/
Super((void *)old_stk);
}
#endif

if (init_rsc())
	{
	rsrc_gaddr(R_TREE, MAILMENU, &menu_adr);
	menu_bar(menu_adr, TRUE);

	memset(&cfg_options, 0, sizeof(cfg_options));
	memset(&mail_options.prt_settings, 0, sizeof(mail_options.prt_settings));

/*	if (CheckSystem())*/
		Pdomain(1);

	naes = check_cookie('nAES');

	get_bootdrive();	
	start_config();
	read_installdir();
	if (init_dialogs() < 0)
		{
		form_alert(1, "[1][Dialog initialization failed!][Ok]");
		goto end;
		}

	load_mail_config(FALSE);

	init_colorcache();
	open_intro();

	scan_fontnames();
	load_toolcollections();
	load_boxcollections();
	load_my_images();
	load_recent();
	load_externals();
	load_confirm();

#ifdef AUDIO
	init_audio();
#endif

	close_intro();
	load_mail_config(TRUE);
		
	graf_mouse(ARROW, 0L);

	init_menu();

	if (argc == 2)
		{
		if (!strcmp(argv[1], "/D"))
			{
			debug = TRUE;
			argc = 1;
			}
		}

	if (argc >= 2)
		{
		if (argc == 2)
			{
			if (!stricmp(argv[1] + strlen(argv[1]) - 4, ".box"))
				scan_mailbox(argv[1]);
			else
				{
				if (!box_open)
					open_box();

				if (box_open)
					external_mail(argv[1]);
				}
			}
		else
			{
			char *to = NULL, *subj = NULL, *box;
			int i;
			
			for (i = 1; i < argc; i++)
				{
				if (argv[i][0] == '-')
					switch (argv[i][1])
						{
						case 's':
							subj = argv[++i];
							break;
						
						case 'b':
							box = argv[++i];
							break;

						default:	
							if (argv[i][0] != '-')
								to = argv[i];
							break;
						}	
				}
			
			if (box)
				scan_mailbox(box);
				
			if (!box_open)
				open_box();

			if (box_open)
				new_mail(to, subj);
			}
		}		
			
	if (box_open && mail_options.startup_check)
		check_id("\0", -1);

#ifdef TRAPIT
	if (!setjmp(trapcont))
#endif	
	while (!exit_flg)
		{
 		if (!stored_msg)
 			{
  	  int bsk;
  	  
			if (!naes)
				{
				graf_mkstate( &evnt);
				bsk = evnt.bstate & 3;
				if ( !(bsk) )
		  		bclicks = 256+2;
				else
				  bclicks = 0;
				 
				bsk = 0;
				}
			else
				{
				evnt_multi(MU_TIMER,
									 258, 3, 0,
									 0, &empty,
									 0, &empty,
									 msg,
									 1L,
									 &evnt, &k_return, &clicks);
				bsk = evnt.bstate & 3;
				bclicks = 256+2;
				}

			which = evnt_multi(MU_MESAG|MU_TIMER|MU_KEYBD|MU_BUTTON,
												 bclicks, 3, bsk,
												 0, &empty,
												 0, &empty,
												 msg,
												 200L,
												 &evnt, &k_return, &clicks);
      }

		stored_msg = FALSE;
		which = evnt_handler(which, msg, &evnt, k_return, clicks);
		app_callback(which, msg, &evnt, k_return, clicks);
		}
	
	menu_bar(menu_adr, FALSE);
	
	if (mail_options.save_opt)
		{
		save_mail_config();
		save_externals();
		}
		
	if (box_open)
		close_box();

	save_recent();
	clear_temp();
	exit_dialogs();
	}
end:

#ifdef TRAPIT
{
long old_stk;

old_stk = Super(0L);
*(long *)0x8 = oldbus;
*(long *)0xC = oldaddr;
/**(long *)0x10 = oldillegal;*/
Super((void *)old_stk);
}
#endif

cls_vwork();
appl_exit();
}

/* -------------------------------------------------------------
   | Application call for handling events during modal dialogs |
   ------------------------------------------------------------- */
void cdecl app_callback( int awhich, int *amsg, EVNTDATA *ev, int kr, int br )
{
DEBUG();
which = awhich;
memcpy(&evnt, ev, sizeof(EVNTDATA));
memcpy(msg, amsg, sizeof(msg));
k_return = kr;
clicks = br;

if (which & MU_TIMER)
	{
	int over, top;
	
	wind_get_int(ROOT, WF_TOP, &top);
	
#ifdef MILAN
	milan_check();
#endif				
	if (top == box_wind) box_move(evnt.x, evnt.y);
	bubble_info(evnt.x, evnt.y);

	over = wind_find(evnt.x, evnt.y);
	
	if (is_listwindow(over) && over == top) 
		column_resize(window_box(over), evnt.x, evnt.y);
	else
		set_normalmouse();
						
	doPoll();
	cursor_control();
	}
	
if ((which & MU_BUTTON) && clicks)
	{
	int top;

/*	if (!(evnt.bstate & 1))*/
		top = wind_find(evnt.x, evnt.y);
/*	else
		wind_get_int(ROOT, WF_TOP, &top);*/
		
	if (top == tree_wind)		tree_button();
	if (top == mail_wind)		mail_button();
	if (top == box_wind)		box_button();
	
	if (top == pop_wind) 		pop_button();
	if (is_listwindow(top)) list_button(window_box(top));
	if (is_viewwindow(top)) view_button(window_view(top));
	}
					
if (which & MU_KEYBD)
	{
	int top;
	
	wind_get_int(ROOT, WF_TOP, &top);
	if (top == mail_wind)		mail_keybd();
	if (is_viewwindow(top)) view_keybd(window_view(top));
	if (is_listwindow(top)) list_keybd(window_box(top));
	system_keys();
	}

if (which & MU_MESAG)		
	message_handler();
}

/* ------------------------
   | handle message queue |
   ------------------------ */
void message_handler(void)
{
DEBUG();
switch(msg[0])
	{
#ifndef RESTRICTED
	case VA_START:
		{
		if (mail_wind > -1)
			if (!close_mailwindow())
				break;
				
		external_mail(*((char **)&msg[3]));
		break;
		}
#endif

	case VA_PROGSTART:
		free_avserver();
		break;
		
	case AV_STARTED:
		{
		char *pointer = *(char **)&msg[3];

		if (pointer)
  		Mfree(pointer);
  	break;
		}
		
	case BUBBLEGEM_ACK:
		{
		char *pointer = *(char **)&msg[5];
		if (bubble_free(pointer))
			msg[0] = 0;
  	break;
  	}
  		
	case AP_TERM:
		exit_flg = TRUE;
		break;

	case 0x5810:
		if (box_open)
			switch(dial_function)
				{
				case 1:		
					{
					check_id(dial_identity, dial_flg); 
					break;
					}
				case 2:		
					send_idqueue(dial_identity, dial_flg); break;
				}
		break;
		
	case MN_SELECTED:
		menu_action(msg);
		break;

	case WM_NEWTOP:	
	case WM_TOPPED:
#ifndef EVAL
		check_editpaste();
#endif		
		cursor_off();
		wind_get_int(ROOT, WF_TOP, &abookset_wind);
		wind_set_int(msg[3], WF_TOP, 0);
		enabler();							
		cursor_on();
		break;

	case WM_FULLED:
		{
		int cx, cy, cw, ch;
		int fx, fy, fw, fh;
		
		wind_get(msg[3], WF_CURRXYWH, &cx, &cy, &cw, &ch);
		wind_get(msg[3], WF_FULLXYWH, &fx, &fy, &fw, &fh);
		if (cx == fx && cy == fy && cw == fw && ch == fh)
			wind_get(msg[3], WF_PREVXYWH, &fx, &fy, &fw, &fh);
			
		cursor_off();
		wind_set(msg[3], WF_CURRXYWH, fx, fy, fw, fh);
		cursor_on();

		if (msg[3] == mail_wind) 		set_mailslider();
		if (msg[3] == pop_wind) 		set_popslider();
		if (is_listwindow(msg[3]))	set_listslider(window_box(msg[3]));
		if (is_viewwindow(msg[3]))	set_viewslider(window_view(msg[3]));
		break;					
		}
		
	case WM_SIZED:
	case WM_MOVED:
		wind_set(msg[3], WF_CURRXYWH, msg[4], msg[5], msg[6], msg[7]);
		if (msg[3] == tree_wind)    redraw_treewind(msg[4], msg[5], msg[6], msg[7]);
		if (msg[3] == box_wind)     
			{
			update_boxinfo();
			redraw_boxwind(msg[4], msg[5], msg[6], msg[7]);
			}
		if (msg[3] == mail_wind) 		set_mailslider();
		if (msg[3] == pop_wind) 		set_popslider();
		if (is_listwindow(msg[3]))	set_listslider(window_box(msg[3]));
		if (is_viewwindow(msg[3]))	set_viewslider(window_view(msg[3]));
		break;

	case WM_REDRAW:
#ifndef EVAL
		if (msg[3] == mail_wind)
			check_editpaste();
#endif
		
	default:
		if (msg[3] == tree_wind)	  { tree_window_message( msg); break; 	}
		if (msg[3] == box_wind)			{ box_window_message(msg); break; 	}
		if (msg[3] == mail_wind) 		{ mail_window_message(msg); break; 	}
		if (msg[3] == pop_wind)	 		{ pop_window_message(msg); break; 	}
		if (is_listwindow(msg[3]))	{ list_window_message(msg, window_box(msg[3])); break; 	}
		if (is_viewwindow(msg[3]))	{ view_window_message(msg, window_view(msg[3])); break; }
		if (msg[3] == poll_wind)		{ poll_window_message(msg); break; }
		break;
	}
}

/* -----------------------------------
   | Check for incoming user message |
   ----------------------------------- */
int check_user_message(void)
{
static time_t last_call = 0;
static long diff;
int bsk, bclicks;

DEBUG();
diff = clock() - last_call;	
if (diff < CLK_TCK / 5)
	return FALSE;

last_call = clock();	
  	  
if (!naes)
	{
	graf_mkstate( &evnt);
	bsk = evnt.bstate & 3;
	if ( !(bsk) )
		bclicks = 256+2;
	else
	  bclicks = 0;
	 
	bsk = 0;
	}
else
	{
	evnt_multi(MU_TIMER,
						 258, 3, 0,
						 0, &empty,
						 0, &empty,
						 msg,
						 1L,
						 &evnt, &k_return, &clicks);
	bsk = evnt.bstate & 3;
	bclicks = 256+2;
	}

evnt.bstate = 0;
which = evnt_multi(MU_MESAG|MU_TIMER|MU_KEYBD|MU_BUTTON,
									 bclicks, 3, bsk,
									 0, &empty,
									 0, &empty,
									 msg,
									 0L,
									 &evnt, &k_return, &clicks);

if (which & MU_BUTTON)
	{
	if (evnt.bstate == 0)
		{
		EVNTDATA gevnt;
		
		graf_mkstate(&gevnt);
		
		if (!gevnt.bstate)
			which &= ~MU_BUTTON;
		else
			evnt.bstate = gevnt.bstate;
		}
	}

if (which & ~MU_TIMER)
	{
	which = evnt_handler(which, msg, &evnt, k_return, clicks);
	app_callback(which, msg, &evnt, k_return, clicks);
	return TRUE;
	}
else
	return FALSE;									 
}


