/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

#define FAULT 0
#define SUCCS 1

typedef unsigned char uchar;

/*******************************
 test match of file name (3.13a)
*******************************/
/*@*/
int matchpattern(uchar *s, uchar *p, uchar cs)
{
  uchar s_c, p_c;
  int matched, reverse;

DEBUG();
  for (;(p_c=*p++)!='\0';)
  {
    s_c=*s++;

    switch (p_c)
    {
      case '\\':
        if (s_c!=*p++)
          return(FAULT);
        break;
      case '?':
        if (s_c=='\0')
          return(FAULT);
        break;
      case '*':
        if (s_c=='\0')
          return((*p=='\0') ? SUCCS :FAULT);
        else
        {
          if (*p!='\0')
          {
            s--;
            while (!matchpattern(s,p,cs))
              if (*++s=='\0')
                return(FAULT);
          }
          return(SUCCS);
        }
      case '[':
        matched=FAULT;

        if (*p=='^')
        {
          reverse=SUCCS;
          p++;
        }
        else
          reverse=FAULT;

        for (; (p_c=*p++)!='\0' && p_c!=']';)
        {
          if (p_c=='-')
          {
            if (s_c<=*p++ && s_c>=p[-3])
              matched=SUCCS;
          }
          else if (p_c=='\\' && s_c==*p++)
            matched=SUCCS;
          else if (s_c==p_c && p_c!='|')
            matched=SUCCS;
        }

        if (matched==reverse)
          return(FAULT);

        break;
      default:
        if( cs )
        {
            if (s_c!=p_c)
              return(FAULT);
        }
        else
        {
          if (tolower(s_c)!=tolower(p_c))
              return(FAULT);
        }
        break;
    }
  }

  return(*s=='\0');
}

int matchpattern2(uchar *s, uchar *e, uchar *p, uchar cs)
{
  uchar s_c, p_c;
  int matched, reverse;
  uchar *start = s;

DEBUG();
  for (;(p_c=*p++)!='\0' && s <= e;)
  {
  	if (s == e)
  		s_c = 0;
  	else
    	s_c=*s++;

    switch (p_c)
    {
      case '\\':
        if (s_c!=*p++)
          return 0;
        break;
      case '?':
        if (s_c=='\0')
          return 0;
        break;
      case '*':
        if (s_c=='\0')
          return((*p=='\0') ? s - start : 0);
        else
        {
         	int len = 0;
          	
          if (*p!='\0')
          {
            s--;
            while ((len = matchpattern2(s,e,p,cs)) == 0)
              if (++s == e)
                return 0;
          }
          return s - start + len;
        }
      case '[':
        matched=FAULT;

        if (*p=='^')
        {
          reverse=SUCCS;
          p++;
        }
        else
          reverse=FAULT;

        for (; (p_c=*p++)!='\0' && p_c!=']';)
        {
          if (p_c=='-')
          {
            if (s_c<=*p++ && s_c>=p[-3])
              matched=SUCCS;
          }
          else if (p_c=='\\' && s_c==*p++)
            matched=SUCCS;
          else if (s_c==p_c && p_c!='|')
            matched=SUCCS;
        }

        if (matched==reverse)
          return 0;

        break;
      default:
        if( cs )
        {
            if (s_c!=p_c)
              return 0;
        }
        else
        {
          if (tolower(s_c)!=tolower(p_c))
              return 0;
        }
        break;
    }
  }

  return s - start;
}
