/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

/* -------------------
   | Initialize menu |
   -------------------*/
void init_menu(void)
{
DEBUG();
menu_icheck(menu_adr, M_BCC, 		mail_options.bcc_flg);
menu_icheck(menu_adr, M_CC, 		mail_options.cc_flg);
menu_icheck(menu_adr, M_FROM, 	mail_options.from_flg);
menu_icheck(menu_adr, M_RPLTO, 	mail_options.replyto_flg);
menu_icheck(menu_adr, M_SUBJCT, mail_options.sub_flg);
menu_icheck(menu_adr, M_ID_LINE, mail_options.idline_flg);
menu_icheck(menu_adr, M_ANSI, 	mail_options.ansi_flg);

default_enable();
}

/* ----------------------
   | Check menu message |
   ---------------------- */
void menu_action(int *msg)
{
int title = msg[3];
int top;

DEBUG();
wind_get_int(ROOT, WF_TOP, &top);

switch(msg[4])
	{
	case M_ABOUT:
		about_info();
		break;

	case M_NEWBOX:
		new_box();
		break;
		
	case M_OPNBOX:
		open_box();
		break;

	case M_CLSBOX:
		close_box();
		break;
		
	case M_NFOLD:
		tree_createfolder();
		break;

	case M_CYCLEWIN:
		next_window();
		break;

	case M_CLOSEWIN:
		close_topwind();
		break;
						
	case SAVE_MSG:
		{
		if (is_listwindow(top))
			save_mail(window_box(top));
		else
			if (is_viewwindow(top))
				save_view(window_view(top));
		break;
		}
		
	case M_SAVATT:
		{
		if (is_viewwindow(top)) open_attachment(window_view(top), 0);
		break;
		}

#ifdef PRINTING
	case M_PSETUP:
		PrinterSetup();
		break;
		
	case M_PRINT:
		if (is_listwindow(top))
			print_mail(window_box(top));
		else
			if (is_viewwindow(top))
				print_view(window_view(top));
		break;
#endif 
		
	case M_QUIT:
		if (mail_wind > -1)
			if (!close_mailwindow())
				break;
		exit_flg = TRUE;
		break;

	case M_UNDO:
		if (top == mail_wind)	undo_edit();
		break;
		
	case M_CUT:
		if (top == mail_wind)	cut_edit();
		break;
		
	case M_COPY:
		if (top == mail_wind)	
			copy_edit();
		else
			if (is_viewwindow(top))
				copy_view(window_view(top));
		break;
		
	case M_PASTE:
		if (top == mail_wind)	paste_edit();
		break;

	case M_PASTEFILE:
		if (top == mail_wind)	paste_file();
		break;
		
	case M_DEL:
		if (top == mail_wind)	
			del_edit();
		else if (top == tree_wind)
			tree_delete(-1);
		else if (is_listwindow(top))
			delete_listmail(window_box(top));
		else if (is_viewwindow(top))
			delete_viewmail(window_view(top));
		else if (top == pop_wind) 
			mark_deletion();			
		break;
		
	case M_SELALL:
		{
		if (top == pop_wind)
			select_all();
		else
			if (is_listwindow(top))
				select_all_list(window_box(top));
			else
				if (top == mail_wind)
					select_all_edit();
		break;
		}

	case M_SEARCH:
		open_find(top);
		break;
		
	case M_REPLACE:
		open_replace(top);
		break;
		
	case M_ABOOK:
		address_book();
		break;
						
	case M_NEWML:
		new_mail(NULL, NULL);
		break;

	case EDITMAIL:
		{
		int i;
		int box = box_number("QUEUE");
		
		for (i = 0; i < box_manager[box].box_entries; i++)
			if (box_manager[box].index[i].sel & 1)
				{
				edit_mail(box, &box_manager[box].index[i]);
				box_manager[box].index[i].sel &= ~2;
				save_box_index(box);
			 	break;
				}
		break;
		}

	case M_SCAN:
		if (top == tree_wind && tree_selection != -1)
			check_id(boxes[tree_selection].identity, FALSE);
		else
			{
			IDENTITY id;
			
			if (getDefaultIdentity(&id))
				check_id(id.name, FALSE);
			}
		break;
		
	case M_SEND:
		if (top == tree_wind && tree_selection != -1)
			send_idqueue(boxes[tree_selection].identity, TRUE);
		else
			{
			IDENTITY id;
			
			if (getDefaultIdentity(&id))
				send_idqueue(id.name, TRUE);
			}
		break;
		
	case M_CHECK:
		if (top == tree_wind && tree_selection != -1)
			check_id(boxes[tree_selection].identity, TRUE);
		else
			{
			IDENTITY id;
			
			if (getDefaultIdentity(&id))
				check_id(id.name, TRUE);
			}
		break;
		
	case M_SENDCHECK:
		if (top == tree_wind && tree_selection != -1)
			{
			if (!send_idqueue(boxes[tree_selection].identity, TRUE))
				check_id(boxes[tree_selection].identity, -1);
			}
		else
			{
			IDENTITY id;
			
			if (getDefaultIdentity(&id))
				if (!send_idqueue(id.name, TRUE))
					check_id(id.name, -1);
			}
		break;
		
	case M_FWD:
		{
		if (is_listwindow(top))
			forward_mail(window_box(top));
		else
			if (is_viewwindow(top))
				forward_view(window_view(top));
		break;
		}

	case M_REPLY:
		{
		if (is_listwindow(top))
			reply_mail(window_box(top));
		else
			if (is_viewwindow(top))
				reply_view(window_view(top));
		break;
		}

	case M_AGAIN:
		{
		if (is_listwindow(top))
			send_again(window_box(top));
		else
			if (is_viewwindow(top))
				send_view_again(window_view(top));
		break;
		}

/*
	case M_SEND:
		{
		if (is_listwindow(top))
			send_idqueue(window_box(top));
		else
			if (is_viewwindow(top))
				send_queue_view(window_view(top));
		break;
		}
*/				
	case M_EDITATT:
		if (top == mail_wind)
			attach_file();
		break;
		
	case M_BCC:
		mail_options.bcc_flg ^= 1;
		menu_icheck(menu_adr, M_BCC, mail_options.bcc_flg);
		redraw_wholemail();
		break;
		
	case M_CC:
		mail_options.cc_flg ^= 1;
		menu_icheck(menu_adr, M_CC, mail_options.cc_flg);
		redraw_wholemail();
		break;
		
	case M_FROM:
		mail_options.from_flg ^= 1;
		menu_icheck(menu_adr, M_FROM, mail_options.from_flg);
		redraw_wholemail();
		break;

	case M_RPLTO:
		mail_options.replyto_flg ^= 1;
		menu_icheck(menu_adr, M_RPLTO, mail_options.replyto_flg);
		redraw_wholemail();
		break;
		
	case M_SUBJCT:
		mail_options.sub_flg ^= 1;
		menu_icheck(menu_adr, M_SUBJCT, mail_options.sub_flg);
		redraw_wholemail();
		break;

	case M_ID_LINE:
		mail_options.idline_flg ^= 1;
		menu_icheck(menu_adr, M_ID_LINE, mail_options.idline_flg);
		redraw_wholemail();
		break;

	case ALLHEAD:
		if (is_viewwindow(top))
			switch_headerview(window_view(top));
		break;
						
	case M_ANSI:
		{
		if (is_viewwindow(top))
			{
			int idx = window_view(top);
			
			switch_ansiview(idx);
			view_enable(idx);
			}
		else
			{
			mail_options.ansi_flg ^= 1;
	/*
			if (mail_wind > -1)
				{
				if (mail_options.ansi_flg)
					convert_from_ansi((unsigned char*)new_buffer.body, new_buffer.body_len);
				else
					convert_to_ansi((unsigned char*)new_buffer.body, new_buffer.body_len);
				redraw_wholemail();
				}
		*/		
			menu_icheck(menu_adr, M_ANSI, mail_options.ansi_flg);
			}
		break;
		}

	case M_FAVOR:
		if (box_wind == -1)
			open_boxwind();
		break;

	case M_FILTER:
		edit_filters(-1);
		break;
		
	case M_SIGNAT:
		edit_signatures();
		break;
		
	case M_IDENTITY:
		edit_identities();
		break;
		
	case M_OPTION:
		set_options();
		break;
	
	case M_EXTERN:
		edit_externals();
		break;
		
	case M_WRAP:
		if (top == mail_wind)
			{
			new_buffer.word_wrap ^= 1;
			menu_icheck(menu_adr, M_WRAP, new_buffer.word_wrap);
			}
		else
			if (is_listwindow(top))
				{
				mail_options.viewwrap ^= 1;
				menu_icheck(menu_adr, M_WRAP, mail_options.viewwrap);
				}
		break;

	case M_SAVOPT:
		save_mail_config();
		break;
	}
	
menu_tnormal(menu_adr, title, TRUE);
}

/* ---------------------
   | Check pressed key |
   --------------------- */
void system_keys(void)
{
int top;

DEBUG();
wind_get_int(ROOT, WF_TOP, &top);

if (evnt.kstate & 4)
	{
	switch(k_return & 0xFF)
		{
		case  1:
			if (!(menu_adr[M_SELALL].ob_state & DISABLED))
				if (top == pop_wind)
					select_all();
				else
					if (is_listwindow(top))
						select_all_list(window_box(top));
					else
						if (top == mail_wind)
							select_all_edit();
			break;

		case 3:
			if (!(menu_adr[M_COPY].ob_state & DISABLED))
				if (top == mail_wind) 
					copy_edit();
				else
					if (is_viewwindow(top))
						copy_view(window_view(top));
			break;

		case  6:
			if (!(menu_adr[M_SEARCH].ob_state & DISABLED))	
				open_find(top);
			break;
			
		case  9:
			about_info();
			break;
						
		case 13:
			break;

		case 14:
			new_box();
			break;
			
		case 15:
			open_box();
			break;
			
		case 16:
			if (!(menu_adr[M_PRINT].ob_state & DISABLED))
				if (is_listwindow(top))
					print_mail(window_box(top));
				else
					if (is_viewwindow(top))
						print_view(window_view(top));
			break;

		case 17:
			if (mail_wind > -1)
				if (!close_mailwindow())
					break;
			exit_flg = TRUE;
			break;

		case 18:
			if (!(menu_adr[M_REPLACE].ob_state & DISABLED))	
				open_replace(top);
			break;
			
		case 19:
			break;
			
		case 20:
			set_options();
			break;

		case 21:		/* Control + U */
			break;
			
		case 22:
			{
			if (evnt.kstate & 3)
				{
				int w = word_wrap;
				word_wrap ^= 1;
				if (top == mail_wind) paste_edit();
				word_wrap = w;
				}
			else
				if (top == mail_wind) paste_edit();
			break;
			}
			
		case 24:
			if (top == mail_wind) cut_edit();
			break;
		}
	}
else
	if (evnt.kstate & 8)
		{
		switch(k_return >> 8)
			{
			case 0x32:
				if (!(menu_adr[M_NEWML].ob_state & DISABLED))	new_mail(NULL, NULL);
				break;

			case 0x25: 	/* Alt + K */
				if (top == tree_wind && tree_selection != -1)
					{
					if (!send_idqueue(boxes[tree_selection].identity, TRUE))
						check_id(boxes[tree_selection].identity, -1);
					}
				else
					{
					IDENTITY id;
					
					if (getDefaultIdentity(&id))
						if (!send_idqueue(id.name, TRUE))
							check_id(id.name, -1);
					}
				break;
				
			case 0x1F:  /* Alt + S */
				if (top == tree_wind && tree_selection != -1)
					check_id(boxes[tree_selection].identity, FALSE);
				else
					{
					IDENTITY id;
					
					if (getDefaultIdentity(&id))
						check_id(id.name, FALSE);
					}
				break;
				
			case 0x10: 	/* Alt + Q */
				if (top == tree_wind && tree_selection != -1)
					send_idqueue(boxes[tree_selection].identity, TRUE);
				else
					{
					IDENTITY id;
					
					if (getDefaultIdentity(&id))
						send_idqueue(id.name, TRUE);
					}
				break;
				
			case 0x2E:	/* Alt + C */
				if (!(menu_adr[M_CHECK].ob_state & DISABLED))	
					check_id("\0", -1);
				break;

			case 0x78:
				mail_options.bcc_flg ^= 1;
				menu_icheck(menu_adr, M_BCC, mail_options.bcc_flg);
				redraw_wholemail();
				break;

			case 0x79:
				mail_options.cc_flg ^= 1;
				menu_icheck(menu_adr, M_CC, mail_options.cc_flg);
				redraw_wholemail();
				break;

			case 0x7A:
				mail_options.from_flg ^= 1;
				menu_icheck(menu_adr, M_FROM, mail_options.from_flg);
				redraw_wholemail();
				break;

			case 0x7B:
				mail_options.sub_flg ^= 1;
				menu_icheck(menu_adr, M_SUBJCT, mail_options.sub_flg);
				redraw_wholemail();
				break;
				
			case 0x7C:
				mail_options.idline_flg ^= 1;
				menu_icheck(menu_adr, M_ID_LINE, mail_options.idline_flg);
				redraw_wholemail();
				break;
			}

		}
	else
		switch(k_return)
			{
			case 0x6100:	/* UNDO */
				if (!(menu_adr[M_UNDO].ob_state & DISABLED))
					{
					if (top == mail_wind)	undo_edit();
					}
				break;
			}
}

/* -----------------------
   | Enable menu options |
   ----------------------- */
void enabler(void)
{
int new_top;

DEBUG();
default_enable();

wind_get_int(ROOT, WF_TOP, &new_top);

if (is_listwindow(new_top)) list_enable(window_box(new_top));
if (is_viewwindow(new_top)) view_enable(window_view(new_top));
if (new_top == pop_wind) 		pop_enable();
if (new_top == mail_wind)		edit_enable();
if (new_top == tree_wind) 	tree_enable();
}
