/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

static char default_table[] = { 0,  0,  0,  0,  0,  0,  0,  0,
															  0,  0,  0,  0,  0,  0,  0,  0,
															  0,  1,  2,  3,  4,  5,  6,  7,
															  8,  9, 10, 11, 12, 13, 14, 15,
															 16, 17, 18, 19, 20, 21, 22, 23,
															 24, 25, 26, 27, 28, 29, 30, 31,
															 32, 33, 34, 35, 36, 37, 38, 39,
															 40, 41, 42, 43, 44, 45, 46, 47,
															 48, 49, 50, 51, 52, 53, 54, 55,
															 56, 57, 58, 59, 60, 61, 62, 63,
															  0,  0,  0,  0,  0,  0,  0,  0,
															  0,  0,  0,  0,  0,  0,  0,  0,
															  0,  0,  0,  0,  0,  0,  0,  0,
															  0,  0,  0,  0,  0,  0,  0,  0,
															 };
															 
/* ---------------------------
   | Decode 7 bit attachment |
   --------------------------- */
void Decode7Bit(int in, int out, char *line, char *boundary)
{
int i;

DEBUG();
do
	{
	if (!get_line(in, line, 1000))
		break;

	i = (int)strlen(line);
	while(i && line[i - 1] == ' ') i--;
	line[i] = 0;
	
	if (!strncmp(line, "--", 2) && !strcmp(line + 2, boundary) && boundary[0])
		break;
			
	WriteStringLF(out, line);
	
	}while(1);
}

/* ---------------------------
   | Decode 8 bit attachment |
   --------------------------- */
void Decode8Bit(int in, int out, char *line, char *boundary)
{
int i;

DEBUG();
do
	{
	if (!get_line(in, line, 1000))
		break;

	i = (int)strlen(line);
	while(i && line[i - 1] == ' ') i--;
	line[i] = 0;
	
	if (!strncmp(line, "--", 2) && !strcmp(line + 2, boundary) && boundary[0])
		break;
			
	WriteStringLF(out, line);
	
	}while(1);
}

/* ----------------------------
   | Decode Quoted attachment |
   ---------------------------- */
void DecodeQuoted(int in, int out, char *line, char *boundary)
{
int i;
int l, v1, v2;
char c;

DEBUG();
do
	{
	if (!get_line(in, line, 1000))
		break;

	i = (int)strlen(line);
	while(i && line[i - 1] == ' ') i--;
	line[i] = 0;
	if (i)
		c = line[i - 1];
	else
		c = 0;
		
	if (!strncmp(line, "--", 2) && !strcmp(line + 2, boundary) && boundary[0])
		break;

	l = 0;
	while(l < i - 2)
		{
		if (line[l] == '=')
			{
			v1 = line[l + 1] - '0'; if (v1 > 9) v1 -= 'A' - '9' - 1;
			v2 = line[l + 2] - '0'; if (v2 > 9) v2 -= 'A' - '9' - 1;
			line[l] = (v1 << 4) | v2;
			memmove(line + l + 1, line + l + 3, i - l - 2);
			i -= 2;
			}
		l++;
		}

	if (c == '=')
		{
		line[i - 1] = 0;
		WriteString(out, line);
		}
	else
		WriteStringLF(out, line);
	
	}while(1);
}

/* --------------------------------
   | Decode base64 bit attachment |
   -------------------------------- */
void DecodeBase64(int in, int out, char *line, char *boundary)
{
int i, l;
char *mem;
int mem_pos, b;
unsigned w;
char m[256];

DEBUG();
l = 0;
while (b64alpha[l])
	{
	m[b64alpha[l]] = l;
	l++;
	}
	
mem = (char *)malloc(30000);
mem_pos = 0;
b = 0;
w = 0;
do
	{
	if (!get_line(in, line, 1000))
		break;

	i = (int)strlen(line);
	while(i && line[i - 1] == ' ') i--;
	line[i] = 0;
	
	if (!strncmp(line, "--", 2) && !strcmp(line + 2, boundary) && boundary[0])
		break;
	
	if (!line[0])
		break;
	
	l = 0;
	while (l < i)
		{
		if (line[l] == '=')
			break;
		
		w <<= 6;
		w |= m[line[l]];
		b += 6;
		
		if (b >= 8)
			{
			mem[mem_pos++] = w >> (b - 8);
			b -= 8;
			if (mem_pos == 30000)
				{
				WriteBytes(out, mem, 30000);
				mem_pos = 0;
				}
			}
			
		l++;
		}
		
	}while(1);

if (mem_pos)
	WriteBytes(out, mem, mem_pos);
	
free(mem);
}

/* -------------------------------
   | Decode uudecoded attachment |
   ------------------------------- */
void DecodeUudecode(int in, int out, char *line, char *boundary, long endpos)
{
int i, l, type = 0, table_set = FALSE;
char *mem;
int mem_pos, b, c, ctotal;
unsigned w;
char table[128];

DEBUG();
mem = (char *)malloc(30000);
mem_pos = 0;
do
	{
	if (get_current_position() >= endpos)
		break;
		
	if (!get_line(in, line, 1000))
		break;

	if (!strcmp(line, "end"))
		break;

	if (type != 2)
		{
		if (!strncmp(line, "table", 5))
			{
			b = 0;
			type = 1;
			table_set = TRUE;
			continue;
			}
		else	
			if (!strncmp(line, "begin", 5) && (line[5] == ' ' || line[5] == '\t') &&
					(!strncmp(line + 6, "644", 3) || !strncmp(line + 6, "600", 3)))
				{
				type = 2;

				if (!table_set)
					{
					if (!strncmp(line + 6, "600", 3))
						memcpy(table + 16, default_table, 128-16);
					else
						memcpy(table, default_table, 128);
					}
					
				continue;
				}
		}
	else
	 	if (!table[line[0]])
	 		break;
				
	if (!strncmp(line, "--", 2) && !strcmp(line + 2, boundary) && boundary[0])
		break;
	i = (int)strlen(line);
	
	switch(type)
		{
		case 1:
			l = 0;
			while (l < i)
				table[line[l++]] = b++;
			break;
			
		case 2:
			l = 1;
			ctotal = table[line[0]];
			c = w = b = 0;
			while (c < ctotal)
				{
				w <<= 6;
				w |= table[line[l]];
				b += 6;
				
				if (b >= 8)
					{
					mem[mem_pos++] = w >> (b - 8);
					c++;
					b -= 8;
					if (mem_pos == 30000)
						{
						WriteBytes(out, mem, 30000);
						mem_pos = 0;
						}
					}
					
				l++;
				}
			break;
		}
		
	}while(1);

if (mem_pos)
	WriteBytes(out, mem, mem_pos);
	
free(mem);
}

/* ----------------------------
   | Decode binary attachment |
   ---------------------------- */
void DecodeBinary(int in, int out, char *line, char *boundary)
{
form_alert(1, nobinatt_msg);
}

/* --------------------------
   | Decode 7 bit to memory |
   -------------------------- */
void MemDecode7Bit(char **mem, long *len, int in, char *line, char *boundary, long max_end)
{
int i;

DEBUG();
do
	{
	if (get_current_position() >= max_end)
		break;
		
	if (!get_line(in, line, 1000))
		break;

	i = (int)strlen(line);
	while(i && line[i - 1] == ' ') i--;
	line[i] = 0;
	
	if (!strncmp(line, "--", 2) && !strcmp(line + 2, boundary) && boundary[0])
		break;
			
	line[i++] = '\r';

	if (*len)
		*mem = (char *)realloc(*mem, *len + i);
	else
		*mem = (char *)malloc(i);
	memcpy(*mem + *len, line, i);	
	*len += i;
		
	}while(1);
}

/* --------------------------
   | Decode 8 bit to memory |
   -------------------------- */
void MemDecode8Bit(char **mem, long *len, int in, char *line, char *boundary, long max_end)
{
int i;

DEBUG();
do
	{
	if (get_current_position() >= max_end)
		break;
		
	if (!get_line(in, line, 1000))
		break;

	i = (int)strlen(line);
	while(i && line[i - 1] == ' ') i--;
	line[i] = 0;
	
	if (!strncmp(line, "--", 2) && !strcmp(line + 2, boundary) && boundary[0])
		break;
			
	line[i++] = '\r';

	if (*len)
		*mem = (char *)realloc(*mem, *len + i);
	else
		*mem = (char *)malloc(i);
	memcpy(*mem + *len, line, i);	
	*len += i;
		
	}while(1);
}

/* ---------------------------
   | Decode quoted to memory |
   --------------------------- */
void MemDecodeQuoted(char **mem, long *len, int in, char *line, char *boundary, long max_end)
{
int i;
int l, v1, v2;
char c;

DEBUG();
do
	{
	if (get_current_position() >= max_end)
		break;
		
	if (!get_line(in, line, 1000))
		break;

	i = (int)strlen(line);
	while(i && line[i - 1] == ' ') i--;
	line[i] = 0;
	if (i)
		c = line[i - 1];
	else
		c = 0;
		
	if (!strncmp(line, "--", 2) && !strcmp(line + 2, boundary) && boundary[0])
		break;

	l = 0;
	while(l < i - 2)
		{
		if (line[l] == '=')
			{
			v1 = line[l + 1] - '0'; if (v1 > 9) v1 -= 'A' - '9' - 1;
			v2 = line[l + 2] - '0'; if (v2 > 9) v2 -= 'A' - '9' - 1;
			line[l] = (v1 << 4) | v2;
			memmove(line + l + 1, line + l + 3, i - l - 2);
			i -= 2;
			}
		l++;
		}

	if (c == '=')
		line[--i] = 0;
	else
		line[i++] = '\r';
		
	if (*len)
		*mem = (char *)realloc(*mem, *len + i);
	else
		*mem = (char *)malloc(i);
	memcpy(*mem + *len, line, i);	
	*len += i;
		
	}while(1);
}

/* ----------------------------
   | Decode Base 64 to memory |
   ---------------------------- */
void MemDecodeBase64(char **mem, long *len, int in, char *line, char *boundary, long max_end)
{
int i, l;
char *imem;
int mem_pos, b;
unsigned w;
char m[256];

DEBUG();
l = 0;
while (b64alpha[l])
	{
	m[b64alpha[l]] = l;
	l++;
	}
	
imem = (char *)malloc(30000);
mem_pos = 0;
b = 0;
w = 0;
do
	{
	if (get_current_position() >= max_end)
		break;
		
	if (!get_line(in, line, 1000))
		break;

	i = (int)strlen(line);
	while(i && line[i - 1] == ' ') i--;
	line[i] = 0;
	
	if (!strncmp(line, "--", 2) && !strcmp(line + 2, boundary) && boundary[0])
		break;
	
	if (!line[0])
		break;
	
	l = 0;
	while (l < i)
		{
		if (line[l] == '=')
			break;
		
		w <<= 6;
		w |= m[line[l]];
		b += 6;
		
		if (b >= 8)
			{
			imem[mem_pos++] = w >> (b - 8);
			b -= 8;
			if (mem_pos == 30000)
				{
				if (*len)
					*mem = (char *)realloc(*mem, *len + mem_pos);
				else
					*mem = (char *)malloc(mem_pos);
				memcpy(*mem + *len, imem, mem_pos);	
				*len += mem_pos;
				mem_pos = 0;
				}
			}
			
		l++;
		}
		
	}while(1);

if (mem_pos)
	{
	if (*len)
		*mem = (char *)realloc(*mem, *len + mem_pos);
	else
		*mem = (char *)malloc(mem_pos);
	memcpy(*mem + *len, imem, mem_pos);	
	*len += mem_pos;
	}
	
free(imem);
}

/* ---------------------------
   | Decode Binary to memory |
   --------------------------- */
void MemDecodeBinary(char **mem, long *len, int in, char *line, char *boundary, long max_end)
{
form_alert(1, nobinatt_msg);
}

/* --------------------------------
   | Decode base64 bit attachment |
   -------------------------------- */
void DecodeBase64Line(char *dest, char *line)
{
int i, l;
int mem_pos, b;
unsigned w;
char m[256];

DEBUG();
l = 0;
while (b64alpha[l])
	{
	m[b64alpha[l]] = l;
	l++;
	}
	
i = (int)strlen(line);
while(i && line[i - 1] == ' ') i--;
line[i] = 0;
	
mem_pos = 0;
b = 0;
w = 0;
l = 0;
while (l < i)
	{
	if (line[l] == '=')
		break;
	
	w <<= 6;
	w |= m[line[l]];
	b += 6;
	
	if (b >= 8)
		{
		dest[mem_pos++] = w >> (b - 8);
		b -= 8;
		}
		
	l++;
	}
dest[mem_pos] = 0;
}
