/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

char *encode_type;

/* ------------------------------------
   | Return TRUE if seven bit is okay |
   ------------------------------------ */
int AnalyseSevenBit(int in, long len)
{
int length = 0;
char *buf, *scan;
int bytes, flg = TRUE;

DEBUG();
buf = (char *)malloc(32000);
while (len > 0)
	{
	if (len > 32000)
		{
		Fread(in, 32000, buf);
		bytes = 32000;
		}
	else
		{
		Fread(in, len, buf);
		bytes = (int)len;
		}
	len -= bytes;

	scan = buf;
	while (bytes--)
		{
		if ((*scan & 0x80) || *scan == '\\' || (iscntrl(*scan) && *scan != '\t' && *scan != '\r' && *scan != '\n'))
			{
			flg = FALSE;
			len = 0;
			break;
			}
			
		if (*scan == '\r')
			length = 0;
		else		
			length++; if (length > 76) { flg = FALSE; len = 0; break; }

		scan++;
		}
	}
free(buf);	

Fseek(0, in, 0);

return flg;
}

/* ------------------------------------
   | Return TRUE if eight bit is okay |
   ------------------------------------ */
int AnalyseEightBit(int in, long len)
{
int length = 0;
char *buf, *scan;
int bytes, flg = TRUE;

DEBUG();
buf = (char *)malloc(32000);
while (len > 0)
	{
	if (len > 32000)
		{
		Fread(in, 32000, buf);
		bytes = 32000;
		}
	else
		{
		Fread(in, len, buf);
		bytes = (int)len;
		}
	len -= bytes;

	scan = buf;
	while (bytes--)
		{
		if (iscntrl(*scan) && *scan != '\t' && *scan != '\r' && *scan != '\n')
			{
			flg = FALSE;
			len = 0;
			break;
			}
			
		if (*scan == '\r')
			length = 0;
		else		
			length++; if (length > 76) { flg = FALSE; len = 0; break; }

		scan++;
		}
	}
free(buf);	

Fseek(0, in, 0);

return flg;
}

/* -----------------
   | Test for text |
   ----------------- */
int TestForText(int in, long len, int eight_allowed)
{
char *buf, *scan, c;
int bytes;
long total = len, special = 0L;

DEBUG();
buf = (char *)malloc(32000);
while (len > 0)
	{
	if (len > 32000)
		{
		Fread(in, 32000, buf);
		bytes = 32000;
		}
	else
		{
		Fread(in, len, buf);
		bytes = (int)len;
		}
	len -= bytes;
	
	scan = buf;
	while (bytes--)
		{
		c = *scan;
		if ((c & 0x80) && !eight_allowed)
			special++;
		else
			{
			if (c < ' ' && c != '\t' && c != '\r' && c != '\n' && c != '\f')
				{
				free(buf);	
				Fseek(0, in, 0);
				return FALSE;
				}
			}
			
		scan++;
		}
	}
free(buf);	

Fseek(0, in, 0);

if (special > total * 20 / 100)
	return FALSE;
else
	return TRUE;
}

/* -----------------
   | Test for text |
   ----------------- */
int DotContaining(int in, long len)
{
char *buf;
int bytes, f = FALSE, srch, num;

DEBUG();
buf = (char *)malloc(32004);
Fread(in, 4, buf);
len -= 4;
bytes = 4;

while (len > 0)
	{
	if (len > 32000)
		{
		num = (int)Fread(in, 32000, buf + 4);
		bytes += num;
		}
	else
		{
		num = (int)Fread(in, len, buf + 4);
		bytes += num;
		}
	len -= num;
	
	for (srch = 0; srch < bytes - 4; srch++)
		if (buf[srch] == '\r')
			if (!strncmp(buf + srch, "\r\n.\r\n", 5))
				{
				len = 0;
				f = TRUE;
				break;
				}
	memmove(buf, buf + bytes - 4, 4);
	}
free(buf);	

Fseek(0, in, 0);

return f;
}

/* -------------------------------
   | Check for a encoded message |
   ------------------------------- */
int IsMessage(int in, long len)
{
char *line;
long p;

DEBUG();
clear_inputbuffer();

p = Fseek(0, in, 1);
line = (char *)malloc(1001);
line[0] = ' ';
while(line[0])
	{
	if (Fseek(0, in, 1) >= p + len)
		break;
		
	if (!get_line(in, line, 1000))
		break;

	if (!strncmp(line, "Content-type:", 13))
		{
		free(line);
		return TRUE;
		}
	}
		
free(line);
Fseek(0, in, 0);

return FALSE;
}

/* --------------------------
   | Start encoding of data |
   -------------------------- */
int EncodingStart(int in, long len, int textflg, int eight_allowed)
{
DEBUG();
if (IsMessage(in, len))
	return 9;

if (textflg || TestForText(in, len, eight_allowed))
	{
	if (DotContaining(in, len))
		return 1;
	else
		if (AnalyseSevenBit(in, len))
			return 0;
		else
			if (eight_allowed && AnalyseEightBit(in, len))
				return 3;
			else
				return 1;
	}
else	
	return 2;
}

/* ------------------------
   | Encode 7/8 bit  data |
   ------------------------ */
void EncodeBit(int in, int out, long len)
{
char *buf;

DEBUG();
buf = (char *)malloc(32000);
while(len)
	{
	if (len >= 32000)
		{
		Fread(in, 32000, buf);
		Fwrite(out, 32000, buf);
		len -= 32000;
		}
	else
		{
		Fread(in, len, buf);
		Fwrite(out, len, buf);
		len = 0;
		}
	}

free(buf);
}

/* -----------------------
   | Encode BASE64  data |
   ----------------------- */
void EncodeBase64(int in, int out, long len)
{
char obuf[76];
int opos, bytes;
unsigned char *buf, *scan;
long coded;

DEBUG();
buf = (unsigned char *)malloc(30000);
opos = 0;

while(len)
	{
	if (len >= 30000)
		{
		Fread(in, 30000, buf);
		bytes = 30000;
		len -= 30000;
		}
	else
		{
		Fread(in, len, buf);
		bytes = (int)len;
		len = 0;
		}
	
	scan = buf;
	while(bytes >= 3)
		{
		coded  = (long)*(scan++) << 16;
		coded |= (long)*(scan++) << 8;
		coded |= *(scan++);
		
		obuf[opos++] = b64alpha[coded >> 18];
		obuf[opos++] = b64alpha[(coded >> 12) & 0x3F];
		obuf[opos++] = b64alpha[(coded >> 6) & 0x3F];
		obuf[opos++] = b64alpha[coded & 0x3F];
		
		bytes -= 3;
		if (opos == 76)
			{
			WriteBytesLF(out, obuf, opos);
			opos = 0;
			}
		}
	}

switch(bytes)
	{
	case 2:
		coded = ((long)*(scan++) << 8) | *(scan++);
		obuf[opos++] = b64alpha[coded >> 10];
		obuf[opos++] = b64alpha[(coded >> 4) & 0x3F];
		obuf[opos++] = b64alpha[(coded << 2) & 0x3F];
		obuf[opos++] = '=';
		break;
			
	case 1:
		coded = *(scan++);
		obuf[opos++] = b64alpha[coded >> 2];
		obuf[opos++] = b64alpha[(coded << 4) & 0x3F];
		obuf[opos++] = '=';
		obuf[opos++] = '=';
		break;
	}
	
if (opos)
	WriteBytesLF(out, obuf, opos);

free(buf);
}

/* ----------------------
   | Encode quoted data |
   ---------------------- */
void EncodeQuoted(int in, int out, long len, int eight_allowed)
{
char *buf, obuf[80];
int bytes, c, pos;

DEBUG();
buf = (char *)malloc(32000);

c = 0;
pos = 0;
bytes = 0;
while (len)
	{
	if (buf[pos] == '\r' && pos + 1 == bytes && len > 1)
		{
		buf[0] = '\r';
		pos = 0;
		Fread(in, 1, buf + 1);
		bytes = 2;
		}
		
	if (pos == bytes)
		{
		if (len >= 32000)
			{
			Fread(in, 32000, buf);
			bytes = 32000;
			}
		else
			{
			Fread(in, len, buf);
			bytes = (int)len;
			}
		pos = 0;
		}
				
	if (buf[pos] == '\r' && pos + 1 < bytes && buf[pos + 1] == '\n')
		{
		obuf[c++] = '\r';
		obuf[c++] = '\n';
		WriteBytes(out, obuf, c);
		c = 0;
		pos += 2;
		len -= 2;
		}
	else
		{
		if (((buf[pos] & 0x80) && !eight_allowed)	|| buf[pos] == '=' || buf[pos] == '.' || buf[pos] == '\\')
			{
			if (c + 3 > 75)
				{
				WriteBytes(out, obuf, c);
				WriteBytesLF(out, "=", 1);
				c = 0;
				}
				
			obuf[c++] = '=';
			obuf[c++] = hexchar[(buf[pos] >> 4) & 0xF];
			obuf[c++] = hexchar[buf[pos++] & 0xF];
			}
		else
			{
			if (c == 75)
				{
				WriteBytes(out, obuf, 75);
				WriteBytesLF(out, "=", 1);
				c = 0;
				}
			obuf[c++] = buf[pos++];
			}
			
		len--;	
		}	
	}

if (c)
	WriteBytes(out, obuf, c);
}

/* ----------------------
   | Encode buffer data |
   ---------------------- */
void EncodeBuffer(int encode, int in, int out, long len, char *info, char *name, int textflg, int eight_allowed)
{
DEBUG();
encoding_offset = Fseek(0L, out, 1);

if (!textflg || encode != 0)
	{
	if (encode != 2)
		{
		if (encode == 5 || encode == 4)
			{
			WriteStringLF(out, "Content-type: text/plain; charset=\"ISO-8859-1\"");
			if (encode == 4)
				encode = 1;
			else
				encode = 3;
			}
		else
			WriteStringLF(out, "Content-type: text/plain; charset=\"US-ASCII\"");
		}
	else
		{
		WriteString(out, "Content-type: Application/octet-stream");
		if (name)
			{
			WriteString(out, "; name=\"");
			WriteString(out, name);
			WriteStringLF(out, "\"");
			WriteString(out, "Content-disposition: attachment; filename=\"");
			WriteString(out, name);
			WriteString(out, "\"");
			}
		WriteStringLF(out, "\0");
		}
	
	WriteString(out, "Content-transfer-encoding: ");
	switch(encode)
		{
		case 0:	WriteStringLF(out, "7BIT"); break;
		case 1: WriteStringLF(out, "quoted-printable"); break;
		case 2: WriteStringLF(out, "base64"); break;
		case 3: WriteStringLF(out, "8BIT"); break;
		}
	if (info)
		{
		WriteString(out, "Content-description: ");
		WriteStringLF(out, info);
		}
	mime_flg = 1;
	}
	
WriteStringLF(out, "\0");

encoded_content = Fseek(0L, out, 1);
switch(encode)
	{
	case 0:		EncodeBit(in, out, len);											break;
	case 1:		EncodeQuoted(in, out, len, eight_allowed);		break;
	case 2:		EncodeBase64(in, out, len);		break;
	case 3:		EncodeBit(in, out, len);			break;
/*	case 4:*/
/*	case 5:*/
	}
}

/* ---------------
   | Encode file |
   --------------- */
int EncodeFile(int in, int out, char *info, int textflg, char *name, long length, int encoding, int eight_allowed)
{
long pos;
int is_ansi;

DEBUG();
is_ansi = (encoding == -2);

pos = Fseek(0L, in, 1);
if (encoding <= -1)
	encoding = EncodingStart(in, length, textflg, eight_allowed);

Fseek(pos, in, 0);

if (encoding == 9)
	{
	encoding_offset = 0;
	CopyPart(out, in, length);
	encode_type = "message";
	}
else
	{
	if (is_ansi)
		if (encoding == 3) 
			encoding = 5;
		else
			if (encoding == 1)
				encoding = 4;
				
	EncodeBuffer(encoding, in, out, length, info, name, textflg, eight_allowed);
	encode_type = enc_tab[encoding];
	}
	
return encoding;
}

/* -----------------------
   | Encode BASE64  data |
   ----------------------- */
void EncodeBase64Line(char *dest, char *line)
{
DEBUG();

EncodeBase64Memory(dest, line, (int)strlen(line));
}

void EncodeBase64Memory(char *dest, char *line, int bytes)
{
int opos;
unsigned char *scan;
long coded;

DEBUG();

opos = 0;
scan = (unsigned char *)line;
while(bytes >= 3)
	{
	coded  = (long)*(scan++) << 16;
	coded |= (long)*(scan++) << 8;
	coded |= *(scan++);
	
	dest[opos++] = b64alpha[coded >> 18];
	dest[opos++] = b64alpha[(coded >> 12) & 0x3F];
	dest[opos++] = b64alpha[(coded >> 6) & 0x3F];
	dest[opos++] = b64alpha[coded & 0x3F];
	
	bytes -= 3;
	}

switch(bytes)
	{
	case 2:
		coded = ((long)*(scan++) << 8) | *(scan++);
		dest[opos++] = b64alpha[coded >> 10];
		dest[opos++] = b64alpha[(coded >> 4) & 0x3F];
		dest[opos++] = b64alpha[(coded << 2) & 0x3F];
		dest[opos++] = '=';
		break;
			
	case 1:
		coded = *(scan++);
		dest[opos++] = b64alpha[coded >> 2];
		dest[opos++] = b64alpha[(coded << 4) & 0x3F];
		dest[opos++] = '=';
		dest[opos++] = '=';
		break;
	}
dest[opos] = 0;	
}
