/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

int debug = FALSE;

void write_log(char *sendstr, int len);

int min(int a, int b)	{	return ((a < b) ? a : b);	}
int max(int a, int b)	{	return ((a > b) ? a : b);	}

static int err;
char in_buffer[32000];
int in_buf, in_ptr;
extern long mark_start, mark_end;
extern int but_disabled[12];
int dial_function;
char dial_identity[21];
int dial_flg;

#define MGLOBAL 0x20

/*
   Mxmask() liefert eine Bitmaske mit der man das Moduswort eines
   Mxalloc() Aufrufs maskieren sollte, falls man Protectionbits
   benutzen mchte. Dies ist notwendig, da Mxalloc() bei der Angabe
   von Protectionbits leider in einigen GEMDOS Implementierungen
   fehlerhafte Resultate liefert, die in der Folge zum Absturz des
   Systems fuehren koennen. ((c) 1994 Martin Osieka)
   Anwendungsbeispiel:
   mxMask = Mxmask();
   p = mxMask ? Mxalloc( size, 0x43 & mxMask) : Malloc( size); */

short Mxmask (VOID)
{
    VOID *svStack;      /* Supervisorstack */
    LONG sRAM, sRAMg;   /* ST-RAM          */
    LONG aRAM, aRAMg;   /* Alternate RAM   */
 
    /*
    // Beispieltabelle moeglicher Werte:
    //           | newfashion  | oldfashion
    // sRAM aRAM | sRAMg aRAMg | sRAMg aRAMg
    //   1    0  |   1     0   |   1     1
    //   0    2  |   0     2   |   2     2
    //   1    2  |   1     2   |   3     3
    */

    svStack = (VOID *) Super( 0);  /* unterbinde Taskwechsel */
 
    sRAM  = (LONG) Mxalloc( -1, 0);
    sRAMg = (LONG) Mxalloc( -1, 0x40); /* im Fehlerfall Mxalloc( -1, 3) */
    aRAM  = (LONG) Mxalloc( -1, 1);
    aRAMg = (LONG) Mxalloc( -1, 0x41); /* im Fehlerfall Mxalloc( -1, 3) */
 
    Super( svStack);  /* erlaube Taskwechsel */

    if (sRAM == -32)
        return 0x0000;  /* Mxalloc() ist nicht implementiert */
 
    else if ( ((sRAM + aRAM) == sRAMg) && ((sRAM + aRAM) == aRAMg) )
        return 0x0003;  /* oldfashion Mxalloc() */
 
    else
        return 0xFFFF;
 
} /* Mxmask */

/* ----------------------------------
   | Case insensitive string search |
   ---------------------------------- */
char *stristr( const char *s1, const char *s2 )
{
int s2len = (int)strlen(s2);

while(*s1)
	{
	if (toupper(*s1) == toupper(*s2))
		if (!strnicmp(s1, s2, s2len))
			return (char *)s1;
	s1++;
	}
return NULL;
}

/* -----------------------------
   | Check for cookie presence |
   ----------------------------- */
long check_cookie(long cook)
{
long old_stk, *old_jar;

old_stk = Super(0L);

old_jar = *(long **)0x5a0;
Super((void *)old_stk);

if (!old_jar)
	return 0L;
	
while(*old_jar)
	{
	if (*old_jar == cook)
		return old_jar[1];
		
	old_jar += 2;
	}

return 0L;
}

/* --------------------------
   | Allocate shared memory |
   -------------------------- */
void *alloc_shared(int size)
{
if (CheckSystem())
  return Mxalloc(size, MGLOBAL|0xB);
else
  return Malloc(size);
}

/* ------------------------
   | Check current system |
   ------------------------ */
int CheckSystem(void)
{
int sys;

sys = 0;
if (check_cookie(MiNT_COOKIE))
	{
  if (_GemParBlk.global[1] > 1 || _GemParBlk.global[1] == -1)
    sys = 3;
  else
  	sys = 1;
	}

if (check_cookie(MagX_COOKIE))
  sys = 2;
  
return sys;
}

/* -------------
   | Copy file |
   ------------- */
void CopyPart(int out, int in, long length)
{
char *buf;

DEBUG();
buf = (char *)malloc(32000);

while (length)
	{
	if (length >= 32000)
		{
		Fread(in, 32000, buf);
		WriteBytes(out, buf, 32000);
		length -= 32000;
		}
	else
		{
		Fread(in, length, buf);
		WriteBytes(out, buf, length);
		length = 0;
		}
	}		
	
free(buf);
}

/* -------------
   | Copy file |
   ------------- */
void CopyFile(int out, char *infile)
{
char *buf;
int in;
long l;

DEBUG();
in = open_file(infile, NULL);
if (!in)
	return;
	
buf = (char *)malloc(32000);
l = Fseek(0, in, 2);
Fseek(0, in, 0);

while (l)
	{
	if (l >= 32000)
		{
		Fread(in, 32000, buf);
		WriteBytes(out, buf, 32000);
		l -= 32000;
		}
	else
		{
		Fread(in, l, buf);
		WriteBytes(out, buf, l);
		l = 0;
		}
	}		
	
Fclose(in);
free(buf);
}

/* -------------
   | Copy file |
   ------------- */
int CopyFile2File(char *outfile, char *infile)
{
char *buf;
int in, out;
long l, f;

DEBUG();
f = Fopen(infile, FO_READ);
if (f < 0)
	return FALSE;
	
in = (int)f;

f = Fcreate(outfile, 0);
if (f < 0)
	{
	Fclose(in);
	return FALSE;
	}
out = (int)f;
	
buf = (char *)malloc(32000);
if (!buf)
	{
	Fclose(in);
	Fclose(out);

	return FALSE;
	}
	
l = Fseek(0, in, 2);
Fseek(0, in, 0);

while (l)
	{
	if (l >= 32000)
		{
		Fread(in, 32000, buf);
		WriteBytes(out, buf, 32000);
		l -= 32000;
		}
	else
		{
		Fread(in, l, buf);
		WriteBytes(out, buf, l);
		l = 0;
		}
	}		
	
Fclose(in);
Fclose(out);
free(buf);
return TRUE;
}
   
/* --------------------
   | Clear error flag |
   -------------------- */
int ClearError(void)
{
int e = err;
err = 0;
return e;
}
   
/* --------------------------------------
   | Write bytes to file (append CR LF) |
   -------------------------------------- */
void WriteBytesLF(int f, char *buf, long len)
{
if (err) return;

if (Fwrite(f, len, buf) != len)
	err = 1;
else
	if (Fwrite(f, 2, "\r\n") != 2)
		err = 1;
}

/* ------------------------
   | Write bytes to file |
   ------------------------ */
void WriteBytes(int f, char *buf, long len)
{
if (err) return;

if (Fwrite(f, len, buf) != len)
	err = 1;
}

/* ---------------------------------------
   | Write string to file (append CR LF) |
   --------------------------------------- */
void WriteStringLF(int f, char *string)
{
int l;

if (err) return;

l = (int)strlen(string);
if (Fwrite(f, l, string) != l)
	err = 1;
else
	if (Fwrite(f, 2, "\r\n") != 2)
		err = 1;
}

/* ------------------------------------------------------------
   | Write strings with a maximum length of 78 (append CR LF) |
   ------------------------------------------------------------ */
void BreakedStringLF(int f, char *str1, char *str2)
{
int l, i;
char *b, *buf;

if (err) return;

l = (int)strlen(str1) + (int)strlen(str2);
if (l < 78)
	{
	l = (int)strlen(str1);
	if (Fwrite(f, l, str1) != l)
		err = 1;
	else
		{
		l = (int)strlen(str2);
		if (Fwrite(f, l, str2) != l)
			err = 1;
		else
			if (Fwrite(f, 2, "\r\n") != 2)
				err = 1;
		}
	}
else
	{
	buf = b = (char *)malloc(l + 1);
	
	strcpy(b, str1);
	strcat(b, str2);
	
	do
		{
		if (l < 77)
			i = l;
		else
			{
			i = 77;
			while(i && !isspace(b[i]))
				i--;
			}
			
		if (!i)
			{
			i++;
			while(i < l && !isspace(b[i]))
				i++;
			}
		
		if (Fwrite(f, i, b) != i)
			err = 1;
		else
			if (Fwrite(f, 2, "\r\n") != 2)
				err = 1;
				
		b += i;
		l -= i;

		if (l) *b = '\t';
		
		}while(l && !err);
	
	free(buf);
	}
}

/* ------------------------
   | Write string to file |
   ------------------------ */
void WriteString(int f, char *string)
{
int l;

if (err) return;

l = (int)strlen(string);
if (Fwrite(f, l, string) != l)
	err = 1;
}

/* ------------------------------------
   | Calculate rectangle intersection |
   ------------------------------------ */
int rc_intersect(GRECT *s1, GRECT *s2)
{
int tx, ty, tw, th;

tw = min(s2->g_x + s2->g_w, s1->g_x + s1->g_w);
th = min(s2->g_y + s2->g_h, s1->g_y + s1->g_h);
tx = s2->g_x = max(s2->g_x, s1->g_x);
ty = s2->g_y = max(s2->g_y, s1->g_y);
s2->g_w = tw - tx;
s2->g_h = th - ty;

return ((tw > tx) && (th > ty));
}

/* ---------------------------
   | Put character in buffer |
   --------------------------- */
void ins_char(char **mem, long *len, int *lines, char **lptr, int c)
{
char *n_mem;

new_buffer.changed = TRUE;
if (!*len)
	n_mem = (char *)malloc(4096);
else
	if (! (*len & 4095))
		n_mem = (char *)realloc(*mem, *len + 4096);
	else
		n_mem = *mem;
if (!n_mem) { printf("Allocation error\n"); return; }

if (!*len)
	*lptr = n_mem;
else
	{
	long offset;
	
	offset = *lptr - *mem;
	*lptr = n_mem + offset;
	memmove(*lptr + 1, *lptr, *len - offset);
	}
**lptr = c;
*mem = n_mem;

if (mark_start != -1)
	{
	if (*lptr <= n_mem + mark_start)
		mark_start++;
	if (*lptr <= n_mem + mark_end)
		mark_end++;
	}

if (c == '\r')
	(*lines)++;
		
if (!(*len)++)
	edit_enable();
}

/* --------------------------------
   | Delete character from buffer |
   -------------------------------- */
void del_char(char **mem, long *len, int *lines, char **lptr)
{
char *n_mem;

new_buffer.changed = TRUE;
if (**lptr == '\r')
	(*lines)--;
		
if (*lptr < *mem + *len)
	memmove(*lptr, *lptr + 1, *len - (*lptr - *mem) - 1);

if (mark_start != -1L)
	{
	if (*lptr < *mem + mark_start)
		mark_start--;

	if (*lptr < *mem + mark_end)
		mark_end--;

	if (mark_start == mark_end && mark_start != -1)
		{
		mark_start = mark_end = -1L;
		edit_enable();
		but_disabled[2] = but_disabled[3] = but_disabled[5] = TRUE;
		redraw_mbbutton(2);
		redraw_mbbutton(3);
		redraw_mbbutton(5);
		}
	}
		
if (*len == 1)
	{
	free(*mem);
	n_mem = NULL;
	}
else
	{
	if (!((*len - 1) & 4095))
		n_mem = (char *)realloc(*mem, *len - 1);
	else
		n_mem = *mem;
	if (!n_mem) return;
	}

*lptr = (*lptr - *mem) + n_mem;
*mem = n_mem;

(*len)--;
}

/* ----------------------
   | Clear input buffer |
   ---------------------- */
void clear_inbuffer(void)
{
in_buf = 0;
in_ptr = 0;
}

/* -------------------
   | Incoming data ? |
   ------------------- */
int TestIncoming(int s)
{
int nx;
struct fd_set fd_chk;
struct timeval tval;

FD_ZERO(&fd_chk);
FD_SET(s, &fd_chk);
		
tval.tv_sec = 0;
tval.tv_usec = 0;
		
nx = select(s + 1, &fd_chk, NULL, NULL, &tval);
return (nx && FD_ISSET(s, &fd_chk));
}

/* ---------------------
   | Socket exception? |
   --------------------- */
int TestException(int s)
{
int nx;
struct fd_set fd_exep;
struct timeval tval;

FD_ZERO(&fd_exep);
FD_SET(s, &fd_exep);
		
tval.tv_sec = 0;
tval.tv_usec = 0;
		
nx = select(s + 1, NULL, NULL, &fd_exep, &tval);
return (nx && FD_ISSET(s, &fd_exep));
}

/* --------------------------
   | Get response of server |
   -------------------------- */
int get_response(int s, char *buf)
{
int n;
int l;
char c;

c = 0;
n = 0;
do
	{
	if (in_ptr == in_buf) 
		in_ptr = in_buf = 0;
		
	if (in_buf < 32000)
		{
		if (TestIncoming(s))
			{
			l = (int)recv(s, NULL, 32000 - in_buf, MSG_PEEK);
			
			if (l > 0)
				l = (int)recv(s, in_buffer + in_buf, l, 0);
			
			if (l > 0)
				in_buf += l;
			}
		else
			if (info_stop())
				return -1;
		}
		
	while(in_ptr < in_buf)
		{
		c = buf[n++] = in_buffer[in_ptr++];
		if (c == '\n')
			break;
		}

	if (!in_buf && TestException(s))
		break;
		
	}while(c != '\n');
	
buf[n++] = 0;

write_log(buf, (int)strlen(buf));

/*Cconws(buf);*/
return atoi(buf);
}

/* --------------------------
   | Send an email sequence |
   -------------------------- */
int send_quoted(int s, char *buf, char *cmd, char *par1, char *par2, char *par3)
{
char *buffer, addr[256], *last_sep, *ptr;
int l, addrlen, e = 200;

l = 5;
if (par1) l += (int)strlen(par1);
if (par2) l += (int)strlen(par2);
if (par3) l += (int)strlen(par3);

buffer = (char *)malloc(l);
buffer[0] = 0;

while (par1 && par1[0] != 1) 
	{
	strcat(buffer, par1);
	par1 = par2;
	par2 = par3;
	par3 = NULL;
	}

last_sep = ptr = buffer;
while(*ptr)
	{
	if (*ptr == '@')
		{
		last_sep = ptr - 1;
		while(last_sep > buffer && 
					(isalnum(*(last_sep - 1)) || *(last_sep - 1) == '.' || 
					 *(last_sep - 1) == '_' || *(last_sep - 1) == '-'))
			last_sep--;
			
		ptr++;
		while(*ptr &&
					(isalnum(*ptr) || *ptr == '.' || *ptr == '_' || *ptr == '-'))
			ptr++;
			
		addrlen = (int)(ptr - last_sep);
		
		addr[0] = '<';
		strncpy(addr + 1, last_sep, addrlen);
		addr[addrlen + 1] = '>';
		addr[addrlen + 2] = 0;

		send_cmd(s, cmd, addr, par1, par2);
		e = get_response(s, buf);
		if (e < 200 || e > 399)
			{
			free(buffer);
			return e;
			}
		}
	else
		if (*ptr == '^')
			{
			char group[21], **addr;
			int gc;
			
			ptr++;
			gc = 0;
			while(*ptr && *ptr != '^')
				{
				if (gc < 20)
					group[gc++] = *ptr++;
				}
			group[gc] = 0;
			
			addr = get_group_address(group);
			
			if (!addr[0])
				{
				/* Fehler */
				}
			else
				{
				gc = 0;
				while(addr[gc])
					{
					send_quoted(s, buf, cmd, addr[gc], NULL, NULL);
					free(addr[gc]);
					gc++;
					}
				}
				
			free(addr);
			}
		else
			ptr++;
	}
	
free(buffer);
return e;
}

/* ---------------------
   | Send mail command |
   --------------------- */
void send_cmd(int s, char *cmd, char *par1, char *par2, char *par3)
{
char *sbuf;

sbuf = (char *)malloc(32000);
if (!sbuf)
	{
	write_log("QUIT", 4);
	send(s, "QUIT\r\n", 6, 0);
	return;
	}
	
strcpy(sbuf, cmd);

if (par1)
	{
	if (sbuf[strlen(sbuf) - 1] != ':')
		strcat(sbuf, " ");
		
	if (par1[0] == 1)
		strcat(sbuf, par1 + 1);
	else
		strcat(sbuf, par1);

	if (par2)
		{
		if (par2[0] == 1)
			{
			strcat(sbuf, " ");
			strcat(sbuf, par2 + 1);
			}
		else
			strcat(sbuf, par2);
	
		if (par3)
			if (par3[0] == 1)
				{
				strcat(sbuf, " ");
				strcat(sbuf, par3 + 1);
				}
			else
				strcat(sbuf, par3);
		}
	}
		
strcat(sbuf, "\r\n");
/*Cconws(sbuf);*/

write_log(sbuf, (int)strlen(sbuf));
send(s, sbuf, strlen(sbuf), 0);

free(sbuf);
}

/* ------------------------------
   | Enable mail window entries |
   ------------------------------ */
void default_enable(void)
{
int has_def = hasDefaultIdentity();

menu_ienable(menu_adr, M_NFOLD, FALSE);
menu_ienable(menu_adr, ALLHEAD, 	FALSE);

menu_ienable(menu_adr, M_UNDO, 		FALSE);
menu_ienable(menu_adr, M_SELALL, 	FALSE);
menu_ienable(menu_adr, M_FWD, 		FALSE);
menu_ienable(menu_adr, M_REPLY, 	FALSE);
menu_ienable(menu_adr, M_AGAIN, 	FALSE);
menu_ienable(menu_adr, M_SEND, 		FALSE);

menu_ienable(menu_adr, EDITMAIL, 	FALSE);

menu_ienable(menu_adr, M_PRINT, 	FALSE);
menu_ienable(menu_adr, SAVE_MSG, 	FALSE);
menu_ienable(menu_adr, M_SAVATT, 	FALSE);
menu_ienable(menu_adr, M_CYCLEWIN, 	get_number_of_windows() > 1);
menu_ienable(menu_adr, M_CLOSEWIN, 	get_number_of_windows() >= 1);

menu_ienable(menu_adr, M_CUT, FALSE);
menu_ienable(menu_adr, M_COPY, FALSE);
menu_ienable(menu_adr, M_DEL, FALSE);
menu_ienable(menu_adr, M_PASTE, FALSE);
menu_ienable(menu_adr, M_SEARCH, FALSE);
menu_ienable(menu_adr, M_REPLACE, FALSE);

menu_ienable(menu_adr, M_EDITATT, FALSE);
menu_ienable(menu_adr, M_WRAP, 	FALSE);

menu_ienable(menu_adr, M_SCAN, 			has_def);
menu_ienable(menu_adr, M_SENDCHECK, has_def);
menu_ienable(menu_adr, M_CHECK, 		has_def);
menu_ienable(menu_adr, M_SEND, 			has_def);

/*menu_ienable(menu_adr, M_PSETUP, mail_options.printmode);*/
menu_icheck(menu_adr, M_ANSI, mail_options.ansi_flg);
}

/* -----------------------------
   | Get installed TOS version |
   ----------------------------- */
void get_aesversion(void)
{
long old_stk;

old_stk = Super(0L);

aes_version = (*(SYSHDR **)(0x4F2))->os_version;

Super((void *)old_stk);
}

/* -------------------------------------
   | Check current state of connection |
   ------------------------------------- */
int check_connection(int function, char *identity, int flg)
{
switch (get_connected())
	{
	case -1:
		form_alert(1, nodriver_msg);
		return FALSE;
	
	case 0:
		{
		int draconis_id;
		
		draconis_id = appl_find("DRACONIS");
		if (draconis_id < 0)
			form_alert(1, offline_msg);
		else
			{
			int msg[8];
			
			msg[0] = 0x5811;
			msg[1] = gl_apid;
			msg[3] = 0;
			appl_write(draconis_id, 16, msg);

			dial_function = function;
			strcpy(dial_identity, identity);
			dial_flg = flg;
			}
		return FALSE;
		}
		
	case 1:
		while (get_connected() == 1)
		  if (get_connected() != 2 && get_connected() != 1)
			  return FALSE;
			
	case 2:
		return TRUE;
	}
return FALSE;
}

/* --------------------------
   | Wait for mouse release |
   -------------------------- */
void wait_mouse_release(void)
{
check_mouse();
/*graf_mkstate( &evnt);*/

if (evnt.bstate & 1)
	evnt_button (1, evnt.bstate & 1, 0x00, &evnt);		/* Warte auf Mausknopf */
}

/* --------------------------
   | Check for a scrap file |
   -------------------------- */
int check_scrap(void)
{
char scrap[256];
long fh;

if (scrp_read(scrap))
	{
	strcat(scrap, "scrap.txt");

	fh = Fopen(scrap, FO_READ);
	if (fh > 0)
		{
		Fclose((int)fh);
		return TRUE;
		}
	else
		return FALSE;
	}
return FALSE;
}

/* ------------------------------------
   | Is it a alphanumeric character ? |
   ------------------------------------ */
int misalnum(unsigned char chr)
{
static long alnumtb[] = {0x00000000L,
												 0x03FF0000L, /* 0-9 */
												 0x87FFFFFEL,	/* A-Z + _ */
												 0x07FFFFFEL, /* a-z */
												 0xC7FFFFFFL,
												 0x01FF00FFL,
												 0x00000000L,
												 0x00000002L};
int r = chr / 32;
int p = chr & 31;

if (alnumtb[r] & (1L << p))
	return TRUE;
else
	return FALSE;
}

/* ---------------------------------------
   | Write log file for server debugging |
   --------------------------------------- */
void write_log(char *sendstr, int len)
{
long fh;

if (!debug) return;

fh = Fopen("MAILLOG", FO_RW);
if (fh == -33)
	fh = Fcreate("MAILLOG", 0);
if (fh > 0)
	{
	Fseek(0L, (int)fh, 2);
	if (!strncmp(sendstr, "PASS ", 5))
		Fwrite((int)fh, 25, "PASS <password supressed>");
	else
		Fwrite((int)fh, len, sendstr);
	Fwrite((int)fh, 2, "\r\n");
	Fclose((int)fh);
	}
}

/* ------------------------------
   | Clear files from temp list |
   ------------------------------ */
void clear_temp(void)
{
FILE *fin;
char buf[256], *s;

DEBUG();
fin = fopen("tmpfiles.dat", "r");
if (fin)
	{
	while(fgets(buf, 256, fin))
		{
		s = strchr(buf, '\n');
		if (s) *s = 0;
		Fdelete(buf);
		}
	fclose(fin);
	Fdelete("tmpfiles.dat");
	}
}

/* ----------------------------
   | Check mouse button state |
   ---------------------------- */
void check_mouse(void)
{
which = evnt_multi(MU_BUTTON|MU_TIMER|MU_MESAG,
					 1, 1, 1,
					 0, &empty,
					 0, &empty,
					 msg,
					 50L,
					 &evnt,
					 &k_return, &clicks);

if (!(which & MU_BUTTON) && !evnt.bstate)
	{
	graf_mkstate(&evnt);
	if (evnt.bstate)
		{
		which |= MU_BUTTON;
		clicks = 1;
		}
	}

which = evnt_handler(which & ~MU_BUTTON, msg, &evnt, k_return, clicks);
}
