/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"
#include "..\include\color.h"

#ifdef MILAN
# define NO_SIGNATURE
#endif

#ifdef EVAL
# define NO_SIGNATURE
# define NO_PRINT
#endif

static int p_codes[]  = {P_INIT, P_EXIT, P_BOLDON, P_BOLDOF, 
												 S_FF};
static int font_id, loaded_test, color_changed;
static char fontname[33];
static char option_audio[256];
static int help_ids[] = {OPTSAVE, OKAYOPT, CANCOPT, 		/* 0-2 */
												 OPTS1, OPTS2, OPTS3, OPTS4,		/* 3-6 */
												 EDITLEN, TABWIDTH, VIEW_WRAP, 	/* 7-9 (First page) */
												 VIEW_STORESIZE, TEXTCOLOUR,		/* 10-11 */
												 QUOTEDCOLOUR, LINKCOLOUR,			/* 12-13 */
												 FONT, FONTSIZE,								/* 14-15 */

												 CHKSTART, ALLOWRD, PROMPT_REPLY,			/* 16-23 (second page) */
												 REMOVE_ATTACHES, OPENNOTIFICATION,
												 PLAYSOUND, SOUNDFILE, TEST_AUDIO,

												 CONFIRM_DELETION, S_ALLHD, BUBBLEGM,		/* 24-30 (third page) */
												 ICON_CACHE, EXIT_SV, S_BSTYLE, S_ISTYLE,
												 
												 S_DPRINT, S_GDOS, P_INIT, P_EXIT, P_BOLDON,	/* 31-40 (forth page) */
												 P_BOLDOF, P_WIDTH, P_HEIGHT, S_PWRAP,
												 S_FF,

												 0};
static int helpinitialized = FALSE;

/* -------------------
   | Set option page |
   ------------------- */
void SetOptPage(OBJECT *tree, int page, int obj)
{
static int tabids[] = {OPTS4, OPTS3, OPTS2, OPTS1};
DEBUG();
SetTabPage(tree, OPTBKG, page, obj, 4, tabids);
}

/* ------------------------
   | Set current settings |
   ------------------------ */
void get_dialog_settings(void)
{
DEBUG();
mail_options.viewlength = atoi(option_tree[opt_pages[0] + EDITLEN].ob_spec.tedinfo->te_ptext);
mail_options.viewtabs = atoi(option_tree[opt_pages[0] + TABWIDTH].ob_spec.tedinfo->te_ptext);

mail_options.viewwrap = IsSelected(option_tree, opt_pages[0] + VIEW_WRAP);
mail_options.saveview = IsSelected(option_tree, opt_pages[0] + VIEW_STORESIZE);

mail_options.text_colour = (option_tree[opt_pages[0] + TEXTCOLOUR].ob_state >> 8) & 0xFF;
mail_options.quoted_colour = (option_tree[opt_pages[0] + QUOTEDCOLOUR].ob_state >> 8) & 0xFF;
mail_options.link_colour = (option_tree[opt_pages[0] + LINKCOLOUR].ob_state >> 8) & 0xFF;

mail_options.font = font_id;

mail_options.fontsize = atoi(option_tree[opt_pages[0] + FONTSIZE].ob_spec.tedinfo->te_ptext);

mail_options.startup_check = IsSelected(option_tree, opt_pages[1] + CHKSTART);
mail_options.play_sound = IsSelected(option_tree, opt_pages[1] + PLAYSOUND);
mail_options.open_notify = IsSelected(option_tree, opt_pages[1] + OPENNOTIFICATION);
mail_options.prompt_reply = IsSelected(option_tree, opt_pages[1] + PROMPT_REPLY);
mail_options.remove_attaches = IsSelected(option_tree, opt_pages[1] + REMOVE_ATTACHES);

mail_options.allow_readconfirm	  = IsSelected(option_tree, opt_pages[2] + ALLOWRD);
mail_options.bubblegem						= IsSelected(option_tree, opt_pages[2] + BUBBLEGM);
mail_options.allheads							= IsSelected(option_tree, opt_pages[2] + S_ALLHD);
mail_options.save_opt 						= IsSelected(option_tree, opt_pages[2] + EXIT_SV);
mail_options.confirm_delete				= IsSelected(option_tree, opt_pages[2] + CONFIRM_DELETION);
mail_options.icon_cache						= IsSelected(option_tree, opt_pages[2] + ICON_CACHE);

mail_options.printmode						= IsSelected(option_tree, opt_pages[3] + S_GDOS);
mail_options.print_ff							= IsSelected(option_tree, opt_pages[3] + S_FF);
mail_options.printwrap						= IsSelected(option_tree, opt_pages[3] + S_PWRAP);

mail_options.pagewidth 	= atoi(option_tree[opt_pages[3] + P_WIDTH].ob_spec.tedinfo->te_ptext);
mail_options.pageheight = atoi(option_tree[opt_pages[3] + P_HEIGHT].ob_spec.tedinfo->te_ptext);

strcpy(mail_options.printinit, 	option_tree[opt_pages[3] + P_INIT].ob_spec.tedinfo->te_ptext);
strcpy(mail_options.printexit, 	option_tree[opt_pages[3] + P_EXIT].ob_spec.tedinfo->te_ptext);
strcpy(mail_options.boldon, 		option_tree[opt_pages[3] + P_BOLDON].ob_spec.tedinfo->te_ptext);
strcpy(mail_options.boldoff, 		option_tree[opt_pages[3] + P_BOLDOF].ob_spec.tedinfo->te_ptext);

if (strcmp(option_audio, mail_options.soundfile))
	{
	strcpy(mail_options.soundfile, option_audio);
#ifdef AUDIO
	load_audio(option_audio);
#endif
	}
}

/* ------------------------
   | Set current settings |
   ------------------------ */
void set_dialog_settings(void)
{
int i;

DEBUG();

DisableObj(option_tree, OPTS4);

itoa(mail_options.viewlength, option_tree[opt_pages[0] + EDITLEN].ob_spec.tedinfo->te_ptext, 10);
itoa(mail_options.viewtabs, 	option_tree[opt_pages[0] + TABWIDTH].ob_spec.tedinfo->te_ptext, 10);
SetSelect(option_tree, opt_pages[0] + VIEW_WRAP, 			mail_options.viewwrap);
SetSelect(option_tree, opt_pages[0] + VIEW_STORESIZE, 			mail_options.saveview);

make_colcell(option_tree, opt_pages[0] + TEXTCOLOUR, mail_options.text_colour);
make_colcell(option_tree, opt_pages[0] + QUOTEDCOLOUR, mail_options.quoted_colour);
make_colcell(option_tree, opt_pages[0] + LINKCOLOUR, mail_options.link_colour);

font_id = mail_options.font;
option_tree[opt_pages[0] + FONT].ob_spec.tedinfo->te_ptext = fontname;
find_font_id(font_id, fontname);

itoa(mail_options.fontsize, option_tree[opt_pages[0] + FONTSIZE].ob_spec.tedinfo->te_ptext, 10);

SetSelect(option_tree, opt_pages[1] + CHKSTART, mail_options.startup_check);
SetSelect(option_tree, opt_pages[1] + PLAYSOUND, mail_options.play_sound);
SetSelect(option_tree, opt_pages[1] + OPENNOTIFICATION, mail_options.open_notify);
SetSelect(option_tree, opt_pages[1] + PROMPT_REPLY, mail_options.prompt_reply);
SetSelect(option_tree, opt_pages[1] + REMOVE_ATTACHES, mail_options.remove_attaches);

strcpy(option_audio, mail_options.soundfile);
if (option_audio[0])
	{
	char *s;
	
	s = strrchr(option_audio, '\\');
	if (s) s++; else s = option_audio;
	strncpy(option_tree[opt_pages[1] + SOUNDFILE].ob_spec.tedinfo->te_ptext, s, 19);
	}
else
	option_tree[opt_pages[1] + SOUNDFILE].ob_spec.tedinfo->te_ptext[0] = 0;
loaded_test = FALSE;
							
SetSelect(option_tree, opt_pages[2] + ALLOWRD, 	mail_options.allow_readconfirm);
SetSelect(option_tree, opt_pages[2] + BUBBLEGM, mail_options.bubblegem);
SetSelect(option_tree, opt_pages[2] + S_ALLHD, 	mail_options.allheads);
SetSelect(option_tree, opt_pages[2] + EXIT_SV, 			mail_options.save_opt);
SetSelect(option_tree, opt_pages[2] + CONFIRM_DELETION, mail_options.confirm_delete);
SetSelect(option_tree, opt_pages[2] + ICON_CACHE, mail_options.icon_cache);

if (mail_options.printmode)
	for (i = 0; i < sizeof(p_codes) / sizeof(int); i++)
		{
		DisableObj(option_tree, opt_pages[3] + p_codes[i]);
		redraw_object(option_tree, opt_pages[3] + p_codes[i]);
		}

if (gdos)
	{
	SetSelect(option_tree, opt_pages[3] + S_DPRINT, mail_options.printmode == 0);
	SetSelect(option_tree, opt_pages[3] + S_GDOS, 	mail_options.printmode != 0);
	EnableObj(option_tree, opt_pages[3] + S_GDOS);
	}
else
	{
	DisableObj(option_tree, opt_pages[3] + S_GDOS);
	SetSelect(option_tree, opt_pages[3] + S_DPRINT, TRUE);
	SetSelect(option_tree, opt_pages[3] + S_GDOS, FALSE);
	}
	
SetSelect(option_tree, opt_pages[3] + S_FF, 		mail_options.print_ff);
SetSelect(option_tree, opt_pages[3] + S_PWRAP, 	mail_options.printwrap);

itoa(mail_options.pagewidth, option_tree[opt_pages[3] + P_WIDTH].ob_spec.tedinfo->te_ptext, 10);
itoa(mail_options.pageheight, option_tree[opt_pages[3] + P_HEIGHT].ob_spec.tedinfo->te_ptext, 10);

strcpy(option_tree[opt_pages[3] + P_INIT].ob_spec.tedinfo->te_ptext, mail_options.printinit);
strcpy(option_tree[opt_pages[3] + P_EXIT].ob_spec.tedinfo->te_ptext, mail_options.printexit);
strcpy(option_tree[opt_pages[3] + P_BOLDON].ob_spec.tedinfo->te_ptext, mail_options.boldon);
strcpy(option_tree[opt_pages[3] + P_BOLDOF].ob_spec.tedinfo->te_ptext, mail_options.boldoff);
}

/* ---------------------------------
   | Handle Option dialogue events |
   --------------------------------- */
static void cdecl option_handler (OBJECT *tree, int obj)
{
DEBUG();
switch(obj)
	{
	case OPTS1:		SetOptPage(option_tree, opt_pages[0], OPTS1);	break;
	case OPTS2:		SetOptPage(option_tree, opt_pages[1], OPTS2);	break;
	case OPTS3:		SetOptPage(option_tree, opt_pages[2], OPTS3);	break;
	case OPTS4:		SetOptPage(option_tree, opt_pages[3], OPTS4);	break;

	case OPTSAVE:
	case OKAYOPT:
		get_dialog_settings();
		if (color_changed)
			{
			GRECT rect;
			wind_get_grect(ROOT, WF_WORKXYWH, &rect);
			form_dial(FMD_FINISH, &empty, &rect);
			}
		enabler();
		if (obj == OPTSAVE)
			{
			save_mail_config();
			break;
			}
		/* Fall through */
		
	case CANCOPT:
		close_dialog(tree, TRUE);
#ifdef AUDIO
		if (mail_options.soundfile[0] && loaded_test)
			load_audio(mail_options.soundfile);
#endif
		break;
		
	default:
		{
		if (IsSelected(option_tree, OPTS1))	/* Browser Page active ? */
			{
			switch(obj - opt_pages[0])
				{
				case TEXTCOLOUR:
				case QUOTEDCOLOUR:
				case LINKCOLOUR:
					{
					int defcol;
					OBJECT *pop;
					
					switch(obj - opt_pages[0])
						{
						case TEXTCOLOUR:		defcol = 1; break;
						case QUOTEDCOLOUR: 	defcol = 2; break;
						case LINKCOLOUR: 		defcol = 4; break;
						}
					
					rsrc_gaddr(R_TREE, CSELPOP, &pop);	
	  			if (color_popup( option_tree, obj, pop, defcol ))
	  				color_changed = TRUE;
					break;
					}
						
				case FONT:
				case SELFONT:
					{
					int x, y;
					
					objc_offset(tree, opt_pages[0] + FONT, &x, &y);
					select_font_popup(x, y, &font_id, fontname);
					redraw_object(option_tree, opt_pages[0] + FONT);
					break;
					}

				case SELSIZE:
					{
					int x, y;
					int sz;
					
					objc_offset(tree, opt_pages[0] + FONTSIZE, &x, &y);
					sz = select_font_size(x, y);
					if (sz > -1)
						{
						itoa(sz, option_tree[opt_pages[0] + FONTSIZE].ob_spec.tedinfo->te_ptext, 10);
						redraw_object(option_tree, opt_pages[0] + FONTSIZE);
						}
					break;
					}
				}
			}
		else if (IsSelected(option_tree, OPTS2))	/* General Page active ? */
			{
			switch(obj - opt_pages[1])
				{
				case SOUNDSEL:
					{
					char buffer[256];
					
					if (fileselect(buffer, "Select audio file...", ".AVR"))
						strcpy(option_audio, buffer);
					else
						option_audio[0] = 0;

					if (option_audio[0])
						{
						char *s;
						
						s = strrchr(option_audio, '\\');
						if (s) s++; else s = option_audio;
						strncpy(option_tree[opt_pages[1] + SOUNDFILE].ob_spec.tedinfo->te_ptext, s, 19);
						}
					else
						option_tree[opt_pages[1] + SOUNDFILE].ob_spec.tedinfo->te_ptext[0] = 0;
							
					redraw_object(option_tree, opt_pages[1] + SOUNDFILE);	
					break;
					}
					
				case TEST_AUDIO:
#ifdef AUDIO
					load_audio(option_audio);
					loaded_test = TRUE;
					play_one();
#endif
					break;
				}				
			}
		else if (IsSelected(option_tree, OPTS3))	/* Browser Page active ? */
			{
			switch(obj - opt_pages[2])
				{
				case S_ISTYLE:
					select_toolbuttons();
					break;
					
				case S_BSTYLE:
					select_boxicons();
					break;
				}				
			}
		else if (IsSelected(option_tree, OPTS4))	/* Browser Page active ? */
			{
			int i;

			switch(obj - opt_pages[3])
				{
				case S_DPRINT:
					for (i = 0; i < sizeof(p_codes) / sizeof(int); i++)
						{
						EnableObj(option_tree, opt_pages[3] + p_codes[i]);
						redraw_object(option_tree, opt_pages[3] + p_codes[i]);
						}
					break;
					
				case S_GDOS:
					for (i = 0; i < sizeof(p_codes) / sizeof(int); i++)
						{
						DisableObj(option_tree, opt_pages[3] + p_codes[i]);
						redraw_object(option_tree, opt_pages[3] + p_codes[i]);
						}
					break;
				}				
			}
		} /* default */
	} /*	switch(obj) */
}

/* -----------------
   | Setup options |
   ----------------- */
void set_options(void)
{
DEBUG();
if (!is_open(option_tree))
	{
	color_changed = TRUE;
	set_dialog_settings();
	
	SetOptPage(option_tree, opt_pages[0], OPTS1);
	open_dialog(option_tree, NULL, iconified_tree);
	
	if (!helpinitialized)
		{
		int i;

		for (i =  7; i <= 15; i++)	help_ids[i] += opt_pages[0];		
		for (i = 16; i <= 23; i++)	help_ids[i] += opt_pages[1];		
		for (i = 24; i <= 30; i++)	help_ids[i] += opt_pages[2];		
		for (i = 31; i <= 40; i++)	help_ids[i] += opt_pages[3];		
		helpinitialized = TRUE;
		}
	
	dlgset_help_info(option_tree, helpfile, help_ids, "OPT_");
	set_active(option_tree, option_handler, TRUE, 0);
	}
else
	open_dialog(option_tree, NULL, iconified_tree);
}

/* ---------------------------
   | Read mail configuration |
   --------------------------- */
void load_mail_config(int flg)
{
long in_cfg;
char buf[256 + 64], id[4];

DEBUG();
mail_options._signature_file[0] = 0;

strcpy(buf, mailpath);
strcat(buf, "\\marathon.mcf");

in_cfg = Fopen(buf, FO_READ);
if (in_cfg > 0)
	{
	Fread((int)in_cfg, sizeof(id), id);
	if (!strnicmp(id, "Mcfg", 4))
		{
		int l = (int)Fread((int)in_cfg, sizeof(mail_options), &mail_options);
		
		if (strncmp(id, "Mcfg", 4))
			{
			char popdesc[20];
			
			mail_options.bubblegem = FALSE;
			mail_options.allheads  = FALSE;
			mail_options.printmode = 0;
			mail_options.print_ff  = 1;
			mail_options.printinit[0] = 
			mail_options.printexit[0] = 0;
			strcpy(mail_options.boldon, "{ESC}E");
			strcpy(mail_options.boldoff, "{ESC}F");
			mail_options.pagewidth = 78;
			mail_options.pageheight = 70;
			mail_options._editwidth = 76;
			mail_options.printwrap = 1;

			Fseek(-16 - 4*51, (int)in_cfg, 1);
			Fread((int)in_cfg, sizeof(mail_options.from_flg), 				&mail_options.from_flg);
			Fread((int)in_cfg, sizeof(mail_options.sub_flg),  				&mail_options.sub_flg);
			Fread((int)in_cfg, sizeof(mail_options.bcc_flg),  				&mail_options.bcc_flg);
			Fread((int)in_cfg, sizeof(mail_options.cc_flg), 	 				&mail_options.cc_flg);
			Fread((int)in_cfg, sizeof(popdesc), 											popdesc);
			Fread((int)in_cfg, sizeof(mail_options.ansi_flg), 				&mail_options.ansi_flg);
			Fread((int)in_cfg, sizeof(mail_options._signature_file), 	mail_options._signature_file);
			Fread((int)in_cfg, sizeof(mail_options.replyto_flg), 			&mail_options.replyto_flg);
			Fread((int)in_cfg, sizeof(mail_options.toolbfile),	 			&mail_options.toolbfile);
			Fread((int)in_cfg, sizeof(mail_options.prt_settings),	 		&mail_options.prt_settings);
			}
		else
			if (l == offsetof(MAIL_OPT, pad))
				{
				mail_options.viewlength = mail_options._editwidth;
				mail_options.viewtabs 	= 8;
				mail_options.viewwrap 	= mail_options._word_wrap;
				mail_options.saveview   = TRUE;
				
				if (colours > 4)
					{
					mail_options.text_colour 		= 1;
					mail_options.quoted_colour 	= 2;
					mail_options.link_colour 		= 4;
					}
				else
					{
					mail_options.text_colour 		=
					mail_options.quoted_colour 	=
					mail_options.link_colour 		= 1;
					}
				mail_options.saveview   			= TRUE;
				mail_options.font 						= 1;
				mail_options.fontsize   			= default_fsize;
				mail_options.prompt_reply 		= TRUE;
				mail_options.remove_attaches 	= FALSE;
				mail_options.open_notify 			= TRUE;
				mail_options.play_sound 			= FALSE;
				mail_options.soundfile[0] 		= 0;
				mail_options.confirm_delete 	= TRUE;
				}
		}
	Fclose((int)in_cfg);

	if (!vq_gdos())
		mail_options.printmode = 0;
	
	if (flg)	
		{
		if (mail_options.maildir[0])
			{
			strcpy(buf, mail_options.maildir);
			strcat(buf, mail_options.mailfile);
			scan_mailbox(buf);
			}
#ifdef AUDIO			
		if (mail_options.soundfile[0])
			load_audio(mail_options.soundfile);
#endif
		}
	}
}

/* ----------------------------
   | Write mail configuration |
   ---------------------------- */
void save_mail_config(void)
{
long in_cfg;
char buf[256 + 64];

DEBUG();
strcpy(buf, mailpath);
strcat(buf, "\\marathon.mcf");

in_cfg = Fcreate(buf, 0);
if (in_cfg > 0)
	{
	Fwrite((int)in_cfg, 4, "Mcfg");
	Fwrite((int)in_cfg, sizeof(mail_options), 	&mail_options);
	Fclose((int)in_cfg);
	}
}
