/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

/* UIDL n 
	liefert UNIQUE - ID im Format
	
	1 IDxxxxxxxx
	
	bzw.
	
	UIDL
	
	1 IDxxxxxxx
	2 IDxxxxxx
	.
	
*/

/* Local prototypes */
static void pollIdle(void);
static void pollResolve(void);
static void pollResolving(void);
static void pollConnect(void);
static void pollConnecting(void);
static void pollWelcome(void);
static void pollAuth(void);
static void pollAuthWait(void);
static void pollPass(void);
static void pollPassWait(void);
static void pollStat(void);
static void pollStatWait(void);
static void pollStatCheck(void);
static void pollUIDL(void);
static void pollUIDLWait(void);
static void pollUIDLCheck(void);
static void pollUIDLComplete(void);
static void pollRetr(void);
static void pollRetrList(void);
static void pollRetrListComplete(void);
static void pollRetrStart(void);
static void pollRetrWait(void);
static void pollRetrStarted(void);
static void pollRetrInProgress(void);
static void pollRetrComplete(void);
static void pollDele(void);
static void pollDeleWait(void);
static void pollDeleComplete(void);
static void pollQuit(void);
static void pollQuitWait(void);
static void pollClose(void);

typedef struct
				{
				int	 identifier;
				int  found;
				long last_checked;
				int  number_of_mails;
				long mail_size;
				char **ids;
				char **recv_ids;
				int  received;
				}IDENTITY_POLL;

#define	POLL_NONE					0
#define POLL_RESOLVE			1
#define POLL_RESOLVING		2
#define	POLL_CONNECT 			3
#define POLL_CONNECTING		4
#define POLL_WELCOME			5
#define POLL_AUTH					6
#define POLL_AUTH_WAIT		7
#define POLL_PASS					8
#define POLL_PASS_WAIT		9
#define POLL_STAT					10
#define POLL_STAT_WAIT		11
#define POLL_STAT_CHECK		12
#define POLL_UIDL					13
#define POLL_UIDL_WAIT		14
#define POLL_UIDL_CHECK		15
#define POLL_UIDL_COMPLETE 16
#define POLL_RECV					17
#define POLL_RECV_LIST		18
#define POLL_RECV_LISTCOMPL	19
#define POLL_RECV_START		20
#define POLL_RECV_WAIT		21
#define POLL_RECV_STARTED	22
#define POLL_RECV_PROGRESS 23
#define POLL_RECV_COMPLETE 24
#define POLL_DELE					25
#define POLL_DELE_WAIT		26
#define POLL_DELE_COMPLETE 27
#define POLL_QUIT					28
#define POLL_QUIT_WAIT		29
#define POLL_CLOSE				30

static IDENTITY_POLL *poll_info;
static int poll_cnt = 0;
static int poll_status, poll_wait;
static int poll_socket, poll_file;
static int poll_idx, new_mail_count = 0, uidl_idx;
static long new_mail_size;
static char **uidl_ids, poll_filename[13];
static IDENTITY current_poll;
static struct sockaddr_in poll_remote;
static char poll_buff[512], locked_server[43], locked_user[43];
static int  poll_buff_cnt, has_new = FALSE, recv_done = FALSE;
static OBJECT *poll_tree;
static FILE_MAIL_MEM poll_in_buf;
static int blocked = 0;

/* ----------------------
   | Redraw poll window |
   ---------------------- */
void redraw_pollwindow(GRECT *g)
{
GRECT t1;

DEBUG();
wind_get_grect(poll_wind, WF_WORKXYWH, &t1);
poll_tree[ROOT].ob_x = t1.g_x;
poll_tree[ROOT].ob_y = t1.g_y;

wind_get_grect(poll_wind, WF_FIRSTXYWH, &t1);
while(t1.g_w && t1.g_h)
	{
	if (rc_intersect(g, &t1))
		{
		dlg_draw(poll_tree, ROOT, MAX_DEPTH, &t1);
		}
	wind_get_grect(poll_wind, WF_NEXTXYWH, &t1);
	}
}

/* --------------------------
   | Message on view window |
   -------------------------- */
void poll_window_message(int *msg)
{
DEBUG();
switch(msg[0])
	{
	case WM_REDRAW:
		redraw_pollwindow((GRECT *)&msg[4]);
		break;
		
	case WM_CLOSED:
		wind_close(poll_wind);
		wind_delete(poll_wind);
		poll_wind = -1;
		break;
	}			
}

/* --------------------
   | Open poll window |
   -------------------- */
void open_poll(void)
{
char ibuf[8], lbuf[11], *ptr, *s;
int i, all_cnt;
long all_size;

DEBUG();
rsrc_gaddr(R_TREE, POLLINFO, &poll_tree);

all_cnt = 0;
all_size = 0L;
for(i = 0; i < poll_cnt; i++)
	{
	all_cnt += poll_info[i].number_of_mails;
	all_size += poll_info[i].mail_size; 
	}
itoa(all_cnt, ibuf, 10);
ltoa(all_size, lbuf, 10);

ptr = poll_tree[POLL_TEXT].ob_spec.tedinfo->te_ptext;
strcpy(ptr, poll_tree[POLL_MSG].ob_spec.free_string);

s = strstr(ptr, "%1");
if (s)
	{
	memmove(s + strlen(ibuf), s + 2, strlen(s + 2) + 1);
	memcpy(s, ibuf, strlen(ibuf)); 
	}
s = strstr(ptr, "%2");
if (s)
	{
	memmove(s + strlen(lbuf), s + 2, strlen(s + 2) + 1);
	memcpy(s, lbuf, strlen(lbuf)); 
	}

if (poll_wind == -1)
	{
	GRECT gc, root;
	int top, nmsg[8];
	
	wind_get_int(ROOT, WF_TOP, &top);
	wind_get_grect(ROOT, WF_WORKXYWH, &root);
	form_center(poll_tree, &gc);
	poll_wind = wind_create(NAME|CLOSER, &root);
	if (poll_wind < 0)
		return;
		
	wind_calc(WC_BORDER, NAME|CLOSER|MOVER, &gc, &gc);
	gc.g_y = root.g_y + root.g_h - gc.g_h;
	gc.g_x = root.g_x + root.g_w - gc.g_w;
	wind_set_string(poll_wind, WF_NAME, poll_tree[POLL_TITLE].ob_spec.free_string);
	wind_open(poll_wind, &gc);
	
	nmsg[0] = WM_TOPPED;
	nmsg[3] = top;
	appl_write(gl_apid, 16, nmsg);
	}
else
	{
	GRECT gr;
	
	wind_get_grect(poll_wind, WF_WORKXYWH, &gr);
	redraw_pollwindow(&gr);
	}
}
   
/* -------------------
   | Initialize poll |
   ------------------- */
void setPollInitialize(void)
{
DEBUG();
poll_status = POLL_NONE;
poll_idx = -1;
poll_wait = 50;
locked_server[0] = 0;
}

/* ----------------
   | Stop polling |
   ---------------- */
void stopPoll(void)
{
DEBUG();
if (poll_status > POLL_CONNECT)
	{
	close_socket(poll_socket);

	if (poll_file)
		{
		char buf[51 * 2 + MAXFILE * 2 + 1];

		Fclose(poll_file);

		base_path(buf);
		strcat(buf, poll_filename);
		Fdelete(buf);
		}
	
	poll_info[poll_idx].last_checked = clock();
	poll_status = POLL_NONE;
	poll_idx++; 
	if (poll_idx == poll_cnt) poll_idx = -1;
	}
}

/* -----------------
   | Block polling |
   ----------------- */
void blockPoll(int flg)
{
if (flg)
	blocked++;
else
	blocked--;
}

/* ------------------
   | Is it polling? |
   ------------------ */
int isPolling(void)
{
return poll_status != POLL_NONE;
}

/* ------------------------
   | is identity in use ? |
   ------------------------ */
int isIdentityUsed(char *server, char *user)
{
DEBUG();
if (poll_status != POLL_NONE)
	{
	char *popserver, *popuser;
	
	if (current_poll.use_providerdata)
		{
		popserver 	= cfg_options.popserver;
		popuser			= pop_username;
		}
	else
		{
		popserver 	= current_poll.popserver;
		popuser			= current_poll.popuser;
		}

	return !strcmp(popserver, server) && !strcmp(popuser, user);
	}
else
	return FALSE;
}
   
/* ------------------------
   | Set lock on identity |
   ------------------------ */
void lockIdentity(char *server, char *user)
{
DEBUG();
strcpy(locked_server, server);
strcpy(locked_user, user);
}

/* ------------------------
   | Set lock on identity |
   ------------------------ */
void unlockIdentity(void)
{
DEBUG();
locked_server[0] = 0;
locked_user[0] = 0;
}

/* --------
   | Poll |
   -------- */
void doPoll(void)
{
DEBUG();
if (get_connected() <= 0)
	return;

/*printf("Idle: %d\n", poll_status);	*/
switch(poll_status)
	{
	case POLL_NONE:
		if (!blocked)
			pollIdle();
		break;

	case POLL_RESOLVE:
		pollResolve();
		break;

	case POLL_RESOLVING:
		pollResolving();
		break;

	case POLL_CONNECT:
		pollConnect();
		break;

	case POLL_CONNECTING:
		pollConnecting();
		break;

	case POLL_WELCOME:
		pollWelcome();
		break;

	case POLL_AUTH:
		pollAuth();
		break;

	case POLL_AUTH_WAIT:
		pollAuthWait();
		break;

	case POLL_PASS:
		pollPass();
		break;
		
	case POLL_PASS_WAIT:
		pollPassWait();
		break;
		
	case POLL_STAT:
		pollStat();
		break;

	case POLL_STAT_WAIT:
		pollStatWait();
		break;

	case POLL_STAT_CHECK:
		pollStatCheck();
		break;

	case POLL_UIDL:
		pollUIDL();
		break;

	case POLL_UIDL_WAIT:
		pollUIDLWait();
		break;
		
	case POLL_UIDL_CHECK:
		pollUIDLCheck();
		break;
	
	case POLL_UIDL_COMPLETE:
		pollUIDLComplete();
		break;
		
	case POLL_RECV:
		pollRetr();
		break;
	
	case POLL_RECV_LIST:
		pollRetrList();
		break;
		
	case POLL_RECV_LISTCOMPL:
		pollRetrListComplete();
		break;
		
	case POLL_RECV_START:
		pollRetrStart();
		break;
		
	case POLL_RECV_WAIT:
		pollRetrWait();
		break;
	
	case POLL_RECV_STARTED:
		pollRetrStarted();
		break;

	case POLL_RECV_PROGRESS:
		pollRetrInProgress();
		break;

	case POLL_RECV_COMPLETE:
		pollRetrComplete();
		break;
		
	case POLL_DELE:
		pollDele();
		break;
		
	case POLL_DELE_WAIT:
		pollDeleWait();
		break;
	
	case POLL_DELE_COMPLETE:
		pollDeleComplete();
		break;
		
	case POLL_QUIT:
		pollQuit();
		break;
		
	case POLL_QUIT_WAIT:
		pollQuitWait();
		break;
		
	case POLL_CLOSE:
		pollClose();
		break;
	}
}

/* ----------------
   | Poll response|
   ---------------- */
static void pollResponse(void)
{
DEBUG();
if (TestIncoming(poll_socket))
	{
	int cnt, n;
	char c;
	
	n = (int)recv(poll_socket, NULL, 255, MSG_PEEK);
	
	while (n-- > 0)
		{
		cnt = (int)recv(poll_socket, &c, 1, 0);
		if (cnt == 1)
			{
			if (c == '\n')
				{
				poll_buff[poll_buff_cnt] = 0;
/*				printf("%s\n", poll_buff);*/
				if (poll_buff[0] == '-')
					{
					poll_status = POLL_QUIT;
					return;
					}
				
				poll_status++;
				return;
				}
			poll_buff[poll_buff_cnt++] = c;
			}
		}
	}
else
	if (TestException(poll_socket))
		{
		close_socket(poll_socket);
		poll_info[poll_idx].last_checked = clock();
		poll_status = POLL_NONE;
		poll_idx++; 
		if (poll_idx == poll_cnt) poll_idx = -1;
		}
}

/* -----------------
   | Free poll ids |
   ----------------- */
void freePollIds(int idx)
{
int i;

DEBUG();
for (i = 0; i < poll_info[idx].number_of_mails; i++)
	free(poll_info[idx].ids[i]);
free(poll_info[idx].ids);
}
  
/* -----------------------------
   | If there is nothing to do |
   ----------------------------- */
static void pollIdle(void)
{
int i, n, j;
IDENTITY *id;

DEBUG();
n = getNumberOfIdentities();
id = getIdentityPtr(0);

if (poll_idx == -1)
	{
	if (has_new)
		{
/*		printf("Has new mail\n");*/
		if (mail_options.play_sound)
#ifdef AUDIO
			play_one();
#endif
		if (mail_options.open_notify)
			open_poll();
		
		has_new = FALSE;
		}
		
	if (poll_wait--)
		return;

	poll_wait = 50;		
	for (i = 0; i < poll_cnt; i++)
		poll_info[i].found = FALSE;

	n = getNumberOfIdentities();
	id = getIdentityPtr(0);
	for (i = 0; i < n; i++, id++)
		{
		IDENTITY_POLL *pollinfo_ptr;

		pollinfo_ptr = poll_info;		
		for (j = 0; j < poll_cnt; j++, pollinfo_ptr++)
			if (pollinfo_ptr->identifier == id->identifier)
				{
				pollinfo_ptr->found = TRUE;
				break;
				}
				
		if (j == poll_cnt)
			{
			if (!poll_cnt)
				poll_info = (IDENTITY_POLL *)malloc(sizeof(IDENTITY_POLL));
			else
				poll_info = (IDENTITY_POLL *)realloc(poll_info, (poll_cnt + 1) * sizeof(IDENTITY_POLL));
			
			pollinfo_ptr = poll_info + poll_cnt++;
			pollinfo_ptr->identifier = id->identifier;
			pollinfo_ptr->found			 = TRUE;
			pollinfo_ptr->last_checked = -1;
			pollinfo_ptr->number_of_mails = 0;
			pollinfo_ptr->mail_size = 0L;
			pollinfo_ptr->ids = NULL;
			pollinfo_ptr->recv_ids = NULL;
			pollinfo_ptr->received = 0;
			}
		}
		
	if (poll_cnt)
		{
		int old = poll_cnt;
		
		for (i = 0; i < poll_cnt; i++)
			if (!poll_info[i].found)
				{
				if (poll_info[i].received)
					{
					for (j = 0; j < poll_info[i].received; j++)
						free(poll_info[i].recv_ids[j]);
					free(poll_info[i].recv_ids);
					}
				freePollIds(i);
				if (i < poll_cnt - 1)
					memmove(poll_info + i, poll_info + i + 1, sizeof(IDENTITY_POLL) * (poll_cnt - i - 1));
				poll_cnt--;
				}

		if (old != poll_cnt)						
			if (!poll_cnt)
				free(poll_info);
			else
				poll_info = (IDENTITY_POLL *)realloc(poll_info, poll_cnt * sizeof(IDENTITY_POLL));
		}
	poll_idx++;
	if (poll_idx == poll_cnt) poll_idx = -1;
	}
else
	{
	for (i = 0; i < n; i++, id++)
		if (poll_info[poll_idx].identifier == id->identifier)
			{
			memcpy(&current_poll, id, sizeof(IDENTITY));
			break;
			}
			
	if (i == n)
		{
		poll_idx++;
		if (poll_idx == poll_cnt) poll_idx = -1;
		}
	else
		poll_status++;
	}
}

/* -------------------
   | Resolve host ip |
   ------------------- */
static void pollResolve(void)
{
char *popserver;
int state;

DEBUG();
if (current_poll.poll_every <= 0 || 
		(poll_info[poll_idx].last_checked != -1 && 
		 clock() - poll_info[poll_idx].last_checked < current_poll.poll_every * CLK_TCK))
	{
	poll_status = POLL_NONE;
	poll_info[poll_idx].last_checked = clock();
	poll_idx++; 
	if (poll_idx == poll_cnt) poll_idx = -1;
	return;
	}
	
poll_remote.sin_family = AF_INET;

if (current_poll.pop_port == -1)
	{
	struct servent *serv;
	
	serv = getservbyname("pop-3", "tcp");
	if (!serv)
		serv = getservbyname("pop3", "tcp");

	if (!serv)
		{
		poll_status = POLL_NONE;
		poll_info[poll_idx].last_checked = clock();
		poll_idx++; 
		if (poll_idx == poll_cnt) poll_idx = -1;
		return;
		}
	poll_remote.sin_port = serv->s_port;
	}
else
	poll_remote.sin_port = current_poll.pop_port;
	
if (current_poll.use_providerdata)
	popserver 	= cfg_options.popserver;
else
	popserver 	= current_poll.popserver;

if (isdigit(popserver[0]))
	{
	poll_remote.sin_addr.s_addr = inet_addr(popserver);
	poll_status += 2; /* Skip resolving hostname */
	return;
	}

state = async_gethostbyname(popserver);
if (state)
	return;
	
poll_status++;
}

/* ---------------------------
   | Still resolving host ip |
   --------------------------- */
static void pollResolving(void)
{
struct hostent *host;

DEBUG();
host = async_queryname();
if ((long)host == -EINPROGRESS)
	return;
	
async_discard();

if (host)
	{
	memcpy(&poll_remote.sin_addr.s_addr, host->h_addr, host->h_length);
	poll_status++;
	}
else
	{
	poll_status = POLL_NONE;
	poll_info[poll_idx].last_checked = clock();
	poll_idx++; 
	if (poll_idx == poll_cnt) poll_idx = -1;
	}
}

/* ---------------
   | Connect pop |
   --------------- */
static void pollConnect(void)
{
int ret;
char *popserver, *popuser;

DEBUG();
if (current_poll.use_providerdata)
	{
	popserver 	= cfg_options.popserver;
	popuser			= pop_username;
	}
else
	{
	popserver 	= current_poll.popserver;
	popuser			= current_poll.popuser;
	}

if (!strcmp(locked_server, popserver) && !strcmp(locked_user, popuser))
	{
	poll_status = POLL_NONE;
	poll_info[poll_idx].last_checked = clock();
	poll_idx++; 
	if (poll_idx == poll_cnt) poll_idx = -1;
	return;
	}
	
poll_buff_cnt = 0;
poll_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
if (!poll_socket)
	{
	poll_status = POLL_NONE;
	poll_info[poll_idx].last_checked = clock();
	poll_idx++; 
	if (poll_idx == poll_cnt) poll_idx = -1;
	return;
	}

if (get_tcpversion() >= 0x00010007L)
	sockfcntl(poll_socket, SO_SETFL, SO_O_NONBLOCK);

ret = connect(poll_socket, (struct sockaddr *)&poll_remote, (int)sizeof(poll_remote));
poll_status++;
if (ret != 0 && (errno == -EINPROGRESS || errno == -EALREADY))
	return;
if (ret && errno == -EISCONN)
	ret = 0;
if (ret)
	{
	close_socket(poll_socket);
	poll_status = POLL_NONE;
	poll_info[poll_idx].last_checked = clock();
	poll_idx++; 
	if (poll_idx == poll_cnt) poll_idx = -1;
	return;
	}
			
if (get_tcpversion() >= 0x00010007L)
	sockfcntl(poll_socket, SO_SETFL, 0);
poll_status++;
}

/* ------------------
   | Connecting pop |
   ------------------ */
static void pollConnecting(void)
{
int ret;

DEBUG();
ret = connect(poll_socket, NULL, 0);
if (ret != 0 && (errno == -EINPROGRESS || errno == -EALREADY))
	return;
if (ret && errno == -EISCONN)
	ret = 0;
if (ret)
	{
	close_socket(poll_socket);
	poll_status = POLL_NONE;
	poll_info[poll_idx].last_checked = clock();
	poll_idx++; 
	if (poll_idx == poll_cnt) poll_idx = -1;
	return;
	}

if (get_tcpversion() >= 0x00010007L)
	sockfcntl(poll_socket, SO_SETFL, 0);
poll_status++;
}

/* ----------------------------
   | Wait for welcome message |
   ---------------------------- */
static void pollWelcome(void)
{
DEBUG();
pollResponse();
}

/* -----------------------
   | Send authentication |
   ----------------------- */
static void pollAuth(void)
{
char *popuser, *poppass;
char pass_id[32];

DEBUG();
if (current_poll.use_providerdata)
	{
	popuser			= pop_username;
	poppass			= pop_password;
	}
else
	{
	popuser			= current_poll.popuser;
	poppass			= current_poll.mailpass;
	}

poll_buff_cnt = 0;
if (current_poll.pass_coded)
	{
	char *s;
	
	s = strchr(poll_buff, '>');
	
	if (s)
		{
		strcpy(s + 1, poppass);
		
		s = strchr(poll_buff, '<');	
		if (s)
			{
			MDString(s, pass_id);
			send_cmd(poll_socket, "APOP", popuser, " ", pass_id);
			poll_status = POLL_PASS_WAIT;
			return;
			}
		}
	}
else
	{
	send_cmd(poll_socket, "USER", popuser, NULL, NULL);
	poll_status++;
	}	
}

/* -------------------------------
   | Get authentication response |
   ------------------------------- */
static void pollAuthWait(void)
{
DEBUG();
pollResponse();
}

/* -----------------------
   | Send authentication |
   ----------------------- */
static void pollPass(void)
{
char *poppass;

DEBUG();
poll_buff_cnt = 0;
if (current_poll.use_providerdata)
	poppass			= pop_password;
else
	poppass			= current_poll.mailpass;
	
poll_buff_cnt = 0;
send_cmd(poll_socket, "PASS", poppass, NULL, NULL);
poll_status++;
}

/* -------------------------
   | Get password response |
   ------------------------- */
static void pollPassWait(void)
{
DEBUG();
pollResponse();
}

/* ----------------
   | Request STAT |
   ---------------- */
static void pollStat(void)
{
DEBUG();
poll_buff_cnt = 0;
send_cmd(poll_socket, "STAT", NULL, NULL, NULL);
poll_status++;
}

/* --------------------------
   | Wait for stat response |
   -------------------------- */
static void pollStatWait(void)
{
DEBUG();
pollResponse();
}

/* --------------------------
   | Wait for stat response |
   -------------------------- */
static void pollStatCheck(void)
{
char *s, *s2;

DEBUG();
s = strchr(poll_buff, ' ');
if (s)
	{
	s++;
	s2 = strchr(s, ' ');
	if (s2)
		{
		*s2 = 0;

		if (new_mail_count)
			{
			int i;
			
			for (i = 0; i < new_mail_count; i++)
				if (uidl_ids[i])
					free(uidl_ids[i]);
			free(uidl_ids);
			}
			
		new_mail_count = atoi(s);
		new_mail_size  = atol(s2 + 1);
		uidl_idx = 0;
		if (new_mail_count)
			{
			long l = sizeof(char *) * new_mail_count;
			uidl_ids = (char **)malloc(l);
			memset(uidl_ids, 0, l);
			}
			
		poll_status++;
		return;
		}
	}
poll_status = POLL_QUIT;
}

/* ----------------
   | Request STAT |
   ---------------- */
static void pollUIDL(void)
{
char buf[5];

DEBUG();
if (uidl_idx == new_mail_count)
	{
	poll_status = POLL_UIDL_COMPLETE;
	return;
	}
	
itoa(uidl_idx + 1, buf, 10);
poll_buff_cnt = 0;
send_cmd(poll_socket, "UIDL", buf, NULL, NULL);
poll_status++;
}

/* --------------------------
   | Wait for UIDL response |
   -------------------------- */
static void pollUIDLWait(void)
{
DEBUG();
pollResponse();
}

/* -----------------------
   | Check UIDL response |
   ----------------------- */
static void pollUIDLCheck(void)
{
char *s;

DEBUG();
s = strchr(poll_buff, ' ');

if (s)
	{
	uidl_ids[uidl_idx] = (char *)malloc(strlen(s + 1) + 1);
	strcpy(uidl_ids[uidl_idx], s + 1);

	poll_status = POLL_UIDL;
	}
else
	poll_status = POLL_QUIT;
}

/* -----------------------
   | Check UIDL response |
   ----------------------- */
static void pollUIDLComplete(void)
{
int i, j, old_n;
char **old_ids;

DEBUG();
old_ids = poll_info[poll_idx].ids;
old_n = poll_info[poll_idx].number_of_mails;

/* Compare for new ids */
for (i = 0; i < new_mail_count; i++)
	{
	for (j = 0; j < old_n; j++)
		if (!strcmp(old_ids[j], uidl_ids[i]))
			break;
	
	if (j == old_n)
		{
		has_new = TRUE;
		break;
		}
	}

for (i = 0; i < poll_info[poll_idx].received; i++)
	{
	for (j = 0; j < new_mail_count; j++)
		if (!strcmp(poll_info[poll_idx].recv_ids[i], uidl_ids[j]))
			break;
	
	if (j == new_mail_count)
		{
		free(poll_info[poll_idx].recv_ids[i]);
		if (i < poll_info[poll_idx].received - 1)
			memmove(poll_info[poll_idx].recv_ids + i, poll_info[poll_idx].recv_ids + i + 1, 
							(poll_info[poll_idx].received - i - 1) * sizeof(char *));
		poll_info[poll_idx].received--;
		if (!poll_info[poll_idx].received)
			free(poll_info[poll_idx].recv_ids);
			
		i--;
		}
	}
	
/* Store ids */
freePollIds(poll_idx);

poll_info[poll_idx].number_of_mails = new_mail_count;
poll_info[poll_idx].mail_size				= new_mail_size;
poll_info[poll_idx].ids 						= uidl_ids;
uidl_ids = NULL;
new_mail_count = 0;
uidl_idx = 0;

poll_status++;
}

/* --------------------------------------
	 | Start receiving mail in background |
	 -------------------------------------- */
static void pollRetr(void)
{
char num[7];
int i;

DEBUG();
if (current_poll.count_only || uidl_idx == poll_info[poll_idx].number_of_mails)
	{
	poll_status = POLL_QUIT;
	return;
	}

for (i = 0; i < poll_info[poll_idx].received; i++)
	if (!strcmp(poll_info[poll_idx].recv_ids[i], poll_info[poll_idx].ids[uidl_idx]))
		{
		uidl_idx++;	/* This index was already downloaded */
		return;
		}

itoa(uidl_idx + 1, num, 10);
poll_buff_cnt = 0;
send_cmd(poll_socket, "LIST", num, NULL, NULL);
poll_status++;
}

/* -----------------------------------------
   | Get status of the message to download |
   ----------------------------------------- */
static void pollRetrList(void)
{
DEBUG();
pollResponse();
}

/* -----------------------------------------
   | Get status of the message to download |
   ----------------------------------------- */
static void pollRetrListComplete(void)
{
char *s, *s2;
long poll_recv_size;

DEBUG();
s = strchr(poll_buff, ' ');
if (s)
	{
	s++;
	s2 = strchr(s, ' ');
	if (s2)
		{
		*s2 = 0;

		poll_recv_size  = atol(s2 + 1);
		
		if (poll_recv_size < current_poll.maxsize || !current_poll.maxsize)
			poll_status++;
		return;
		}
	}
uidl_idx++;
poll_status = POLL_RECV;
}

/* ------------------
   | Start retrieve |
   ------------------ */
static void pollRetrStart(void)
{
char num[7];

DEBUG();
itoa(uidl_idx + 1, num, 10);
poll_buff_cnt = 0;
send_cmd(poll_socket, "RETR", num, NULL, NULL);
poll_status++;
}

/* ------------------------------
   | Receive mail in background |
   ------------------------------ */
static void pollRetrWait(void)
{
DEBUG();
pollResponse();
}

/* -------------------------------
   | Retrieve command progressed |
   ------------------------------- */
static void pollRetrStarted(void)
{
DEBUG();
if (poll_buff[0] == '+')
	{
	poll_file = create_outgoing(poll_filename);
	if (poll_file)
		{
		memset(&poll_in_buf, 0, sizeof(poll_in_buf));
	
		poll_in_buf.magic 		= 0x4104;
		poll_in_buf.version 	= SVERSION_ID20;
		poll_in_buf.body_pos = sizeof(poll_in_buf);
		poll_in_buf.part_count = -1;
		strcpy(poll_in_buf.identity, current_poll.name);
		Fwrite(poll_file, sizeof(poll_in_buf), &poll_in_buf);
	
		poll_status++;
		return;
		}
	}

uidl_idx++;
poll_status = POLL_RECV;
}

/* -------------------------
   | Process incoming data |
   ------------------------- */
static void pollRetrInProgress(void)
{
DEBUG();
if (TestIncoming(poll_socket))
	{
	int cnt, n;
	
	n = (int)recv(poll_socket, NULL, 255, MSG_PEEK);
	cnt = (int)recv(poll_socket, poll_buff, n, 0);
	poll_buff[cnt] = 0;
	
	if (cnt >= 3 && !strcmp(poll_buff + cnt - 3, ".\r\n"))
		{
		/* End of data */
		if (cnt > 3)
			Fwrite(poll_file, cnt - 3, poll_buff);
		poll_status++;
		return;
		}
	
	Fwrite(poll_file, cnt, poll_buff);
	}
else
	if (TestException(poll_socket))
		{
		close_socket(poll_socket);
		poll_info[poll_idx].last_checked = clock();
		poll_status = POLL_NONE;
		poll_idx++; 
		if (poll_idx == poll_cnt) poll_idx = -1;
		}
}

/* ---------------------
   | receive completed |
   --------------------- */
static void pollRetrComplete(void)
{
DEBUG();
poll_in_buf.body_len = Fseek(0, poll_file, SEEK_CUR) - sizeof(poll_in_buf);

Fseek(0, poll_file, SEEK_SET);
Fwrite(poll_file, sizeof(poll_in_buf), &poll_in_buf);

Fclose(poll_file);
poll_file = 0;
	
if (store_incoming(poll_filename))
	{
	int in_box = box_number("IN");
	int wx, wy, ww, wh, r;

	set_listinfo(box_number("IN"));
	if (box_manager[in_box].list_wind > -1) 
		{
		wind_get(box_manager[in_box].list_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
		redraw_listwindow(in_box, wx, wy, ww, wh);
		}
	
	r = poll_info[poll_idx].received++;
	if (!r)
		poll_info[poll_idx].recv_ids = (char **)malloc(sizeof(char *));
	else
		poll_info[poll_idx].recv_ids = (char **)realloc(poll_info[poll_idx].recv_ids, 
																										sizeof(char *) * (r + 1));
	poll_info[poll_idx].recv_ids[r] = (char *)malloc(strlen(poll_info[poll_idx].ids[uidl_idx]) + 1);
	strcpy(poll_info[poll_idx].recv_ids[r], poll_info[poll_idx].ids[uidl_idx]);
	
	recv_done = TRUE;
	poll_status++;
	}
else
	{
	uidl_idx++;
	poll_status = POLL_RECV;	
	}
}

/* -------------------------------------
	 | Start deleting mail in background |
	 ------------------------------------- */
static void pollDele(void)
{
char num[7];

DEBUG();
if (!current_poll.delete_rcvd)
	{
	uidl_idx++;
	poll_status = POLL_RECV;
	return;
	}
	
itoa(uidl_idx + 1, num, 10);
poll_buff_cnt = 0;
send_cmd(poll_socket, "DELE", num, NULL, NULL);
poll_status++;
}

/* -----------------------------
   | Delete mail in background |
   ----------------------------- */
static void pollDeleWait(void)
{
DEBUG();
pollResponse();
}

/* -----------------------------
   | Poll delete was correctly |
   ----------------------------- */
static void pollDeleComplete(void)
{
DEBUG();
uidl_idx++;
poll_status = POLL_RECV;
}

/* ----------------
   | Request QUIT |
   ---------------- */
static void pollQuit(void)
{
DEBUG();
poll_buff_cnt = 0;
send_cmd(poll_socket, "QUIT", NULL, NULL, NULL);
poll_status++;
}

/* --------------------------
   | Wait for stat response |
   -------------------------- */
static void pollQuitWait(void)
{
DEBUG();
pollResponse();
}

/* ----------------
   | Close socket |
   ---------------- */
static void pollClose(void)
{
DEBUG();
close_socket(poll_socket);

if (recv_done)
	{
	int box = box_number("IN");
	if (boxes[box].auto_apply)
		apply_filter(-1);
	recv_done = FALSE;
	}
	
poll_info[poll_idx].last_checked = clock();
poll_status = POLL_NONE;
poll_idx++; 
if (poll_idx == poll_cnt) poll_idx = -1;
}

/* --------------------
   | Load receive ids |
   -------------------- */
void load_recvids(void)
{
long id_fl;
char buf[256];
int i, j;

DEBUG();
if (poll_cnt)
	{
	for (i = 0; i < poll_cnt; i++)
		{
		if (poll_info[i].received)
			{
			for (j = 0; j < poll_info[i].received; j++)
				free(poll_info[i].recv_ids[j]);
			free(poll_info[i].recv_ids);
			}
		freePollIds(i);
		}
	free(poll_info);
	poll_cnt = 0;
	}
	
base_path(buf);
strcat(buf, "RECV.DAT");

id_fl = Fopen(buf, FO_READ);
if (id_fl > 0)
	{
	int k, n;
	long ver;
	char name[21];
	IDENTITY *id;
	
	Fread((int)id_fl, sizeof(ver), &ver);

	while (Fread((int)id_fl, 21, name) == 21)
		{
		n = getNumberOfIdentities();
		id = getIdentityPtr(0);
		
		for (j = 0; j < n; j++, id++)
			if (!strcmp(id->name, name))
				{
				IDENTITY_POLL *pollinfo_ptr;
				int received;
				
				if (!poll_cnt)
					poll_info = (IDENTITY_POLL *)malloc(sizeof(IDENTITY_POLL));
				else
					poll_info = (IDENTITY_POLL *)realloc(poll_info, sizeof(IDENTITY_POLL) * (poll_cnt + 1));
					
				pollinfo_ptr = poll_info + poll_cnt++;
				pollinfo_ptr->identifier = id->identifier;
				pollinfo_ptr->found			 = TRUE;
				pollinfo_ptr->last_checked = -1;
				pollinfo_ptr->number_of_mails = 0;
				pollinfo_ptr->mail_size = 0L;
				pollinfo_ptr->ids = NULL;

				Fread((int)id_fl, sizeof(received), &received);
				pollinfo_ptr->received = received;

				if (received)
					{
					pollinfo_ptr->recv_ids = (char **)malloc(sizeof(char *) * received);
					for (k = 0; k < received; k++)
						{
						int l;
				
						Fread((int)id_fl, sizeof(l), &l);
						pollinfo_ptr->recv_ids[k] = (char *)malloc(l + 1);
						Fread((int)id_fl, l, pollinfo_ptr->recv_ids[k]);
						pollinfo_ptr->recv_ids[k][l] = 0;
						}
					}
						
				break;
				}
		
		if (j == n)
			{
			int received;
			
			Fread((int)id_fl, sizeof(received), &received);
			for (k = 0; k < received; k++)
				{
				int l;
				
				Fread((int)id_fl, sizeof(l), &l);
				Fseek(l, (int)id_fl, SEEK_CUR);
				}
			}
		}				  
		
	Fclose((int)id_fl);
	}
}
   
/* ----------------------------
   | Save current receive ids |
   ---------------------------- */
void save_recvids(void)
{
long id_fl;
char buf[256];
int i, n;

DEBUG();
base_path(buf);
strcat(buf, "RECV.DAT");
id_fl = Fcreate(buf, 0);
if (id_fl > 0)
	{
	long ver = VERSION_ID20;
	IDENTITY *id;
	
	Fwrite((int)id_fl, sizeof(ver), &ver);
	
	n = getNumberOfIdentities();
	id = getIdentityPtr(0);
	for (i = 0; i < n; i++, id++)
		{
		IDENTITY_POLL *pollinfo_ptr;
		int j, k; 

		pollinfo_ptr = poll_info;		
		for (j = 0; j < poll_cnt; j++, pollinfo_ptr++)
			if (pollinfo_ptr->identifier == id->identifier)
				{
				Fwrite((int)id_fl, 21, id->name);
				Fwrite((int)id_fl, sizeof(pollinfo_ptr->received), &pollinfo_ptr->received);
				
				for (k = 0; k < pollinfo_ptr->received; k++)
					{
					int l = (int)strlen(pollinfo_ptr->recv_ids[k]);
					
					Fwrite((int)id_fl, sizeof(l), &l);
					Fwrite((int)id_fl, l, pollinfo_ptr->recv_ids[k]);
					}
					
				break;
				}
		}	
	Fclose((int)id_fl);
	}
}
