/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

int retrieve_msg(int sk, int msg, char *file);
int remove_mail(int sk, int msg);
void remove_pop_entry(int msg);
void resize_popcolumn(int col);
void set_popsorting(void);
void get_datevalues(char *date, int *r_day, int *r_month, int *r_year,
										int *r_hour, int *r_min, int *r_sec);
void sort_popbox(void);

static POP_MEM *pop_mem = NULL;
static long pop_size;
static int pop_count = 0;
static char pop_title[256];
static char pop_info[256];
static long pop_total_size, pop_marked, pop_loaded;
static int pop_page, pop_left, pop_current;
static int pop_marks;
static char buf[3000];
static int sort_order;

static int pbut_pos[5];
static int pbut_no[5] = {73-1, 76-1, 79-1, 82-1, 85-1};
static int pbut_disabled[5], pbut_selected[5];

static int defsize[] = {23, 23, 23, 8, 17, 22};

static int inuse_ids[] = {CANCEL_OP,
													CANCEL_BG,
													0};

/* ------------------------------------------------
   | Calculate box column positions               |
   | From / To / Subject / Size / Date / Identity |
   ------------------------------------------------ */
void calc_popcolumn_positions(int *col_pos, int *col_size)
{
int i, n;

DEBUG();
for (i = 0; i < 6; i++)
	if (pop_cols[i] < 0)
		pop_cols[i] = defsize[i];												/* Default old data */

n = 3;		
for (i = 0; i < 6; i++)
	{
	col_pos[i] = n;
	if (pop_cols[i] > 0)
		{
		col_size[i] = pop_cols[i];
		n += col_size[i] + 3;
		}
	else
		col_size[i] = 0;
	}
}

/* ---------------------------------------
   | Calculate horizontal button offsets |
   --------------------------------------- */
void calc_popbutpos(void)
{
int ispace, ox;

DEBUG();
ispace = isize / 2;
	
		 pbut_pos[0] = 2;																/* Mark for retrival 		*/
ox = pbut_pos[1] = 2  + isize + 4;									/* Mark for deletion 		*/
ox = pbut_pos[2] = ox + isize + 4;									/* Mark for both		 		*/
ox = pbut_pos[3] = ox + isize + ispace + 4;					/* Do it	*/
     pbut_pos[4] = ox + isize + ispace + 4;					/* Close		*/
}

/* -----------------------------
   | Select mail editor button |
   ----------------------------- */
void select_popbutton(int no)
{
DEBUG();
pbut_selected[0] = 
pbut_selected[1] = 
pbut_selected[2] = 
pbut_selected[3] = 
pbut_selected[4] = FALSE;

if (no != -1)
	pbut_selected[no] = TRUE;
}
   
/* -------------------------
   | Draw a toolbar button |
   ------------------------- */
void draw_popbutton(int no, int x, int y)
{
int bnum;

DEBUG();
bnum = pbut_no[no]; 
if (pbut_disabled[no])
	bnum += 2;
else
	if (pbut_selected[no])
		bnum++;
		
blt_copy(mw_toolbar, 0, bnum * iheight, NULL, x, y, isize, iheight, S_ONLY);
}

/* ----------------------
   | Select mail button |
   ---------------------- */
void redraw_popbutton(int x)
{
GRECT t1;
int xy[4];
int wx, wy, ww, wh;

DEBUG();
calc_popbutpos();

wind_update(BEG_UPDATE);
graf_mouse(M_OFF, 0L);

wind_get(pop_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);

wind_get(pop_wind, WF_FIRSTXYWH, &t1.g_x, &t1.g_y, &t1.g_w, &t1.g_h);
while(t1.g_w && t1.g_h)
	{
	xy[0] = t1.g_x;
	xy[1] = t1.g_y;
	xy[2] = t1.g_x + t1.g_w - 1;
	xy[3] = t1.g_y + t1.g_h - 1;
	vs_clip(vdi_handle, 1, xy);

	draw_popbutton(x, wx + pbut_pos[x], wy);

	vs_clip(vdi_handle, 0, xy);
		
	wind_get(pop_wind, WF_NEXTXYWH, &t1.g_x, &t1.g_y, &t1.g_w, &t1.g_h);
	}

graf_mouse(M_ON, 0L);
wind_update(END_UPDATE);
}

/* -------------------------------------
   | Enable menu entries in pop window |
   ------------------------------------- */
void pop_enable(void)
{
int pdisabled[8], i;

DEBUG();
memcpy(pdisabled, pbut_disabled, sizeof(pdisabled));

/*if (!pop_marks)
	{
	menu_ienable(menu_adr, M_RECEIV, FALSE);
	menu_ienable(menu_adr, M_RCVDEL, FALSE);
	menu_ienable(menu_adr, M_DELETE, FALSE);
	}
else
	{
	menu_ienable(menu_adr, M_RECEIV, TRUE);
	menu_ienable(menu_adr, M_RCVDEL, TRUE);
	menu_ienable(menu_adr, M_DELETE, TRUE);
	}
*/
menu_ienable(menu_adr, M_FWD, FALSE);
menu_ienable(menu_adr, M_REPLY, FALSE);
menu_ienable(menu_adr, M_AGAIN, FALSE);
menu_ienable(menu_adr, M_SEND, FALSE);

if (pop_count)
	menu_ienable(menu_adr, M_SELALL, TRUE);
else
	menu_ienable(menu_adr, M_SELALL, FALSE);

pbut_disabled[4] = FALSE;			/* Close 		*/

if (pop_wind > -1)
	for (i = 0; i < 5; i++)
		if (pdisabled[i] != pbut_disabled[i])
			redraw_popbutton(i);
}

/* --------------
   | Pop slider |
   -------------- */
void set_popslider(void)
{
int t_xy[8];
int th, tw;
int wx, wy, ww, wh;
int size, pos, new, old;
int col_pos[6], col_size[6], hsize;

DEBUG();
calc_popcolumn_positions(col_pos, col_size);
hsize = col_pos[5] + col_size[5]/* + 2*/;

set_default_text();
vqt_extent(vdi_handle, "x", t_xy);
tw = (t_xy[4] - t_xy[0]);
th = (t_xy[5] - t_xy[1]);

wind_get(pop_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
if (ww + pop_left * tw >= hsize * tw)
	{
	new = hsize - ww / tw;
	if (new < 0) new = 0;
	if (new != pop_left)
		{
		pop_left = new;
		redraw_popwindow(wx, wy, ww, wh);
		}
	}
	
if (ww >= hsize * tw)
	{
	size = 1000;
	pos = 0;
	}
else
	{
	pos = (int)(pop_left * 1000L / (hsize - ww / tw));
	size = (int)(ww * 1000L / tw / hsize);
	}
	
wind_get_int(pop_wind, WF_HSLSIZE, &old);
if (size != old)
	wind_set_int(pop_wind, WF_HSLSIZE, size);
	
wind_get_int(pop_wind, WF_HSLIDE, &old);
if (pos != old)
	wind_set_int(pop_wind, WF_HSLIDE, pos);

if (wh / th - 1 + pop_page >= pop_count)
	{
	new = pop_count - wh / th + 1;
	if (new < 0) new = 0;
	if (new != pop_page)
		{
		pop_page = new;
		redraw_popwindow(wx, wy, ww, wh);
		}
	}
	
if (wh / th - 1 >= pop_count)
	{
	if (pop_page > 0)
		{
		pop_page = 0;
		redraw_popwindow(wx, wy, ww, wh);
		}
	size = 1000;
	pos = 0;
	}
else
	{
	pos = (int)(pop_page * 1000L / (pop_count - wh / th + 1));
	size = (int)((wh / th - 1) * 1000L / pop_count);
	}

wind_get_int(pop_wind, WF_VSLSIZE, &old);
if (size != old)
	wind_set_int(pop_wind, WF_VSLSIZE, size);

wind_get_int(pop_wind, WF_VSLIDE, &old);
if (pos != old)
	wind_set_int(pop_wind, WF_VSLIDE, pos);
}

/* ---------------------
   | Set pop info line |
   --------------------- */
void set_popinfo(void)
{
DEBUG();
itoa(pop_count, pop_info,	10);
if (pop_count > 1)
	strcat(pop_info, s_msgs);
else
	strcat(pop_info, s_msg);
	
ltoa(pop_total_size, pop_info + strlen(pop_info), 10);
strcat(pop_info, " bytes)");

if (pop_marked)
	{
	strcat(pop_info, " [");

	itoa(pop_marks, pop_info + strlen(pop_info), 10);
	if (pop_marks > 1)
		strcat(pop_info, s_msgs);
	else
		strcat(pop_info, s_msg);
	
	ltoa(pop_marked, pop_info + strlen(pop_info), 10);
	strcat(pop_info, s_mrkd);
	}

if (pop_wind != -1)	
	wind_set_string(pop_wind, WF_INFO, pop_info);
}		

/* -------------------
   | Open POP server |
   ------------------- */
void pop_window(void)
{
DEBUG();
if (pop_wind > -1)
	{
	abookset_wind = -1;
	wind_set_int(pop_wind, WF_TOP, 0);
	pop_enable();
	}
else
	{
	if (!pop_count)
		{
		form_alert(1, nonews_msg);
		}
	else
		{
		GRECT wrec;

		menu_ienable(menu_adr, M_CHECK, FALSE);
				
		wind_get_grect(ROOT, WF_WORKXYWH, &wrec);
		pop_wind = wind_create(NAME|CLOSER|FULLER|MOVER|INFO|SIZER|
													 UPARROW|DNARROW|VSLIDE|LFARROW|RTARROW|
													 HSLIDE, &wrec);

		if (pop_wd[0] == -1)
			{
			pop_wd[0] =  wrec.g_x + wrec.g_w / 6;
			pop_wd[1] =  wrec.g_y + wrec.g_h / 6;
			pop_wd[2] =  wrec.g_w - wrec.g_w / 3;
			pop_wd[3] =  wrec.g_h - wrec.g_h / 3;
			}
	
		if (pop_wd[0] > wrec.g_x + wrec.g_w)
			pop_wd[0] =  wrec.g_x + wrec.g_w / 6;
	
		if (pop_wd[1] > wrec.g_y + wrec.g_h)
			pop_wd[1] =  wrec.g_y + wrec.g_h / 6;
			
		if (pop_wd[2] > wrec.g_w)
			pop_wd[2] = wrec.g_w;
			
		if (pop_wd[3] > wrec.g_h)
			pop_wd[3] = wrec.g_h;

		if (pop_wd[0] < 0)	pop_wd[0] = 0;
		if (pop_wd[1] < 0)	pop_wd[1] = 0;
			
		strcpy(pop_title, "Mail server: ");
		strcat(pop_title, pop_username);
		strcat(pop_title, " (");
		strcat(pop_title, cfg_options.popserver);
		strcat(pop_title, ")");	
		wind_set_string(pop_wind, WF_NAME, pop_title);

		set_popinfo();

		wind_open(pop_wind, (GRECT *)&pop_wd[0]);
		sort_popbox();
		register_window(pop_wind);
		pop_page = 0;
		pop_left = 0;
		set_popslider();
		pop_enable();
		}
	}
}

/* --------------------
   | Close POP window |
   -------------------- */
void close_popwindow(void)
{
DEBUG();
if (pop_wind == -1)
	return;
	
wind_get(pop_wind, WF_CURRXYWH, &pop_wd[0], &pop_wd[1], &pop_wd[2], &pop_wd[3]);

if (pop_count)
	free(pop_mem);

pop_count = 0;
pop_size = 0;
pop_total_size = 0L;
	
wind_close(pop_wind);
unregister_window(pop_wind);
wind_delete(pop_wind);
pop_wind = -1;

menu_ienable(menu_adr, M_CHECK, TRUE);
/*menu_ienable(menu_adr, M_RECEIV, FALSE);
menu_ienable(menu_adr, M_RCVDEL, FALSE);
menu_ienable(menu_adr, M_DELETE, FALSE);*/
}

/* ----------------------------------------
   | Redraw toolbar of mail editor window |
   ---------------------------------------- */
void redraw_popbar(void)
{
int wx, wy, ww, wh;
int x, w;

DEBUG();
wind_get(pop_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);

x = wx;
w = ww;
while (w > 0)
	{
	blt_copy(mw_toolbar, 0, 0 * iheight, NULL, x, wy, isize, iheight, S_ONLY);
	x += isize;
	w -= isize;
	}	

draw_popbutton(0, wx + pbut_pos[0], wy);			/* Mark for retrival	*/
draw_popbutton(1, wx + pbut_pos[1], wy);			/* Mark for deletion	*/
draw_popbutton(2, wx + pbut_pos[2], wy);			/* Mark for both		 	*/
draw_popbutton(3, wx + pbut_pos[3], wy);			/* Do it					 		*/
draw_popbutton(4, wx + pbut_pos[4], wy);			/* Close					 		*/
}

/* ---------------------
   | Redraw POP window |
   --------------------- */
void redraw_popwindow(int rx, int ry, int rw, int rh)
{
static int head_id[] = {FROMTEXT, TOTEXT, SUBJTEXT, SIZETEXT, DATETEXT, IDENTITYTEXT};
GRECT t1, t2;
int xy[4], xy2[4];
int wx, wy, ww, wh, x;
int t_xy[8];
int th, tw, hg, l, msg;
char t_buf[128];

char weekday[80];
int  day, j;
char month[20];
int  year;
int  hour,minute;
int col_pos[6], col_size[6], i;

DEBUG();
t2.g_x = rx;
t2.g_y = ry;
t2.g_w = rw;
t2.g_h = rh;

wind_update(BEG_UPDATE);
graf_mouse(M_OFF, 0L);

set_default_text();
vqt_extent(vdi_handle, "x", t_xy);
tw = (t_xy[4] - t_xy[0]);
th = (t_xy[5] - t_xy[1]);

vsl_color(vdi_handle, 1);
vsl_type(vdi_handle, 1);
vsl_width(vdi_handle, 1);

vst_color(vdi_handle, 1);

wind_get(pop_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);

hg = (wh + th - 1 - iheight) / th - 1;
x = wx - pop_left * tw;

calc_popcolumn_positions(col_pos, col_size);

wind_get(pop_wind, WF_FIRSTXYWH, &t1.g_x, &t1.g_y, &t1.g_w, &t1.g_h);
while(t1.g_w && t1.g_h)
	{
	if (rc_intersect(&t2, &t1))
		{
		int y;
		
		xy[0] = t1.g_x;
		xy[1] = t1.g_y;
		xy[2] = t1.g_x + t1.g_w - 1;
		xy[3] = t1.g_y + t1.g_h - 1;
		vs_clip(vdi_handle, 1, xy);

		vswr_mode(vdi_handle, MD_REPLACE);
		
		vsf_interior(vdi_handle, FIS_HOLLOW);
		vr_recfl(vdi_handle, xy);

		redraw_popbar();

		y = wy + iheight;
		vswr_mode(vdi_handle, MD_TRANS);		

		if (colours > 4)
			{
			vsf_color(vdi_handle, 8);
			vsf_interior(vdi_handle, FIS_SOLID);

			xy2[0] = wx;
			xy2[1] = y;
			xy2[2] = wx + ww - 1;
			xy2[3] = y + th - 1;
			vr_recfl(vdi_handle, xy2);
			}
				
			/* Headlines */
		for (i = 0; i < 6; i++)
			if (col_size[i])
				{
				strcpy(t_buf, header_tree[head_id[i]].ob_spec.free_string);
				t_buf[col_size[i]] = 0;
				v_gtext(vdi_handle, x + col_pos[i] * tw, y, t_buf);
				}

		vswr_mode(vdi_handle, MD_REPLACE);
		xy2[0] = wx;
		xy2[2] = wx + ww - 1;
		xy2[1] = xy2[3] = wy + iheight;
		v_pline(vdi_handle, 2, xy2);
		
		if (pop_sort)
			{
			int col = pop_sort, sgn;
			int xy[8];
			
			if (pop_sort < 0)
				{
				col = -col;
				sgn = 0;
				}
			else
				sgn = 1;
			
			col--;
			if (col_size[col])
				{
				xy[0] = x + (col_pos[col] + (int)strlen(header_tree[head_id[col]].ob_spec.free_string)) * tw;
				xy[2] = xy[0] + tw;
				xy[4] = xy[0] + tw / 2;
				xy[6] = xy[0];
				
				xy[1] = wy + th - 4 - sgn * th / 2 + iheight;
				xy[3] = xy[1];
				xy[5] = wy + th - 4 - th / 2 + sgn * th / 2 + iheight;
				xy[7] = xy[1];
				
				v_pline(vdi_handle, 4, xy);
				}
			}

		vswr_mode(vdi_handle, MD_TRANS);
		for (msg = pop_page, l = 0; l < hg && msg < pop_count; l++, msg++)
			{
			if (pop_mem[msg].sel & 1)	
				{
				xy2[0] = wx; xy2[2] = wx + ww - 1;
				xy2[1] = y + (l + 1) * th;
				xy2[3] = xy2[1] + th - 1;
				vsf_interior(vdi_handle, FIS_SOLID);
				vsf_color(vdi_handle, 1);
				vr_recfl(vdi_handle, xy2);
				
				vst_color(vdi_handle, 0);
				}

			if (pop_mem[msg].sel & 2)
				{
				icon_copy(symbols, 10, (smallfont ? 8 : 16) * 4, 
							 		x + 6, y + (l + 1) * th, 
							 		10, smallfont ? 8 : 16);
				}

			if (pop_mem[msg].sel & 4)
				{
				icon_copy(symbols, 10, (smallfont ? 8 : 16) * 5, 
							 		x + 14, y + (l + 1) * th, 
							 		10, smallfont ? 8 : 16);
				}
					
			if (col_size[0])
				{
				strncpy(t_buf, pop_mem[msg].from, col_size[0]);
				t_buf[col_size[0]] = 0;
				v_gtext(vdi_handle, x + col_pos[0] * tw,  y + (l + 1) * th, t_buf);
				}
				
			if (col_size[1])
				{
				strncpy(t_buf, pop_mem[msg].to, col_size[1]);
				t_buf[col_size[1]] = 0;
				v_gtext(vdi_handle, x + col_pos[1] * tw,  y + (l + 1) * th, t_buf);
				}				
				
			if (col_size[2])
				{
				strncpy(t_buf, pop_mem[msg].subject, col_size[2]);
				t_buf[col_size[2]] = 0;
				v_gtext(vdi_handle, x + col_pos[2] * tw,  y + (l + 1) * th, t_buf);
				}				

			if (col_size[3])
				{
				sprintf(t_buf, "%8ld", pop_mem[msg].size);
				t_buf[col_size[3]] = 0;
				v_gtext(vdi_handle, x + col_pos[3] * tw, y + (l + 1) * th, t_buf);
				}

			if (col_size[4])
				{			
				if (sscanf(pop_mem[msg].date, "%[^0-9]%d %[A-Za-z]%d %d:%d:", 
										weekday, &day, month, &year, &hour, &minute) != 6)
					{
					j = 5;
				  while(pop_mem[msg].date[j] && pop_mem[msg].date[j++] != ','); 	/* find start of date */
					sscanf(pop_mem[msg].date + j, "%[^0-9]%d %[A-Za-z]%d %d:%d:", 
								 weekday, &day, month, &year, &hour, &minute);				
					}
				else
				  if (year > 100) year %= 100;
	
				sprintf(t_buf, "%2d %03s %02d,%02d:%02d", day, month, year, hour, minute);
				t_buf[col_size[4]] = 0;
				v_gtext(vdi_handle, x + col_pos[4] * tw, y + (l + 1) * th, t_buf);
				}

			if (col_size[5])
				{
				strncpy(t_buf, pop_mem[msg].identity, col_size[5]);
				t_buf[col_size[5]] = 0;
				v_gtext(vdi_handle, x + col_pos[5] * tw, y + (l + 1) * th, t_buf);
				}
				
			if (pop_mem[msg].sel)	vst_color(vdi_handle, 1);
			}

		vswr_mode(vdi_handle, MD_REPLACE);
		
		xy2[0] = wx;
		xy2[2] = wx + ww - 1;
		xy2[1] = xy2[3] = wy + th - 1 + iheight;
		v_pline(vdi_handle, 2, xy2);

		xy2[1] = wy + iheight;
		xy2[3] = wy + wh - 1;

		for (i = 0; i < 6; i++)
			if (col_size[i])
				{
				xy2[0] = xy2[2] = x + (col_pos[i] + col_size[i] + 2) * tw + tw / 2;
				v_pline(vdi_handle, 2, xy2);
				}

		vs_clip(vdi_handle, 0, xy);
		}

	wind_get(pop_wind, WF_NEXTXYWH, &t1.g_x, &t1.g_y, &t1.g_w, &t1.g_h);
	}

graf_mouse(M_ON, 0L);
wind_update(END_UPDATE);
}

/* --------------------------
   | Message on mail window |
   -------------------------- */
void pop_window_message(int *msg)
{
int wx, wy, ww, wh;
int t_xy[8], tw, th;
int col_pos[6], col_size[6], hsize;

DEBUG();
set_default_text();
vqt_extent(vdi_handle, "x", t_xy);
tw = (t_xy[4] - t_xy[0]);
th = (t_xy[5] - t_xy[1]);

calc_popcolumn_positions(col_pos, col_size);
hsize = col_pos[5] + col_size[5]/* + 2*/;
		
wind_get(pop_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
switch(msg[0])
	{
	case WM_REDRAW:
		cursor_off();
		redraw_popwindow(msg[4], msg[5], msg[6], msg[7]);
		cursor_on();
		break;
		
	case WM_CLOSED:
		close_popwindow();
		enabler();
		break;
		
	case WM_ARROWED:
		{
		switch(msg[4])
			{
			case WA_UPPAGE:
				if (pop_page > 0) 
					{
					pop_page -= wh / th - 1;
					if (pop_page < 0) pop_page = 0;
					redraw_popwindow(wx, wy, ww, wh);
					set_popslider();
					}
				break;
				
			case WA_DNPAGE:
				if (pop_page + (wh / th - 1) < pop_count)
					{
					pop_page += wh / th - 1;
					if (pop_page + (wh / th - 1) >= pop_count) 
						pop_page = pop_count - wh / th + 1;

					redraw_popwindow(wx, wy, ww, wh);
					set_popslider();
					}
				break;
				
			case WA_UPLINE:
				if (pop_page > 0) 
					{
					pop_page--;
					redraw_popwindow(wx, wy, ww, wh);
					set_popslider();
					}
				break;
				
			case WA_DNLINE:
				if (pop_page + (wh / th - 1) < pop_count)
					{
					pop_page++;
					redraw_popwindow(wx, wy, ww, wh);
					set_popslider();
					}
				break;
				
			case WA_LFPAGE:
				if (pop_left > 0) 
					{
					pop_left -= ww / tw;
					if (pop_left < 0) pop_left = 0;
					redraw_popwindow(wx, wy, ww, wh);
					set_popslider();
					}
				break;
				
			case WA_RTPAGE:
				if (pop_left + (ww / tw) < hsize /*24 + 24 + 9 + 15*/)
					{
					pop_left += ww / tw;
					if (pop_left + (ww / tw) >= hsize /*24 + 24 + 9 + 15*/) 
						pop_left = hsize /*24 + 24 + 9 + 15*/ - ww / tw;

					redraw_popwindow(wx, wy, ww, wh);
					set_popslider();
					}
				break;
				
			case WA_LFLINE:
				if (pop_left > 0)
					{
					pop_left--;
					redraw_popwindow(wx, wy, ww, wh);
					set_popslider();
					}
				break;
				
			case WA_RTLINE:
				if (/*(24 + 24 + 9 + 15)*/ hsize - pop_left > ww / tw)
					{
					pop_left++;
					redraw_popwindow(wx, wy, ww, wh);
					set_popslider();
					}
				break;
			}
		break;
		}
		
	case WM_HSLID:
	  pop_left = (int)((long)msg[4] * (hsize /*24 + 24 + 9 + 15*/ - ww / tw) / 1000);
		redraw_popwindow(wx, wy, ww, wh);
		set_popslider();
		break;
		
	case WM_VSLID:
	  pop_page = (int)((long)msg[4] * (pop_count - wh / th + 1) / 1000);
		redraw_popwindow(wx, wy, ww, wh);
		set_popslider();
		break;
	}			
}

/* ------------------------------------
   | Get button number in mail window |
   ------------------------------------ */
int get_popwind_button(int mousex, int mousey)
{
int i;
int wx, wy, ww, wh;

DEBUG();
if (pop_wind == -1)
	return -1;
	
wind_get(pop_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);

if (mousey >= wy && mousey < wy + iheight)
	for (i = 0; i < 5; i++)
		if (mousex >= wx + pbut_pos[i] && mousex < wx + pbut_pos[i] + isize)
			return i;

return -1;
}

/* ---------------------------
   | Redraw all list buttons |
   --------------------------- */
void redraw_all_popbuttons(void)
{
int i;

DEBUG();
for (i = 0; i < 5; i++)
	redraw_popbutton(i);
}

/* ---------------------
   | Press mail button |
   --------------------- */
void press_popbutton(int n)
{
DEBUG();
pbut_selected[n] = TRUE;
redraw_popbutton(n);

wait_mouse_release();
	
pbut_selected[n] = FALSE;
redraw_popbutton(n);
}

/* ----------------------
   | Mark for retrieval |
   ---------------------- */
void mark_retrieval(void)
{
int j;
int wx, wy, ww, wh, th, t_xy[8];

DEBUG();
set_default_text();
vqt_extent(vdi_handle, "x", t_xy);
th = (t_xy[5] - t_xy[1]);

wind_get(pop_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
for (j = 0; j < pop_count; j++)
	if (pop_mem[j].sel & 1)
		{
		pop_mem[j].sel ^= 2 | 1;
		if (pop_mem[j].sel & 2)
			{
			pop_marked += pop_mem[j].size;
			pop_marks++;
			}
		else
			{
			pop_marked -= pop_mem[j].size;
			pop_marks--;
			}
		
		if (j >= pop_page && j < pop_page + (wh - th - iheight + th - 1) / th)
			redraw_popwindow(wx, wy + j * th + th + iheight, ww, th);
		set_popinfo();
		}
}

/* ---------------------
   | Mark for deletion |
   --------------------- */
void mark_deletion(void)
{
int j;
int wx, wy, ww, wh, th, t_xy[8];

set_default_text();
vqt_extent(vdi_handle, "x", t_xy);
th = (t_xy[5] - t_xy[1]);

wind_get(pop_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
for (j = 0; j < pop_count; j++)
	if (pop_mem[j].sel & 1)
		{
		pop_mem[j].sel ^= 4 + 1;
		
		if (j >= pop_page && j < pop_page + (wh - th - iheight + th - 1) / th)
			redraw_popwindow(wx, wy + j * th + th + iheight, ww, th);
		set_popinfo();
		}
}

/* -----------------
   | Mark for both |
   ----------------- */
void mark_both(void)
{
int j;
int wx, wy, ww, wh, th, t_xy[8];

DEBUG();
set_default_text();
vqt_extent(vdi_handle, "x", t_xy);
th = (t_xy[5] - t_xy[1]);

wind_get(pop_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
for (j = 0; j < pop_count; j++)
	if (pop_mem[j].sel & 1)
		{
		if (pop_mem[j].sel & 2)
			{
			pop_marked -= pop_mem[j].size;
			pop_marks--;
			}

		if ((pop_mem[j].sel & 6) == 6)
			pop_mem[j].sel = 0;
		else
			pop_mem[j].sel = 6;

		if (pop_mem[j].sel & 2)
			{
			pop_marked += pop_mem[j].size;
			pop_marks++;
			}
		
		if (j >= pop_page && j < pop_page + (wh - th - iheight + th - 1) / th)
			redraw_popwindow(wx, wy + j * th + th + iheight, ww, th);
		set_popinfo();
		}
}

/* ---------------------
   | Execute selection |
   --------------------- */
static void do_pop_action(int sk_pop, char *pop_ident)
{
int sk, rcv, i, j, stopped = FALSE;
char in_file[13], id[21];
IDENTITY identity;
char *popserver;
char *popuser, *poppass;
char zwuser[43], zwpass[43];

/*start_info(rcvdel_text, popconn_text);*/
DEBUG();
pop_loaded = 0;
pop_current = 0;
start_progressinfo(pop_marks, pop_marked, clock(), TRUE);

if (pop_ident)
	strcpy(id, pop_ident);
else
	id[0] = 0;
	
sk = sk_pop;
for (i = 0; i < pop_count && !stopped; i++)
	{
	pop_mem[i].sel &= ~128;
	if (pop_mem[i].sel & 6)
		{
		if (strcmp(id, pop_mem[i].identity))
			{
			if (sk)
				{
				disconnect_pop(sk);
				sk = 0;
				}
				
			if (!getIdentity(pop_mem[i].identity, &identity))
				continue;
			
			if (identity.use_providerdata)
				{
				popserver 	= cfg_options.popserver;
				popuser			= pop_username;
				poppass			= pop_password;
				}
			else
				{
				strcpy(zwuser, identity.popuser);
				strcpy(zwpass, identity.mailpass);
	
				popserver 	= identity.popserver;
				popuser			= zwuser;
				poppass			= zwpass;
				}
			
			change_info(popconn_text);
			sk = connect_pop(popserver, identity.pop_port, popuser, poppass, identity.pass_coded);
			if (!sk)
				show_error(FALSE);
			}
			
		rcv = 0;
		if (sk)
			{
			for (j = i; j < pop_count; j++)
				if ((pop_mem[j].sel & 6) != 0 && !strcmp(pop_mem[j].identity, pop_mem[i].identity))
					{
					if (pop_mem[j].sel & 2)
						{
						int ret;
						
						pop_current++;
						ret = retrieve_msg(sk, j, in_file);
						if (ret == 1)
							{
							if (store_incoming(in_file))
								{
								int in_box = box_number("IN");
								int wx, wy, ww, wh;
				
								rcv++;
								set_listinfo(box_number("IN"));
								if (box_manager[in_box].list_wind > -1) 
									{
									wind_get(box_manager[in_box].list_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
									redraw_listwindow(in_box, wx, wy, ww, wh);
									}
								pop_mem[j].sel &= ~2;				/* Don't do it again	*/
								}
							else
								{
								pop_mem[j].sel |= 128;
								continue;
								}
							}
						else
							{
							pop_mem[j].sel |= 128;
							if (ret == -1)
								{
								while (j < pop_count)
									{
									if (pop_mem[j].sel & 6)
										pop_mem[j].sel |= 128;
									j++;
									}
								stopped = TRUE;
								break;
								}
							continue;
							}
						}
						
					if ((pop_mem[j].sel & 4))
						if (remove_mail(sk, j))
							{
							remove_pop_entry(j);
/*							pop_mem[j].sel &= ~4;*/
							j--;
							}
						else
							pop_mem[j].sel |= 128;
					}					
			}
		}
	}

if (sk)
	disconnect_pop(sk);

for (i = 0; i < pop_count; i++)
	pop_mem[i].num = pop_mem[i].new_num;
	
end_info();
set_popinfo();
	
for (i = 0; i < pop_count; i++)
	if (pop_mem[i].sel & 128)
		break;
		
if (i == pop_count)
	close_popwindow();

if (rcv)
	list_window(box_number("IN"));
}
   
/* -----------------------
   | Mail button pressed |
   ----------------------- */
void pop_button_action(void)
{
int but;

DEBUG();
but = get_popwind_button(evnt.x, evnt.y);
if (pbut_disabled[but])
	return;

switch(but)
	{
	case 0:														/* Mark for retrieval */
		press_popbutton(0);
		mark_retrieval();
		break;
		
	case 1:														/* Mark for deletion */
		press_popbutton(1);
		mark_deletion();
		break;

	case 2:														/* Mark for both */
		press_popbutton(2);
		mark_both();
		break;
				
	case 3:														/* Do it */
		press_popbutton(3);
		do_pop_action(0, NULL);
		break;

	case 4:														/* Close */
		press_popbutton(4);
		close_popwindow();
		break;
	}
}

/* -------------------------------
   | List header selection popup |
   ------------------------------- */
static void pop_columnpopup(void)
{
OBJECT *poptree;
static int show_ids[] = {POPHEAD_FROM, POPHEAD_TO, POPHEAD_SUBJECT, POPHEAD_SIZE, 
												 POPHEAD_DATE, POPHEAD_ID};
int i, obj;

DEBUG();
rsrc_gaddr(R_TREE, POPHEAD_POPUP, &poptree);

for (i = 0; i < 6; i++)
	if (pop_cols[i] > 0)
		CheckObj(poptree, show_ids[i]);
	else
		UncheckObj(poptree, show_ids[i]);

obj = popup(poptree);
for (i = 0; i < 6; i++)
	if (obj == show_ids[i])
		{
		int wx, wy, ww, wh;
		
		if (pop_cols[i] > 0)
			pop_cols[i] = 0;
		else
			pop_cols[i] = defsize[i];

		wind_get(pop_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
		redraw_popwindow(wx, wy, ww, wh);
		set_popslider();
		break;
		}
}

static void pop_contentpopup(void)
{
OBJECT *poptree;
int j;

DEBUG();
for (j = 0; j < pop_count; j++)
	if (pop_mem[j].sel & 1)
		break;
		
rsrc_gaddr(R_TREE, M_POPUP, &poptree);
if (j == pop_count)
	{
	poptree[MP_RECV].ob_state 	|= DISABLED;
	poptree[MP_RCVDL].ob_state 	|= DISABLED;
	poptree[MP_DEL].ob_state 		|= DISABLED;
	}
else
	{
	poptree[MP_RECV].ob_state 	&= ~DISABLED;
	poptree[MP_RCVDL].ob_state 	&= ~DISABLED;
	poptree[MP_DEL].ob_state 		&= ~DISABLED;
	}
	
switch(popup(poptree))
	{
	case MP_SELAL:	
		select_all();				
		break;
	case MP_RECV:		
		mark_retrieval();			
		break;
	case MP_RCVDL:	
		mark_both();			
		break;
	case MP_DEL:		
		mark_deletion();			
		break;
	case MP_CLOSE:	
		close_popwindow(); 	
		break;
	}
}

/* ------------------------
   | Click on mail window |
   ------------------------ */
void pop_button(void)
{
int i, j;
int wx, wy, ww, wh;
int t_xy[8], th;

DEBUG();
wind_get(pop_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);

if (evnt.bstate & 2)
	{
	if (evnt.y > wy + iheight && evnt.y < wy + tx_height + iheight)
		pop_columnpopup();
	else
		pop_contentpopup();
	}
else
	{
	if (evnt.y >= wy && evnt.y < wy + iheight)
		{
		pop_button_action();
		return;
		}

	if (evnt.y > wy + iheight && evnt.y < wy + tx_height + iheight)
		{
		set_popsorting();
		return;
		}
		
	set_default_text();
	vqt_extent(vdi_handle, "x", t_xy);
	th = (t_xy[5] - t_xy[1]);

	if (evnt.x < wx || evnt.x > wx + ww - 1 ||
			evnt.y < wy + th || evnt.y > wy + wh - 1) 
		return;
	
	i = pop_page + (evnt.y - wy - th - iheight) / th;
	if (i >= 0 && i < pop_count)
		{
		if (!(evnt.kstate & 3))
			for (j = 0; j < pop_count; j++)
				if (pop_mem[j].sel & 1)
					{
					pop_mem[j].sel ^= 1;
					
					if (j >= pop_page && j < pop_page + (wh - th - iheight + th - 1) / th)
						redraw_popwindow(wx, wy + (j - pop_page) * th + th + iheight, ww, th);
					}

		pop_mem[i].sel ^= 1;
		redraw_popwindow(wx, wy + (i - pop_page) * th + th + iheight, ww, th);
		}
	else
		{
		for (j = 0; j < pop_count; j++)
			if (pop_mem[j].sel & 1)
				{
				pop_mem[j].sel ^= 1;
				
				if (j >= pop_page && j < pop_page + (wh - th - iheight + th - 1) / th)
					redraw_popwindow(wx, wy + j * th + th + iheight, ww, th);
				}
		}
		
	pop_enable();	
	set_popinfo();
	}
}

/* -------------------------
   | Disconnect POP server |
   ------------------------- */
void disconnect_pop(int s)
{
DEBUG();
send_cmd(s, "QUIT", NULL, NULL, NULL);
get_response(s, buf);
	
close_socket(s);

unlockIdentity();
blockPoll(FALSE);
}

static int inuse_cancelled;

/* ------------------------
   | Handle info dialogue |
   ------------------------ */
static void cdecl inuse_handler (OBJECT *tree, int obj)
{
(void)tree;

switch (obj & 0x7FFF)
	{
	case CANCEL_BG:
		{
		inuse_cancelled = 1;
		break;
		}
	
	case CANCEL_OP:
		{
		inuse_cancelled = 2;
		break;
		}
	}
}

/* ----------------------
   | Connect pop server |
   ---------------------- */
int connect_pop(char *server, int port, char *user, char *pass, int coded)
{
struct servent *serv;
struct hostent *host;
struct sockaddr_in remote;
int s, stop_scan, stop_trans;
int login, ret;
char pass_id[32];

DEBUG();
lockIdentity(server, user);
blockPoll(TRUE);

if (isIdentityUsed(server, user))
	{
	OBJECT *inuse;
	
	inuse_cancelled = 0;
	rsrc_gaddr(R_TREE, POLL_IN_PROGRESS, &inuse);
	open_dialog(inuse, NULL, iconified_tree);
	dlgset_help_info(inuse, helpfile, inuse_ids, "INUS_");
	redraw_dialog(inuse);
	set_active(inuse, inuse_handler, TRUE, 0);

	while (isIdentityUsed(server, user) && !inuse_cancelled)
		{
		check_user_message();
		doPoll();
		}
		
	if (inuse_cancelled == 2)
		{
		return FALSE;
		}
	else
		if (inuse_cancelled == 1)
			stopPoll();
		
	close_dialog(inuse, TRUE);
	}

remote.sin_family = AF_INET;

if (port == -1)
	{
	serv = getservbyname("pop-3", "tcp");
	if (!serv)
		serv = getservbyname("pop3", "tcp");
		
	if (!serv)
		{
		form_alert(1, pservice_msg);
		return FALSE;
		}
	remote.sin_port = serv->s_port;
	}
else
	remote.sin_port = port;
	
if (isdigit(server[0]))
	remote.sin_addr.s_addr = inet_addr(server);
else
	{
	int	state = async_gethostbyname(server);
	while (state)
		{
		long stime = clock();
		if (!info_stop())
			{
			if (clock() - stime >= CLK_TCK)
				state = async_gethostbyname(server);
			}
		else
			{
			return FALSE;
			}
		}
				
	do
		{
		host = async_queryname();
		if (info_stop())
			{
			async_discard();
			return FALSE;
			}
					
		}while((long)host == -EINPROGRESS);
	async_discard();

/*	host = gethostbyname(server);*/
	if (!host)
		{
		form_alert(1, unknownserver);
		return FALSE;
		}
	memcpy(&remote.sin_addr.s_addr, host->h_addr, host->h_length);
	}

s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
if (!s)
	{
	form_alert(1, sockfail_msg);
	return FALSE;
	}
	
stop_scan = FALSE;
stop_trans = FALSE;

clear_inbuffer();

if (get_tcpversion() >= 0x00010007L)
	sockfcntl(s, SO_SETFL, SO_O_NONBLOCK);

ret = connect(s, (struct sockaddr *)&remote, (int)sizeof(remote));
while(ret != 0 && (errno == -EINPROGRESS || errno == -EALREADY))
	{
	if (info_stop())
		{
		close_socket(s);
		return 0;
		}
	ret = connect(s, NULL, 0);
	}
if (ret && errno == -EISCONN)
	ret = 0;
	
if (get_tcpversion() >= 0x00010007L)
	sockfcntl(s, SO_SETFL, 0);

if (!ret)
	{
	get_response(s, buf);
	if (buf[0] == '-')	
		{
		store_error("POP", 1, buf);
		stop_scan = TRUE; 
		}
	else 
		stop_trans = TRUE;
	
	login = FALSE;
	
	if (coded && !stop_scan)
		{
		change_info(encrypt_text);	
		if (strchr(buf, '>'))
			{
			strcpy(strchr(buf, '>') + 1, pass);
			
			if (strchr(buf, '<'))
				{
				MDString(strchr(buf, '<'), pass_id);
				send_cmd(s, "APOP", user, " ", pass_id);
				get_response(s, buf);
				if (buf[0] == '-')	
					{
					store_error("POP", 2, buf);
					stop_scan = TRUE; 
					}
				else 
					login = TRUE;
				}
			}
		}
	
	if (!login)	
		{
		if (stop_scan)
			{
			close_socket(s);
			stop_scan = FALSE;
			
			s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
			if (!s)
				{
				form_alert(1, sockfail_msg);
				return FALSE;
				}
	
			if (get_tcpversion() >= 0x00010007L)
				sockfcntl(s, SO_SETFL, SO_O_NONBLOCK);

			clear_inbuffer();
			
			ret = connect(s, (struct sockaddr *)&remote, (int)sizeof(remote));
			while(ret != 0 && (errno == -EINPROGRESS || errno == -EALREADY))
				{
				if (info_stop())
					{
					close_socket(s);
					return 0;
					}
				ret = connect(s, NULL, 0);
				}
			if (ret && errno == -EISCONN)
				ret = 0;
			
			if (get_tcpversion() >= 0x00010007L)
				sockfcntl(s, SO_SETFL, 0);

			get_response(s, buf);
			if (buf[0] == '-')	
				{
				store_error("POP", 1, buf);
				stop_scan = TRUE; 
				}
			else 
				stop_trans = TRUE;
			}
		
		if (!stop_scan)
			{
			change_info(unsecure_text);	
	
			send_cmd(s, "USER", user, NULL, NULL);
			get_response(s, buf);
			if (buf[0] == '-')	
				{
				store_error("POP", 3, "\0");
				stop_scan = TRUE;
				}
				
			if (!stop_scan) 
				{
				send_cmd(s, "PASS", pass, NULL, NULL);
				get_response(s, buf);
				if (buf[0] == '-')	
					{
					store_error("POP", 3, "\0");
					stop_scan = TRUE;
					}
				}
			}
		}
	}
else
	{
	stop_scan = TRUE;
	form_alert(1, connfail_msg);
	}
	
if (stop_scan)
	{
	if (stop_trans)
		disconnect_pop(s);
	else
		close_socket(s);
	return 0;
	}
return s;
}	

/* --------------------
   | POP server login |
   -------------------- */
int pop_login(char *user, char *pass)
{
OBJECT *logtree;
int e;

DEBUG();
rsrc_gaddr(R_TREE, D_AUTH, &logtree);

strcpy(logtree[AU_USER].ob_spec.tedinfo->te_ptext, user);
strcpy(logtree[AU_PASS].ob_spec.tedinfo->te_ptext, pass);

open_dialog(logtree, NULL, iconified_tree);
e = do_dialog(logtree, user[0] ? AU_USER : AU_PASS, app_callback);
close_dialog(logtree, TRUE);

if (e == AU_OKAY)
	{
	strcpy(user, logtree[AU_USER].ob_spec.tedinfo->te_ptext);
	strcpy(pass, logtree[AU_PASS].ob_spec.tedinfo->te_ptext);
	return TRUE;
	}
else
	return FALSE;
}

/* -------------------------
   | Clear pop information |
   ------------------------- */
void clear_pop(void)
{
DEBUG();
if (pop_count)
	free(pop_mem);

pop_marked = 0;
pop_marks = 0;
pop_count = 0;
pop_size = 0;
pop_total_size = 0L;
}

/* -------------------
   | Scan pop server |
   ------------------- */
int scan_pop(char *server, int port, char *user, char *pass, int coded, int receive, int del_rcvd, long maxsize,
						 char *identity)
{
int s, i;
int stop_scan, pop_start;
char *eptr, *cptr, info_buf[40], pcnt_num[6];

DEBUG();
if (!pass[0])
	if (!pop_login(user, pass))
		return -2;

start_info(mailchk_text, popconn_text);	
s = connect_pop(server, port, user, pass, coded);

if (s)
	{
	stop_scan = FALSE;
	change_info(getminf_text);	
	send_cmd(s, "STAT", NULL, NULL, NULL);
	get_response(s, buf);
	if (buf[0] == '-')	
		{
		store_error("POP", 4, buf);
		stop_scan = TRUE;
		}

	pop_start = pop_count;
	if (!stop_scan && !info_stop())
		{
		send_cmd(s, "LIST", NULL, NULL, NULL);
		get_response(s, buf);
		if (buf[0] == '-' || info_stop())	
			{
			if (buf[0] == '-')
				store_error("POP", 5, buf);

			stop_scan = TRUE;
			}
		else
			{
			do
				{
				get_response(s, buf);
				
				if (!strcmp(buf, ".\r\n"))
					break;
				else
					{
					if (!pop_size)
						{
						pop_mem = (POP_MEM *)malloc(30 * sizeof(POP_MEM));
						pop_size = 30;
						}
					else
						if (pop_count == pop_size)
							{
							pop_size += 30;
							pop_mem = (POP_MEM *)realloc(pop_mem, pop_size * sizeof(POP_MEM));
							}

					strcpy(pop_mem[pop_count].from, "???");
					strcpy(pop_mem[pop_count].subject, "???");
					strcpy(pop_mem[pop_count].date, "???");
					strcpy(pop_mem[pop_count].to, "???");

					pop_mem[pop_count].sel 	= 0;		
					pop_mem[pop_count].new_num = 
					pop_mem[pop_count].num 	= (int)strtol(buf, &eptr, 10);
					pop_mem[pop_count].size = 		 strtol(eptr, &eptr, 10);
					pop_total_size += pop_mem[pop_count].size;
					strncpy(pop_mem[pop_count].subject, eptr, 50);
					pop_mem[pop_count].subject[50] = 0;
					strcpy(pop_mem[pop_count].identity, identity);
					pop_count++;
					}
				}while(!info_stop());
			}
		}

	itoa(pop_count - pop_start, pcnt_num, 10);
	for (i = pop_start; i < pop_count && !info_stop(); i++)
		{
		char num[6];

		pop_mem[i].sel = 0;
		itoa(pop_mem[i].num, num, 10);

		strcpy(info_buf, s_scanhd);
		strcat(info_buf, num);
		strcat(info_buf, "/");
		strcat(info_buf, pcnt_num);
		strcat(info_buf, ")");
		
		change_info(info_buf);	
		
		send_cmd(s, "TOP", num, " 1", NULL);
		
		get_response(s, buf);
		if (buf[0] == '+')
			{
			do
				{
				get_response(s, buf);

				cptr = buf;
				eptr = NULL;
				if (!strnicmp(buf, "From:", 5))
					{
					eptr = pop_mem[i].from;
					cptr += 5;
					}
				else
					if (!strnicmp(buf, "Subject:", 8))
						{
						eptr = pop_mem[i].subject;
						cptr += 8;
						}
					else
						if (!strnicmp(buf, "Date:", 5))
							{
							eptr = pop_mem[i].date;
							cptr += 5;
							}
						else
							if (!strnicmp(buf, "To:", 3))
								{
								eptr = pop_mem[i].to;
								cptr += 3;
								}

				if (eptr)
					{
					while (*cptr == ' ' || *cptr == '\t') cptr++;
					
					if  (eptr != pop_mem[i].date)
						{
						strncpy(eptr, cptr, 60);
						eptr[60] = 0;
						}
					else
						{
						strncpy(eptr, cptr, 50);
						eptr[50] = 0;
						}
						
					if (strchr(eptr, '\r'))	*strchr(eptr, '\r') = 0;
					if (strchr(eptr, '\n'))	*strchr(eptr, '\n') = 0;
					}
					
				}while(strcmp(buf, ".\r\n"));
			}
		else
			if (buf[0] == '-')
				store_error("POP", 6, buf);
		}

/*	pop_count = i;*/

	if (receive)
		{	
		for (i = pop_start; i < pop_count; i++)
			if (pop_mem[i].size <= maxsize || !maxsize)
				{
				if (del_rcvd)
					pop_mem[i].sel |= 6;
				else
					pop_mem[i].sel |= 2;
				pop_marked += pop_mem[i].size;
				pop_marks++;
				}
		
		end_info();
		if (pop_marks > 0)
			do_pop_action(s, identity);
		else
			disconnect_pop(s);
				
		if (!pop_count)
			return -2;
		}
	else
		disconnect_pop(s);
	}
else
	{
	store_error("POP", 1, buf);
	show_error(FALSE);
	stop_scan = TRUE;
	}
	
end_info();

return (stop_scan == FALSE);
}

/* ---------------------------
   | Select all pop messages |
   --------------------------- */
void select_all(void)
{
int i;
int wx, wy, ww, wh;

DEBUG();
for (i = 0; i < pop_count; i++)
	pop_mem[i].sel ^= 1;

pop_marks = pop_count;
pop_marked = pop_total_size;
	
wind_get(pop_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
redraw_popwindow(wx, wy, ww, wh);

set_popinfo();
pop_enable();
}

/* --------------------
   | Retrieve message |
   -------------------- */
int retrieve_msg(int sk, int msg, char *file)
{
char num[7];
int f, bl;
long pos, len;
FILE_MAIL_MEM in_buf;

DEBUG();
f = create_outgoing(file);
if (f)
	{
	memset(&in_buf, 0, sizeof(in_buf));
	
	in_buf.magic 		= 0x4104;
	in_buf.version 	= SVERSION_ID20;
	in_buf.body_pos = sizeof(in_buf);
	in_buf.part_count = -1;
	strcpy(in_buf.identity, pop_mem[msg].identity);
	Fwrite(f, sizeof(in_buf), &in_buf);
	
	pos = 0L;	
	len = pop_mem[msg].size;
	itoa(pop_mem[msg].num, num, 10);
	send_cmd(sk, "RETR", num, NULL, NULL);
	get_response(sk, buf);
	if (buf[0] == '+')
		{
		int flg = TRUE;
		do
			{
			change_progressinfo(pop_current, pos, len, pop_loaded + pos);

			get_response(sk, buf);
			if (!strcmp(buf, ".\r\n"))
				break;
			
			bl = (int)strlen(buf);
			pos += bl;
			if (bl && (buf[bl - 1] == '\r' || buf[bl - 1] == '\n'))
				{
				pos--;
				if (bl >= 2 && (buf[bl - 2] == '\r' || buf[bl - 2] == '\n') &&
						buf[bl - 2] == buf[bl - 1])
					pos--;
				}
			Fwrite(f, bl, buf);
			in_buf.body_len += bl;
			
			if (info_stop())
				flg = FALSE;
			}while(flg);
		
		Fseek(0, f, 0);
		Fwrite(f, sizeof(in_buf), &in_buf);
		
		Fclose(f);
		
		if (flg) pop_loaded += len;
		return !flg ? -1 : 1;
		} 
	else
		{
		store_error("POP", 7, buf);
		Fclose(f);
		return 0;
		}
	}
else
	return 0;
}

/* ---------------------------
   | Remove mail from server |
   --------------------------- */
int remove_mail(int sk, int msg)
{
char num[7];

DEBUG();
itoa(pop_mem[msg].num, num, 10);
send_cmd(sk, "DELE", num, NULL, NULL);

get_response(sk, buf);
/*
Cconws(buf);
Cconws("\r\n");
*/
if (buf[0] == '+')
	return TRUE;
else
	{
	store_error("POP", 8, buf);
	return FALSE;
	}
}

/* ----------------------------
   | Remove pop message entry |
   ---------------------------- */
void remove_pop_entry(int msg)
{
int j;
int wx, wy, ww, wh;

DEBUG();
for (j = 0; j < pop_count; j++)
	if (pop_mem[j].num > pop_mem[msg].num && !strcmp(pop_mem[j].identity, pop_mem[msg].identity))
		pop_mem[j].new_num--;

if (pop_mem[msg].sel & 2)
	{
	pop_marked -= pop_mem[msg].size;
	pop_marks--;
	}
					
if (msg < pop_count - 1)
	memmove(pop_mem + msg, pop_mem + msg + 1, sizeof(POP_MEM) * (pop_count - msg - 1));

pop_count--;

if (pop_count)
	{
	if (pop_wind != -1)
		{
		wind_get(pop_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
		redraw_popwindow(wx, wy, ww, wh);
		}
	}
else
	free(pop_mem);
}

/* -------------------------
   | Set sort order/column |
   ------------------------- */
void set_popsorting(void)
{
int wx, wy, ww, wh;
int col, t_xy[8], tw;
int col_pos[6], col_size[6], i;

DEBUG();
set_default_text();
vqt_extent(vdi_handle, "x", t_xy);
tw = (t_xy[4] - t_xy[0]);

wind_get(pop_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
col = (evnt.x - wx) / tw + pop_left;

calc_popcolumn_positions(col_pos, col_size);
for (i = 0; i < 6; i++)
	if (col_size[i])
		if (col == col_pos[i] + col_size[i] + 2)
			{
			resize_popcolumn(i);
			return;
			}
		else
			if (col >= col_pos[i] && col <= col_pos[i] + col_size[i])
				if (pop_sort != i + 1)
					pop_sort	= i + 1;
				else
					pop_sort	= -(i + 1);
	
sort_popbox();
redraw_popwindow(wx, wy, ww, wh);
}

/* -----------------
   | Resize column |
   ----------------- */
void resize_popcolumn(int col)
{
static int maxsizes[] = {63, 63, 63, 11, 20, 22};
int col_pos[6], col_size[6];
EVNTDATA gev;
int xy[4];
int wx, wy, ww, wh, changed, old_size;
unsigned pattern, opattern;
long last;

DEBUG();
calc_popcolumn_positions(col_pos, col_size);

wind_get(pop_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);

vswr_mode(vdi_handle, MD_XOR);

xy[1] = wy + iheight;
xy[3] = wy + wh - 1;

xy[0] = xy[2] = wx - pop_left * tx_width + (col_pos[col] + col_size[col] + 2) * tx_width + tx_width / 2;
old_size = col_size[col];

wind_update(BEG_MCTRL);
graf_mouse(M_OFF, 0L);

vsl_color(vdi_handle, 1);
vsl_type(vdi_handle, 7);
vsl_width(vdi_handle, 1);
pattern = 0xF0F0;
vsl_udsty(vdi_handle, pattern);

v_pline(vdi_handle, 2, xy);

last = clock();
changed = FALSE;

do
	{
	evnt_multi(MU_BUTTON|MU_TIMER,
						 1, 1, 1,
						 0, &empty,
						 0, &empty,
						 NULL,
						 50L,
						 &gev,
						 &k_return, &clicks);
					 
	graf_mkstate(&gev);

	if (gev.x < xy[2] - tx_width / 2)
		{
		if (col_size[col] > 1)
			{
			col_size[col]--;
			changed = TRUE;
			}
		}
	else
		if (gev.x > xy[2] + tx_width / 2)
			{
			if (col_size[col] < maxsizes[col])
				{
				col_size[col]++;
				changed = TRUE;
				}
			}
		
	if (clock() - last > CLK_TCK / 16 || changed)
		{	
		opattern = pattern;
		pattern = (pattern << 1) | (pattern >> 15);

		if (!changed)
			{
			vsl_udsty(vdi_handle, pattern);
			v_pline(vdi_handle, 2, xy);
			}

		vsl_udsty(vdi_handle, opattern);
		v_pline(vdi_handle, 2, xy);
		
		if (changed)
			{
			xy[0] = xy[2] = wx - pop_left * tx_width + (col_pos[col] + col_size[col] + 2) * tx_width + tx_width / 2;
			vsl_udsty(vdi_handle, pattern);
			v_pline(vdi_handle, 2, xy);
			changed = FALSE;
			}
			
		last = clock();
		}
	
	}while(gev.bstate & 1);

vsl_udsty(vdi_handle, pattern);
v_pline(vdi_handle, 2, xy);

vswr_mode(vdi_handle, MD_REPLACE);

graf_mouse(M_ON, 0L);
wind_update(END_MCTRL);

if (old_size != col_size[col])
	{
	pop_cols[col] = col_size[col];
	redraw_popwindow(wx, wy, ww, wh);
	set_popslider();
	}	
}

/* ------------------------
   | Compare 'from' field |
   ------------------------ */
static int compare_from(void *a, void *b)
{
if (sort_order == -1)
	return -stricmp(((POP_MEM *)a)->from, ((POP_MEM *)b)->from);
else	
	return stricmp(((POP_MEM *)a)->from, ((POP_MEM *)b)->from);
}

/* ----------------------
   | Compare 'to' field |
   ----------------------- */
static int compare_to(void *a, void *b)
{
if (sort_order == -1)
	return -stricmp(((POP_MEM *)a)->to, ((POP_MEM *)b)->to);
else	
	return stricmp(((POP_MEM *)a)->to, ((POP_MEM *)b)->to);
}

/* ---------------------------
   | Compare 'subject' field |
   --------------------------- */
static int compare_subject(void *a, void *b)
{
if (sort_order == -1)
	return -stricmp(((POP_MEM *)a)->subject, ((POP_MEM *)b)->subject);
else	
	return stricmp(((POP_MEM *)a)->subject, ((POP_MEM *)b)->subject);
}

/* ------------------------
   | Compare 'size' field |
   ------------------------ */
static int compare_size(void *a, void *b)
{
long a_len, b_len;

a_len = ((POP_MEM *)a)->size;
b_len = ((POP_MEM *)b)->size;

if (a_len < b_len)
	return -sort_order;
else
	if (a_len > b_len)
		return sort_order;
	else
		return 0;
}

/* ------------------------
   | Compare 'date' field |
   ------------------------ */
static int compare_date(void *a, void *b)
{
char *a_date, *b_date;
int a_day, a_month, a_year;
int b_day, b_month, b_year;
int a_hour, a_minute, a_second;
int b_hour, b_minute, b_second;

a_date = ((POP_MEM *)a)->date;
b_date = ((POP_MEM *)b)->date;

get_datevalues(a_date, &a_day, &a_month, &a_year, &a_hour, &a_minute, &a_second);
get_datevalues(b_date, &b_day, &b_month, &b_year, &b_hour, &b_minute, &b_second);

if (a_year < b_year)
	return -sort_order;
else
	if (a_year > b_year)
		return sort_order;

if (a_month < b_month)
	return -sort_order;
else
	if (a_month > b_month)
		return sort_order;

if (a_day < b_day)
	return -sort_order;
else
	if (a_day > b_day)
		return sort_order;
		
if (a_hour < b_hour)
	return -sort_order;
else
	if (a_hour > b_hour)
		return sort_order;
		
if (a_minute < b_minute)
	return -sort_order;
else
	if (a_minute > b_minute)
		return sort_order;
		
if (a_second < b_second)
	return -sort_order;
else
	if (a_second > b_second)
		return sort_order;
		
return 0;		
}

/* ----------------------------
   | Compare 'identity' field |
   ---------------------------- */
static int compare_identity(void *a, void *b)
{
if (sort_order == -1)
	return -stricmp(((POP_MEM *)a)->identity, ((POP_MEM *)b)->identity);
else	
	return stricmp(((POP_MEM *)a)->identity, ((POP_MEM *)b)->identity);
}

/* --------------
   | Sort a box |
   -------------- */
void sort_popbox(void)
{
static int (*compar[])(void *a, void *b) = 
		{compare_from, compare_to, compare_subject, compare_size,
		 compare_date, compare_identity};

DEBUG();
if (!pop_sort) return;

if (pop_sort < 0)
	sort_order = -1;
else
	sort_order = 1;

qsort(pop_mem, pop_count, sizeof(POP_MEM),
			compar[pop_sort * sort_order - 1]);
}
/*
void start_progressinfo(int number, long bytes, clock_t starttime, int download)
void change_progressinfo(int numproc, long currbytes, long currsize, long currtotal)
*/