/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#ifdef PRINTING
#include <PORTAB.H>
#include	<TOS.H>
#include	<VDI.H>
#include	<VDICOL.H>														/* neue VDI-Funktionen */
#include	<MT_AES.H>

#include "extern.h"

#ifndef EVAL

int	prn_work_out[57];
int	prn_handle;
PRN_DIALOG	*prn_dialog;

int InitPrint(char *mod_settings);
void ExitPrint(void);
int GetPageWidth(void);
int GetPageHeight(void);
void StartPage(void);
void EndPage(void);
void GdosFooter(int page, int page_width, int page_height, int prn_tx_height);

/* ----------------------------
   | Write a printer sequence |
   ---------------------------- */
void writeSequence(long pfh, char *seq)
{
char *s, sbuf[51];
int c;

s = seq;
c = 0;
while(*s)
	{
	if (!strnicmp(s, "{Esc}", 5))
		{
		s += 5;
		sbuf[c++] = 27;
		}
	else
		if (*s == '\\')
			if (isdigit(s[1]) && isdigit(s[2]) && isdigit(s[3]))
				{
				sbuf[c++] = (s[1] - '0') * 100 +
										(s[2] - '0') * 10 +
										(s[3] - '0');
				s += 4;
				}
			else
				{
				sbuf[c++] = s[1];
				s += 2;
				}
		else
			sbuf[c++] = *(s++);
	}
if (c)
	Fwrite((int)pfh, c, sbuf);
}

/* ---------------------
   | Print left border |
   --------------------- */
void print_leftborder(long pfh)
{
if (mail_options.left_margin)
	{
	int i;
	
	i = mail_options.left_margin;
	while (i-- > 0)
		Fwrite((int)pfh, 1, " ");
	}
}

/* ---------------------
   | Print page footer |
   --------------------- */
void print_footer(long pfh, int page)
{
char  footer[256], *tmstr, num[9], *s;
time_t mtime;
int i, pwidth;

i = mail_options.bottom_margin;
while (i-- > 0)
	Fwrite((int)pfh, 2, "\r\n");

pwidth = mail_options.pagewidth - (mail_options.left_margin > 0 ? mail_options.left_margin : 0);

memset(footer, '-', mail_options.pagewidth);
print_leftborder(pfh);
Fwrite((int)pfh, pwidth, footer);
Fwrite((int)pfh, 2, "\r\n");

memset(footer, ' ', mail_options.pagewidth);

mtime = time(NULL);
tmstr = ctime(&mtime);

s = strchr(tmstr, '\n');
if (s) *s = 0;

if (strlen(tmstr) < mail_options.pagewidth)
	memcpy(footer, tmstr, strlen(tmstr));

itoa(page, num, 10);
memcpy(footer + pwidth - strlen(s_page) - strlen(num) - 1, s_page, strlen(s_page));
memcpy(footer + pwidth - strlen(num), num, strlen(num));

print_leftborder(pfh);
Fwrite((int)pfh, pwidth, footer);
Fwrite((int)pfh, 2, "\r\n");
	
if (mail_options.print_ff)
	Fwrite((int)pfh, 1, "\014");
}

/*
							int	 print_header;
*/

/* -----------------
   | Print message |
   ----------------- */
void do_print(int box, MAIL_INFO *entry, int part_no)
{
char 	*print_mem, *x;
long 	print_blen, *print_ptr, print_fh;
int 	print_width;
int 	print_lines, i, len, page;
int		old_width, lcount, bold, owrap;
int		ansi;

DEBUG();
if (mail_options.reformat)
	{
	old_width = mail_options.viewlength;
	owrap = mail_options.viewwrap;
	mail_options.viewlength = mail_options.pagewidth;
	mail_options.viewwrap = mail_options.printwrap;
	}

if (part_no == -1)
	{
	if (!get_body("Print mail",
							boxes[box].file, entry, 
							&print_mem, &print_blen, &print_ptr,
					 		&print_width, &print_lines, TRUE))
		return;
	}
else
	{
	if (!get_part("Print mail",
							 boxes[box].file, entry, 
							 &print_mem, &print_blen, 
							 &print_ptr, &print_width, 
							 &print_lines, &ansi, TRUE, part_no))
		return;
	}

if (mail_options.reformat)
	{
	mail_options.viewlength = old_width;
	mail_options.viewwrap = owrap;
	}
	
if (!mail_options.printmode)
	{
	print_fh = Fopen("U:\\DEV\\PRN", FO_RW);
	if (print_fh == -33)
		print_fh = Fdup(3);
		
	if (print_fh > 0)
		{
		bold = FALSE;
		writeSequence(print_fh, mail_options.printinit);
		lcount = 0;
		page = 1;
		
		i = 0;
		while(i < print_lines)
			{
			x = print_ptr[i] + print_mem;
			if (i < print_lines - 1)
				len = (int)(print_ptr[i + 1] - print_ptr[i]);
			else
				len = (int)(print_blen - print_ptr[i]);

			if (*x == 3)
				x++;
			else
				if (*x == 1) 
					{
					if (!bold)
						{
						bold = TRUE;
						writeSequence(print_fh, mail_options.boldon);
						}
						
					x++;
					len--;
					}
				else
					if (bold)
						{
						bold = FALSE;
						writeSequence(print_fh, mail_options.boldoff);
						}

			if (len && (x[len - 1] == '\r' || x[len - 1] == '\n'))
				{
				len--;
				if (len && (x[len - 1] == '\r' || x[len - 1] == '\n') &&
						x[len - 1] != x[len])
					len--;
				}

			if (lcount == 0)
				{
				int i;

				i = mail_options.top_margin;
				while (i-- > 0)
					{
					Fwrite((int)print_fh, 2, "\r\n");
					lcount++;
					}
				}
			
			print_leftborder(print_fh);
					
			Fwrite((int)print_fh, len, x);
			Fwrite((int)print_fh, 2, "\r\n");
			lcount++;
			
			if (lcount == mail_options.pageheight - mail_options.bottom_margin - (mail_options.print_footer ? 2 : 0))
				{
				int i;

				i = mail_options.bottom_margin;
				while (i-- > 0)
					{
					Fwrite((int)print_fh, 2, "\r\n");
					lcount++;
					}
					
				if (mail_options.print_footer)
					print_footer(print_fh, page++);
				lcount = 0;
				}
				
			i++;
			}

		if (mail_options.print_attaches)
			{
			MAIL_MEM buffer;
			FILE_MAIL_MEM header;
			int f;

			memset(&buffer, 0, sizeof(buffer));

			f = open_file(entry->file, boxes[box].file);
			if (f)
				{
				restore_header(f, &header, &buffer);

				if (header.part_count > 1)
					{
					int cnt;
		
					edit_parts = (PART_INFO *)malloc(sizeof(PART_INFO) * header.part_count);
					Fseek(header.partinfo_pos, f, 0);
					Fread(f, sizeof(PART_INFO) * header.part_count, edit_parts);

					for (cnt = 0; cnt < header.part_count; cnt++)
						{
						if (cnt != part_no && (part_no != -1 || cnt != header.main_idx))
							{
							int l, c;
							
							if (edit_parts[cnt].desc[0])
								{
								c = (int)strlen(edit_parts[cnt].name) + 2 + (int)strlen(edit_parts[cnt].desc) + 
										(mail_options.left_margin > 0 ? mail_options.left_margin : 0);
								if (c > mail_options.pagewidth)
									l = 2;
								else
									l = 1;
								}
							else
								l =  1;
							
							if (lcount + l > mail_options.pageheight - mail_options.bottom_margin - (mail_options.print_footer ? 2 : 0))
								{
								int i;
				
								i = mail_options.bottom_margin;
								while (i-- > 0)
									{
									Fwrite((int)print_fh, 2, "\r\n");
									lcount++;
									}
									
								if (mail_options.print_footer)
									print_footer(print_fh, page++);
								lcount = 0;
								}

							if (mail_options.left_margin > 0)
								{
								print_leftborder(print_fh);
								c = mail_options.left_margin;
								}
							else
								c = 0;

							l = strlen(edit_parts[cnt].name);
							Fwrite((int)print_fh, l, edit_parts[cnt].name);
							
							c += l;
							if (edit_parts[cnt].desc[0] != 0)
								{
								l = strlen(edit_parts[cnt].desc);
								
								if (c + 2 + l > mail_options.pagewidth)
									{
									Fwrite((int)print_fh, 2, "\r\n");
									print_leftborder(print_fh);
									lcount++;
									}
								else
									Fwrite((int)print_fh, 2, "  ");
									
								Fwrite((int)print_fh, l, edit_parts[cnt].desc);
								}
							Fwrite((int)print_fh, 2, "\r\n");
							lcount++;
							}
						}
						
					free(edit_parts);
					}
				clear_buffer(&buffer);
				Fclose((int)f);
				}
			}
			
		if (lcount)
			{
			int i;

			while (lcount < mail_options.pageheight - mail_options.bottom_margin - (mail_options.print_footer ? 2 : 0))
				{
				Fwrite((int)print_fh, 2, "\r\n");
				lcount++;
				}
				
			i = mail_options.bottom_margin;
			while (i-- > 0)
				{
				Fwrite((int)print_fh, 2, "\r\n");
				lcount++;
				}
				
			if (mail_options.print_footer)
				print_footer(print_fh, page);
			}
			
		writeSequence(print_fh, mail_options.printexit);
		Fclose((int)print_fh);
		}
	}
else
	gdos_print(print_mem, print_ptr, print_lines, box, entry, part_no);

if (print_mem) free(print_mem);
if (print_ptr) free(print_ptr);
		
free(print_ptr);
}

/* --------------------
   | Print using GDOS |
   -------------------- */
void gdos_print(char *mem, long *ptr, int lines, int box, MAIL_INFO *entry, int part_no)
{
int page_width;
int page_height;
int page, bold = FALSE, prn_tx_height, hor, ver;
int msg, lw, y, clip_xy[4], t_attr[10];
long position;
char *buf;
int wout[57], work_out[57], /*ein_mm_x, */ein_mm_y, dummy;

DEBUG();
InitPrint(mail_options.prt_settings);

page_width = GetPageWidth();
page_height = GetPageHeight();

vq_extnd(prn_handle, FALSE, work_out);
vq_extnd(prn_handle, TRUE, wout);
if (!wout[0])
	ein_mm_y = 2520 / work_out[4];
else
	ein_mm_y = 10;

page = 1;
clip_xy[0] = mail_options.left_margin;
clip_xy[1] = mail_options.top_margin;
clip_xy[2] = page_width - mail_options.left_margin - mail_options.right_margin;
clip_xy[3] = page_height - mail_options.top_margin - mail_options.bottom_margin;

StartPage();
vs_clip(prn_handle, 1, clip_xy);

vst_alignment(prn_handle, 0, 5, &hor, &ver);
vst_font(prn_handle, mail_options.prt_font);
vst_height(prn_handle, mail_options.prt_fontsize * ein_mm_y / 10, &dummy, &dummy, &dummy, &dummy);

vqt_attributes(prn_handle, t_attr);
prn_tx_height = t_attr[9] + mail_options.line_spacing;
 
buf = (char *)malloc(255);
y = mail_options.top_margin;
for (msg = 0; msg < lines; msg++)
	{
	lw = (int)(ptr[msg + 1] - ptr[msg] - 1);
	position = ptr[msg];
	
	vst_color(prn_handle, mail_options.text_colour);
	switch(mem[position])
		{
		case 0x3:
			vst_color(prn_handle, mail_options.quoted_colour);
							/* Fall through */

		case 0x1:
			if (mem[position] == 1)
				if (!bold)
					{
					vst_effects(prn_handle, 1);
					bold = TRUE;
					}
			position++;
			lw--;
							/* Fall through */
							
		default:
			if (lw > 254) lw = 254;
			memcpy(buf, mem + position, lw);
			buf[lw] = 0;

			v_gtext(prn_handle, mail_options.left_margin, y, buf);
			if (bold) 
				{
				vst_effects(prn_handle, 0);
				bold = FALSE;
				}
				
			y += prn_tx_height;
			break;
		}
	
	if (y + (mail_options.print_footer ? prn_tx_height * 3 : 0) >= clip_xy[3] && msg < lines - 1)
		{
		if (mail_options.print_footer)
			GdosFooter(page, page_width, page_height, prn_tx_height);
		EndPage();
		page++;
		StartPage();
		y = mail_options.top_margin;
		}
	}

if (mail_options.print_attaches)
	{
	MAIL_MEM buffer;
	FILE_MAIL_MEM header;
	int f;

	memset(&buffer, 0, sizeof(buffer));

	f = open_file(entry->file, boxes[box].file);
	if (f)
		{
		restore_header(f, &header, &buffer);

		if (header.part_count > 1)
			{
			int cnt;

			edit_parts = (PART_INFO *)malloc(sizeof(PART_INFO) * header.part_count);
			Fseek(header.partinfo_pos, f, 0);
			Fread(f, sizeof(PART_INFO) * header.part_count, edit_parts);

			for (cnt = 0; cnt < header.part_count; cnt++)
				{
				if (cnt != part_no && (part_no != -1 || cnt != header.main_idx))
					{
					int l, c, extent[8];
					
					if (edit_parts[cnt].desc[0])
						{
						vqt_extent(prn_handle, edit_parts[cnt].name, extent);
						c = abs(extent[6] - extent[2]) + 20;
						vqt_extent(prn_handle, edit_parts[cnt].desc, extent);
						c += abs(extent[6] - extent[2]);
						
						if (mail_options.left_margin > 0)				c += mail_options.left_margin;
						if (mail_options.right_margin > 0)			c += mail_options.right_margin;
								
						if (c > page_width)
							l = 2;
						else
							l = 1;
						}
					else
						l =  1;
					
					if (y + l * prn_tx_height > page_height - mail_options.top_margin - mail_options.bottom_margin - 
																			(mail_options.print_footer ? 2 * prn_tx_height : 0))
						{
						if (mail_options.print_footer)
							GdosFooter(page, page_width, page_height, prn_tx_height);
						EndPage();
						page++;
						StartPage();
						y = mail_options.top_margin;
						}

					v_gtext(prn_handle, mail_options.left_margin, y, edit_parts[cnt].name);
					if (edit_parts[cnt].desc[0] != 0)
						{
						if (l == 2)
							{
							y += prn_tx_height;
							c = mail_options.left_margin;
							}
						else
							{
							vqt_extent(prn_handle, edit_parts[cnt].name, extent);
							c = abs(extent[6] - extent[2]) + 20;
							}
							
						v_gtext(prn_handle, c, y, edit_parts[cnt].desc);
						}
						
					y += prn_tx_height;
					}
				}
				
			free(edit_parts);
			}
		clear_buffer(&buffer);
		Fclose((int)f);
		}
	}
			
if (mail_options.print_footer && y > mail_options.top_margin)
	GdosFooter(page, page_width, page_height, prn_tx_height);

vs_clip(prn_handle, 0, clip_xy);
ExitPrint();

free(buf);
}

/* -----------------------
   | Initialize printing |
   ----------------------- */
int InitPrint(char *mod_settings)
{
PRN_SETTINGS *settings;
int i;
int8 dummy[50], devname[50];
int16 devexist;

DEBUG();
settings = (PRN_SETTINGS *)(mod_settings + 128);

prn_dialog = pdlg_create( PDLG_3D );						/* Speicher anfordern, Treiber scannen */
if ( prn_dialog )
	{
	if ( pdlg_validate_settings( prn_dialog, settings ) == 0 )	/* schwerwiegender Fehler? */
		pdlg_dflt_settings( prn_dialog, settings );								/* und initialisieren */

	if (mod_settings[0])
		{
		for (i = 21; i < 30; i++)
			{
			vq_ext_devinfo(vdi_handle, i, &devexist, dummy, dummy, devname);
			if (devexist)
				if (!strcmp(devname, mod_settings))
					{
					settings->driver_id = i;
					if ( pdlg_validate_settings( prn_dialog, settings ) == 0 )	/* schwerwiegender Fehler? */
						pdlg_dflt_settings( prn_dialog, settings );								/* und initialisieren */
					break;
					}
			}
		}
			
	prn_handle = v_opnprn(gl_apid, settings, prn_work_out );
	}

return prn_handle;						/* konnte der Druckertreiber geffnet werden? */
}

/* ----------------
   | End printing |
   ---------------- */
void ExitPrint(void)
{
pdlg_delete( prn_dialog );								/* Speicher freigeben */
v_clswk( prn_handle );
}

/* ------------------------
   | Determine page width |
   ------------------------ */
int GetPageWidth(void)
{
return prn_work_out[0];
}

/* -------------------------
   | Determine page height |
   ------------------------- */
int GetPageHeight(void)
{
return prn_work_out[1];
}

/* --------------------
   | Start a new page |
   -------------------- */
void StartPage(void)
{
}

/* ---------------
   | End of page |
   --------------- */
void GdosFooter(int page, int page_width, int page_height, int prn_tx_height)
{
int xy[4], extent[8];
char  pagetext[20], *tmstr, *s;
time_t mtime;

xy[0] = mail_options.left_margin;
xy[1] = page_height - mail_options.bottom_margin - prn_tx_height - 3;
xy[2] = page_width - mail_options.left_margin - mail_options.right_margin;
xy[3] = page_height - mail_options.bottom_margin - prn_tx_height - 3;
v_pline( prn_handle, 2, xy);

mtime = time(NULL);
tmstr = ctime(&mtime);

s = strchr(tmstr, '\n');
if (s) *s = 0;

v_gtext(prn_handle, mail_options.left_margin, page_height - mail_options.bottom_margin - prn_tx_height, tmstr);

strcpy(pagetext, s_page);
strcat(pagetext, " ");
itoa(page, pagetext + strlen(s_page), 10);

vqt_extent(prn_handle, pagetext, extent);
v_gtext(prn_handle, page_width - mail_options.right_margin - abs(extent[6] - extent[2]), page_height - 20 - prn_tx_height, pagetext);
}

/* ---------------
   | End of page |
   --------------- */
void EndPage(void)
{
v_updwk( prn_handle );								/* Seite aufbauen */
v_clrwk( prn_handle );								/* Seitenvorschub */
}


#endif
/*
							int	 use_colours, reformat;
							int	 print_header;
							int	 print_attaches;
*/
#endif