/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

static int f_provs[] = {PROV_1, PROV_2, PROV_3, PROV_4,
												PROV_5, PROV_6, PROV_7};
												
/* -------------------------
   | Fill in provider list |
   ------------------------- */
void fill_provider(OBJECT *tree, int base)
{
int i, obj;

DEBUG();
for (i = 0; i < 7; i++)
	{
	obj = f_provs[i];
	
	if (base + i < prov_cnt)
		{
		strcpy(tree[obj].ob_spec.free_string, providers[base + i].provider);
		tree[obj].ob_state &= ~DISABLED;
		}
	else
		{
		tree[obj].ob_spec.free_string[0] = 0;
		tree[obj].ob_state |= DISABLED;
		}
	}
}

/* -------------------
   | Select provider |
   ------------------- */
int provider_sel(void)
{
OBJECT *sel_prov;
int obj, i;
int prov_base = 0;

DEBUG();
if (!providers)
	scan_provider();
	
rsrc_gaddr(R_TREE, SEL_PROV, &sel_prov);
for (i = 0; i < 7; i++)
	sel_prov[f_provs[i]].ob_state &= ~SELECTED;
	
if (prov_cnt > 7)
	{
	sel_prov[SLIDER].ob_height = sel_prov[SLD_BKG].ob_height / prov_cnt * 7;
	if (sel_prov[SLIDER].ob_height < sel_prov[UP].ob_height)
		sel_prov[SLIDER].ob_height = sel_prov[UP].ob_height;

	sel_prov[SLIDER].ob_flags &= ~HIDETREE;
	sel_prov[SLD_BKG].ob_flags &= ~HIDETREE;
	sel_prov[UP].ob_flags &= ~HIDETREE;
	sel_prov[DOWN].ob_flags &= ~HIDETREE;
	}
else
	{
	sel_prov[SLIDER].ob_flags |= HIDETREE;
	sel_prov[SLD_BKG].ob_flags |= HIDETREE;
	sel_prov[UP].ob_flags |= HIDETREE;
	sel_prov[DOWN].ob_flags |= HIDETREE;
	}
	
fill_provider(sel_prov, prov_base);
open_dialog(sel_prov, NULL, NULL);
do
	{
	if (prov_cnt > 7)
		{
		sel_prov[SLIDER].ob_y = (sel_prov[SLD_BKG].ob_height - sel_prov[SLIDER].ob_height)  * prov_base / (prov_cnt - 7);
		redraw_object(sel_prov, SLD_BKG);
		}
			
	obj = do_dialog(sel_prov, 0);

	switch(obj & 0x7FFF)
		{
		case PROV_1:
		case PROV_2:
		case PROV_3:
		case PROV_4:
		case PROV_5:
		case PROV_6:
		case PROV_7:
			if (sel_prov[obj & 0x7FFF].ob_spec.free_string[0])
				{
				set_scrollselect(sel_prov, obj & 0x7FFF); 
				if (obj & 0x8000)
					obj = PROV_OK;
				}
			break;
		
		case UP:
			if (prov_base > 0)
				{
				prov_base--;
				fill_provider(sel_prov, prov_base);
				redraw_object(sel_prov, PROV_BKG);
				}
			break;
			
		case DOWN:
			if (prov_base + 7 < prov_cnt)
				{
				prov_base++;
				fill_provider(sel_prov, prov_base);
				redraw_object(sel_prov, PROV_BKG);
				}
			break;
							
		case SLIDER:
			prov_base = graf_slidebox(sel_prov, SLD_BKG, SLIDER, 1) * (prov_cnt - 7) / 1000;
			fill_provider(sel_prov, prov_base);
			redraw_object(sel_prov, PROV_BKG);
			break;
			
		case SLD_BKG:
			break;

		case PRV_LOAD:
			if (load_config(NULL))
				{
				close_dialog(sel_prov, TRUE);
				close_popwindow();
				return TRUE;
				}
		}
	
	}while(obj != PROV_OK && obj != PROV_CNC);
close_dialog(sel_prov, TRUE);

wait_mouse_release();

if (obj == PROV_OK)
	{
	for (i = 0; i < 7 && !(sel_prov[f_provs[i]].ob_state & SELECTED); i++);

	if (i < 7)
		if (load_config(providers[prov_base + i].prov_path))
			{
			close_popwindow();
			return TRUE;
			}		
	}
return FALSE;
}

