/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */

#ifdef PRINTING
#include <vdicol.h>
#include "extern.h"

static int prt_font_id;
static char prt_fontname[33];
static char prt_settings[1024];

static int pset_ids[] = { PRT_NAME,
													PRT_SETUP,
													LEFT_MARGIN,
													RIGHT_MARGIN,
													TOP_MARGIN,
													BOTTOM_MARGIN,
													LINE_SPACING,
													PRT_USE_COLOURS,
													PRT_REFORMAT,
													PRT_SUMMARY,
													PRT_HEADERS,
													PRT_FOOTER,
													PRT_FONT,
													PRT_FONTSIZE,
													PSETUP_CANCEL,
													PSETUP_OKAY,
													0};
							
/* ------------------------
   | Module setup handler |
   ------------------------ */ 
int print_setup()
{
PRN_DIALOG	*prn_dialog;
WORD	button;
PRN_SETTINGS *settings;
int i;
int8 dummy[50], devname[50];
int16 devexist;

DEBUG();
prn_dialog = pdlg_create( PDLG_3D );						/* Speicher anfordern, Treiber scannen */
if ( prn_dialog )
	{
	settings = (PRN_SETTINGS *)(prt_settings + 128);
	
	if ( pdlg_validate_settings( prn_dialog, settings ) == 0 )	/* schwerwiegender Fehler? */
		pdlg_dflt_settings( prn_dialog, settings );								/* und initialisieren */
	
	if (prt_settings[0])
		{
		for (i = 21; i < 30; i++)
			{
			vq_ext_devinfo(vdi_handle, i, &devexist, dummy, dummy, devname);
			if (devexist)
				if (!strcmp(devname, prt_settings))
					{
					settings->driver_id = i;
					if ( pdlg_validate_settings( prn_dialog, settings ) == 0 )	/* schwerwiegender Fehler? */
						pdlg_dflt_settings( prn_dialog, settings );								/* und initialisieren */
					break;
					}
			}
		}

	button = pdlg_do(prn_dialog, settings, "Marathon Mail", PDLG_PREFS);
	pdlg_delete( prn_dialog );								/* Speicher freigeben */
	
	if (button)
		vq_ext_devinfo(vdi_handle, settings->driver_id, &devexist, 
									 dummy, dummy, prt_settings);
	return (int)button;
	}

return 0;
}

/* --------------------------------
   | Set page setup configuration |
   -------------------------------- */
static void set_pagesettings(OBJECT *psetup)
{
DEBUG();
prt_font_id = mail_options.prt_font;
psetup[PRT_FONT].ob_spec.tedinfo->te_ptext = prt_fontname;

find_font_id(prt_font_id, prt_fontname);
itoa(mail_options.prt_fontsize, psetup[PRT_FONTSIZE].ob_spec.tedinfo->te_ptext, 10);

itoa(mail_options.left_margin, psetup[LEFT_MARGIN].ob_spec.tedinfo->te_ptext, 10);
itoa(mail_options.right_margin, psetup[RIGHT_MARGIN].ob_spec.tedinfo->te_ptext, 10);
itoa(mail_options.top_margin, psetup[TOP_MARGIN].ob_spec.tedinfo->te_ptext, 10);
itoa(mail_options.bottom_margin, psetup[BOTTOM_MARGIN].ob_spec.tedinfo->te_ptext, 10);
itoa(mail_options.line_spacing, psetup[LINE_SPACING].ob_spec.tedinfo->te_ptext, 10);

SetSelect(psetup, PRT_REFORMAT, mail_options.reformat);
SetSelect(psetup, PRT_SUMMARY, mail_options.print_attaches);
SetSelect(psetup, PRT_HEADERS, mail_options.print_header);
SetSelect(psetup, PRT_FOOTER, mail_options.print_footer);
SetSelect(psetup, PRT_USE_COLOURS, mail_options.use_colours);

memcpy(prt_settings, mail_options.prt_settings, sizeof(prt_settings));
}

/* --------------------------------
   | Get page setup configuration |
   -------------------------------- */
static void get_pagesettings(OBJECT *psetup)
{
DEBUG();
mail_options.prt_font = prt_font_id;
mail_options.prt_fontsize = atoi(psetup[PRT_FONTSIZE].ob_spec.tedinfo->te_ptext);

mail_options.left_margin = atoi(psetup[LEFT_MARGIN].ob_spec.tedinfo->te_ptext);
mail_options.right_margin = atoi(psetup[RIGHT_MARGIN].ob_spec.tedinfo->te_ptext);
mail_options.top_margin = atoi(psetup[TOP_MARGIN].ob_spec.tedinfo->te_ptext);
mail_options.bottom_margin = atoi(psetup[BOTTOM_MARGIN].ob_spec.tedinfo->te_ptext);
mail_options.line_spacing = atoi(psetup[LINE_SPACING].ob_spec.tedinfo->te_ptext);

mail_options.reformat = IsSelected(psetup, PRT_REFORMAT);
mail_options.print_attaches = IsSelected(psetup, PRT_SUMMARY);
mail_options.print_header = IsSelected(psetup, PRT_HEADERS);
mail_options.print_footer = IsSelected(psetup, PRT_FOOTER);
mail_options.use_colours = IsSelected(psetup, PRT_USE_COLOURS);

memcpy(mail_options.prt_settings, prt_settings, sizeof(prt_settings));
}

/* --------------------------------------
   | Handle Printer setup dialog events |
   -------------------------------------- */
static void cdecl psetup_handler (OBJECT *tree, int obj)
{
switch (obj)
	{
	case PRT_SETUP:
		print_setup();
		redraw_object(tree, PRT_NAME);
		break;
		
	case PRT_FONT:
	case PRT_SELFONT:
		{
		int x, y;
		
		objc_offset(tree, PRT_FONT, &x, &y);
		select_font_popup(x, y, &prt_font_id, prt_fontname);
		redraw_object(tree, PRT_FONT);
		break;
		}

	case PRT_SELSIZE:
		{
		int x, y;
		int sz;
		
		objc_offset(tree, PRT_FONTSIZE, &x, &y);
		sz = select_font_size(x, y);
		if (sz > -1)
			{
			itoa(sz, tree[PRT_FONTSIZE].ob_spec.tedinfo->te_ptext, 10);
			redraw_object(tree, PRT_FONTSIZE);
			}
		break;
		}
	
	case PSETUP_OKAY:
		get_pagesettings(tree);
		
	case PSETUP_CANCEL:
		close_dialog(tree, TRUE);
		break;
	}
}

/* -----------------
   | Printer setup |
   ----------------- */
void PrinterSetup(void)
{
OBJECT* psetup;

rsrc_gaddr(R_TREE, PRINT_SETUP, &psetup);

set_pagesettings(psetup);

if (mail_options.printmode)
	{
	psetup[PRT_NAME].ob_spec.tedinfo->te_ptext = prt_settings;
	EnableObj(psetup, PRT_NAME);
	EnableObj(psetup, PRT_SETUP);
	EnableObj(psetup, LINE_SPACING);
	EnableObj(psetup, PRT_USE_COLOURS);
	EnableObj(psetup, PRT_FONT);
	EnableObj(psetup, PRT_SELFONT);
	EnableObj(psetup, PRT_FONTSIZE);
	EnableObj(psetup, PRT_SELSIZE);
	EnableObj(psetup, RIGHT_MARGIN);
	}
else
	{
	psetup[PRT_NAME].ob_spec.tedinfo->te_ptext = "---";
	DisableObj(psetup, PRT_NAME);
	DisableObj(psetup, PRT_SETUP);
	DisableObj(psetup, LINE_SPACING);
	DisableObj(psetup, PRT_USE_COLOURS);
	DisableObj(psetup, PRT_FONT);
	DisableObj(psetup, PRT_SELFONT);
	DisableObj(psetup, PRT_FONTSIZE);
	DisableObj(psetup, PRT_SELSIZE);
	DisableObj(psetup, RIGHT_MARGIN);
	}
	
open_dialog(psetup, psetup[PRTSETUP_TITLE].ob_spec.free_string, iconified_tree);
dlgset_help_info(psetup, helpfile, pset_ids, "PSET_");
set_active(psetup, psetup_handler, TRUE, 0);
}
#endif

