/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

extern long mark_start, mark_end;

/* ----------------
   | Popup folder |
   ---------------- */
void identity_popup(int box)
{
OBJECT *id_pop;

DEBUG();
rsrc_gaddr(R_TREE, P_POPUP, &id_pop);

switch(popup(id_pop))
	{
	case PP_SCAN:
		check_id(boxes[box].identity, FALSE);
		break;
		
	case PP_SEND:
		send_idqueue(boxes[box].identity, TRUE);
		break;

	case PP_CHECK:
		check_id(boxes[box].identity, TRUE);
		break;
		
	case PP_SENDCHK:
		if (!send_idqueue(boxes[box].identity, TRUE))
			check_id(boxes[box].identity, -1);
		break;
				
	case PP_DELET:
		delete_one(box);
		break;
		
	case PP_FREMOVE:
		{
		int idx = boxes[box].fav_index;
		int i;
		
		boxes[box].fav_index = -1;
		for (i = 0; i < box_count; i++)
			if (boxes[i].fav_index > idx)
				boxes[i].fav_index--;
				
		update_boxinfo();
		redraw_boxwind(-1, 0, 0, 0);
		break;
		}
	
	case PP_LOGIN:
		break;
		
	case PP_PROPS:
		folder_properties(box);
		update_boxinfo();
		break;
	}
}

/* ----------------
   | Popup folder |
   ---------------- */
void queue_popup(int box)
{
OBJECT *que_pop;

DEBUG();
rsrc_gaddr(R_TREE, O_POPUP, &que_pop);

switch(popup(que_pop))
	{
	case OP_OPEN:
		list_window(box);
		break;

	case OP_RENAM:
		folder_properties(box);
		update_boxinfo();
		break;
						
	case OP_SENDA:
		send_idqueue("\0", TRUE);
		break;
		
	case OP_FREMOVE:
		{
		int idx = boxes[box].fav_index;
		int i;
		
		boxes[box].fav_index = -1;
		for (i = 0; i < box_count; i++)
			if (boxes[i].fav_index > idx)
				boxes[i].fav_index--;
				
		update_boxinfo();
		redraw_boxwind(-1, 0, 0, 0);
		break;
		}
	}
}

/* ----------------
   | Popup folder |
   ---------------- */
void folder_popup(int box)
{
OBJECT *box_pop;

DEBUG();
rsrc_gaddr(R_TREE, B_POPUP, &box_pop);

if (box == box_number("IN") || box == box_number("SENT") ||
		box == box_number("SENTERR"))
	DisableObj(box_pop, BP_DELET);
else
	EnableObj(box_pop, BP_DELET);

switch(popup(box_pop))
	{
	case BP_OPEN:
		list_window(box);
		break;

	case BP_FREMOVE:
		{
		int idx = boxes[box].fav_index;
		int i;
		
		boxes[box].fav_index = -1;
		for (i = 0; i < box_count; i++)
			if (boxes[i].fav_index > idx)
				boxes[i].fav_index--;
				
		update_boxinfo();
		redraw_boxwind(-1, 0, 0, 0);
		break;
		}

	case BP_EDITFILTER:
		edit_filters(box);
		break;
		
	case BP_APPLYFILTER:
		apply_filter(box);
		break;
		
	case BP_DELET:
		delete_one(box);
		break;
		
	case BP_PROPS:
		folder_properties(box);
		update_boxinfo();
		break;
	}
}

/* ----------------
   | Popup folder |
   ---------------- */
void editor_popup(void)
{
OBJECT *edit_pop;

DEBUG();
rsrc_gaddr(R_TREE, MAILEDIT_POPUP, &edit_pop);

if (mark_start != -1)
	{
	EnableObj(edit_pop, EDIT_CUT);
	EnableObj(edit_pop, EDIT_COPY);
	EnableObj(edit_pop, EDIT_DELETE);
	}
else
	{
	DisableObj(edit_pop, EDIT_CUT);
	DisableObj(edit_pop, EDIT_COPY);
	DisableObj(edit_pop, EDIT_DELETE);
	}
	
if (check_scrap())
	{
	EnableObj(edit_pop, EDIT_PASTE_WRAPPED);
	EnableObj(edit_pop, EDIT_PASTE_NOTWRAPPED);
	}
else
	{
	DisableObj(edit_pop, EDIT_PASTE_WRAPPED);
	DisableObj(edit_pop, EDIT_PASTE_NOTWRAPPED);
	}
	
switch(popup(edit_pop))
	{
	case EDIT_CUT:
		cut_edit();
		break;
		
	case EDIT_COPY:
		copy_edit();
		break;
		
	case EDIT_PASTE_WRAPPED:
		{
		int wrap = new_buffer.word_wrap;
		
		new_buffer.word_wrap = 1;
		paste_edit();
		new_buffer.word_wrap = wrap;
		break;
		}
		
	case EDIT_PASTE_NOTWRAPPED:
		{
		int wrap = new_buffer.word_wrap;
		
		new_buffer.word_wrap = 0;
		paste_edit();
		new_buffer.word_wrap = wrap;
		break;
		}
		
	case EDIT_DELETE:
		del_edit();
		break;
		
	case EDIT_SELALL:
		select_all_edit();
		break;
	}
}
