/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

/* Recent list */
static int recent_count = 0, recent_len = 0, recent_chg = 0;
static char *recent_list;

#ifdef MILAN
#  define	RECENT_SIZE 500
#else
#	 ifdef EVAL
#		 define RECENT_SIZE 500
#  else
#		 define RECENT_SIZE 5000
#  endif			
#endif

/* ------------------------------------
   | Find first matching recent entry |
   ------------------------------------ */
int find_matchingrecent(char *set)
{
char *xsrch;
int no, l;

l = (int)strlen(set);
xsrch = recent_list;
no = 0;
while (no < recent_count)
	{
	if (!strnicmp(xsrch, set, l))
		{
		strcpy(set, xsrch);
		return TRUE;
		}
		
	xsrch += strlen(xsrch) + 1;
	no++;
	}
return FALSE;
}

/* ------------------------------------------
   | Add mail addresses to recent mail list |
   ------------------------------------------ */
void add_recent(char *addr)
{
char *x, *xsrch;
int p, p2, xcount, l;

DEBUG();
x = (char *)malloc(strlen(addr) + 1);
p2 = 0;
while(addr[p2])
	{
	while(isspace(addr[p2]))
		p2++;
		
	if (addr[p2])
		{
		p = 0;
		while(addr[p2])
			{
			if (addr[p2] == ',' || addr[p2] == ';')
				break;
				
			x[p++] = addr[p2++];
			}
			
		while(p >= 1 && isspace(x[p - 1]))
			p--;
		x[p++] = 0;

		if (x[0])
			{
			xsrch = recent_list;
			for (xcount = 0; xcount < recent_count; xcount++)
				{
				if (!strcmp(xsrch, x))
					{
					memmove(recent_list + p, recent_list, xsrch - recent_list);
					strcpy(recent_list, x);
					break;
					}
					
				xsrch += strlen(xsrch) + 1;
				}
			
			if (xcount == recent_count)
				{
				l = (int)strlen(x) + 1;
				
				if (!recent_count)
					recent_list = (char *)malloc(l);
				else
					recent_list = (char *)realloc(recent_list, 
																				recent_len + l);

				memmove(recent_list + l, recent_list, recent_len);
				strcpy(recent_list, x);
				recent_len += l;
				recent_count++;
				
				if (recent_len > RECENT_SIZE)
					{
					xsrch = recent_list;
					xcount = --recent_count;
					while(xcount--)
						xsrch += strlen(xsrch) + 1;
					recent_len -= (int)strlen(xsrch) + 1;
					}
				}
					
			recent_chg = 1;
			}
		
		if (addr[p2] == ',' || addr[p2] == ';')
			p2++;
		}
	}
}

/* --------------------
   | Fill recent list |
   -------------------- */
void fill_recent(OBJECT *rec_list, int no, int redraw)
{
int rec, pag;
char *xsrch;

DEBUG();
set_slider_objects(rec_list, REC_SLD, REC_DRG, recent_count, no, 9, redraw);	

xsrch = recent_list;
pag = no;
while(pag--)
	xsrch += strlen(xsrch) + 1;

rec = REC_1;
while (rec <= REC_9 && no < recent_count)
	{
	EnableObj(rec_list, rec);
	rec_list[rec].ob_flags |= SELECTABLE|TOUCHEXIT|RBUTTON;
	strncpy(rec_list[rec++].ob_spec.tedinfo->te_ptext + 1, xsrch, 48);
	no++;
	xsrch += strlen(xsrch) + 1;
	}

while(rec <= REC_9)
	{
	DisableObj(rec_list, rec);
	rec_list[rec].ob_flags &= ~(SELECTABLE|TOUCHEXIT|RBUTTON);
	rec_list[rec++].ob_spec.tedinfo->te_ptext[1] = 0;
	}

if (redraw)
	redraw_object(rec_list, REC_BKG);
}

/* ----------------
   | Paste recent |
   ---------------- */
void paste_recent(int lin, int num, char **ptr, int *max_len)
{
char *recent_ptr;
int l;

DEBUG();
recent_ptr = recent_list;			
while (num--)
	recent_ptr += strlen(recent_ptr) + 1;
	
if (ptr)
	{
	if ((int)strlen(*ptr) + (int)strlen(recent_ptr) + 3 > *max_len)
		{
		l = (int)(strlen(recent_ptr) + 2 + 59);
		l /= 60;
		l *= 60;
		
		*max_len += l;
		*ptr = (char *)realloc(*ptr, *max_len);
		}
	if (**ptr)
		strcat(*ptr, ", ");
	strcat(*ptr, recent_ptr);

	redraw_topedit(lin, *ptr);
	}
else
	if (get_dialog_wind(fwd_tree) != -1)
		{
#ifndef EVAL
		fwd_paste(recent_ptr);	
#endif
		}
}

/* -----------------------------------
   | Select address from recent list |
   ----------------------------------- */
void recent_select(int lin, char **ptr, int *maxlen)
{
OBJECT *rec_list;
int obj, page;
EVNTDATA gev;

DEBUG();
if (!recent_count)
	return;

rsrc_gaddr(R_TREE, RECENT, &rec_list);

align_slider_objects(rec_list, REC_SBK, REC_UP, REC_SLD, REC_DRG, REC_DWN);

fill_recent(rec_list, page = 0, FALSE);
open_dialog(rec_list, NULL, iconified_tree);
do
	{
	obj = do_dialog(rec_list, 0, app_callback);
	
	switch(obj & 0x7FFF)
		{
		case REC_UP:
			if (page > 0)
				{
				page--;
				delay(200);
				fill_recent(rec_list, page, TRUE);
				}
			break;
			
		case REC_DWN:
			if (page < recent_count - 9)
				{
				page++;
				delay(200);
				fill_recent(rec_list, page, TRUE);
				}
			break;
			
		case REC_SLD:
			{
			int ox, oy;
			EVNTDATA ev;
			
			graf_mkstate(&ev);
			objc_offset(rec_list, REC_DRG, &ox, &oy);
			if (ev.y < oy)
				{
				page -= 9;
				if (page < 0) page = 0;
				}
			else
				{
				page += 9;
				if (page > recent_count - 9)
					page = recent_count - 9;
				}
			delay(200);
			fill_recent(rec_list, page, TRUE);
			break;
			}
			
		case REC_DRG:
			{
			int pos = graf_slidebox(rec_list, REC_SLD, REC_DRG, 1);
			page = (int)(pos * (long)(recent_count - 9) / 1000);
			fill_recent(rec_list, page, TRUE);
			break;
			}
			
		case RECPASTE:
			{
			int rec;
			
			rec = REC_1;
			while(rec <= REC_9)
				{
				if (IsSelected(rec_list, rec))
					break;
				rec++;
				}
					
			if (rec <= REC_9)
				paste_recent(lin, rec + page - REC_1, ptr, maxlen);
			break;
			}
			
		default:
			if ((obj & 0x7FFF) >= REC_1 && (obj & 0x7FFF) <= REC_9)
				{
				int c;
				
				c = REC_1;
				while (c <= REC_9)
					{
					if (IsSelected(rec_list, c) && (obj & 0x7FFF) != c)
						{
						DeselectObj(rec_list, c);
						redraw_object(rec_list, c);
						}
					c++;
					}
				
				if (!IsSelected(rec_list, obj & 0x7FFF))
					{
					SelectObj(rec_list, obj & 0x7FFF);
					redraw_object(rec_list, obj & 0x7FFF);
					}
				
				if (obj & 0x8000)
					{
					paste_recent(lin, (obj & 0x7FFF) + page - REC_1, ptr, maxlen);
					do
						{
						graf_mkstate(&gev);
						}while(gev.bstate);
					}
				}
			break;
		}

	}while(obj != RECCLOSE);
close_dialog(rec_list, TRUE);
}

/* ----------------------------
   | Save recent address list |
   ---------------------------- */
void save_recent(void)
{
long rec_fl;

DEBUG();
if (!recent_chg) return;

rec_fl = Fcreate("RECENT.LIS", 0);
if (rec_fl > 0)
	{
	Fwrite((int)rec_fl, sizeof(recent_count), &recent_count);
	Fwrite((int)rec_fl, sizeof(recent_len), 	&recent_len);
	
	if (recent_len)
		Fwrite((int)rec_fl, recent_len, recent_list);
		
	Fclose((int)rec_fl);
	}
}

/* ----------------------------
   | Save recent address list |
   ---------------------------- */
void load_recent(void)
{
long rec_fl;

DEBUG();
rec_fl = Fopen("RECENT.LIS", FO_READ);
if (rec_fl > 0)
	{
	Fread((int)rec_fl, sizeof(recent_count), 	&recent_count);
	Fread((int)rec_fl, sizeof(recent_len), 		&recent_len);

	if (recent_len)
		{
		recent_list = (char *)malloc(recent_len);
		Fread((int)rec_fl, recent_len, recent_list);
		}
		
	Fclose((int)rec_fl);
	}
}
