/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

static int quote_all = TRUE;
static int include_header = TRUE;
static int replyto_replyto = TRUE;
static int replyto_all = FALSE;
static char reply_identity[21];

static int ropt_ids[] = {	QUOTE_MAIL,
													QUOTE_SELECTION,
													INCLUDEHEADER,
													REPLY_FROM,
													USE_REPLYTO,
													SEND_ALL,
													ROPT_IDSEL,
													RPLYOPT_CANCEL,
													RPLYOPT_OKAY,
													0};

/* ----------------------------
   | Prompt for reply options |
   ---------------------------- */
int prompt_reply(int has_replyto, int has_morereceipients, int has_selection, char *identity)
{
OBJECT *replyoptions;
int obj;
char nidentity[21], sidentity[21];

DEBUG();
rsrc_gaddr(R_TREE, REPLYOPTIONS, &replyoptions);

if (identity)
	{
	strcpy(replyoptions[ROPT_IDSEL].ob_spec.tedinfo->te_ptext, identity);
	strcpy(nidentity, identity);
	}
else
	{
	replyoptions[ROPT_IDSEL].ob_spec.tedinfo->te_ptext[0] = 0;
	nidentity[0] = 0;
	}
	
if (has_selection)
	{
	EnableObj(replyoptions, 	QUOTE_SELECTION);
	SelectObj(replyoptions, 	QUOTE_SELECTION);
	DeselectObj(replyoptions, QUOTE_MAIL);
	}
else
	{
	DisableObj(replyoptions, 	QUOTE_SELECTION);
	DeselectObj(replyoptions, QUOTE_SELECTION);
	SelectObj(replyoptions, 	QUOTE_MAIL);
	}

if (has_replyto)
	{
	EnableObj(replyoptions, 	USE_REPLYTO);
	SelectObj(replyoptions, 	USE_REPLYTO);
	DeselectObj(replyoptions, REPLY_FROM);
	}
else
	{
	DisableObj(replyoptions, 	USE_REPLYTO);
	DeselectObj(replyoptions, USE_REPLYTO);
	SelectObj(replyoptions, 	REPLY_FROM);
	}

if (has_morereceipients)
	EnableObj(replyoptions, 	SEND_ALL);
else
	DisableObj(replyoptions, 	SEND_ALL);
	
open_dialog(replyoptions, replyoptions[REPLYOPTIONS_TITLE].ob_spec.free_string, iconified_tree);
dlgset_help_info(replyoptions, helpfile, ropt_ids, "ROPT_");
do
	{
	obj = do_dialog(replyoptions, 0, app_callback) & 0x7FFF;
	
	if (obj == ROPT_IDSEL || obj == ROPT_IDSELDROP)
		{
		int x, y;
			
		objc_offset(replyoptions, ROPT_IDSEL, &x, &y);
		if (DialogIdentitySelect(sidentity, x, y, 
														 replyoptions[ROPT_IDSELDROP].ob_x + replyoptions[ROPT_IDSELDROP].ob_width - 
														 replyoptions[ROPT_IDSEL].ob_x,
														 "Default"))
			{
			if (!sidentity[0])
				strcpy(replyoptions[ROPT_IDSEL].ob_spec.tedinfo->te_ptext, "Default");
			else
				strcpy(replyoptions[ROPT_IDSEL].ob_spec.tedinfo->te_ptext, sidentity);
			strcpy(nidentity, sidentity);
			redraw_object(replyoptions, ROPT_IDSEL);
			}
		}
	} while (obj != RPLYOPT_CANCEL && obj != RPLYOPT_OKAY);
close_dialog(replyoptions, TRUE);

if (obj == RPLYOPT_OKAY)
	{
	quote_all = IsSelected(replyoptions, QUOTE_MAIL);
	include_header = IsSelected(replyoptions, INCLUDEHEADER);
	replyto_replyto = IsSelected(replyoptions, USE_REPLYTO);
	replyto_all = IsSelected(replyoptions, SEND_ALL);
	strcpy(reply_identity, nidentity);
	return TRUE;
	}
else
	return FALSE;
}

/* ----------------------
   | Reply to a message |
   ---------------------- */
void do_reply(int box, MAIL_INFO *entry, 
							char *mem, long *ptr, int lines, 
							long len, long start, long end)
{
char 	*reply_mem, *x, *y;
long 	reply_blen, *reply_ptr;
int 	reply_width;
int 	reply_lines, i, first_replyoffset, f, head_seperator;
long offset;
IDENTITY setid;
int has_identity;

DEBUG();
if (entry->identity[0])
	has_identity = getIdentity(entry->identity, &setid);
else
	{
	int sbox = box;
	int l;
	
	has_identity = FALSE;
	do
		{
		if (boxes[sbox].identity[0])
			has_identity = getIdentity(boxes[sbox].identity, &setid);
		
		if (!boxes[sbox].level) break;
		l = boxes[sbox].level;
		
		while(sbox > -1 && boxes[sbox].level >= l)
			sbox--;
		
		}while (!has_identity && sbox > -1);
	}
	
if (!has_identity)
	has_identity = getDefaultIdentity(&setid);
	
if (mail_options.prompt_reply)
	{
	int ansi;
	int has_replyto = FALSE;
	int receipients = 0;
	
	if (!mem)
		{
		if (!get_part("Reply mail",
									boxes[box].file, entry, 
									&reply_mem, &reply_blen, &reply_ptr,
							 		&reply_width, &reply_lines, &ansi, TRUE, 32767))
			return;
		}
	else
		{
		reply_mem = mem;
		reply_lines = lines;
		reply_ptr = ptr;
		reply_blen = len;
		}
		
	i = 0;
	while(i < reply_lines)
		{
		x = reply_ptr[i] + reply_mem;
				
		while(*x == 1 || *x == 3 || *x == ' ' || *x == '\t')		x++;
	
		if (*x == '\r') break;
		
		if (!strnicmp(x, "Reply-To:", 9))
			has_replyto = TRUE;
		else
			if ((!strnicmp(x, "To:", 3) || !strnicmp(x, "CC:", 3)))
				{
				x += 3;
				while(*x == ' ' || *x == '\t')	x++;
				
				while (*x != '\r' || !strncmp(x + 1, "\1              ", 14))
					{
					if (*x != '\r')
						{
						if (*x == '@')
							receipients++;						
						}
					else
						x += 14;
					x++;
					}
			}
								
		i++;
		}

	if (!mem)
		{	
		free(reply_mem);
		free(reply_ptr);
		}
		
	if (!prompt_reply(has_replyto, receipients > 1, start != -1, has_identity ? setid.name : NULL))
		return;
	}
else
	{
	quote_all = TRUE;
	include_header = TRUE;
	replyto_replyto = TRUE;
	replyto_all = FALSE;
	strcpy(reply_identity, entry->identity);
	}

if (!mem)
	{
	if (!get_body("Reply mail",
								boxes[box].file, entry, 
								&reply_mem, &reply_blen, &reply_ptr,
						 		&reply_width, &reply_lines, TRUE))
		return;
	}
else
	{
	reply_blen = len;
	reply_lines = lines;

	reply_mem = (char *)malloc(len);
	memcpy(reply_mem, mem, len);

	reply_ptr = (long *)malloc(sizeof(long) * lines);
	memcpy(reply_ptr, ptr, sizeof(long) * lines);
	}
	
strcpy(mail_title, reply_text);

memset(&new_buffer, 0, sizeof(new_buffer));
create_empty_header();

reply_mem = (char *)realloc(reply_mem, reply_blen + reply_lines * 2);

f = open_file(entry->file, boxes[box].file);
if (f)
	{
	MAIL_MEM buffer;
	FILE_MAIL_MEM header;
	
	restore_header(f, &header, &buffer);
	Fclose(f);
	clear_buffer(&buffer);
	}

strcpy(new_buffer.identity, reply_identity);
		
i = 0;
while(i < reply_lines)
	{
	x = reply_ptr[i] + reply_mem;
			
	while(*x == 1 || *x == 3 || *x == ' ' || *x == '\t')		x++;

	if (*x == '\r') break;
	
	if (!strnicmp(x, "From:", 5) && !new_buffer.to[0])
		{
		x += 5;
		while(*x == ' ' || *x == '\t')	x++;
		
		y = new_buffer.to;
		while (*x != '\r' || !strncmp(x + 1, "\1              ", 14))
			{
			if (*x != '\r')
				{
				if (y - new_buffer.to + 2 > new_buffer.to_max)
					{
					offset = y - new_buffer.to;
					new_buffer.to_max += 60;
					new_buffer.to = (char *)realloc(new_buffer.to, new_buffer.to_max);
					y = new_buffer.to + offset;
					}
				*y++ = *x;
				}
			else
				x += 14;
			x++;
			}
		*y = 0;
		}
	else
		if (!strnicmp(x, "Reply-To:", 9) && replyto_replyto)
			{
			x += 9;
			while(*x == ' ' || *x == '\t')	x++;
			
			y = new_buffer.to;
			while (*x != '\r' || !strncmp(x + 1, "\1              ", 14))
				{
				if (*x != '\r')
					{
					if (y - new_buffer.to + 2 > new_buffer.to_max)
						{
						offset = y - new_buffer.to;
						new_buffer.to_max += 60;
						new_buffer.to = (char *)realloc(new_buffer.to, new_buffer.to_max);
						y = new_buffer.to + offset;
						}
					*y++ = *x;
					}
				else
					x += 14;
				x++;
				}
			*y = 0;
			}
		else
			if (!strnicmp(x, "Subject:", 8))
				{
				x += 8;
				while(*x == ' ' || *x == '\t')	x++;
		
				if (new_buffer.subj_max < 60)
					{
					new_buffer.subj_max = 60;
					new_buffer.subject = (char *)realloc(new_buffer.subject, 60);
					}
									
				y = new_buffer.subject;
				strcpy(y, "Re: ");
				y += 4;
				while (*x != '\r' || !strncmp(x + 1, "\1             ", 14))
					{
					if (*x != '\r')
						{
						if (y - new_buffer.subject + 2 > new_buffer.subj_max)
							{
							offset = y - new_buffer.subject;
							new_buffer.subj_max += 60;
							new_buffer.subject = (char *)realloc(new_buffer.subject, new_buffer.subj_max);
							y = new_buffer.subject + offset;
							}
						*y++ = *x;
						}
					else
						x += 14;
					x++;
					}
				*y = 0;				
				}
			else
				if ((!strnicmp(x, "To:", 3) || !strnicmp(x, "CC:", 3)) && replyto_all)
					{
					x += 3;
					while(*x == ' ' || *x == '\t')	x++;
					
					y = new_buffer.cc;
					if (*y)
						{
						y += strlen(y);
						if (y - new_buffer.cc + 2 > new_buffer.cc_max)
							{
							offset = y - new_buffer.cc;
							new_buffer.cc_max += 60;
							new_buffer.cc = (char *)realloc(new_buffer.cc, new_buffer.cc_max);
							y = new_buffer.cc + offset;
							}
						strcpy(y, ", ");
						y += 2;
						}
						
					while (*x != '\r' || !strncmp(x + 1, "\1              ", 14))
						{
						if (*x != '\r')
							{
							if (y - new_buffer.cc + 2 > new_buffer.cc_max)
								{
								offset = y - new_buffer.cc;
								new_buffer.cc_max += 60;
								new_buffer.cc = (char *)realloc(new_buffer.cc, new_buffer.cc_max);
								y = new_buffer.cc + offset;
								}
							*y++ = *x;
							}
						else
							x += 14;
						x++;
						}
					*y = 0;
					}
				
	i++;
	}
head_seperator = i;

first_replyoffset = -1L;
i = reply_lines - 1;
while(i >= 0)
	{
	if (reply_ptr[i] < reply_blen)
		{
		x = reply_ptr[i] + reply_mem;
	
		if (*x == 2 || (*x == 1 && (!include_header || i > head_seperator)))
			{
			if (i < reply_lines - 1)
				{
				memmove(x, reply_ptr[i + 1] + reply_mem, reply_blen - reply_ptr[i + 1]);
				reply_blen -= reply_ptr[i + 1] - reply_ptr[i];
				reply_lines--;
				}
			else
				reply_blen = reply_ptr[i];
				
			if (*x == 2)
				first_replyoffset = -1L;
			}
		else
			{
			if (start != -1L && !quote_all && *x != 1)
				{
				if (i < reply_lines - 1 && reply_ptr[i + 1] <= start)
					{
					memmove(x, reply_ptr[i + 1] + reply_mem, reply_blen - reply_ptr[i + 1]);
					reply_blen -= reply_ptr[i + 1] - reply_ptr[i];
					reply_lines--;
					i--;
					continue;
					}
				if (reply_ptr[i] < start)
					{
					memmove(x, start + reply_mem, reply_blen - start);
					reply_blen -= start - reply_ptr[i];
					end -= start - reply_ptr[i];
					}
				if (reply_ptr[i + 1] >= end)
					{
					if (reply_ptr[i] >= end)
						{
						reply_blen = reply_ptr[i];
						reply_lines--;
						i--;
						continue;
						}
					else
						reply_blen = end;
					}
				}
				
			if (*x == 1 || *x == 3)
				{
				if (*x == 1)
					if (first_replyoffset == -1L)
						first_replyoffset = reply_ptr[i + 1];
				
				if (first_replyoffset > reply_ptr[i])
					first_replyoffset += 1;
							
				memmove(x + 1, x,	reply_blen - reply_ptr[i]);
				reply_blen++;
				}
			else
				{
				memmove(x + 2, x,	reply_blen - reply_ptr[i]);
	
				if (first_replyoffset > reply_ptr[i])
					first_replyoffset += 2;
					
				reply_blen += 2;
				}
			x[0] = '>';
			x[1] = ' ';
			}
		}
	i--;
	}
free(reply_ptr);

if (start != -1 && include_header)
	{
	x = reply_mem + first_replyoffset;
	memmove(x + 2, x, reply_blen - first_replyoffset);
	reply_blen += 2;
	*(x++) = '>';
	*x = '\r';
	reply_lines++;
	}

new_buffer.changed = TRUE;
reply_mem = (char *)realloc(reply_mem, (reply_blen + 4095L) & ~4095L);
new_buffer.body = reply_mem;
new_buffer.body_len = reply_blen;
new_buffer.body_lines = reply_lines;
new_buffer.max_length = reply_width;

if (has_identity)
	{
	if (setid.username[0])
		{
		new_buffer.from[0] = '"';
		strcpy(new_buffer.from + 1, setid.username);
		strcat(new_buffer.from, "\" <");
		}

	if (setid.use_providerdata)
		{
		strcat(new_buffer.from, cfg_options.email);
		strcat(new_buffer.from, cfg_options.email2);
		}
	else
		{
		strcat(new_buffer.from, setid.email);
		strcat(new_buffer.from, setid.email2);
		}

	if (setid.username[0])
		strcat(new_buffer.from, ">");

	if (setid.organization[0])
		strcpy(new_buffer.organization, setid.organization);

	new_buffer.mime_allowed = setid.mime_allowed;
	new_buffer.word_wrap = setid.word_wrap;
	new_buffer.editwidth = setid.editwidth;

		/* Get priority from identity */
	if (setid.priority == -1)
		{
		if (setid.last_priority != -1)
			new_buffer.priority = setid.last_priority;
		}
	else
		new_buffer.priority = setid.priority;
		
		/* Get confirm receipt from identity */
	if (setid.confirm_rcpt == -1)
		{
		if (setid.last_rcpt != -1)
			new_buffer.confirm_rcpt = setid.last_rcpt;
		}
	else
		new_buffer.confirm_rcpt = setid.confirm_rcpt;
		
	if (setid.confirm_read == -1)
		{
		if (setid.last_read != -1)
			new_buffer.confirm_read = setid.last_read;
		}
	else
		new_buffer.confirm_read = setid.confirm_read;

	strcpy(new_buffer.signature, setid.signature);
	}
else
	{
	new_buffer.mime_allowed = FALSE;
	new_buffer.editwidth = 78;
	}

if (!has_identity || !setid.reply_to1[0])	
	strcpy(new_buffer.replyto, new_buffer.from);
else
	{
	strcpy(new_buffer.replyto, setid.reply_to1);
	strcat(new_buffer.replyto, setid.reply_to2);
	}
	
entry->sel |= 4;
save_box_index(box);
forward_mode = FALSE;
edit_mode = FALSE;
open_mailwindow();
}
