/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"
#include "..\include\color.h"

static int strgs[] 		= {FWD_TEXT, REP_TEXT, AUTHTEXT,
												 CONTACT,  CLOSECON, SEND_TXT,  
												 POP_CONN, RCV, RCVDEL,  	
												 DEL, MAILCHK, VIEWMAIL,  
												 SAVE_ATT, PROCESSI, ANALYSE, 	
												 SENDING, ENCRYPT, UNSECURE, 	
												 GETMINF, DISCONN, ATT_TEXT,  
												 MFOLLOWS, UNKN_ATT, FWD_MSG,
												 S_MSGS, S_MSG, S_MRKD,
												 S_SCANHD, S_RETR, S_RETR2, S_DELE,
												 S_EDIT, S_FWDM, S_FWD, 
												 SENDPERC, S_MESSGS, S_MESSG, S_NOMSSG,
												 S_MARKED, S_PAGE, FAV_TITLE,

												 H_FROM, H_CC, H_BCC, H_REPLYTO, H_SUBJECT, H_TO,
												 H_IDENTITY, H_SIGNATURE,	ALL_STRING, NONE_STRING,
												 DEF_IDSTRING, LANGUAGE_CODE, IDFOLDER_LABEL, IDFOLDER_ENTRY,
#ifdef MILAN
												 HOMEEDIT,
#endif
												 };

static char **strs[] 	= {&fwd_text, &reply_text, &auth_text,
												 &connect_msg, &disconnect_msg, &send_text,
												 &popconn_text, &rcv_text, &rcvdel_text,
												 &del_text, &mailchk_text, &viewmail_text,
												 &saveatt_text, &processing_text, &analysing,
												 &sending_text, &encrypt_text, &unsecure_text,
												 &getminf_text, &disconn_text, &attachm_text,
												 &follows_text, &attachinf_text, &forwardmsg_text,
												 &s_msgs, &s_msg, &s_mrkd,
												 &s_scanhd, &s_retr, &s_retr2, &s_dele,
												 &s_edit, &s_fwdm, &s_fwd,
												 &send_perc, &s_messgs, &s_messg, &s_nomssg,
												 &s_marked, &s_page, &fav_title, 
												 &h_from, &h_cc, &h_bcc, &h_replyto, &h_subject, &h_to,
												 &h_identity, &h_signature, &all_string, &none_string,
												 &defid_string, &languagecode, &idfolder_label, &idfolder_entry,
												 
#ifdef MILAN
												 &homeedition,
#endif
												 };

static int alertid[] = {INVAL_I, INVALCNF, 
												NODRIVER, YOFFLINE, TOMANYW,  
												CONNFAIL, PSERVUNK, BOXEXIST, 
												SMTPUNK, BIGATT, SOCKFAIL, 
												NONEWMSG, NOBINATT, SMTPCONN, 
												BOXFAIL, R_NOMARK, RD_NOMRK,  
												D_NOMARK, SCANERR, NOADDR,   	
												NOSUBJ, FILEERR, DELFOLDQ,
												SCRAPERR, DELMSG, WASERROR,
												PRINTMSG, FILEOERR, NOTPALET,
												CHGD, DELETETHIS, DELETENODE,
												UNKNOWN_SERVER,
#ifdef MILAN												
												ISONE, NOMILAN,
#endif
												};

static char **alerts[]={&inval_box, &invalid_conf,
												&nodriver_msg, &offline_msg, &tomanywin_msg,
												&connfail_msg, &pservice_msg, &boxexists_msg,
												&smtp_unk_msg, &bigatt_msg, &sockfail_msg,
												&nonews_msg, &nobinatt_msg, &smtpprob_msg,
												&boxfail_msg, &rnomark_msg, &rdnomark_msg,
												&dnomark_msg, &scanerr_msg, &noaddress_text,
												&nosubject_text, &filec_error, &delfoldq,
												&scrap_error, &deletemsg, &waserror,
												&printmsg, &fileoerr, &notpalet, &changed,
												&deletethis, &deletenode, &unknownserver,
#ifdef MILAN												
												&isone,	&nomilan,
#endif
												};

static int opttabs[] = {MOPT1, MOPT2, MOPT3, MOPT4};

static int cdecl iconified_obj(PARMBLK *parm);
static USERBLK iconified_ublk 	= {iconified_obj, 0L};

/* ----------------------------
   | User object address book |
   ---------------------------- */
static int cdecl iconified_obj(PARMBLK *parm)
{
int xy[4];

DEBUG();
obj_clip(parm, TRUE);

vswr_mode(vdi_handle, MD_REPLACE);
vsf_interior(vdi_handle, FIS_HOLLOW);
xy[0] = parm->pb_x;
xy[1] = parm->pb_y;
xy[2] = parm->pb_x + parm->pb_w - 2;
xy[3] = parm->pb_y + parm->pb_h - 1;
vr_recfl(vdi_handle, xy);

icon_copy(symbols, 0, (smallfont ? 8 : 16) * 6, 
				 	parm->pb_x + (parm->pb_w - 20) / 2, 
				 	parm->pb_y + (parm->pb_h - (smallfont ? 8 : 16)) / 2, 
					20, smallfont ? 8 : 16);

obj_clip(parm, FALSE);
vswr_mode(vdi_handle, MD_REPLACE);

return 0;
}

/* -----------------------
   | Initialize resource |
   ----------------------- */
int init_rsc(void)
{
int i;
OBJECT *strings;
OBJECT *tree, *tab_x;
GRECT fc;

DEBUG();
if (!rsrc_load("MARATHON.RSC"))
	{
	form_alert(1,"[3][Resource-file|'MARATHON.RSC' missing.][Oops!]");
	return FALSE;
	}
	
rsrc_gaddr(R_TREE, LIMITS, &limits);
rsrc_gaddr(R_TREE, STRINGS, &strings);

/* Alerts */
for (i = 0; i < sizeof(alertid) / sizeof(int); i++)
	rsrc_gaddr(R_STRING, alertid[i],  alerts[i]);

/* Strings */
for (i = 0; i < sizeof(strgs) / sizeof(int); i++)
	*(strs[i]) = strings[strgs[i]].ob_spec.free_string;
	
rsrc_gaddr(R_TREE, HEADER, &header_tree);

#ifdef EVAL
	rsrc_gaddr(R_TREE, 	 NOTDEMO,   &notdemo);
#endif

/* Combine multipage dialogs */
option_tree = NULL;
rsrc_gaddr(R_TREE, MOPTIONS, &tree);
for (i = 0; i < 4; i++)
	{
	rsrc_gaddr(R_TREE, opttabs[i],	&tab_x);
	if (!i)
		opt_pages[i] = CombineDialogs(&option_tree, tree, tab_x);
	else
		opt_pages[i] = CombineDialogs(&option_tree, option_tree, tab_x);
	}

rsrc_gaddr(R_TREE, CSELPOP, &tree);
create_cpopup( tree);

rsrc_gaddr(R_TREE, FIND_OPT, &find_tree);
find_tree[FND_WORD].ob_spec.tedinfo->te_ptext[0] = 0;

rsrc_gaddr(R_TREE, REPL_OPT, &repl_tree);
repl_tree[REPL_WORD].ob_spec.tedinfo->te_ptext[0] = 0;
repl_tree[REPL_WORD_SET].ob_spec.tedinfo->te_ptext[0] = 0;

rsrc_gaddr(R_TREE, EDITORSELECT, &editorselect);
form_center(editorselect, &fc);

rsrc_gaddr(R_TREE, ICONIFIED_TREE, &iconified_tree);
form_center(iconified_tree, &fc);

iconified_tree[1].ob_type = G_USERDEF;
iconified_tree[1].ob_spec.userblk = &iconified_ublk;

return TRUE;
}
