/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

/* Local prototype */
int cdecl signat_obj(PARMBLK *parm);

static USERBLK signat_ublk 	= {signat_obj, 0L};
static int signature_chg, signature_page, selected_signature, cfg_signature = -1;
static int signature_count = 0;
static SIGNATURE *signatures;
static OBJECT *signature_conf;
static int sigedits[] = {	SIG_01, SIG_02, SIG_03, SIG_04, SIG_05,
													SIG_06, SIG_07, SIG_08, SIG_09, SIG_10,
													SIG_11, SIG_12, SIG_13};

static SIGNATURE defaultentry = {"\0", 0L, NULL};

static int slist_ids[] = {SIGNATURELIST,
													NEW_SIGNATURE,
													EDIT_SIGNATURE,
													DEL_SIGNATURE,
													CLOSE_SIGNATURE,
													0};
static int sconf_ids[] = {SIGNATURELABEL,
													SIG_01,
													SIG_02,
													SIG_03,
													SIGEDIT_CANCEL,
													SIGEDIT_OKAY,
													0};

/* --------------------------------------------------
   | Get identity pointer (for short time use only) |
   -------------------------------------------------- */
SIGNATURE *getSignaturePtr(char *sig)
{
int i;

DEBUG();
for (i = 0; i < signature_count; i++)
	if (!strcmp(signatures[i].name, sig))
		return signatures + i;
return NULL;
}

/* -------------------------------
	 | Fill identity configuration |
	 ------------------------------- */
void fill_sig_conf(int num)
{
SIGNATURE *cfg;
int i, p, e;
char *s;

DEBUG();
if (num == -1)
	{
	cfg = &defaultentry;
	signature_conf[SIGNATURELABEL].ob_spec.tedinfo->te_ptext[0] = 0;
	}
else
	{
  cfg = signatures + num;
  strcpy(signature_conf[SIGNATURELABEL].ob_spec.tedinfo->te_ptext, cfg->name);
  }
  
for (i = 0; i < 13; i++)
	signature_conf[sigedits[i]].ob_spec.tedinfo->te_ptext[0] = 0;

i = 0;
p = 0;
e = 0;
s = cfg->sig;
while(p < cfg->len)
	{
	if (s[p] == '\r')
		{
		signature_conf[sigedits[i]].ob_spec.tedinfo->te_ptext[e] = 0;
		e = 0;
		i++;
		if (i == 13) return;
		}
	else
		{
		signature_conf[sigedits[i]].ob_spec.tedinfo->te_ptext[e++] = s[p];
		}
	p++;
	}
signature_conf[sigedits[i]].ob_spec.tedinfo->te_ptext[e] = 0;
}

/* ------------------------------
	 | Get identity configuration |
	 ------------------------------ */
void store_sig_conf(int num)
{
SIGNATURE newcfg;
char *buf = malloc(13 * 64 + 1);
int i;

DEBUG();
strcpy(newcfg.name, signature_conf[SIGNATURELABEL].ob_spec.tedinfo->te_ptext);

buf[0] = 0;
for (i = 0; i < 13; i++)
	{
	strcat(buf, signature_conf[sigedits[i]].ob_spec.tedinfo->te_ptext);
	strcat(buf, "\r");
	}
i = (int)strlen(buf);
while(i > 0 && buf[i - 1] == '\r')
	buf[--i] = 0;
if (i)
	{
	strcat(buf, "\r");
	i++;
	buf = (char *)realloc(buf, i + 1);
	}
else
	free(buf);
	
newcfg.sig = buf;
newcfg.len = i;	
if (num != -1)
	{
	if (signatures[num].len > 0) free(signatures[num].sig);
	memcpy(signatures + num, &newcfg, sizeof(SIGNATURE));
	}
else
	{
	if (!signature_count)
		signatures = (SIGNATURE *)malloc(sizeof(SIGNATURE));
	else
		signatures = (SIGNATURE *)realloc(signatures, sizeof(SIGNATURE) * (signature_count + 1));
		
	memcpy(signatures + signature_count, &newcfg, sizeof(SIGNATURE));
	signature_count++;
	}
}

/* ----------------------------------------
   | Handle Identity configuration events |
   ---------------------------------------- */
static void cdecl sigedit_handler (OBJECT *tree, int obj)
{
DEBUG();
switch(obj)
	{
	case SIGEDIT_OKAY:
		{
		OBJECT *siglist;
		
		signature_chg = TRUE;
		store_sig_conf(cfg_signature);

		rsrc_gaddr(R_TREE, SIGNATURES, &siglist);

		if (is_open(siglist))
			redraw_object(siglist, FILTERLIST);
		}
		
	case SIGEDIT_CANCEL:
		close_dialog(tree, TRUE);
		break;
	}
}

/* --------------------------
	 | Identity configuration |
	 -------------------------- */
void config_signature(int num)
{
DEBUG();
if (!signature_conf)
	{
	rsrc_gaddr(R_TREE, SIGEDIT, &signature_conf);
	}

cfg_signature = num;
open_dialog(signature_conf, NULL, iconified_tree);
dlgset_help_info(signature_conf, helpfile, sconf_ids, "SCNF_");
fill_sig_conf(num);
set_active(signature_conf, sigedit_handler, TRUE, 0);
}

/* --------------------------
   | Handle Identity events |
   -------------------------- */
static void cdecl signature_handler (OBJECT *tree, int obj)
{
DEBUG();
switch(obj)
	{
	case SIGNATURELIST:
		{
		int ox, oy, pos;
		EVNTDATA ev;
		
		objc_offset(tree, SIGNATURELIST, &ox, &oy);
		graf_mkstate(&ev);
		
		pos = (ev.y - oy) / tx_height + signature_page;
		if (pos >= 0 && pos < signature_count)
			{
			if (selected_signature == -1)
				{
				EnableObj(tree, DEL_SIGNATURE);
				EnableObj(tree, EDIT_SIGNATURE);
				redraw_object(tree, DEL_SIGNATURE);
				redraw_object(tree, EDIT_SIGNATURE);
				}
			selected_signature = pos;
			}
		else
			{
			if (selected_signature != -1)
				{
				DisableObj(tree, DEL_SIGNATURE);
				DisableObj(tree, EDIT_SIGNATURE);
				redraw_object(tree, DEL_SIGNATURE);
				redraw_object(tree, EDIT_SIGNATURE);
				}
			selected_signature = -1;
			}
		redraw_object(tree, SIGNATURELIST);
		break;
		}

	case NEW_SIGNATURE:
		config_signature(-1);
		break;
		
	case EDIT_SIGNATURE:
		config_signature(selected_signature);
		break;

	case DEL_SIGNATURE:
		{
		if (cfg_signature == selected_signature && is_open(signature_conf))
			close_dialog(signature_conf, TRUE);
			
		if (cfg_signature > selected_signature) cfg_signature--;
		if (signature_count > 1 && selected_signature < signature_count - 1)
			memmove(signatures + selected_signature, signatures + selected_signature + 1,
							sizeof(SIGNATURE) * (signature_count - selected_signature - 1)) ;
		else
			free(signatures);	
		signature_count--;
		signature_chg = TRUE;
		selected_signature = -1;
		redraw_object(tree, SIGNATURELIST);
		break;
		}
		
	case CLOSE_SIGNATURE:
		save_signatures();
		close_dialog(tree, TRUE);
		break;
	}
}


/* ----------------------------
   | User object address book |
   ---------------------------- */
int cdecl signat_obj(PARMBLK *parm)
{
int xy[4], i, y, lines;
int dummy;

DEBUG();
obj_clip(parm, TRUE);

vst_height(vdi_handle, default_fsize, &dummy, &dummy, &dummy, &dummy);
vswr_mode(vdi_handle, MD_REPLACE);
vsf_interior(vdi_handle, FIS_HOLLOW);
xy[0] = parm->pb_x;
xy[1] = parm->pb_y;
xy[2] = parm->pb_x + parm->pb_w - 2;
xy[3] = parm->pb_y + parm->pb_h - 1;
vr_recfl(vdi_handle, xy);

lines = (parm->pb_tree[SIGNATURELIST].ob_height + tx_height - 1) / tx_height;

y = xy[1];
for (i = signature_page; i < signature_count && i < signature_page + lines; i++, y += tx_height)
	{
	v_gtext(vdi_handle, parm->pb_x + 2 * tx_width, y, signatures[i].name);

	if (selected_signature == i)
		{
		vswr_mode(vdi_handle, MD_XOR);
		vsf_interior(vdi_handle, FIS_SOLID);
		vsf_color(vdi_handle, 1);

		xy[0] = parm->pb_x;
		xy[1] = y;
		xy[2] = parm->pb_x + parm->pb_w - 2;
		xy[3] = y + tx_height - 1;
		vr_recfl(vdi_handle, xy);
		vswr_mode(vdi_handle, MD_TRANS);
		}
	}

obj_clip(parm, FALSE);
vswr_mode(vdi_handle, MD_REPLACE);

return 0;
}

/* -------------------
   | Edit signatures |
   ------------------- */
void edit_signatures(void)
{
OBJECT *signaturelist;

DEBUG();
rsrc_gaddr(R_TREE, SIGNATURES, &signaturelist);

if (!is_open(signaturelist))
	{
	if (signaturelist[SIGNATURELIST].ob_type != G_USERDEF)
		{
		signaturelist[SIGNATURELIST].ob_type = G_USERDEF;
		signaturelist[SIGNATURELIST].ob_spec.userblk = &signat_ublk;
	
		signature_page = 0;
		selected_signature = -1;
		DisableObj(signaturelist, DEL_SIGNATURE);
		DisableObj(signaturelist, EDIT_SIGNATURE);
		}
		
	align_slider_objects(signaturelist, SIG_SLIDER, SIG_UP, SIG_SLDBKG, SIG_SLDBOX, SIG_DOWN);
	set_slider_objects(signaturelist, SIG_SLDBKG, SIG_SLDBOX, signature_count, signature_page, 
										 signaturelist[SIGNATURELIST].ob_height / tx_height, FALSE);	
	}
		
open_dialog(signaturelist, NULL, iconified_tree);
dlgset_help_info(signaturelist, helpfile, slist_ids, "SLST_");
set_active(signaturelist, signature_handler, TRUE, 0);
}

/* -------------------
   | Select a folder |
   ------------------- */
int select_signature(int x, int y, int w, char *sig)
{
char **strings;
char *buffer, *ptr;
int i, ret, num;

DEBUG();
num = signature_count;
strings = (char **)malloc(sizeof(char *) * (num + 1));
if (!strings)
	return FALSE;
	
ptr = buffer = (char *)malloc(num * 21);
if (!buffer)
	{
	free(strings);
	return FALSE;
	}
	
for (i = 0; i < num; i++)
	{
	strings[i] = ptr;
	strcpy(ptr, signatures[i].name);
	ptr += 21;
	}
strings[i] = NULL;

ret = free_droplist(x, y, w, strings);
if (ret != -1)
	strcpy(sig, strings[ret]);

free(buffer);	
free(strings);

return ret != -1;
}

/* ------------------------
   | Load identities file |
   ------------------------ */
void load_signatures(void)
{
long id_fl;
char buf[256];
int i;

DEBUG();
if (signature_count)
	{
	for (i = 0; i < signature_count; i++)
		if (signatures[i].len > 0)
			free(signatures[i].sig);
			
	free(signatures);
	}
	
base_path(buf);
strcat(buf, "SIGNAT.DAT");

id_fl = Fopen(buf, FO_READ);
if (id_fl > 0)
	{
	long ver;
	
	Fread((int)id_fl, sizeof(ver), &ver);
	Fread((int)id_fl, sizeof(signature_count), &signature_count);
	if (signature_count)
		{
		signatures = (SIGNATURE *)malloc(sizeof(SIGNATURE) * 
																		   signature_count);
																		   
		Fread((int)id_fl, sizeof(SIGNATURE) * signature_count,
				  signatures);
				  
		for (i = 0; i < signature_count; i++)
			{
			if (signatures[i].len > 0)
				{
				signatures[i].sig = (char *)malloc(signatures[i].len + 1);
				Fread((int)id_fl, signatures[i].len, signatures[i].sig);
				signatures[i].sig[signatures[i].len] = 0;
				}
			}
		}
		
	Fclose((int)id_fl);
	}
else
	{
	signature_count = 0;
	if (mail_options._signature_file[0])
		{
		long sig_fh, sig_len, lc;
		
		sig_fh = Fopen(mail_options._signature_file, FO_READ);
		if (sig_fh > 0)
			{
			sig_len = Fseek(0L, (int)sig_fh, 2);
			Fseek(0L, (int)sig_fh, 0);
			
			signatures = (SIGNATURE *)malloc(sizeof(SIGNATURE));
			signatures->sig = (char *)malloc(sig_len + 1);
			
			Fread((int)sig_fh, sig_len, signatures->sig);
			
			lc = 0;
			while(lc < sig_len)
				{
				if (signatures->sig[lc] == '\n')
					{
					if (sig_len - lc - 1)	/* 63 */
						memmove(signatures->sig + lc, 
										signatures->sig + lc + 1,
										sig_len - lc - 1);
					sig_len--;
					}
				else
					lc++;
				}
			Fclose((int)sig_fh);
			signatures->len = sig_len;
			strcpy(signatures->name, "Default");
			signature_count = 1;
			
			mail_options._signature_file[0] = 0;
			signature_chg = TRUE;
			return;
			}
		}
	}
	
signature_chg = FALSE;
}

/* ------------------------
   | Save identities file |
   ------------------------ */
void save_signatures(void)
{
long id_fl;
char buf[256];
int i;

DEBUG();
if (!signature_chg) return;

base_path(buf);
strcat(buf, "SIGNAT.DAT");
id_fl = Fcreate(buf, 0);
if (id_fl > 0)
	{
	long ver = VERSION_ID20;
	
	Fwrite((int)id_fl, sizeof(ver), &ver);
	Fwrite((int)id_fl, sizeof(signature_count), &signature_count);
	if (signature_count)
		{
		Fwrite((int)id_fl, sizeof(SIGNATURE) * signature_count,
					 signatures);

		for (i = 0; i < signature_count; i++)
			{
			if (signatures[i].len > 0)
				Fwrite((int)id_fl, signatures[i].len, signatures[i].sig);
			}
		}
	Fclose((int)id_fl);
	signature_chg = FALSE;
	}
}
