/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"
#include "md5glbal.h"
#include "md5.h"

void MDPrint(unsigned char *digest, char *dest);

/* ----------------------
   | Create CRAM Answer |
   ---------------------- */
void create_cram(unsigned char *challenge, int chal_len, 
								 unsigned char *password, int pass_len,
								 char *buffer)
{
MD5_CTX ctx;
unsigned char response[16];
int i;
unsigned char ipad[64];
unsigned char opad[64];
unsigned char hash_passwd[16];
int resp_len = 16;
	
if (pass_len > sizeof (ipad))
  {
  MD5Init (&ctx);
  MD5Update (&ctx, password, pass_len);
  MD5Final (hash_passwd, &ctx);
  password = hash_passwd; pass_len = sizeof (hash_passwd);
  }

memset (ipad, 0, sizeof (ipad));
memset (opad, 0, sizeof (opad));
memcpy (ipad, password, pass_len);
memcpy (opad, password, pass_len);

for (i = 0; i < 64; i++) 
 	{
  ipad[i] ^= 0x36;
  opad[i] ^= 0x5c;
  }

MD5Init (&ctx);
MD5Update (&ctx, ipad, sizeof (ipad));
MD5Update (&ctx, challenge, chal_len);
MD5Final (response, &ctx);

MD5Init (&ctx);
MD5Update (&ctx, opad, sizeof (opad));
MD5Update (&ctx, response, resp_len);
MD5Final (response, &ctx);
    
MDPrint(response, buffer);
}

/* ---------------
   | Plain login |
   --------------- */
int cram_login(int smtp_s, IDENTITY *identity)
{
int e;
char buf[256];
char buf2[256];

send_cmd(smtp_s, "AUTH", "CRAM-MD5", NULL, NULL);
e = get_response(smtp_s, buf);
if (e == 334)
	{
	char *ptr = buf + 3;
	
	while (isspace(*ptr))
		ptr++;
		
	DecodeBase64Line(buf2, ptr);

	create_cram((unsigned char *)buf2, (int)strlen(buf2), 
							(unsigned char *)identity->smtppass, (int)strlen(identity->smtppass),
							buf);

	strlwr(buf);
	strcpy(buf2, identity->smtpuser);
	strcat(buf2, " ");
	strcat(buf2, buf);
	
	EncodeBase64Line(buf, buf2);

	send_cmd(smtp_s, buf, NULL, NULL, NULL);
	}

e = get_response(smtp_s, buf);
if (e == 235)
	return TRUE;

store_error("SMTP", e, buf);
return FALSE;
}

/* ---------------
   | Plain login |
   --------------- */
int login_login(int smtp_s, IDENTITY *identity)
{
int e;
char buf[256];

send_cmd(smtp_s, "AUTH", "LOGIN", NULL, NULL);
e = get_response(smtp_s, buf);
if (e == 334)
	{
	EncodeBase64Line(buf, identity->smtpuser);
	send_cmd(smtp_s, buf, NULL, NULL, NULL);
	}
	
e = get_response(smtp_s, buf);
if (e == 334)
	{
	EncodeBase64Line(buf, identity->smtppass);
	send_cmd(smtp_s, buf, NULL, NULL, NULL);
	}

e = get_response(smtp_s, buf);
if (e == 235)
	return TRUE;

store_error("SMTP", e, buf);
return FALSE;
}

/* ---------------
   | Plain login |
   --------------- */
int plain_login(int smtp_s, IDENTITY *identity)
{
int e;
char buf[80], buf2[80 * 4 / 3 + 4];
int len;
	
buf[0] = 0;
strcpy(buf + 1, identity->smtpuser);
	
len = 1 + (int)strlen(identity->smtpuser) + 1;
strcpy(buf + len, identity->smtppass);
len += (int)strlen(identity->smtppass);
	
EncodeBase64Memory(buf2, buf, len);
send_cmd(smtp_s, "AUTH", "PLAIN ", buf2, NULL);

e = get_response(smtp_s, buf);
if (e == 235)
	return TRUE;

store_error("SMTP", e, buf);
return FALSE;
}

/* -----------------------
   | SMTP authentication |
   ----------------------- */
int smtp_auth(int smtp_s, IDENTITY *identity, int type)
{
switch (type)
	{
	case 1:
		return plain_login(smtp_s, identity);
		
	case 2:
		return login_login(smtp_s, identity);
		
	case 3:
		return cram_login(smtp_s, identity);
	}
return FALSE;
}
