/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

static int sort_order;

/* ------------------------
   | Compare 'from' field |
   ------------------------ */
static int compare_from(void *a, void *b)
{
if (sort_order == -1)
	return -stricmp(((MAIL_INFO *)a)->from, ((MAIL_INFO *)b)->from);
else	
	return stricmp(((MAIL_INFO *)a)->from, ((MAIL_INFO *)b)->from);
}

/* ----------------------
   | Compare 'to' field |
   ----------------------- */
static int compare_to(void *a, void *b)
{
if (sort_order == -1)
	return -stricmp(((MAIL_INFO *)a)->to, ((MAIL_INFO *)b)->to);
else	
	return stricmp(((MAIL_INFO *)a)->to, ((MAIL_INFO *)b)->to);
}

/* ---------------------------
   | Compare 'subject' field |
   --------------------------- */
static int compare_subject(void *a, void *b)
{
if (sort_order == -1)
	return -stricmp(((MAIL_INFO *)a)->subject, ((MAIL_INFO *)b)->subject);
else	
	return stricmp(((MAIL_INFO *)a)->subject, ((MAIL_INFO *)b)->subject);
}

/* ------------------------
   | Compare 'size' field |
   ------------------------ */
static int compare_size(void *a, void *b)
{
long a_len, b_len;

a_len = ((MAIL_INFO *)a)->body_len;
b_len = ((MAIL_INFO *)b)->body_len;

if (a_len < b_len)
	return -sort_order;
else
	if (a_len > b_len)
		return sort_order;
	else
		return 0;
}

/* -----------------------
   | Analyse date values |
   ----------------------- */
void get_datevalues(char *date, int *r_day, int *r_month, int *r_year,
										int *r_hour, int *r_min, int *r_sec)
{
char weekday[20];
int  day, j;
char month[20];
int  year, mon;
int  hour,minute, second;
static char *mons[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
											 "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

if (date[0] == 0x02)
	{
	sscanf(date + 1, "%d %[A-Za-z] %d,%d:%d", 
				 &day, month, &year, &hour, &minute);
	second = 0;
	}						
else
	{
	if (sscanf(date, "%[^0-9]%d %[A-Za-z]%d %d:%d:%d", 
						 weekday, &day, month, &year, &hour, &minute, &second) < 6)
		{
		j = 5;
	  while(date[j] && date[j++] != ','); 	/* find start of date */
			sscanf(date + j, "%[^0-9]%d %[A-Za-z]%d %d:%d:%d", 
						 weekday, &day, month, &year, &hour, &minute, &second);				
		}
	}
mon = 0;
while(mon < 12 && strnicmp(mons[mon], month, 3))
	mon++;

mon++;
if (mon < 1 || mon > 12)	mon = 1;	/* Safety */
if (year < 47) year += 2000;
if (year < 100) year += 1900;

*r_year = year;
*r_month = mon;
*r_day = day;
*r_hour = hour;
*r_min = minute;
*r_sec = second;
}

/* ------------------------
   | Compare 'date' field |
   ------------------------ */
static int compare_date(void *a, void *b)
{
char *a_date, *b_date;
int a_day, a_month, a_year;
int b_day, b_month, b_year;
int a_hour, a_minute, a_second;
int b_hour, b_minute, b_second;

a_date = ((MAIL_INFO *)a)->date;
b_date = ((MAIL_INFO *)b)->date;

get_datevalues(a_date, &a_day, &a_month, &a_year, &a_hour, &a_minute, &a_second);
get_datevalues(b_date, &b_day, &b_month, &b_year, &b_hour, &b_minute, &b_second);

if (a_year < b_year)
	return -sort_order;
else
	if (a_year > b_year)
		return sort_order;

if (a_month < b_month)
	return -sort_order;
else
	if (a_month > b_month)
		return sort_order;

if (a_day < b_day)
	return -sort_order;
else
	if (a_day > b_day)
		return sort_order;
		
if (a_hour < b_hour)
	return -sort_order;
else
	if (a_hour > b_hour)
		return sort_order;
		
if (a_minute < b_minute)
	return -sort_order;
else
	if (a_minute > b_minute)
		return sort_order;
		
if (a_second < b_second)
	return -sort_order;
else
	if (a_second > b_second)
		return sort_order;
		
return 0;		
}

/* ----------------------------
   | Compare 'identity' field |
   ---------------------------- */
static int compare_identity(void *a, void *b)
{
if (sort_order == -1)
	return -stricmp(((MAIL_INFO *)a)->identity, ((MAIL_INFO *)b)->identity);
else	
	return stricmp(((MAIL_INFO *)a)->identity, ((MAIL_INFO *)b)->identity);
}

/* --------------
   | Sort a box |
   -------------- */
void sort_box(int box)
{
static int (*compar[])(void *a, void *b) = 
		{compare_from, compare_to, compare_subject, compare_size,
		 compare_date, compare_identity};

if (!box_manager[box].box_sort) return;

if (box_manager[box].box_sort < 0)
	sort_order = -1;
else
	sort_order = 1;

qsort(box_manager[box].index, box_manager[box].box_entries, 
			sizeof(MAIL_INFO),
			compar[box_manager[box].box_sort * sort_order - 1]);
}

