/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

static int	tree_icony_flg = FALSE;
static char tree_title[90];
static int *boxlist, boxlist_size;

void tree_popup(int box);
void tree_idpopup(int box);
static void tree_upline(void);
static void tree_downline(void);
static void tree_uppage(void);
static void tree_downpage(void);
static void tree_vslide(int pos);

/* -------------------------------------------
	 | is checknode a child of basenode or one |
	 | of it's subnodes?                       |
	 ------------------------------------------- */
int isChildOf(int basenode, int checknode)
{
DEBUG();
if (boxes[basenode].type == -2)
	{
	int lvl;
					
	lvl = boxes[basenode++].level;
	while (basenode < box_count && boxes[basenode].level > lvl)
		{
		if (basenode == checknode)
			return TRUE;
		basenode++;
		}
	}
	
return FALSE;
}

/* ------------------------------------
   | Enable tree related menu entries |
   ------------------------------------ */
void tree_enable(void)
{
int has_def = hasDefaultIdentity();

DEBUG();
menu_ienable(menu_adr, M_SCAN, 			(tree_selection != -1 && boxes[tree_selection].type == -1) || has_def);
menu_ienable(menu_adr, M_SENDCHECK, (tree_selection != -1 && boxes[tree_selection].type == -1) || has_def);
menu_ienable(menu_adr, M_CHECK, 		(tree_selection != -1 && boxes[tree_selection].type == -1) || has_def);
menu_ienable(menu_adr, M_SEND, 			(tree_selection != -1 && boxes[tree_selection].type == -1) || has_def);
	
menu_ienable(menu_adr, M_DEL, 			tree_selection != -1 && tree_selection != box_number("IN") &&
																		tree_selection != box_number("SENT") &&  tree_selection != box_number("SENTERR") &&
																	  tree_selection != box_number("QUEUE"));

#ifndef MILAN
	menu_ienable(menu_adr, M_NFOLD, tree_selection != -1);
#endif
}

/* ---------------
   | Tree window |
   --------------- */
void open_tree(void)
{
GRECT win, wout, wrec;

DEBUG();
wind_get_grect(ROOT, WF_WORKXYWH, &wrec);

win.g_x = win.g_y = 0;
win.g_w = 22 * tx_width;
win.g_h = 0;
wind_calc(WC_BORDER, NAME|CLOSER|MOVER|SIZER|
										 UPARROW|DNARROW|VSLIDE|SMALLER,
										 &win, &wout);
					
tree_wind = wind_create(NAME|CLOSER|MOVER|SIZER|
											 UPARROW|DNARROW|VSLIDE|SMALLER, &wrec);
if (tree_wind < 0)
	return;
	
strcpy(tree_title, mail_options.mailfile);											 
wind_set_string(tree_wind, WF_NAME, tree_title);

if (tree_wd[0] == -1)
	{
	tree_wd[0] = wrec.g_x; tree_wd[1] = wrec.g_y;
	tree_wd[2] = wout.g_w; tree_wd[3] = wrec.g_h;
	}

if (tree_wd[0] < 0)	tree_wd[0] = 0;
if (tree_wd[1] < 0)	tree_wd[1] = 0;
	
if (tree_wd[1] < wrec.g_y)
	tree_wd[1] = wrec.g_y;
if (tree_wd[3] > wrec.g_h)
	tree_wd[3] = wrec.g_h;
if (tree_wd[0] > wrec.g_x + wrec.g_w)
	tree_wd[0] = wrec.g_x + wrec.g_w - wout.g_w;
if (tree_wd[1] > wrec.g_y + wrec.g_h)	
	tree_wd[1] = wrec.g_y + wrec.g_h - tree_wd[3];
	
wind_open(tree_wind, (GRECT *)&tree_wd[0]);
register_window(tree_wind);
}

/* ---------------------
   | Close tree window |
   --------------------- */
void close_tree(void)
{
DEBUG();
wind_get(tree_wind, WF_CURRXYWH, &tree_wd[0], &tree_wd[1], &tree_wd[2], &tree_wd[3]);
wind_close(tree_wind);
unregister_window(tree_wind);
wind_delete(tree_wind);
tree_wind = -1;
}

/* ---------------------------------------
   | Build a list with the visible nodes |
   --------------------------------------- */
static void build_visible_boxlist(void)
{
int i, *bp;

DEBUG();
bp = boxlist = (int *)malloc(sizeof(int) * box_count);

i = 0;
while (i < box_count)
	{
	*(bp++) = i;
	
	if (boxes[i].type == -2 && boxes[i].expanded == FALSE)
		{
		int lvl;
					
		lvl = boxes[i++].level;
		while (i < box_count && boxes[i].level > lvl)
			i++;
/*		i--;*/
		}
	else
		i++;
	}
boxlist_size = bp - boxlist;
}

/* --------------------------
   | Redraw box icon window |
   -------------------------- */
void redraw_treewind(int rx, int ry, int rw, int rh)
{
GRECT t1, t2;
int xy[4], top;
int wx, wy, ww, wh;
int i, y;
int size, oldsize;
int pos, oldpos, iconheight;

DEBUG();
wind_get(tree_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
if (rx == -1)
	{
	t2.g_x = wx;
	t2.g_y = wy;
	t2.g_w = ww;
	t2.g_h = wh;
	}
else
	{
	t2.g_x = rx;
	t2.g_y = ry;
	t2.g_w = rw;
	t2.g_h = rh;
	}
	
wind_update(BEG_UPDATE);
graf_mouse(M_OFF, 0L);

set_default_text();

build_visible_boxlist();

top = 0;
while(boxlist[top] < tree_top)
	top++;
if (top > boxlist_size - wh / tx_height)
	{
	top = boxlist_size - wh / tx_height;
	if (top < 0) top = 0;
	tree_top = boxlist[top];
	}
	
wind_get(tree_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
iconheight = icon_palheader.height;

wind_get(tree_wind, WF_FIRSTXYWH, &t1.g_x, &t1.g_y, &t1.g_w, &t1.g_h);
while(t1.g_w && t1.g_h)
	{
	if (rc_intersect(&t2, &t1))
		{
		xy[0] = t1.g_x;
		xy[1] = t1.g_y;
		xy[2] = t1.g_x + t1.g_w - 1;
		xy[3] = t1.g_y + t1.g_h - 1;
		vs_clip(vdi_handle, 1, xy);

		if (tree_icony_flg)
			{
			iconified_tree[0].ob_x = wx;
			iconified_tree[0].ob_y = wy;
			iconified_tree[0].ob_width = ww;
			iconified_tree[0].ob_height = wh;
			
			iconified_tree[1].ob_x = (ww - iconified_tree[1].ob_width) >> 1;
			iconified_tree[1].ob_y = (wh - iconified_tree[1].ob_height) >> 1;

			objc_draw(iconified_tree, 0, MAX_DEPTH, &t1);
			}
		else
			{
			vswr_mode(vdi_handle, MD_REPLACE);
			vsf_interior(vdi_handle, FIS_HOLLOW);
			vr_recfl(vdi_handle, xy);
			vswr_mode(vdi_handle, MD_TRANS);
	
			i = top;
			y = wy + 2;
			
			while (y < xy[3] && i < boxlist_size)
				{
				int idx = boxlist[i];
				int sel = (tree_selection == idx);
				
				v_gtext(vdi_handle, wx + 20 * boxes[idx].level + 25, y, boxes[idx].desc);
				
				if (boxes[idx].type == -2)
					{
					if (boxes[idx].expanded == FALSE)
						{
						icon_copy(boxicons, 0, iconheight * 27, 
										 	wx + 20 * boxes[idx].level + 2, y, 
										 	20, smallfont ? 8 : 16);
						}
					else
						icon_copy(boxicons, 0, iconheight * 28, 
										 	wx + 20 *  boxes[idx].level + 2, y, 
										 	20, smallfont ? 8 : 16);
					}
				else
					icon_copy(boxicons, 0, iconheight * (28 + boxes[idx].icon), 
									 wx + 20 * boxes[idx].level + 2, y, 
									 20, smallfont ? 8 : 16);
	
				if (sel)
					{
					int xy[4];
					
					vswr_mode(vdi_handle, MD_XOR);
					vsf_interior(vdi_handle, FIS_SOLID);
					vsf_color(vdi_handle, 1);
			
					xy[0] = wx + 20 * boxes[idx].level + 2;
					xy[1] = y;
					xy[2] = wx + ww;
					xy[3] = y + tx_height - 1;
					vr_recfl(vdi_handle, xy);
					vswr_mode(vdi_handle, MD_TRANS);
					}
			
				y += tx_height;
				i++;
				}
			}
			
		vs_clip(vdi_handle, 0, xy);
		}

	wind_get(tree_wind, WF_NEXTXYWH, &t1.g_x, &t1.g_y, &t1.g_w, &t1.g_h);
	}

graf_mouse(M_ON, 0L);
wind_update(END_UPDATE);

if (boxlist_size * tx_height <= wh)
	{
	pos = 0;
	size = 1000;
	}
else
	{
	pos = (int)(1000L * top / (boxlist_size - wh / tx_height));
	size = (wh / tx_height) * 1000 / boxlist_size;
	}

wind_get_int(tree_wind, WF_VSLIDE, &oldpos);
wind_get_int(tree_wind, WF_VSLSIZE, &oldsize);

if (size != oldsize || pos != oldpos)
	{
	wind_set_int(tree_wind, WF_VSLIDE, pos);
	wind_set_int(tree_wind, WF_VSLSIZE, size);
	}

free(boxlist);
}

/* ------------------------------
   | Process box window message |
   ------------------------------ */
void tree_window_message(int *msg)
{
int wx, wy, ww, wh;

DEBUG();
wind_get(tree_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);

switch(msg[0])
	{
	case WM_REDRAW:
		redraw_treewind(msg[4], msg[5], msg[6], msg[7]);
		break;
		
	case WM_CLOSED:
		close_box();
		break;
	
	case WM_ICONIFY:
	case WM_ALLICONIFY:
		tree_icony_flg = TRUE;
		wind_set(msg[3], WF_ICONIFY, msg[4], msg[5], msg[6], msg[7]);
		break;

	case WM_UNICONIFY:
		tree_icony_flg = FALSE;
		wind_set(msg[3], WF_UNICONIFY, msg[4], msg[5], msg[6], msg[7]);
		break;
				
	case WM_ARROWED:
		{
		switch(msg[4])
			{
			case WA_UPPAGE:
				{
				tree_uppage();
				break;
				}
				
			case WA_DNPAGE:
				{
				tree_downpage();
				break;
				}
				
			case WA_UPLINE:
				{
				tree_upline();
				break;
				}
				
			case WA_DNLINE:
				{
				tree_downline();
				break;
				}
			}
		break;
		}
		
	case WM_VSLID:
		tree_vslide(msg[4]);
		break;
	}			
}

/* ----------------
   | Get tree row |
   ---------------- */
int get_treerow(int box)
{
int i, r;

DEBUG();
i = tree_top;
r = 0;
while (i < box)
	{
	if (boxes[i].type == -2)
		{
		if (boxes[i].expanded == FALSE)
			{
			int lvl;
					
			lvl = boxes[i++].level;
			while (i < box_count && boxes[i].level > lvl)
				i++;
			i--;
			}
		}

	r++;
	i++;
	}
return r;
}

/* -----------------
   | View drag box |
   ----------------- */
static void view_treedrag(int x, int y, int w, int h, int *cxy)
{
int ln[10];

vs_clip(vdi_handle, TRUE, cxy);
vswr_mode(vdi_handle, MD_XOR);
graf_mouse(M_OFF, 0L);

ln[0] = ln[6] = ln[8] = x;
ln[2] = ln[4] = x + w - 1;

ln[1] = ln[3] = ln[9] = y;
ln[5] = ln[7] = y + h - 1;

vsl_color(vdi_handle, 1);
v_pline(vdi_handle, 5, ln);

graf_mouse(M_ON, 0L);
vswr_mode(vdi_handle, MD_REPLACE);
vs_clip(vdi_handle, FALSE, cxy);
}

/* ---------------------------------
   | Drag inside tree or favorites |
   --------------------------------- */
static void drag_tree(int drag_box)
{
int last_x = evnt.x, last_y = evnt.y;
int sel_box, w, cxy[4], ext[8];
int mw2, mh2;
int wx, wy, ww, wh;
int mwind;

DEBUG();
wind_get(tree_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);

set_default_text();
vqt_extent(vdi_handle, boxes[drag_box].desc, ext);
w = abs(ext[2] - ext[0]) + 25;
mw2 = w >> 1;
mh2 = tx_height >> 1;

wind_get(ROOT, WF_WORKXYWH, &cxy[0], &cxy[1], &cxy[2], &cxy[3]);
cxy[2] += cxy[0] - 1;
cxy[3] += cxy[1] - 1;

wind_update(BEG_MCTRL);
view_treedrag(evnt.x - mw2, evnt.y - mh2, w, tx_height, cxy);
if (boxes[drag_box].fav_index == -1)
	sel_box = box_move(evnt.x + mw2, evnt.y + mh2);
else
	sel_box = -1;
do
	{
/*  graf_mkstate( &evnt);*/
	which = evnt_multi(MU_BUTTON|MU_TIMER,
									  1, 1, 0,
										0, &empty,
										0, &empty,
										NULL,
										50L,
										&evnt,
										&k_return, &clicks);
			
	if (last_x != evnt.x || last_y != evnt.y)
		{
		int last_selbox = sel_box;
		
		view_treedrag(last_x - mw2, last_y - mh2, w, tx_height, cxy);

		mwind = wind_find(evnt.x, evnt.y);
		
		if (mwind == box_wind)
			{
			if (boxes[drag_box].fav_index == -1)
				sel_box = box_move(evnt.x + mw2, evnt.y + mh2);

			if (sel_box != last_selbox)
				{
				if (sel_box == -1)
					graf_mouse(BUSYBEE, 0L);
				else
					graf_mouse(POINT_HAND, 0L);
				}
			}
		else
			if (mwind == tree_wind)
				{
				int i, r, row;
				
				row = (evnt.y - wy - 2) / tx_height;

				i = tree_top;
				r = 0;
				while (i < box_count && r < row)
					{
					if (boxes[i].type == -2)
						{
						if (boxes[i].expanded == FALSE)
							{
							int lvl;
									
							lvl = boxes[i++].level;
							while (i < box_count && boxes[i].level > lvl)
								i++;
							i--;
							}
						}
				
					r++;
					i++;
					}

				if (i < box_count && r == row)
					{
					if (i != tree_selection)
						{
						if (tree_selection != -1)
							{
							int rrow = get_treerow(tree_selection);
							tree_selection = -1;
							redraw_treewind(wx, wy + 2 + rrow * tx_height, ww, tx_height);
							}
							
						if (!isChildOf(drag_box, i))
							{
							tree_selection = i;
							redraw_treewind(wx, wy + 2 + row * tx_height, ww, tx_height);
							}
						}
					}
				else
					if (tree_selection != -1)
						{
						int rrow = get_treerow(tree_selection);
						tree_selection = -1;
						redraw_treewind(wx, wy + 2 + rrow * tx_height, ww, tx_height);
						}
			}
				
		view_treedrag(evnt.x - mw2, evnt.y - mh2, w, tx_height, cxy);
		last_x = evnt.x;
		last_y = evnt.y;
		}

	}while(! (which & MU_BUTTON));			

view_treedrag(last_x - mw2, last_y - mh2, w, tx_height, cxy);
graf_mouse(ARROW, 0L);
wind_update(END_MCTRL);

mwind = wind_find(evnt.x, evnt.y);
if (mwind == box_wind)
	{
	if (boxes[drag_box].fav_index == -1)
		{
		int i, idx;
		
		sel_box = box_move(evnt.x + mw2, evnt.y + mh2);
		if (sel_box != -1)
			idx = boxes[sel_box].fav_index;
		else
			idx = number_of_favorites();

		for (i = 0; i < box_count; i++)
			if (boxes[i].fav_index >= idx)
				boxes[i].fav_index++;
		
		boxes[drag_box].fav_index = idx;
		update_boxinfo();
		redraw_boxwind(-1, -1, -1, -1);
		}
	}
else
	if (mwind == tree_wind && tree_selection != drag_box)
		{
		BOX_INFO *zw_info;
		BOX_MANAGER *zw_manag;
		int num, olvl, i, lvl;
		
		if (boxes[drag_box].type == -2)
			{
			num = 1;								
			i = drag_box;
			lvl = boxes[i++].level;
			while (i < box_count && boxes[i].level > lvl)
				{
				num++;
				i++;
				}
			}
		else
			num = 1;
			
		zw_info = (BOX_INFO *)malloc(sizeof(BOX_INFO) * num);
		zw_manag = (BOX_MANAGER *)malloc(sizeof(BOX_MANAGER) * num);
		
		memcpy(zw_info, boxes + drag_box, sizeof(BOX_INFO) * num);
		memcpy(zw_manag, box_manager + drag_box, sizeof(BOX_MANAGER) * num);

		memmove(boxes + drag_box, boxes + drag_box + num, sizeof(BOX_INFO) * (box_count - drag_box - num));
		memmove(box_manager + drag_box, box_manager + drag_box + num, sizeof(BOX_MANAGER) * (box_count - drag_box - num));
		if (tree_selection > drag_box) tree_selection -= num;
		
		if (tree_selection != -1)
			if (boxes[tree_selection].type == -2)
				{
				i = tree_selection;
				lvl = boxes[i++].level;
				while (i < box_count - num && boxes[i].level > lvl)
					i++;
				
				if (i < box_count - num)
					{
					memmove(boxes + i + num, boxes + i, sizeof(BOX_INFO) * (box_count - i - num));
					memmove(box_manager + i + num, box_manager + i, sizeof(BOX_MANAGER) * (box_count - i - num));
					}
				memcpy(boxes + i, zw_info, sizeof(BOX_INFO) * num);
				memcpy(box_manager + i, zw_manag, sizeof(BOX_MANAGER) * num);
				
				olvl = boxes[i].level;
				while(num--)
					{
					boxes[i].level = lvl + 1 + (boxes[i].level - olvl);
					i++;
					}
				}
			else
				{
				memmove(boxes + tree_selection + num, boxes + tree_selection, sizeof(BOX_INFO) * (box_count - tree_selection - num));
				memmove(box_manager + tree_selection + 1, box_manager + tree_selection, sizeof(BOX_MANAGER) * (box_count - tree_selection - num));
				memcpy(boxes + tree_selection, zw_info, sizeof(BOX_INFO) * num);
				memcpy(box_manager + tree_selection, zw_manag, sizeof(BOX_MANAGER) * num);

				i = tree_selection;
				lvl = boxes[i + num].level;
				olvl = boxes[i].level;
				while(num--)
					{
					boxes[i].level = lvl + (boxes[i].level - olvl);
					i++;
					}
				}
		else
			{
			i = box_count - num;
			memcpy(boxes + i, zw_info, sizeof(BOX_INFO) * num);
			memcpy(box_manager + i, &zw_manag, sizeof(BOX_MANAGER) * num);

			olvl = boxes[i].level;
			while(num--)
				{
				boxes[i].level = (boxes[i].level - olvl);
				i++;
				}
			}
		free(zw_info);
		free(zw_manag);
		redraw_treewind(-1, -1, -1, -1);
		}
}

/* ------------------------------
	 | Mouse button event on tree |
	 ------------------------------ */
int tree_select(int x, int y)
{
int row, i, r;
int wx, wy, ww, wh;

(void)x;

DEBUG();
wind_get(tree_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
row = (y - wy - 2) / tx_height;

i = tree_top;
r = 0;
while (i < box_count && r < row)
	{
	if (boxes[i].type == -2)
		{
		if (boxes[i].expanded == FALSE)
			{
			int lvl;
					
			lvl = boxes[i++].level;
			while (i < box_count && boxes[i].level > lvl)
				i++;
			i--;
			}
		}

	r++;
	i++;
	}

if (i < box_count && r == row)
	{
	if (i != tree_selection)
		{
		if (tree_selection != -1)
			{
			int rrow = get_treerow(tree_selection);
			tree_selection = -1;
			redraw_treewind(wx, wy + 2 + rrow * tx_height, ww, tx_height);
			}
		tree_selection = i;
		redraw_treewind(wx, wy + 2 + row * tx_height, ww, tx_height);
		}
	}
else
	if (tree_selection != -1)
		{
		tree_selection = -1;
		redraw_treewind(-1, -1, -1, -1);
		}
return tree_selection;
}
	
/* ------------------------------
	 | Mouse button event on tree |
	 ------------------------------ */
void tree_button(void)
{
int row, i, r;
int wx, wy, ww, wh;

DEBUG();
wind_get(tree_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);
row = (evnt.y - wy - 2) / tx_height;

i = tree_top;
r = 0;
while (i < box_count && r < row)
	{
	if (boxes[i].type == -2)
		{
		if (boxes[i].expanded == FALSE)
			{
			int lvl;
					
			lvl = boxes[i++].level;
			while (i < box_count && boxes[i].level > lvl)
				i++;
			i--;
			}
		}

	r++;
	i++;
	}

if (i < box_count && r == row)
	{
	if (evnt.bstate & 2)
		{
		if (i != -1 && boxes[i].type == -1)
			tree_idpopup(i);
		else
			tree_popup(i);
		}
	else
		if (clicks == 2)
			{
			if (boxes[i].type == -2)
				{
				boxes[i].expanded ^= TRUE;
				redraw_treewind(-1, -1, -1, -1);
				}
			else
				if (boxes[i].type == -1)
					{
					if (!send_idqueue(boxes[i].identity, TRUE))
						check_id(boxes[i].identity, -1);
					}
				else
					list_window(i);
			}
		else
			{
			if (i != tree_selection)
				{
				if (tree_selection != -1)
					{
					int rrow = get_treerow(tree_selection);
					tree_selection = -1;
					redraw_treewind(wx, wy + 2 + rrow * tx_height, ww, tx_height);
					}
				tree_selection = i;
				redraw_treewind(wx, wy + 2 + row * tx_height, ww, tx_height);
				}
			
			check_mouse();
			if ((evnt.bstate & 1) && tree_selection >= 0)
				drag_tree(tree_selection);
			}
	}
else
	{
	if (evnt.bstate & 2)
		{
	/*	tree_popup(-1);*/
		}
	else	
		if (tree_selection != -1)
			{
			tree_selection = -1;
			redraw_treewind(-1, -1, -1, -1);
			}
	}
tree_enable();
}

/* ------------------------------
   | Add tree item to favorites |
   ------------------------------ */
static void tree_add2favor(int box)
{
int i, idx;

DEBUG();
idx = 0;
for (i = 0; i < box_count; i++)
	if (boxes[i].fav_index >= idx)
		idx = boxes[i].fav_index + 1;

boxes[box].fav_index = idx;
update_boxinfo();
redraw_boxwind(-1, -1, -1, -1);
}
		
/* -------------------------
   | Remove item from tree |
   ------------------------- */
void tree_delete(int box)
{
DEBUG();
if (box == -1)
	if (tree_selection == -1)
		return;
	else
		box = tree_selection;
		
if (form_alert(2, deletenode) == 1)
	{
	int lvl = boxes[box].level;
	int i, upd_box;

	if (boxes[box].fav_index > -1)
		{
		i = 0; 
		while (i < box_count)
			{
			if (boxes[i].fav_index > boxes[box].fav_index)
				boxes[i].fav_index--;
			i++;
			}
		upd_box = TRUE;
		}
	else
		upd_box = FALSE;
		
	i = box + 1;
	while (i < box_count && boxes[i].level > lvl)
		i++;
	i--;
	while (i >= box)
		{
		if (boxes[i].type >= 0)
			{
			if (delete_folder(boxes[i].file))
				free_index(i);
			else
				break;
			}

		if (i < box_count - 1)
			{
			memmove(boxes + i, boxes + i + 1, sizeof(BOX_INFO) * (box_count - i - 1));
			memmove(box_manager + i, box_manager + i + 1, sizeof(BOX_MANAGER) * (box_count - i - 1));
			}
		move_filterfolders(i + 1, -1);
		box_count--;

		if (box_top > 0)
			box_top--;		
		
		i--;
		}
		
	redraw_treewind(-1, -1, -1, -1);
	if (upd_box)
		{
		update_boxinfo();
		redraw_boxwind(-1, -1, -1, -1);
		}
	}
}

/* ----------------
   | Popup folder |
   ---------------- */
void tree_popup(int box)
{
OBJECT *tree_pop;

DEBUG();
rsrc_gaddr(R_TREE, TREE_POPUP, &tree_pop);

if (box == -1)
	{
	DisableObj(tree_pop, TREE_DELETE);
	DisableObj(tree_pop, TREE_EXPAND);
	DisableObj(tree_pop, TREE_OPEN);
	DisableObj(tree_pop, TREE_PROPS);
	DisableObj(tree_pop, TREE_FAVOR);
	DisableObj(tree_pop, TREE_EDITFILTER);
	}
else
	{
	if (box == box_number("QUEUE"))
		{
		EnableObj(tree_pop, TREE_SEND);
		DisableObj(tree_pop, TREE_DELETE);
		DisableObj(tree_pop, TREE_EDITFILTER);
		DisableObj(tree_pop, TREE_APPLYFILTER);
		}
	else
		{
		DisableObj(tree_pop, TREE_SEND);
		if (box == box_number("IN") || box == box_number("SENT") ||
				box == box_number("SENTERR"))
			DisableObj(tree_pop, TREE_DELETE);
		else
			EnableObj(tree_pop, TREE_DELETE);
		EnableObj(tree_pop, TREE_EDITFILTER);
		EnableObj(tree_pop, TREE_APPLYFILTER);
		}
	
	if (boxes[box].type == -2)
		{
		EnableObj(tree_pop, TREE_EXPAND);
		DisableObj(tree_pop, TREE_OPEN);
		DisableObj(tree_pop, TREE_EDITFILTER);
		DisableObj(tree_pop, TREE_APPLYFILTER);
		}
	else
		{
		EnableObj(tree_pop, TREE_OPEN);
		DisableObj(tree_pop, TREE_EXPAND);
		}
	
	if (boxes[box].fav_index == -1 && boxes[box].type != -2)
		EnableObj(tree_pop, TREE_FAVOR);
	else
		DisableObj(tree_pop, TREE_FAVOR);

	EnableObj(tree_pop, TREE_PROPS);
	}
			
switch(popup(tree_pop))
	{
	case TREE_NEW:
		folder_properties(box | 0x8000);
		break;

	case TREE_OPEN:
		list_window(box);
		break;

	case TREE_PROPS:
		folder_properties(box);
		update_boxinfo();
		break;

	case TREE_DELETE:
		{
		tree_delete(box);
		break;
		}
		
	case TREE_FAVOR:
		{
		tree_add2favor(box);
		break;
		}

	case TREE_SEND:
		send_idqueue("\0", TRUE);
		break;
		
	case TREE_EDITFILTER:
		edit_filters(box);
		break;
		
	case TREE_APPLYFILTER:
		apply_filter(box);
		break;
		
	case TREE_EXPAND:
		boxes[box].expanded ^= TRUE;
		redraw_treewind(-1, -1, -1, -1);
		break;
	}
}

/* ----------------
   | Popup folder |
   ---------------- */
void tree_idpopup(int box)
{
OBJECT *tree_pop;

DEBUG();
rsrc_gaddr(R_TREE, TREE_IDPOPUP, &tree_pop);

if (boxes[box].fav_index == -1)
	EnableObj(tree_pop, TREE_IDFAVOR);
else
	DisableObj(tree_pop, TREE_IDFAVOR);
		
switch(popup(tree_pop))
	{
	case TREE_IDNEW:
		folder_properties(box | 0x8000);
		break;

	case TREE_OPEN:
		list_window(box);
		break;

	case TREE_IDPROPS:
		folder_properties(box);
		update_boxinfo();
		break;

	case TREE_SCANID:
		check_id(boxes[box].identity, FALSE);
		break;
		
	case TREE_SENDID:
		send_idqueue(boxes[box].identity, TRUE);
		break;
		
	case TREE_CHECKID:
		check_id(boxes[box].identity, TRUE);
		break;
		
	case TREE_BOTHID:
		if (!send_idqueue(boxes[box].identity, TRUE))
			check_id(boxes[box].identity, -1);
		break;

	case TREE_IDDELETE:
		{
		tree_delete(box);
		break;
		}
		
	case TREE_IDFAVOR:
		{
		tree_add2favor(box);
		break;
		}
	}
}

/* --------------------------
   | Create new tree folder |
   -------------------------- */
void tree_createfolder(void)
{
DEBUG();
if (tree_selection != -1)
	folder_properties(tree_selection | 0x8000);
}

/* --------------------
   | Tree one line up |
   -------------------- */
static void tree_upline(void)
{
int wx, wy, ww, wh;
int i;

DEBUG();
wind_get(tree_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);

build_visible_boxlist();

if (tree_top > boxlist[0])
	{
	i = boxlist_size - 1;
	while(boxlist[i] > tree_top)
		i--;

	if (tree_top == boxlist[i]) i--;

	tree_top = boxlist[i];		
	redraw_treewind(-1, -1, -1, -1);
	}
free(boxlist);
}

/* ----------------------
   | Tree one line down |
   ---------------------- */
static void tree_downline(void)
{
int wx, wy, ww, wh;
int i;

DEBUG();
wind_get(tree_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);

build_visible_boxlist();

if (boxlist_size > wh / tx_height && tree_top < boxlist[boxlist_size - wh / tx_height])
	{
	i = 0;
	while(boxlist[i] < tree_top)
		i++;

	if (tree_top == boxlist[i]) i++;

	tree_top = boxlist[i];		
	redraw_treewind(-1, -1, -1, -1);
	}
free(boxlist);
}

/* --------------------
   | Tree one page up |
   -------------------- */
static void tree_uppage(void)
{
int wx, wy, ww, wh;
int i;

DEBUG();
wind_get(tree_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);

build_visible_boxlist();

if (tree_top > boxlist[0])
	{
	i = boxlist_size - 1;
	while(boxlist[i] > tree_top)
		i--;

	if (tree_top == boxlist[i]) 
		i -= wh / tx_height;
	else
		i -= (wh / tx_height) - 1;
	
	if (i < 0) i = 0;
	
	tree_top = boxlist[i];		
	redraw_treewind(-1, -1, -1, -1);
	}
free(boxlist);
}

/* ----------------------
   | Tree one line down |
   ---------------------- */
static void tree_downpage(void)
{
int wx, wy, ww, wh;
int i;

DEBUG();
wind_get(tree_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);

build_visible_boxlist();

if (boxlist_size > wh / tx_height && tree_top < boxlist[boxlist_size - wh / tx_height])
	{
	i = 0;
	while(boxlist[i] < tree_top)
		i++;

	if (tree_top == boxlist[i]) 
		i += wh / tx_height;
	else
		i += (wh / tx_height) - 1; 

	if (i > boxlist_size - wh / tx_height)		i = boxlist_size - wh / tx_height;
	if (i < 0) i = 0;
	
	tree_top = boxlist[i];		
	redraw_treewind(-1, -1, -1, -1);
	}
free(boxlist);
}

/* ----------------------
   | Tree one line down |
   ---------------------- */
static void tree_vslide(int pos)
{
int wx, wy, ww, wh;
int i;

DEBUG();
wind_get(tree_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);

build_visible_boxlist();

if (boxlist_size > wh / tx_height)
	{
	i = (int)((long)pos * (boxlist_size - wh / tx_height) / 1000);
	tree_top = boxlist[i];		
	redraw_treewind(-1, -1, -1, -1);
	}
free(boxlist);
}


